Package: salmonMSE
Title: Management Strategy Evaluation for Salmon Species
Version: 2.1.0
Authors@R: 
    person("Quang", "Huynh", , "quang@bluematterscience.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-7835-4376"))
Date: 2026-04-20
Maintainer: Quang Huynh <quang@bluematterscience.com>
Description: Simulation tools to evaluate the long-term effects of salmon management strategies, including a combination of habitat, harvest, and
  habitat actions. The stochastic age-structured operating model accommodates complex life histories, including freshwater survival across 
  early life stages, juvenile survival and fishery exploitation in the marine life stage, partial maturity by age class, and fitness impacts of 
  hatchery programs on natural spawning populations. 'salmonMSE' also provides an age-structured conditioning model to develop operating models 
  fitted to data.
License: GPL (>= 3)
Depends: R (>= 3.5.0)
Imports: abind, MSEtool (>= 3.7.2), RTMB (>= 1.9), dplyr, ggplot2,
        grDevices, gsl, methods, reshape2, rlang, rmarkdown, stats,
        utils
Suggests: ggrepel, rstan, scales, testthat (>= 3.0.0), tmbstan
Encoding: UTF-8
LazyData: yes
LazyLoad: yes
RoxygenNote: 7.3.3
URL: https://docs.salmonmse.com/,
        https://github.com/Blue-Matter/salmonMSE
BugReports: https://github.com/Blue-Matter/salmonMSE/issues
NeedsCompilation: no
Packaged: 2026-04-24 22:04:44 UTC; quang
Author: Quang Huynh [aut, cre] (ORCID: <https://orcid.org/0000-0001-7835-4376>)
Repository: CRAN
Date/Publication: 2026-04-25 09:40:13 UTC
Built: R 4.5.2; ; 2026-05-01 18:29:27 UTC; unix

Package: salmonMSE
Title: Management Strategy Evaluation for Salmon Species
Version: 3.0.0
Authors@R: 
    person("Quang", "Huynh", , "quang@bluematterscience.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-7835-4376"))
Date: 2026-08-18
Maintainer: Quang Huynh <quang@bluematterscience.com>
Description: Simulation tools to evaluate the long-term effects of salmon management strategies, including a combination of habitat, harvest, and
  habitat actions. The stochastic age-structured operating model accommodates complex life histories, including freshwater survival across 
  early life stages, juvenile survival and fishery exploitation in the marine life stage, partial maturity by age class, and fitness impacts of 
  hatchery programs on natural spawning populations. 'salmonMSE' also provides an age-structured conditioning model to develop operating models 
  fitted to data.
License: GPL (>= 3)
Depends: R (>= 4.1.0)
Imports: abind, RTMB (>= 1.9), dplyr, ggplot2, grDevices, gsl, methods,
        reshape2, rlang, rmarkdown, parallel, stats, utils
Suggests: bookdown, EnvStats, ggrepel, knitr, rstan, scales, testthat
        (>= 3.0.0), tmbstan
Encoding: UTF-8
LazyData: yes
LazyLoad: yes
RoxygenNote: 7.3.3
URL: https://docs.salmonmse.com/,
        https://github.com/Blue-Matter/salmonMSE
BugReports: https://github.com/Blue-Matter/salmonMSE/issues
NeedsCompilation: no
Packaged: 2026-08-19 23:29:45 UTC; quang
Author: Quang Huynh [aut, cre] (ORCID: <https://orcid.org/0000-0001-7835-4376>)
Repository: CRAN
Date/Publication: 2026-08-20 10:00:02 UTC
Built: R 4.5.2; ; 2026-08-20 12:11:03 UTC; unix

# Generated by roxygen2: do not edit by hand

export(AHA)
export(CM2SOM)
export(CM_CWT_ER)
export(CM_ER)
export(CM_F)
export(CM_M)
export(CM_MSY)
export(CM_Megg)
export(CM_Njuv)
export(CM_SRR)
export(CM_Sgen)
export(CM_Srep)
export(CM_covariate)
export(CM_esc)
export(CM_fit_CWTcatch)
export(CM_fit_CWTesc)
export(CM_fit_esc)
export(CM_fit_pHOS)
export(CM_int)
export(CM_maturity)
export(CM_pairs)
export(CM_prod)
export(CM_recr)
export(CM_surv)
export(CM_surv2)
export(CM_trace)
export(CM_vul)
export(CM_wt)
export(CM_wto)
export(PNI50)
export(PNI80)
export(P_PNI50)
export(P_PNI80)
export(P_SMSY80)
export(P_SMSY85)
export(P_Sgen100)
export(P_WILD50)
export(ProjectSOM)
export(SMSY85)
export(SOM)
export(Sgen100)
export(WILD50)
export(calc_MSY)
export(calc_SRR)
export(calc_SRRpars)
export(calc_Sgen)
export(calc_Sgen_Ricker)
export(calc_Smsy_Ricker)
export(calc_Umsy_Ricker)
export(calc_pwild)
export(calc_ref)
export(calc_smolt)
export(check_SOM)
export(compare)
export(compare_escapement)
export(compare_fitness)
export(compare_spawners)
export(compare_statevar_hist)
export(compare_statevar_ts)
export(fit_CM)
export(get_CMdata)
export(get_report)
export(plot_Kobe)
export(plot_LHG)
export(plot_RS)
export(plot_decision_table)
export(plot_decision_table2)
export(plot_escapement)
export(plot_fishery)
export(plot_fitness)
export(plot_spawners)
export(plot_statevar_hist)
export(plot_statevar_ts)
export(plot_tradeoff)
export(report)
export(report_CM)
export(salmonMSE)
export(salmonMSE_env)
export(sample_CM)
export(simpleSOM)
export(simple_salmonMSE)
exportClasses(Bio)
exportClasses(Habitat)
exportClasses(Harvest)
exportClasses(Hatchery)
exportClasses(Historical)
exportClasses(SMSE)
exportClasses(SOM)
exportClasses(simpleSOM)
exportMethods(report)
import(RTMB)
import(ggplot2)
importFrom(abind,abind)
importFrom(dplyr,"%>%")
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,filter)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(grDevices,hcl.colors)
importFrom(grDevices,rainbow)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,box)
importFrom(graphics,grid)
importFrom(graphics,hist)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,matlines)
importFrom(graphics,matplot)
importFrom(graphics,pairs)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,rect)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(gsl,lambert_W0)
importFrom(methods,"slot<-")
importFrom(methods,.hasSlot)
importFrom(methods,.slotNames)
importFrom(methods,new)
importFrom(methods,slot)
importFrom(methods,slotNames)
importFrom(parallel,detectCores)
importFrom(parallel,makeCluster)
importFrom(parallel,parLapplyLB)
importFrom(parallel,parSapplyLB)
importFrom(parallel,stopCluster)
importFrom(rlang,.data)
importFrom(rlang,.env)
importFrom(rmarkdown,render)
importFrom(stats,cor)
importFrom(stats,density)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,nlminb)
importFrom(stats,optimize)
importFrom(stats,quantile)
importFrom(stats,rlnorm)
importFrom(stats,sd)
importFrom(utils,browseURL)
importFrom(utils,packageVersion)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# salmonMSE

> Management Strategy Evaluation for Salmon Species

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/salmonMSE)](https://cran.r-project.org/package=salmonMSE)
[![R-CMD-check](https://github.com/Blue-Matter/salmonMSE/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/Blue-Matter/salmonMSE/actions/workflows/R-CMD-check.yaml)
[![License: GPL (\>=
3)](https://img.shields.io/badge/license-GPL%20(%3E=%203)-blue.svg)](https://cran.r-project.org/web/licenses/GPL-3)

<!-- badges: end -->

salmonMSE is a decision-support tool for Pacific salmon focusing on
strategic trade-offs among harvest, hatchery and habitat management
levers.

Funding for the development of salmonMSE is provided through the
[Pacific Salmon Strategy
Initiative](https://www.dfo-mpo.gc.ca/campaign-campagne/pss-ssp/index-eng.html)
in collaboration with the Department of Fisheries and Oceans Canada.

## Installation

salmonMSE can be downloaded from [CRAN](https://CRAN.R-project.org)
with:

``` r
install.packages("salmonMSE")
```

And the development version is available on
[GitHub](https://github.com/) with:

``` r
# install.packages("remotes")
remotes::install_github("Blue-Matter/salmonMSE")
```

#' @import MSEtool
#' @importFrom dplyr %>% pull filter
#' @importFrom methods new slot slotNames slot<-
#' @importFrom stats rlnorm
#' @importFrom rlang .data .env
NULL

setClassUnion("num.array", c("numeric", "array"))
setClassUnion("num.logical", c("numeric", "logical"))
setClassUnion("num.matrix", c("numeric", "matrix"))
setClassUnion("num.function", c("numeric", "function"))

# ---- Bio Class -----
#' Class \code{"Bio"}
#'
#' The component of the operating model that controls biological dynamics, i.e., natural production.
#'
#' Various parameters can be stochastic (length `nsim`) or input as a single numeric
#' (value identical across all simulations).
#'
#' @name Bio-class
#' @docType class
#' @slot Name Character. Identifying name
#' @template Bio_template
#'
#' @section Creating Object:
#' Objects can be created by calls of the form \code{new("Bio")}
#'
#' @export
#' @keywords classes
#' @examples
#' showClass("Bio")
setClass(
  "Bio",
  slots = c(
    Name = "character",
    maxage = "numeric",
    n_g = "numeric",
    p_LHG = "numeric",
    p_mature = "num.array",
    SRrel = "character",
    capacity = "numeric",
    kappa = "numeric",
    Smax = "numeric",
    phi = "numeric",
    tau = "numeric",
    Mjuv_NOS = "num.array",
    fec = "num.array",
    p_female = "numeric",
    s_enroute = "numeric"
  )
)



# ---- Habitat Class -----
#' Class \code{"Habitat"}
#'
#' The component of the operating model that controls survival in the freshwater environment. Includes changes in survival from either
#' environmental/climate effects or habitat mitigation.
#'
#' @name Habitat-class
#' @docType class
#' @slot Name Character. Identifying name
#' @template Habitat_template
#'
#' @section Creating Object:
#' Objects can be created by calls of the form \code{new("Habitat")}
#'
#' @export
#' @keywords classes
#' @examples
#' showClass("Habitat")
setClass(
  "Habitat",
  slots = c(
    Name = "character",
    use_habitat = "logical",
    prespawn_rel = "character",
    prespawn_prod = "numeric",
    prespawn_capacity = "numeric",
    egg_rel = "character",
    egg_prod = "numeric",
    egg_capacity = "numeric",
    fry_rel = "character",
    fry_prod = "numeric",
    fry_capacity = "numeric",
    fry_sdev = "matrix",
    smolt_rel = "character",
    smolt_prod = "numeric",
    smolt_capacity = "numeric",
    smolt_sdev = "matrix"
  )
)

# ---- Harvest Class -----
#' Class \code{"Harvest"}
#'
#' The component of the operating model that controls marine harvest.
#'
#' @name Harvest-class
#' @docType class
#' @slot Name Character. Identifying name
#' @template Harvest_template
#'
#' @section Creating Object:
#' Objects can be created by calls of the form \code{new("Harvest")}
#'
#' @export
#' @keywords classes
#' @examples
#' showClass("Harvest")
setClass(
  "Harvest",
  slots = c(
    Name = "character",
    type_PT = "character",
    type_T = "character",
    u_preterminal = "numeric",
    u_terminal = "numeric",
    K_PT = "numeric",
    K_T = "numeric",
    MSF_PT = "logical",
    MSF_T = "logical",
    release_mort = "numeric",
    vulPT = "num.matrix",
    vulT = "num.matrix"
  )
)


# ---- Historical Class -----
#' Class \code{"Historical"}
#'
#' Optional component of the operating model that specifies the historical dynamics. The default option starts the projection with 1000
#' natural-origin juveniles and 1000 hatchery-origin juveniles (if there is hatchery production) in the oldest age class, representing
#' single-brood year returns since there is only one age class in the population. Specify the abundance in all age class to simulate multiple
#' brood-year returns.
#'
#' @name Historical-class
#' @docType class
#' @slot Name Character. Identifying name
#' @template Historical_template
#'
#' @section Creating Object:
#' Objects can be created by calls of the form \code{new("Historical")}
#'
#' @export
#' @keywords classes
#' @examples
#' showClass("Historical")
setClass(
  "Historical",
  slots = c(
    Name = "character",
    #InitNOS = "num.array",
    #InitHOS = "num.array",
    InitNjuv_NOS = "num.array",
    InitNjuv_HOS = "num.array"
  )
)


# ---- Hatchery Class -----
#' Class \code{"Hatchery"}
#'
#' The component of the operating model that controls the hatchery management and in-river removals.
#'
#' Various parameters can be stochastic (length `nsim`) or input as a single numeric
#' (value identical across all simulations).
#'
#' @name Hatchery-class
#' @docType class
#' @slot Name Character. Identifying name
#' @template Hatchery_template
#'
#' @section Creating Object:
#' Objects can be created by calls of the form \code{new("Hatchery")}
#'
#' @export
#' @keywords classes
#' @examples
#' showClass("Hatchery")
setClass(
  "Hatchery",
  slots = c(
    Name = "character",
    n_r = "numeric",
    n_yearling = "numeric",           # Management lever. No hatchery if both this line and next line are zero
    n_subyearling = "numeric",        # Management lever. No hatchery if both this line and previous line are zero
    yearling_DD = "logical",
    subyearling_DD = "logical",
    s_prespawn = "numeric",           # Survival prior to spawning
    s_egg_smolt = "numeric",          # Survival of eggs in hatchery
    s_egg_subyearling = "numeric",
    Mjuv_HOS = "num.array",
    p_mature_HOS = "num.array",
    stray_external = "matrix",
    gamma = "numeric",
    m = "numeric",
    f_brood = "function",
    brood_import = "numeric",
    pmax_esc = "numeric",
    pmax_NOB = "numeric",
    ptarget_NOB = "numeric",
    phatchery = "num.logical",
    premove_HOS = "num.function",
    premove_NOS = "num.function",
    fec_brood = "num.array",
    fitness_type = "character",
    theta = "numeric",
    rel_loss = "numeric",
    zbar_start = "num.array",
    phenotype_variance = "numeric",
    fitness_variance = "numeric",
    heritability = "numeric",
    fitness_floor = "numeric"
  )
)

#' @importFrom methods .hasSlot
setMethod("initialize", "Hatchery",
          function(.Object, ...) {
            dots <- list(...)
            for (i in names(dots)) {
              if (.hasSlot(.Object, i)) slot(.Object, i) <- dots[[i]]
            }
            if (!length(.Object@phatchery)) .Object@phatchery <- NA_real_
            return(.Object)
          })


setClassUnion("Bio.list", c("Bio", "list"))
setClassUnion("Habitat.list", c("Habitat", "list"))
setClassUnion("Hatchery.list", c("Hatchery", "list"))
setClassUnion("Harvest.list", c("Harvest", "list"))
setClassUnion("Historical.list", c("Historical", "list"))


# ---- SOM Class -----
#' Class \code{"SOM"}
#'
#' An object containing all the parameters for a salmon operating model (SOM).
#'
#' @name SOM-class
#' @section Objects from the Class: Objects can be created by calls of the form
#' \code{new("SOM", Bio, Habitat, Hatchery, Harvest, Historical)}.
#'
#' @slot Name Character. Identifying name
#' @slot nsim Integer. Number of simulations
#' @slot proyears Integer. The number of projected years
#' @slot seed Integer. A random seed to ensure users can reproduce results exactly
#' @slot Bio \linkS4class{Bio} object informing biological parameters and natural production. Provide a list of Bio objects for multi-population models.
#' @slot Habitat \linkS4class{Habitat} object containing management levers for controlling survival in the freshwater environment. Provide a list of Habitat objects for multi-population models.
#' @slot Hatchery \linkS4class{Hatchery} object containing management levers for hatchery production and in-river removals. Provide a list of Hatchery objects for multi-population models.
#' @slot Harvest \linkS4class{Harvest} object containing management levers for marine harvest. Provide a list of Harvest objects for multi-population models.
#' @slot Historical \linkS4class{Historical} object to inform historical reconstruction and informing starting abundance for the projection. Provide a list of Historical objects for multi-population models.
#' @slot stray Matrix `[np, np]` where `np = length(Bio)` and row `p` indicates the re-assignment of hatchery fish to each population when they mature (at the recruitment life stage). For example,
#' `SOM@stray <- matrix(c(0.75, 0.25, 0.25, 0.75), 2, 2)` indicates that 75 percent of mature fish return to their natal river and 25 percent stray in both populations. By default, an identity matrix is used (no straying).
#' @keywords classes
#'
#' @export
SOM <- setClass(
  "SOM",
  slots = c(
    Name = "character",
    nsim = "numeric",
    #nyears = "numeric",
    proyears = "numeric",
    seed = "numeric",
    Bio = "Bio.list",
    Habitat = "Habitat.list",
    Hatchery = "Hatchery.list",
    Harvest = "Harvest.list",
    Historical = "Historical.list",
    stray = "array"
  )
)

#' @importFrom utils packageVersion
setMethod("initialize", "SOM",
          function(.Object, Bio, Habitat, Hatchery, Harvest, Historical,
                   nsim = 3, proyears = 20, seed = 1, Name = "Salmon operating model", ...) {

            dots <- list(...)

            .Object@nsim <- nsim
            #.Object@nyears <- nyears
            .Object@proyears <- proyears
            .Object@seed <- seed

            if (!missing(Bio)) .Object@Bio <- Bio
            if (!missing(Habitat)) .Object@Habitat <- Habitat
            if (!missing(Hatchery)) .Object@Hatchery <- Hatchery
            if (!missing(Harvest)) .Object@Harvest <- Harvest
            if (!missing(Historical)) .Object@Historical <- Historical

            attr(.Object, "version") <- paste("salmonMSE", packageVersion("salmonMSE"), "with MSEtool", packageVersion("MSEtool"))
            attr(.Object, "date") <- date()
            attr(.Object, "R.version") <- getRversion()

            return(.Object)
          })


# ---- SMSE Class -----
#' Class \code{"SMSE"}
#'
#' Stores the outputs from the simulation of salmon operating models.
#'
#' @name SMSE-class
#' @docType class
#' @slot Name Character. Identifying name
#' @slot proyears Integer. The number of projected years
#' @slot nsim Integer. The number of simulations
#' @slot nstocks Integer. The number of stocks
#' @slot Snames Character. Stock names
#' @slot Egg_NOS Array `[nsim, nstocks, proyears]`. Spawning output, i.e., egg production, of natural origin spawners.
#' @slot Egg_HOS Array `[nsim, nstocks, proyears]`. Spawning output of hatchery origin spawners.
#' @slot Fry_NOS Array `[nsim, nstocks, proyears]`. Fry that are offspring of natural origin spawners.
#' @slot Fry_HOS Array `[nsim, nstocks, proyears]`. Fry that are offspring of hatchery origin spawners.
#' @slot Smolt_NOS Array `[nsim, nstocks, proyears]`. Smolts that are offspring of natural origin spawners.
#' @slot Smolt_HOS Array `[nsim, nstocks, proyears]`. Smolts that are offspring of hatchery origin spawners.
#' @slot Smolt_Rel Array `[nsim, nstocks, proyears]`. Smolts that are offspring of broodtake, i.e., hatchery releases.
#' @slot Njuv_NOS Array `[nsim, nstocks, nage, proyears]`. Abundance of juvenile natural origin fish at the beginning of the year.
#' @slot Njuv_HOS Array `[nsim, nstocks, nage, proyears]`. Abundance of juvenile hatchery origin fish at the beginning of the year.
#' @slot Return_NOS Array `[nsim, nstocks, nage, proyears]`. Mature fish that will be natural origin spawners.
#' @slot Return_HOS Array `[nsim, nstocks, nage, proyears]`. Mature fish that will be hatchery origin spawners.
#' @slot Escapement_NOS Array `[nsim, nstocks, nage, proyears]`. The escapement of mature fish that will be natural origin spawners.
#' @slot Escapement_HOS Array `[nsim, nstocks, nage, proyears]`. The escapement of mature fish that will be hatchery origin spawners.
#' @slot NOB Array `[nsim, nstocks, proyears]`. Natural origin broodtake.
#' @slot HOB Array `[nsim, nstocks, proyears]`. Hatchery origin broodtake (local + strays).
#' @slot HOB_stray Array `[nsim, nstocks, proyears]`. Hatchery origin broodtake (strays only).
#' @slot HOB_import Array `[nsim, nstocks, proyears]`. Imported hatchery origin broodtake used for hatchery production.
#' @slot NOS Array `[nsim, nstocks, nage, proyears]`. Natural origin spawners.
#' @slot HOS Array `[nsim, nstocks, nage, proyears]`. Hatchery origin spawners (local + strays).
#' @slot HOS_stray Array `[nsim, nstocks, nage, proyears]`. Hatchery origin spawners (strays only).
#' @slot HOS_effective Array `[nsim, nstocks, nage, proyears]`. Hatchery origin spawners (local + strays) discounted by `gamma`.
#' @slot KPT_NOS Array `[nsim, nstocks, proyears]`. Pre-terminal fishery kept catch of natural origin spawners.
#' @slot KT_NOS Array `[nsim, nstocks, proyears]`. Terminal fishery kept catch of natural origin spawners.
#' @slot KPT_HOS Array `[nsim, nstocks, proyears]`. Pre-terminal fishery kept catch of hatchery origin spawners.
#' @slot KT_HOS Array `[nsim, nstocks, proyears]`. Terminal fishery kept catch of hatchery origin spawners.
#' @slot DPT_NOS Array `[nsim, nstocks, proyears]`. Pre-terminal fishery released catch (live and dead) of natural origin spawners.
#' @slot DT_NOS Array `[nsim, nstocks, proyears]`. Terminal fishery released catch (live and dead) of natural origin spawners.
#' @slot DPT_HOS Array `[nsim, nstocks, proyears]`. Pre-terminal fishery released catch (live and dead) of hatchery origin spawners.
#' @slot DT_HOS Array `[nsim, nstocks, proyears]`. Terminal fishery released catch (live and dead) hatchery origin spawners.
#' @slot UPT_NOS Array `[nsim, nstocks, nage, proyears]`. Pre-terminal fishery harvest rate (from kept catch) of natural origin spawners.
#' @slot UT_NOS Array `[nsim, nstocks, nage, proyears]`. Terminal fishery harvest rate of natural origin spawners.
#' @slot UPT_HOS Array `[nsim, nstocks, nage, proyears]`. Pre-terminal fishery harvest rate of hatchery origin spawners.
#' @slot UT_HOS Array `[nsim, nstocks, nage, proyears]`. Terminal fishery harvest rate of hatchery origin spawners.
#' @slot ExPT_NOS Array `[nsim, nstocks, nage, proyears]`. Pre-terminal fishery exploitation rate (from kept catch and dead releases) of natural origin spawners.
#' @slot ExT_NOS Array `[nsim, nstocks, nage, proyears]`. Terminal fishery exploitation rate of natural origin spawners.
#' @slot ExPT_HOS Array `[nsim, nstocks, nage, proyears]`. Pre-terminal fishery exploitation rate of hatchery origin spawners.
#' @slot ExT_HOS Array `[nsim, nstocks, nage, proyears]`. Terminal fishery exploitation rate of hatchery origin spawners.
#' @slot fitness Array `[nsim, nstocks, 2, proyears]`. Fitness of the population in the natural (1) and hatchery (2) environments.
#' @slot pNOB Array `[nsim, nstocks, proyears]`. Proportion of natural fish in the brood.
#' @slot pHOS_census Array `[nsim, nstocks, proyears]`. Proportion of spawners of hatchery origin, weighted by age class fecundity.
#' @slot pHOS_effective Array `[nsim, nstocks, proyears]`. Proportion of spawners of hatchery origin, discounted by `gamma`, weighted by age class fecundity.
#' @slot PNI Array `[nsim, nstocks, proyears]`. Proportionate natural influence, index of gene flow from hatchery to the natural environment.
#' @slot p_wild Array `[nsim, nstocks, proyears]`. Proportion of wild spawners, natural spawners whose parents were also produced in the natural environment assuming
#' non-assortative mating, defined under Canada's Wild Salmon Policy.
#' @slot Mjuv_loss Array `[nsim, nstocks, nage, proyears]`. Realized juvenile natural mortality, which may differ from inputs due to fitness loss.
#' @slot Misc List. Miscellaneous output:
#'
#' - `Ref` for reference points
#' - `SHist` for the [salmonMSE::SHist-class] object (primarily for developer use)
#' - `SOM` for the [salmonMSE::SOM-class] object (updated by `check_SOM()`).
#' - `LHG` list `nstocks` long containing state variables by natural-origin life history group
#' - `RS` list `nstocks` long containing state variables by hatchery-origin release strategy
#' @details
#' In generation \eqn{t}, proportionate natural influence (PNI) is defined as:
#'
#' \deqn{\textrm{PNI}_t = \dfrac{p^\textrm{NOB}_t}{p^\textrm{NOB}_t + p^\textrm{HOSeff}_t}}
#'
#' with \eqn{p^\textrm{HOSeff} = \textrm{HOSeff}/(\textrm{NOS} + \textrm{HOSeff})}.
#'
#' The proportion of wild salmon is defined as:
#'
#' \deqn{p^{\textrm{WILD}}_t = q^\textrm{HOScen}_t
#' \dfrac{(q^\textrm{HOScen}_{t-1})^2}
#' {(q^\textrm{HOScen}_{t-1})^2 + 2\gamma \times p^\textrm{HOScen}_{t-1} q^\textrm{HOScen}_{t-1} +
#' \gamma^2 (p^\textrm{HOScen}_{t-1})^2}}
#'
#' where \eqn{q = 1-p} and \eqn{p^\textrm{HOScen} = \textrm{HOS}/(\textrm{NOS} + \textrm{HOS})}.
#'
#' @references
#' Withler et al. 2018. Genetically Based Targets for Enhanced Contributions to Canadian Pacific Chinook Salmon Populations.
#' DFO Can. Sci. Advis. Sec. Res. Doc. 2018/019. xii + 88 p.
#'
#' @section Creating Object:
#' Objects can be created by calls of the form \code{new("SMSE")}
#'
#' @export
#' @keywords classes
#' @examples
#' showClass("SMSE")
setClass(
  "SMSE",
  slots = c(
    Name = "character",
    proyears = "numeric",
    nsim = "numeric",
    nstocks = "numeric",
    Snames = "character",
    Egg_NOS = "array",
    Egg_HOS = "array",
    Fry_NOS = "array",
    Fry_HOS = "array",
    Smolt_NOS = "array",
    Smolt_HOS = "array",
    Smolt_Rel = "array",
    Njuv_NOS = "array",
    Njuv_HOS = "array",
    Return_NOS = "array",
    Return_HOS = "array",
    Escapement_NOS = "array",
    Escapement_HOS = "array",
    NOB = "array",
    HOB = "array",
    HOB_stray = "array",
    HOB_import = "array",
    NOS = "array",
    HOS = "array",
    HOS_stray = "array",
    HOS_effective = "array",
    KPT_NOS = "array",
    KT_NOS = "array",
    KPT_HOS = "array",
    KT_HOS = "array",
    DPT_NOS = "array",
    DT_NOS = "array",
    DPT_HOS = "array",
    DT_HOS = "array",
    UPT_NOS = "array",
    UT_NOS = "array",
    UPT_HOS = "array",
    UT_HOS = "array",
    ExPT_NOS = "array",
    ExT_NOS = "array",
    ExPT_HOS = "array",
    ExT_HOS = "array",
    fitness = "array",
    pNOB = "array",
    pHOS_census = "array",
    pHOS_effective = "array",
    Mjuv_loss = "array",
    PNI = "array",
    p_wild = "array",
    Misc = "list"
  )
)


setMethod("initialize", "SMSE",
          function(.Object, ...) {

            dots <- list(...)
            if (length(dots)) {
              for (i in names(dots)) slot(.Object, i) <- dots[[i]]
            }

            attr(.Object, "version") <- paste("salmonMSE", packageVersion("salmonMSE"), "with MSEtool", packageVersion("MSEtool"))
            attr(.Object, "date") <- date()
            attr(.Object, "R.version") <- getRversion()

            return(.Object)
          })



# ---- SHist Class -----
#' Class \code{"SHist"}
#'
#' Stores the outputs from the historical reconstruction of salmon operating models. Primarily used for development and debugging.
#'
#' @name SHist-class
#' @docType class
#' @slot Name Character. Identifying name
#' @slot nyears Integer, number of historica years
#' @slot nsim Integer. The number of simulations
#' @slot nstocks Integer. The number of stocks
#' @slot Snames Character. Stock names
#' @slot Egg_NOS Array `[nsim, nstocks, nyears]`. Spawning output, i.e., egg production, of natural origin spawners.
#' @slot Egg_HOS Array `[nsim, nstocks, nyears]`. Spawning output of hatchery origin spawners.
#' @slot Smolt Array `[nsim, nstocks, nyears]`. Natural smolt production (sum of offspring of natural and hatchery spawners).
#' @slot Smolt_Rel Array `[nsim, nstocks, proyears]`. Smolts that are offspring of broodtake, i.e., hatchery releases.
#' @slot Njuv_NOS Array `[nsim, nstocks, nage, nyears]`. Abundance of juvenile natural origin fish at the beginning of the year.
#' @slot Njuv_HOS Array `[nsim, nstocks, nage, nyears]`. Abundance of juvenile hatchery origin fish at the beginning of the year.
#' @slot Return_NOS Array `[nsim, nstocks, nage, nyears]`. Mature fish that will be natural origin spawners.
#' @slot Return_HOS Array `[nsim, nstocks, nage, nyears]`. Mature fish that will be hatchery origin spawners.
#' @slot Escapement_NOS Array `[nsim, nstocks, nage, nyears]`. The escapement of mature fish that will be natural origin spawners.
#' @slot Escapement_HOS Array `[nsim, nstocks, nage, nyears]`. The escapement of mature fish that will be hatchery origin spawners.
#' @slot NOS Array `[nsim, nstocks, proyears]`. Natural origin spawners.
#' @slot HOS Array `[nsim, nstocks, proyears]`. Hatchery origin spawners.
#' @slot HOS_effective Array `[nsim, nstocks, proyears]`. Hatchery origin spawners discounted by `gamma`.
#' @slot KPT_NOS Array `[nsim, nstocks, proyears]`. Pre-terminal fishery kept catch of natural origin spawners.
#' @slot KT_NOS Array `[nsim, nstocks, proyears]`. Terminal fishery kept catch of natural origin spawners.
#' @slot KPT_HOS Array `[nsim, nstocks, proyears]`. Pre-terminal fishery kept catch of hatchery origin spawners.
#' @slot KT_HOS Array `[nsim, nstocks, proyears]`. Terminal fishery kept catch of hatchery origin spawners.
#' @slot DPT_NOS Array `[nsim, nstocks, proyears]`. Pre-terminal fishery released catch (live and dead) of natural origin spawners.
#' @slot DT_NOS Array `[nsim, nstocks, proyears]`. Terminal fishery released catch (live and dead) of natural origin spawners.
#' @slot DPT_HOS Array `[nsim, nstocks, proyears]`. Pre-terminal fishery released catch (live and dead) of hatchery origin spawners.
#' @slot DT_HOS Array `[nsim, nstocks, proyears]`. Terminal fishery released catch (live and dead) hatchery origin spawners.
#' @slot UPT_NOS Array `[nsim, nstocks, proyears]`. Pre-terminal fishery harvest rate (from kept catch) of natural origin spawners.
#' @slot UT_NOS Array `[nsim, nstocks, proyears]`. Terminal fishery harvest rate of natural origin spawners.
#' @slot UPT_HOS Array `[nsim, nstocks, proyears]`. Pre-terminal fishery harvest rate of hatchery origin spawners.
#' @slot UT_HOS Array `[nsim, nstocks, proyears]`. Terminal fishery harvest rate of hatchery origin spawners.
#' @slot ExPT_NOS Array `[nsim, nstocks, proyears]`. Pre-terminal fishery exploitation rate (from kept catch and dead releases) of natural origin spawners.
#' @slot ExT_NOS Array `[nsim, nstocks, proyears]`. Terminal fishery exploitation rate of natural origin spawners.
#' @slot ExPT_HOS Array `[nsim, nstocks, proyears]`. Pre-terminal fishery exploitation rate of hatchery origin spawners.
#' @slot ExT_HOS Array `[nsim, nstocks, proyears]`. Terminal fishery exploitation rate of hatchery origin spawners.
#' @slot Misc List. Miscellaneous output
#' @export
#' @keywords classes
#' @examples
#' showClass("SHist")
setClass(
  "SHist",
  slots = c(
    Name = "character",
    nyears = "numeric",
    nsim = "numeric",
    nstocks = "numeric",
    Snames = "character",
    Egg_NOS = "array",
    Egg_HOS = "array",
    Smolt = "array",
    Smolt_Rel = "array",
    Njuv_NOS = "array",
    Njuv_HOS = "array",
    Return_NOS = "array",
    Return_HOS = "array",
    Escapement_NOS = "array",
    Escapement_HOS = "array",
    NOS = "array",
    HOS = "array",
    HOS_effective = "array",
    KPT_NOS = "array",
    KT_NOS = "array",
    KPT_HOS = "array",
    KT_HOS = "array",
    DPT_NOS = "array",
    DT_NOS = "array",
    DPT_HOS = "array",
    DT_HOS = "array",
    UPT_NOS = "array",
    UT_NOS = "array",
    UPT_HOS = "array",
    UT_HOS = "array",
    ExPT_NOS = "array",
    ExT_NOS = "array",
    ExPT_HOS = "array",
    ExT_HOS = "array",
    Misc = "list"
  )
)


setMethod("initialize", "SHist",
          function(.Object, ...) {

            dots <- list(...)
            if (length(dots)) {
              for (i in names(dots)) slot(.Object, i) <- dots[[i]]
            }

            attr(.Object, "version") <- paste("salmonMSE", packageVersion("salmonMSE"), "with MSEtool", packageVersion("MSEtool"))
            attr(.Object, "date") <- date()
            attr(.Object, "R.version") <- getRversion()

            return(.Object)
          })



#' @export
setGeneric("report", function(object, ...) standardGeneric("report"))

#' @name report
#' @title Generate markdown reports
#'
#' @description Generate a markdown report for outcomes from a single operating model projection
#'
#' @param object \linkS4class{SMSE} object
#' @param name Character string for the model name to include in the report, e.g., model run number.
#' @param filename Character string for the name of the markdown and HTML files.
#' @param dir The directory in which the markdown and HTML files will be saved.
#' @param open_file Logical, whether the HTML document is opened after it is rendered.
#' @param render_args List of arguments to pass to [rmarkdown::render()].
#' @param ... Additional arguments (not used)
#' @importFrom utils browseURL
#' @aliases report,SMSE-method
#' @importFrom rmarkdown render
#' @seealso [compare()]
#' @return Returns invisibly the output of [rmarkdown::render()], typically the path of the output file
#' @export
setMethod("report", "SMSE",
          function(object, name = object@Name, filename = "SMSE", dir = tempdir(), open_file = TRUE, render_args = list(), ...) {

            if (missing(name)) name <- substitute(object) %>% as.character()

            dots <- list(...)
            SMSE <- object # Needed for markdown file

            ####### Function arguments for rmarkdown::render
            rmd <- system.file("include", "SMSEreport.Rmd", package = "salmonMSE") %>% readLines()
            rmd_split <- split(rmd, 1:length(rmd))

            name_ind <- grep("NAME", rmd)
            rmd_split[[name_ind]] <- paste("#", name, "{.tabset}")

            stock_ind <- grep("ADD RMD BY STOCK", rmd)
            n_g <- sapply(SMSE@Misc$SOM@Bio, slot, "n_g")
            n_r <- sapply(SMSE@Misc$SOM@Hatchery, slot, "n_r")
            rmd_split[[stock_ind]] <- Map(make_rmd_stock, s = 1:SMSE@nstocks, sname = SMSE@Snames, n_g = n_g, n_r = n_r) %>% unlist()

            if (SMSE@nstocks > 1) {
              rmd_stock_compare <- make_rmd_stock_comparison()
              rmd_split[[stock_ind]] <- c(rmd_split[[stock_ind]], rmd_stock_compare)
            }

            filename_rmd <- paste0(filename, ".Rmd")

            render_args$input <- file.path(dir, filename_rmd)
            if (is.null(render_args$quiet)) render_args$quiet <- TRUE

            # Generate markdown report
            if (!dir.exists(dir)) {
              message("Creating directory: ", dir)
              dir.create(dir)
            }
            write(unlist(rmd_split), file = file.path(dir, filename_rmd))

            # Rendering markdown file
            message("Rendering markdown file: ", file.path(dir, filename_rmd))
            output_filename <- do.call(rmarkdown::render, render_args)
            message("Rendered file: ", output_filename)

            if (open_file) browseURL(output_filename)
            invisible(output_filename)
          })

plot_SOM <- function(object, var = "kappa", figure = TRUE, xlab, ylab = "Frequency",
                     type = c("hist", "age", "age_ts", "ts"),
                     maxage, nsim, proyears, g = 1, surv = FALSE, ...) {

  type <- match.arg(type)
  x <- slot(object, var)

  if (!length(x)) return(invisible())

  if (surv) x <- exp(-x)

  quant <- c(0.025, 0.5, 0.975)

  if (type == "hist") {
    xplot <- x
    if (figure) {
      if (missing(xlab)) xlab <- var
      hist(xplot, xlab = xlab, main = NULL, ...)
    }
  } else if (type == "age") {

    if (is.matrix(x) && all(dim(x) == c(nsim, maxage))) { # nsim x age
      xplot <- apply(x, 2, quantile, quant)
    } else if (is.array(x)) {
      if (length(dim(x)) == 4) x <- x[, , , g]
      if (all(dim(x) == c(nsim, maxage, proyears))) {
        xplot <- apply(x[, , proyears], 2, quantile, quant)
      }
    } else if (is.numeric(x) && length(x) == maxage) {
      xplot <- matrix(x, 3, maxage, byrow = TRUE)
    } else {
      xplot <- NULL
      figure <- FALSE
    }

    if (figure && any(xplot > 0, na.rm = TRUE)) {
      if (missing(xlab)) xlab <- "Age"
      ylim <- c(0, 1.1) * range(xplot)

      plot(seq(1, maxage), xplot[2, ], xlab = xlab, ylab = ylab, ylim = ylim, type = "o")
      matlines(seq(1, maxage), t(xplot[c(1, 3), ]), lty = 2, col = 1)
    }

  } else if (type == "age_ts") {

  }

  invisible(xplot)
}

# x - array by sim x age x g
#' @importFrom graphics matlines
plot_Mjuv_LHG <- function(x, ylab = "Juvenile natural mortality rate", figure = TRUE, LHG_names, palette = "Dark 2", surv = FALSE) {

  if (surv) x <- exp(-x)

  quant <- c(0.025, 0.5, 0.975)
  xplot <- apply(x, 2:3, quantile, quant)

  n_g <- dim(x)[3]
  maxage <- dim(x)[2]

  if (figure) {

    if (missing(LHG_names)) LHG_names <- paste("LHG", 1:n_g)

    col <- grDevices::hcl.colors(n_g, palette = palette)

    ylim <- c(0, 1.1) * range(xplot)
    Age <- seq(1, maxage-1)

    plot(Age, NULL, xlab = "Age", ylab = ylab, type = "n", ylim = ylim)
    for (g in 1:n_g) {
      lines(Age, xplot[2, , g], type = "o", col = col[g], pch = 16)
      matlines(Age, t(xplot[c(1, 3), , g]), col = col[g], lty = 2)
    }
    legend('topright', legend = LHG_names, col = col, lty = 1, pch = 16, bty = "n")
  }

  invisible(xplot)
}

plot_Mjuv_RS <- function(x, ylab = "Juvenile natural mortality rate", figure = TRUE, RS_names, palette = "Cold", surv = FALSE) {
  plot_Mjuv_LHG(x, ylab, figure, RS_names, palette, surv)
}

#' @importFrom grDevices rainbow
#' @importFrom graphics axis
plot_stray <- function(stray, xlab = "Destination", ylab = "Origin") {

  ns <- nrow(stray)

  vcol <- rainbow(100, end = 0.45)

  graphics::plot.default(
    NULL, xlab = xlab, ylab = ylab, xaxs = "i", yaxs = "i",
    xaxt = "n", yaxt = "n", xlim = c(1, ns+1), ylim = c(1, ns+1)
  )

  for(x in 1:ns) {
    for(y in 1:ns) {
      m_yx <- round(stray[y, x], 2)
      rect(xleft = x, ybottom = y, xright = x+1, ytop = y+1, col = vcol[100 * m_yx])
      text(x + 0.5, y + 0.5, m_yx)
    }
  }

  axis(2, at = 1:ns + 0.5, labels = 1:ns)
  axis(1, at = 1:ns + 0.5, labels = 1:ns)

  invisible(stray)
}

plot_SRR <- function(SOM, s = 1, quant = FALSE, surv = FALSE, figure = TRUE, check = TRUE, ylim, ...) {
  if (check) SOM <- check_SOM(SOM)
  Bio <- SOM@Bio[[s]]
  maxage <- Bio@maxage

  # Smolt production at replacement
  real_phi <- sapply(1:SOM@nsim, function(x) {
    calc_phi(
      Mjuv = matrix(Bio@Mjuv_NOS[x, , 1, ], Bio@maxage-1, Bio@n_g),
      p_mature = matrix(Bio@p_mature[x, , 1], Bio@maxage, Bio@n_g),
      p_female = Bio@p_female,
      fec = matrix(Bio@fec[x, , 1], Bio@maxage, Bio@n_g),
      s_enroute = Bio@s_enroute,
      n_g = Bio@n_g,
      p_LHG = Bio@p_LHG
    )
  })
  smolt0 <- sapply(1:SOM@nsim, function(x) {
    calc_smolt(
      N1 = real_phi[x],
      kappa = Bio@kappa[x],
      capacity = Bio@capacity[x],
      Smax = Bio@Smax[x],
      phi = Bio@phi[x],
      tau = Bio@tau[x],
      SRrel = Bio@SRrel,
      per_recruit = TRUE
    )
  })

  # Egg production
  E0 <- smolt0 * real_phi
  E <- seq(0, 1.5 * max(E0), length.out = 100)

  Smolt <- sapply(1:SOM@nsim, function(x) {
    sapply(E, function(N) {
      calc_smolt(
        N1 = N,
        kappa = Bio@kappa[x],
        capacity = Bio@capacity[x],
        Smax = Bio@Smax[x],
        phi = Bio@phi[x],
        tau = Bio@tau[x],
        SRrel = Bio@SRrel
      )
    })
  })

  if (surv) {
    x <- t(Smolt/E)
    x[, 1] <- SOM@Bio[[s]]@kappa/SOM@Bio[[s]]@phi
  } else {
    x <- t(Smolt)
  }

  if (!quant) {
    xplot <- x
  } else {
    xplot <- apply(x, 2, quantile, c(0.025, 0.5, 0.975), na.rm = TRUE)
  }

  if (figure && any(xplot > 0, na.rm = TRUE)) {
    xlab <- "Egg production"
    ylab <- ifelse(surv, "Egg-smolt survival", "Smolt production")

    if (missing(ylim)) ylim <- c(0, 1.1) * range(xplot, na.rm = TRUE)

    if (!quant) {
      matplot(E, t(xplot), type = 'l', col = "grey40", ylim = ylim, lty = 1,
              xlab = xlab, ylab = ylab, ...)
    } else {
      matplot(E, t(xplot), type = 'o', pch = c(NA, 1, NA), col = 1, lty = c(2, 1, 2), ylim = ylim,
              xlab = xlab, ylab = ylab, ...)
      abline(v = median(E0), lty = 3)
    }
  }

  invisible(xplot)
}

.onLoad <- function(libname, pkgname) {
  salmonMSE_env$Ford <- data.frame()
  salmonMSE_env$N <- data.frame()
  salmonMSE_env$stateN <- data.frame()
  salmonMSE_env$H <- data.frame()
  salmonMSE_env$stateH <- data.frame()
}

if(getRversion() >= "2.15.1") {
  utils::globalVariables(
    c(".", "2.5%", "50%", "97.5%", "Age", "Design1", "Design2", "Release Strategy",
      "Var1", "Var2", "Var3", "Var4", "Year", "value", "variable", "glossary"
    )
  )
}

#' @importFrom graphics grid matplot abline par
plot.default <- function(..., zero_line = FALSE) {
  if (zero_line) {
    graphics::plot.default(..., panel.first = {graphics::grid(); abline(h = 0, col = "grey60")})
  } else {
    graphics::plot.default(..., panel.first = graphics::grid())
  }
}

matplot <- function(..., zero_line = FALSE) {
  if (zero_line) {
    graphics::matplot(..., panel.first = {graphics::grid(); abline(h = 0, col = "grey60")})
  } else {
    graphics::matplot(..., panel.first = graphics::grid())
  }
}

#' @importFrom graphics hist legend
#' @importFrom stats sd
hist.numeric <- function(x, ...) {
  n <- length(x)

  # Remove NA's
  if (any(is.na(x))) {
    na_rate <- mean(is.na(x))
    x <- x[!is.na(x)]
    legend_na <- paste0(round(100 * na_rate), "% NA's")
  } else {
    legend_na <- NULL
  }

  # Calculate skewness NA's already removed, n is the original length of x
  skewness <- (sum((x - mean(x))^3)/n)/(sum((x - mean(x))^2)/n)^1.5
  cv <- sd(x)/mean(x)

  # Remove outliers
  if (!is.na(skewness) && cv > 0.1) {
    if (skewness > 3) {
      max_x_plot <- quantile(x, 0.95)
      p_outlier <- signif(100 * mean(x > max_x_plot), 2)
      x <- x[x <= max_x_plot]
      legend_skew1 <- paste0(p_outlier, "% > ", round(max_x_plot, 2))
    } else {
      legend_skew1 <- NULL
    }
    if (skewness < -3) {
      min_x_plot <- quantile(x, 0.05)
      x <- x[x >= min_x_plot]
      p_outlier <- signif(100 * mean(x < min_x_plot), 2)
      legend_skew2 <- paste0(p_outlier, "% < ", round(min_x_plot, 2))
    } else {
      legend_skew2 <- NULL
    }
  } else {
    legend_skew1 <- legend_skew2 <- NULL
  }

  # Plot histogram
  if (all(!diff(signif(x, 3)))) { # If all identical values
    x <- signif(x, 3)
    if (all(!x)) { # if x is all zeros
      breaks <- c(-0.1, 0.1)
      xlim <- c(-1, 1)
    } else {
      breaks <- c(0.99, 1.01) * x[1]
      xlim <- c(x[1] - 0.2 * abs(x[1]), x[1] + 0.2 * abs(x[1]))
    }
    r <- graphics::hist.default(x, breaks = breaks, xlim = xlim, ...)
  } else {
    r <- graphics::hist.default(x, ...)
  }

  # Make legend
  legend_text <- c(legend_na, legend_skew1, legend_skew2)
  if (!is.null(legend_text)) {
    legend("topright", legend = legend_text, bty = "n", text.col = "red")
  }
  invisible(r)
}

#' @name glossary
#' @title salmonMSE glossary
#' @description Glossary of terms and parameters used in salmonMSE
#' @examples
#' data(glossary)
#' glossary[1:2, ]
NULL

#' Fit conditioning model to historical data
#'
#' @description{
#' Bayesian stock reconstruction model of natural and hatchery origin fish population.
#' Maturity and age-1 natural mortality are estimated from coded wire tag catch and escapement at age.
#' A separate series of observed escapement, and hatchery releases reconstructs the population of interest,
#' informed by natural mortality and maturity from CWT ([Korman and Walters 2024](https://publications.gc.ca/site/eng/9.940685/publication.html)).
#' The model estimates time-varying maturity rate as well as time-varying ocean survival as a linear model of covariates (separate covariates
#' for age 1 vs. ages 2+).
#' The model can include either a preterminal juvenile fishery, terminal return fishery, or both (see Data and start sections of the documentation).
#'
#' `fit_CM()` generates the RTMB model from data which can then be passed to `sample_CM()` to run the MCMC in Stan. Generate a markdown report with [`report_CM()`].
#'
#' More information is available on the [salmonMSE](https://docs.salmonmse.com/articles/conditioning.html) website
#' }
#'
#' @param data  A list containing data inputs. See details.
#' @param start An optional list containing parameter starting values. See details.
#' @param map An optional list that describes how parameters are fixed in the model. See [`TMB::MakeADFun()`].
#' @param lower Named list containing lower bounds for parameters. See details.
#' @param upper Named list containing upper bounds for parameters. See details.
#' @param do_fit Logical, whether to do the fit and estimate the Hessian.
#' @param silent Logical, whether to silence output from RTMB to the console.
#' @param control List, `control` argument to pass to [`stats::nlminb()`].
#' @param ... For `fit_CM`, arguments to [`RTMB::MakeADFun()`]. For `sample_CM`, arguments to `rstan::sampling()`
#' @import RTMB
#' @importFrom stats nlminb
#' @seealso [report_CM()]
#' @returns
#' - `fit_CM()` returns a named list containing the RTMB model (`obj`), nlminb output (`opt`), standard errors (`SD`),
#' and parameter bounds (`lower` and `upper`)
#' - `sample_CM()` returns a `stanfit` object containing the MCMC chains
#'
#' @section Data:
#' Data should passed through a named list with the following entries.
#'
#' - `Nages` Integer, number of age classes in the model
#' - `Ldyr` Integer, number of years in the model
#' - `lht` Integer, life history type. Should be 1 for now
#' - `n_r` Integer, number of release strategies for CWT, subset of a hatchery-origin brood year that differ in maturity rate. Default is 1.
#' - `cwtrelease` Matrix `[Ldyr, n_r]`, coded wire tag (CWT) releases by **release year and release strategy**
#' - `cwtesc` Array `[Ldyr, Nages, n_r]`. CWT escapement **by release year, age, and release strategy** (assumes broodyear is release year minus 1). Poisson likelihood.
#' - `cwtcatPT` Array `[Ldyr, Nages, n_r]`. CWT preterminal catch (juvenile fish), **by release year, age, and release strategy**. Poisson likelihood. Set all values to zero to turn off
#' parameters related to the preterminal fishery.
#' - `cwtcatT` Array `[Ldyr, Nages, n_r]`. CWT terminal catch (returning, mature fish), **by release year, age, and release strategy**. Poisson likelihood. Set all values to zero to turn off
#' parameters related to the terminal fishery.
#'
#' - `bvulPT` Vector length `Nages`. Prior mean for the vulnerability at age to the preterminal fishery.
#' - `bvulT` Vector length `Nages`. Prior mean for the vulnerability at age to the terminal fishery.
#'
#' - `RelRegFPT` Vector `Ldyr`. Trend in relative regional preterminal fishing mortality. Fishing mortality is estimated by estimating a scaling
#' coefficient and annual deviations from this vector. Default is `rep(1, d$Ldyr)` (no prior trend) if `cwtcatPT` is provided, otherwise zero.
#' - `RelRegFT` Vector `Ldyr`. Trend in relative regional terminal fishing mortality.
#' Default is `rep(1, d$Ldyr)` (no prior trend) if `cwtcatT` is provided, otherwise zero.
#'
#' - `bmatt` Vector length `Nages`. Proportion maturity at age, base values for calculating the unfished replacement line. Also the prior means if year-specific
#' maturity rates are estimated.
#'
#' - `mobase`. Vector length `Nages`. Natural mortality at age, base values for calculating the unfished replacement line and the
#' the equilibrium spawners at age.
#'
#' - `covariate1` *Optional*. Matrix `Ldyr, ncov1` of linear covariates that predict natural mortality for age 1.
#' - `covariate` *Optional*. Matrix `Ldyr, ncov` of linear covariates that predict natural mortality for ages 2+.
#'
#' - `hatchsurv` Numeric, survival of hatchery releases into the smolt life stage. Density-independent. Default is 1. If less than 1, then hatchery origin fish have
#' lower survival to age 2 (after first year of marine life stage) compared to natural origin fish.
#' - `gamma` *Optional*. Numeric, the relative spawning success of hatchery origin spawners. Default is 1.
#' - `ssum` Numeric, proportion of spawners that is female. Can also be a vector `Nages`
#'
#' - `fec` Vector length `Nages`. Fecundity, egg production at age
#'
#' - `r_matt` Integer, the release strategy for which to use maturity parameter for the natural system. Default is 1.
#' - `obsescape` Vector length `Ldyr` by **return year**, total observed escapement from fisheries, i.e., return to river (all ages and both hatchery/natural fish). Lognormal likelhood.
#' - `propwildspawn` Vector length `Ldyr` by **return year**, proportion of the escapement that spawn (accounts for en-route mortality and broodtake)
#' - `hatchrelease` Vector length `Ldyr+1` by **release year**, number of hatchery releases for the population. Default is zero.
#' - `obs_pHOS` *Optional*. Vector length `Ldyr` by **return year**, observations of proportion of hatchery origin spawners (census) (between 0-1) *by brood year*. Fitted to model with logistic-normal likelihood.
#' - `pHOS_sd` Numeric, logistic-normal standard deviation of pHOS observations. Default is 1.
#' - `pHOS_init` Numeric, initial pHOS for equilibrium abundance in the first year of the model. Default is 0.
#' - `s_enroute` Numeric, survival of escapement to spawning grounds. Default is 1.
#'
#' - `so_mu` Numeric, the prior mean for spawners at unfished replacement in logspace. Default is `log(3 * max(data$obsescape))`.
#' - `so_sd` Numeric, the prior standard deviation for spawners at unfished replacement in logspace. Default is 0.5.
#'
#' - `finitPT` Numeric, initial preterminal fishing mortality for calculating the equilibrium juvenile proportions at age in the first year of the model. Default is 0. Set to `"estimate"` to allow the model to estimate
#' the equilibrium condition.
#' - `finitT` Numeric, initial terminal fishing mortality for calculating the equilibrium juvenile proportions at age in the first year of the model. Default is 0. Set to `"estimate"` to allow the model to estimate
#' the equilibrium condition.
#' - `spawn_init` Numeric, initial spawners to calculate equilibrium abundance in the first year of the model. Default is `obsescape[1]`.
#'
#' - `cwtExp` Numeric, the CWT expansion factor, typically the reciprocal of the catch sampling rate (higher factors for lower sampling rate).
#' The model scales down the CWT predictions to match the observations. In other words,
#' the model assumes that the CWT catch and escapement are not expanded. For example, `cwtExp = 10` divides the CWT predictions by 10 for the likelihood. Default is 1.
#' The Poisson distribution is used for the likelihood of the CWT observations, and the expansion parameter can be used to downweight the CWT likelihood relative to the escapement time series.
#' However it requires adjustments of the CWT catches prior to fitting to ensure the proper population scale.
#' If the expanded catch is 100, then the input CWT catch should be 10 and 50 with `cwtExp` of 10 and 2, respectively, to maintain the same population scale.
#' The Poisson variance scales with the mean and is higher with `cwtExp = 2`.
#' - `fitness` Logical, whether to calculate fitness effects on survival. Default is `FALSE`.
#' - `theta` Vector length 2, the optimum phenotype value for the natural and hatchery environments. Default is 100 and 80, respectively. See
#' [online article](https://docs.salmonmse.com/articles/equations.html#fitness-effects-on-survival) for more information.
#' - `rel_loss` Vector length 3, the loss in fitness apportioned between the egg, fry (both prior to density-dependence), and smolt (after density-dependence) life stage. The three values should sum to 1.
#' - `zbar_start` Vector length 2, the mean phenotype of the spawners and broodtake in the natural and hatchery environment, respectively, at the start of the model. Default values of 100 and 100, implying maximum fitness at
#' for the natural environment at the start of the model.
#' - `fitness_variance` Numeric. The variance (omega-squared) of the fitness function. Assumed identical between the natural and hatchery environments. Default is 100.
#' - `phenotype_variance` Numeric. The variance (sigma-squared) of the phenotypic trait (with optimum theta). Assumed identical between the natural and hatchery environments. Default is 10.
#' - `heritability` Numeric. The heritability (h-squared) of the phenotypic trait. Between 0-1. Default is 0.5.
#' - `fitness_floor` Numeric. The minimum fitness value in the natural and hatchery environments. Default is 0.5.
#'
#' @section start:
#' Starting values for parameters can be provided through a named list:
#'
#' - `log_cr` Numeric, log of the compensation ratio (productivity). Default is 3.
#' - `log_so` Numeric, unfished spawners in logspace. Default is `log(3 * max(data$obsescape))`.
#' - `moadd` Numeric, additive term to base natural mortality rate for age 1 juveniles. Default is zero.
#' - `wt` Vector `Ldyr`. Annual deviates in natural mortality during the freshwater life stage (affects egg to smolt survival).
#' Estimated with normal prior with mean zero and standard deviation `p$wt_sd`. Default is zero.
#' - `wto` Vector `Ldyr`. Annual deviates in natural mortality for age 1 juveniles (marine life stage).
#' Estimated with normal prior with mean zero and standard deviation `p$wto_sd`. Default is zero.
#' - `log_FbasePT` Numeric, scaling coefficient to estimate preterminal fishing mortality from `data$RelRegFPT`. Default is `log(0.1)`.
#' - `log_FbaseT` Numeric, scaling coefficient to estimate preterminal fishing mortality from `data$RelRegFT`. Default is `log(0.1)`.
#' - `log_fanomalyPT` Vector `Ldyr`. Annual lognormal deviates from `exp(log_FbasePT) * data$RelRegFPT` to estimate preterminal fishing mortality.
#' Estimated with normal prior with mean zero and standard deviation `p$fanomaly_sd`. Default is zero.
#' - `log_fanomalyT` Vector `Ldyr`. Annual lognormal deviates from `exp(log_FbaseT) * data$RelRegFT` to estimate terminal fishing mortality.
#' Estimated with normal prior with mean zero and standard deviation `p$fanomalyPT_sd`. Default is zero.
#' - `lnE_sd` Numeric, lognormal standard deviation of the observed escapement. Estimated with hierarchical `gamma(2, 5)` prior. Default is 0.1.
#' - `wt_sd` Numeric, lognormal standard deviation of the egg to smolt (freshwater) natural mortality deviates. Estimated with hierarchical `gamma(2, 5)` prior. Default is 1.
#' - `wto_sd` Numeric, lognormal standard deviation of the age 1 (marine) natural mortality deviates. Estimated with hierarchical `gamma(2, 5)` prior. Default is 1.
#' - `fanomalyPT_sd` Numeric, lognormal standard deviation of `fanomalyPT`. Estimated with hierarchical `gamma(2, 5)` prior. Default is 1.
#' - `fanomalyT_sd` Numeric, lognormal standard deviation of `fanomalyT`. Estimated with hierarchical `gamma(2, 5)` prior. Default is 1.
#' - `logit_vulPT` Vector `Nages-2` of preterminal vulnerability at age in logit space. Fixed to zero and one at age 1 and the maximum age, respectively.
#' Default is `qlogis(data$bvul_PT[-c(1, data$Nages)])`.
#' - `logit_vulT` Vector `Nages-2` of terminal vulnerability at age in logit space. Fixed to zero and one at age 1 and the maximum age, respectively.
#' Default is `qlogis(data$bvul_T[-c(1, data$Nages)])`.
#' - `logit_matt` Matrix `Ldyr, Nages-2` maturity by year and age in logit space. Maturity is fixed to zero and one at age 1 and the maximum age, respectively.
#' Default is `matrix(qlogis(data$bmatt[-c(1, data$Nages)]), data$Ldyr, data$Nages-2, byrow = TRUE)`.
#' - `sd_matt` Vector `Nages-2`. Logit standard deviation of maturity (`logit_matt`) by age class. Default is 0.5.
#' - `b1` Vector `ncov1` of coefficients for linear covariates that predict natural mortality for age 1. Default is zero.
#' - `b` Vector `ncov` of coefficients for linear covariates that predict natural mortality for ages 2+. Default is zero.
#'
#' @section Bounds:
#'
#' By default, the standard deviation parameters and parameters in normal space (e.g., `FbasePT`, `Fbase_T`) have a lower bound of zero.
#' `moadd` has a lower bound of zero by default, but it is feasible that this parameter can be negative as well.
#' Deviation parameters centred around zero are bounded between -3 to 3.
#' The `log_cr` parameter has a lower bound of zero.
#'
#' All other parameters are unbounded.
#'
#' @section Covariates on natural mortality:
#'
#' Natural mortality is modeled as the sum of a base value \eqn{M^\textrm{base}}, additional scaling factor for age 1 \eqn{M^\textrm{add}},
#' a linear system of covariates \eqn{X} and coefficients \eqn{b}:
#'
#' \deqn{
#' M_{y,a} =
#' \begin{cases}
#' M^\textrm{base}_a + M^\textrm{add} + \sum_j b^1_j X^1_{y,j} & \quad a = 1\\
#' M^\textrm{base}_a + \sum_j b_j X_{y,j} & \quad a = 2, \ldots, A
#' \end{cases}
#' }
#' @author Q. Huynh from Stan code provided by J. Korman and C. Walters
#' @references
#' Korman, J. and Walters, C. 2024. A life cycle model for Chinook salmon population dynamics. Canadian Contractor Report of Hydrography and Ocean
#' Sciences 62: vi + 60 p.
#' @seealso [CM2SOM()]
#' @export
fit_CM <- function(data, start = list(), map = list(), lower = list(), upper = list(), do_fit = TRUE, silent = TRUE,
                   control = list(eval.max = 1e5, iter.max = 1e5), ...) {

  data <- check_data(data)
  p <- make_CMpars(start, data)
  map <- make_map(map, p, data)

  f <- function(p) salmonMSE::CM_int(p, d = data) # :: is needed for parallel MCMC sampling
  obj <- RTMB::MakeADFun(func = f, parameters = p, map = map, silent = silent, ...)

  bounds <- make_bounds(names(obj$par), data, lower, upper)

  if (do_fit) {
    opt <- nlminb(
      obj$par, obj$fn, obj$gr,
      lower = bounds$lower, upper = bounds$upper, control = control
    )
    SD <- RTMB::sdreport(obj)
  } else {
    opt <- SD <- NULL
  }

  res <- list(obj = obj, opt = opt, SD = SD, lower = bounds$lower, upper = bounds$upper)
  return(res)
}

#' @rdname fit_CM
#' @param fit List of output from `fit_CM()`
#' @export
sample_CM <- function(fit, ...) {
  if (!requireNamespace("tmbstan", quietly = TRUE)) stop("tmbstan package is needed.")
  samp <- tmbstan::tmbstan(fit$obj, lower = fit$lower, upper = fit$upper, ...)
  samp@.MISC$CMfit <- fit
  return(samp)
}


#' Convert conditioning model to operating model
#'
#' @description
#' Creates an operating model from MCMC samples and data inputs of the conditioning model.
#' Management actions for habitat, hatchery production, and harvest still need to be specified in the operating model.
#'
#' **Note: the function assumes the maturity values in the last conditioning year for the projection, which are likely not well informed by CWT data. Consider updating the maturity using some historical average (e.g., across most recent completed brood years).**
#'
#' @param stanfit Output from [`sample_CM()`]
#' @param sims Optional, a vector of integers indicating the MCMC iterations to convert to operating model simulations. Otherwise,
#' use argument `nsim` in order to sample a subset of the MCMC.
#' @param nsim Integer, total number of simulations in the operating model. Only used if `sims` is missing.
#' @param seed Integer, seed for sampling the MCMC output. Only used if `sims` is missing.
#' @param proyears Integer, the number of projection years in the operating model
#' @return \linkS4class{SOM} object.
#' @importFrom abind abind
#' @export
CM2SOM <- function(stanfit, sims, nsim = 2, seed = 1, proyears = 40) {

  if (!requireNamespace("rstan", quietly = TRUE)) stop("rstan package is needed.")
  pars <- rstan::extract(stanfit)

  if (missing(sims)) {
    nsim_stan <- length(pars[["lp__"]])

    set.seed(seed)
    sims <- sample(nsim_stan, nsim)
  }

  fit <- stanfit@.MISC$CMfit
  if (is.null(fit)) stop("CM fitted object not found in stanfit@.MISC$CMfit")
  data <- get_CMdata(fit)

  report <- get_report(stanfit, sims)
  nsim_om <- length(sims)
  nyears <- data$Ldyr

  phi <- sapply(report, getElement, "epro")
  tau <- sapply(report, getElement, "spro")

  SRbeta <- sapply(report, getElement, "beta")
  Emax <- 1/SRbeta
  Smax <- Emax * tau/phi

  # Sim, age, year, rs
  matt_HO <- sapply(report, getElement, "matt", simplify = "array") %>%
    aperm(c(4, 2, 1, 3))
  matt_NO <- matt_HO[, , , data$r_matt]

  mo <- sapply(report, getElement, "mo", simplify = "array") %>%
    aperm(3:1)

  Njuv <- sapply(report, getElement, "N", simplify = "array") %>%
    aperm(c(4, 2, 1, 3)) # nsim x age x year x origin
  Spawner <- sapply(report, getElement, "syear", simplify = "array") %>%
    aperm(c(4, 2, 1, 3))
  FPT <- sapply(1:2, function(...) sapply(report, getElement, "FPT"), simplify = "array") %>%
    aperm(c(2, 1, 3))
  FT <- sapply(1:2, function(...) sapply(report, getElement, "FT"), simplify = "array") %>%
    aperm(c(2, 1, 3))

  Bio <- new(
    "Bio",
    maxage = data$Nages,
    n_g = 1,
    p_LHG = 1,
    p_mature = expand_array(matt_NO, proyears),
    SRrel = "Ricker",
    kappa = as.numeric(exp(pars$log_cr[sims])),
    Smax = Smax,
    phi = phi,
    tau = tau,
    Mjuv_NOS = expand_array(mo, proyears),
    fec = data$fec,
    p_female = data$ssum,
    s_enroute = data$s_enroute
  )

  M_HO_proj <- expand_array(mo, proyears)
  M_HO_proj2 <- array(M_HO_proj, c(dim(M_HO_proj), data$n_r))
  matt_HO_proj <- local({
    x <- matt_HO
    xlast <- x[, , dim(x)[3], ]
    xproj <- array(xlast, c(dim(x)[c(1, 2, 4)], proyears)) %>% aperm(c(1, 2, 4, 3))
    abind::abind(x, xproj, along = 3) %>%
      structure(dimnames = NULL)
  })

  Hatchery <- new(
    "Hatchery",
    n_r = data$n_r,
    Mjuv_HOS = M_HO_proj2,
    p_mature_HOS = matt_HO_proj,
    gamma = data$gamma
  )

  Harvest <- new(
    "Harvest",
    vulPT = sapply(report, getElement, "vulPT") %>% t(),
    vulT = sapply(report, getElement, "vulT") %>% t()
  )

  Habitat <- new("Habitat", use_habitat = FALSE)

  Historical <- new(
    "Historical",
    HistNjuv_NOS = array(Njuv[, , nyears, 1], c(nsim_om, data$Nages, 1)),
    HistNjuv_HOS = array(Njuv[, , nyears, 2], c(nsim_om, data$Nages, 1)),
  )

  if (data$fitness) {
    Hatchery@fitness_type <- c("Ford", "none")
    Hatchery@theta <- data$theta
    Hatchery@rel_loss <- data$rel_loss

    zbar <- sapply(report, getElement, "zbar", simplify = "array") %>%
      aperm(c(3, 1, 2))
    Hatchery@zbar_start <- zbar[, nyears - seq(1, data$Nages) + 1, ]

    Hatchery@phenotype_variance <- data$phenotype_variance
    Hatchery@fitness_variance <- data$fitness_variance
    Hatchery@heritability <- data$heritability
    Hatchery@fitness_floor <- data$fitness_floor
  } else {
    Hatchery@fitness_type <- c("none", "none")
  }

  SOM <- new("SOM", Bio, Habitat, Hatchery, Harvest, Historical,
             nsim = nsim_om, proyears = proyears)

  return(SOM)
}

#' @name CMfigures
#' @title Figures for conditioning model results
#'
#' @description Functions used by the markdown report to generate summary figures from the age-structured conditoning model
#' @param stanfit Output from [sample_CM()]
#' @param vars Character vector for variable names (see `names(stanfit@sim$samples[[1]])`). Regex and partial matching supported because it is passed to the `pattern` argument of [`grepl()`]
#' @param inc_warmup Logical, whether to include warmup MCMC samples
#' @returns
#' - `CM_trace()` returns a ggplot showing the MCMC trace plot (aka wormplot)
#' @importFrom dplyr bind_rows bind_cols mutate
#' @export
CM_trace <- function(stanfit, vars, inc_warmup = FALSE) {
  if (missing(vars)) vars <- names(stanfit@sim$samples[[1]])

  val <- lapply(1:stanfit@sim$chains, function(i) {
    x <- stanfit@sim$samples[[i]]
    vars_regex <- paste0(vars, collapse = "|")

    x_pars <- x[grepl(vars_regex, names(x))]

    if (length(x_pars)) {

      if (inc_warmup) {
        warmup <- 1
      } else {
        warmup <- stanfit@stan_args[[i]]$warmup
      }
      thin <- stanfit@stan_args[[i]]$thin
      iter <- stanfit@stan_args[[i]]$iter

      it <- seq(1, iter, by = thin)

      x[grepl(vars_regex, names(x))] %>%
        bind_cols() %>%
        as.data.frame() %>%
        mutate(Chain = i, Iteration = it) %>%
        filter(.data$Iteration > warmup) %>%
        reshape2::melt(id.vars = c("Chain", "Iteration"))
    } else {
      data.frame()
    }

  }) %>%
    bind_rows()

  if (nrow(val)) {
    g <- ggplot(val, aes(.data$Iteration, .data$value, colour = factor(.data$Chain))) +
      geom_line() +
      facet_wrap(vars(variable), scales = "free_y") +
      labs(colour = "Chain")
    g
  }
}


#' @rdname CMfigures
#' @returns
#' - `CM_pairs()` returns output from [graphics::pairs()], a matrix of scatterplots of MCMC posterior samples
#' @importFrom stats cor
#' @importFrom graphics pairs rect strwidth text box
#' @export
CM_pairs <- function(stanfit, vars = c("log_so", "log_cr"), inc_warmup = FALSE) {
  panel.hist <- function(x, ...) {
    usr <- par("usr"); on.exit(par(usr = usr))
    par(usr = c(usr[1:2], 0, 1.5) )
    h <- hist(x, plot = FALSE)
    breaks <- h$breaks; nB <- length(breaks)
    y <- h$counts; y <- y/max(y)
    rect(breaks[-nB], 0, breaks[-1], y, col = "cyan", ...)
  }

  panel.cor <- function(x, y, digits = 2, prefix = "", cex.cor, ...) {
    usr <- par("usr"); on.exit(par(usr = usr))
    par(usr = c(0, 1, 0, 1))
    r <- abs(cor(x, y))
    txt <- format(c(r, 0.123456789), digits = digits)[1]
    txt <- paste0(prefix, txt)
    if(missing(cex.cor)) cex.cor <- 0.8/strwidth(txt)
    text(0.5, 0.5, txt, cex = 1.3)
  }

  val <- lapply(1:stanfit@sim$chains, function(i) {
    x <- stanfit@sim$samples[[i]]
    vars_str <- paste0(vars, collapse = "|")
    xout <- x[grepl(vars_str, names(x))] %>%
      bind_cols()

    if (nrow(xout)) {

      if (inc_warmup) {
        warmup <- 1
      } else {
        warmup <- stanfit@stan_args[[i]]$warmup
      }

      thin <- stanfit@stan_args[[i]]$thin
      iter <- stanfit@stan_args[[i]]$iter

      it <- data.frame(iter = seq(1, iter, by = thin))
      it$n <- 1:nrow(it)

      xout <- xout[it$n[it$iter > warmup], ]
    }
    return(xout)
  }) %>%
    bind_rows()


  if (nrow(val)) {
    if (ncol(val) > 1) {
      pairs(x = val, pch = 19, cex = 0.2, diag.panel = panel.hist, upper.panel = panel.cor, gap = 0)
    } else {
      hist(val[[1]], col = "cyan", main = NULL, xlab = names(val)[1])
      box()
    }
  }

}


#' @rdname CMfigures
#' @param report List, output of state variables from individual MCMC samples, obtained with `get_report()`
#' @param d List of data variables, obtained with `get_CMdata()`
#' @param year Vector of years
#' @returns
#' - `CM_fit_esc()` returns base graphics with fit to total escapement time series
#' @export
#' @importFrom graphics points polygon
CM_fit_esc <- function(report, d, year) {
  obs <- d$obsescape

  if (missing(year)) year <- 1:length(obs)

  esc <- exp(sapply(report, getElement, "logpredesc")) %>%
    apply(1, quantile, probs = c(0.025, 0.5, 0.975))

  plot(NULL, NULL, type = "n", pch = 16, xlim = range(year), ylim = c(0, 1.1) * range(esc, obs, na.rm = TRUE), xlab = "Return Year", ylab = "Escapement")
  polygon(c(rev(year), year), c(rev(esc[1, ]), esc[3, ]), col = alpha("grey", 0.5), border = NA)
  lines(year, esc[2, ], lwd = 1.5)

  points(year, obs, pch = 16)
  lines(year, obs, lty = 3)

  invisible()
}

#' @rdname CMfigures
#' @returns
#' - `CM_fit_pHOS()` returns base graphics with fit to pHOS (census) observations
#' @export
CM_fit_pHOS <- function(report, d, year) {
  obs <- d$obs_pHOS

  if (!is.null(obs) && sum(obs, na.rm = TRUE)) {

    if (missing(year)) year <- 1:length(obs)
    year <- year - 1 # Brood year

    pred <- sapply(report, getElement, "pHOScensus_brood") %>%
      apply(1, quantile, probs = c(0.025, 0.5, 0.975), na.rm = TRUE)

    plot(NULL, NULL, type = "n", pch = 16, xlim = range(year), ylim = c(0, 1.1) * range(pred, obs, na.rm = TRUE), xlab = "Brood Year", ylab = "pHOS census")
    polygon(c(rev(year), year), c(rev(pred[1, ]), pred[3, ]), col = alpha("grey", 0.5), border = NA)
    lines(year, pred[2, ], lwd = 1.5)

    points(year, obs, pch = 16)
    lines(year, obs, lty = 3)
  }

  invisible()
}

CM_data <- function(obs, year, ylab, xlab = "Year") {

  if (!is.null(obs) && sum(obs)) {
    if (missing(year)) year <- 1:length(obs)
    plot(year, obs, type = "o", pch = 16, xlim = range(year),
         ylim = c(0, 1.1) * range(obs, na.rm = TRUE), xlab = xlab, ylab = ylab)
  }

  invisible()
}

CM_CWTrel <- function(obs, year1, rs_names) {
  if (missing(rs_names)) rs_names <- seq(1, ncol(obs))

  dat <- obs %>%
    structure(dimnames = list(RelYear = 1:nrow(obs) + year1 - 1, `Release Strategy` = rs_names)) %>%
    reshape2::melt() %>%
    mutate(`Release Strategy` = factor(`Release Strategy`, rs_names), `Brood Year` = .data$RelYear - 1)

  g <- ggplot(dat, aes(.data$`Brood Year`, .data$value, colour = .data$`Release Strategy`)) +
    geom_point() +
    geom_line() +
    labs(x = "Brood Year", y = "CWT release") +
    theme(legend.position = "bottom")

  if (length(unique(dat$`Release Strategy`)) == 1) {
    g <- g +
      guides(colour = "none") +
      scale_colour_manual(values = "black")
  }

  g
}


#' @rdname CMfigures
#' @param year1 Numeric, first year of model
#' @param rs_names Character vector of hatchery release strategies
#' @returns
#' - `CM_fit_CWTesc()` returns ggplot of fit to CWT escapement at age
#' @export
CM_fit_CWTesc <- function(report, d, year1 = 1, rs_names) {

  if (missing(rs_names)) rs_names <- seq(1, d$n_r)

  ebrood <- sapply(report, getElement, "ebrood", simplify = "array") %>%
    apply(1:3, quantile, probs = c(0.025, 0.5, 0.975)) %>%
    reshape2::melt() %>%
    mutate(RelYear = Var2 + year1 - 1, BroodYear = .data$RelYear - 1) %>%
    mutate(Age = paste("Age", Var3)) %>%
    mutate(`Release Strategy` = factor(rs_names[Var4], rs_names)) %>%
    reshape2::dcast(Age + BroodYear + `Release Strategy` ~ Var1, value.var = "value")

  dat <- d$cwtesc

  nyears <- dim(dat)[1]
  nage <- dim(dat)[2]
  n_r <- dim(dat)[3]

  cwtesc <- dat %>%
    structure(dimnames = list(RelYear = year1 + seq(1, nyears) - 1, Age = seq(1, nage), `Release Strategy` = rs_names)) %>%
    reshape2::melt() %>%
    mutate(Age = paste("Age", Age),
           BroodYear = .data$RelYear - 1,
           `Release Strategy` = factor(`Release Strategy`, rs_names)) %>%
    dplyr::filter(value > 0)

  g <- ggplot(ebrood, aes(.data$BroodYear, colour = .data$`Release Strategy`, fill = .data$`Release Strategy`)) +
    geom_ribbon(aes(ymin = .data$`2.5%`, ymax = .data$`97.5%`), colour = NA, alpha = ifelse(n_r > 1, 0.25, 0.5)) +
    geom_line(aes(y = .data$`50%`), linewidth = 0.75) +
    facet_wrap(vars(Age), scales = "free_y") +
    geom_point(data = cwtesc, aes(y = .data$value)) +
    geom_line(data = cwtesc, aes(y = .data$value), linetype = 3) +
    labs(x = "Brood Year", y = "CWT Escapement") +
    theme(legend.position = "bottom")

  if (n_r == 1) {
    g <- g +
      guides(colour = "none", fill = "none") +
      scale_colour_manual(values = "black") +
      scale_fill_manual(values = "grey")
  }

  g
}

#' @rdname CMfigures
#' @param PT Logical, whether to plot preterminal catch, otherwise (plot terminal catch)
#' @returns
#' - `CM_fit_CWTcatch()` returns ggplot of fit to CWT catch at age
#' @export
CM_fit_CWTcatch <- function(report, d, PT = TRUE, year1 = 1, rs_names) {
  dat <- d[[ifelse(PT, "cwtcatPT", "cwtcatT")]]

  if (sum(dat)) {

    nyears <- dim(dat)[1]
    nage <- dim(dat)[2]
    n_r <- dim(dat)[3]

    if (missing(rs_names)) rs_names <- seq(1, n_r)

    cwtcat <- dat %>%
      structure(dimnames = list(RelYear = year1 + seq(1, nyears) - 1, Age = seq(1, nage), `Release Strategy` = rs_names)) %>%
      reshape2::melt() %>%
      mutate(Age = paste("Age", Age),
             BroodYear = .data$RelYear - 1,
             `Release Strategy` = factor(`Release Strategy`, rs_names)) %>%
      dplyr::filter(value > 0)

    cbrood <- sapply(report, getElement, ifelse(PT, "cbroodPT", "cbroodT"), simplify = "array") %>%
      apply(1:3, quantile, probs = c(0.025, 0.5, 0.975)) %>%
      reshape2::melt() %>%
      mutate(RelYear = Var2 + year1 - 1, BroodYear = .data$RelYear - 1) %>%
      mutate(Age = paste("Age", Var3)) %>%
      mutate(`Release Strategy` = factor(rs_names[Var4], rs_names)) %>%
      reshape2::dcast(Age + BroodYear + `Release Strategy` ~ Var1, value.var = "value")

    g <- ggplot(cbrood, aes(.data$BroodYear, colour = .data$`Release Strategy`, fill = .data$`Release Strategy`)) +
      geom_ribbon(aes(ymin = .data$`2.5%`, ymax = .data$`97.5%`), colour = NA, alpha = ifelse(n_r > 1, 0.25, 0.5)) +
      geom_line(aes(y = .data$`50%`), linewidth = 0.75) +
      facet_wrap(vars(Age), scales = "free_y") +
      geom_point(data = cwtcat, aes(y = .data$value)) +
      geom_line(data = cwtcat, aes(y = .data$value), linetype = 3) +
      labs(x = "Brood Year", y = ifelse(PT, "CWT preterminal catch", "CWT terminal catch")) +
      theme(legend.position = "bottom")

    if (n_r == 1) {
      g <- g +
        guides(colour = "none", fill = "none") +
        scale_colour_manual(values = "black") +
        scale_fill_manual(values = "grey")
    }

    g
  }
}

#' @rdname CMfigures
#' @param r Integer, the release strategy for the figure (only if `annual = FALSE`)
#' @param brood Logical, whether to show results by brood year or return year (FALSE)
#' @param annual Logical, whether to show panel figure by individual year (TRUE) or a single time series figure
#' @returns
#' - `CM_maturity()` returns ggplot of estimated maturity at age
#' @export
#' @importFrom dplyr rename
CM_maturity <- function(report, d, year1 = 1, r = 1, brood = TRUE, annual = FALSE, rs_names) {
  n_r <- d$n_r
  if (missing(rs_names)) rs_names <- seq(1, n_r)

  if (annual) {

    if (brood) {
      matt <- sapply(report, function(i) {
        sapply(1:n_r, function(rr) CY2BY(i[["matt"]][, , rr]), simplify = 'array') # This is actually release year
      }, simplify = 'array')
    } else {
      matt <- sapply(report, getElement, "matt", simplify = 'array')
    }
    matt_q <- apply(matt, 1:3, quantile, probs = c(0.025, 0.5, 0.975), na.rm = TRUE) %>%
      reshape2::melt() %>%
      mutate(Year = Var2 + year1 - 1) %>%
      rename(Age = Var3) %>%
      mutate(`Release Strategy` = factor(rs_names[Var4], rs_names)) %>%
      reshape2::dcast(Age + Var2 + Year + `Release Strategy` ~ Var1) %>%
      filter(!is.na(`50%`))

    if (brood) {
      matt_q$Year <- matt_q$Year - 1 # Release year offset by 1
    }

    g <- matt_q %>%
      ggplot(aes(Age, .data$`50%`, colour = .data$`Release Strategy`, fill = .data$`Release Strategy`)) +
      geom_line() +
      geom_point() +
      geom_ribbon(aes(ymin = `2.5%`, ymax = `97.5%`), colour = NA, alpha = ifelse(n_r > 1, 0.25, 0.5)) +
      facet_wrap(vars(Year)) +
      theme(
        panel.spacing = unit(0, "in"),
        legend.position = "bottom",
        strip.background = element_blank()
      ) +
      labs(x = "Age", y = "Proportion mature", colour = "Release strategy", fill = "Release strategy",
           title = ifelse(brood, "Brood Year", "Return Year"))

    if (n_r == 1) {
      g <- g +
        guides(colour = "none", fill = "none") +
        scale_colour_manual(values = "black") +
        scale_fill_manual(values = "grey")
    }
  } else {

    bmatt <- data.frame(Age = 1:d$Nages, value = d$bmatt) %>%
      dplyr::filter(Age > 1)

    if (brood) {
      matt <- sapply(report, function(i) CY2BY(i[["matt"]][, , r]), simplify = 'array')
    } else {
      matt <- sapply(report, function(i) i[["matt"]][, , r], simplify = 'array')
    }
    matt_q <- apply(matt, 1:2, quantile, probs = c(0.025, 0.5, 0.975), na.rm = TRUE) %>%
      reshape2::melt() %>%
      mutate(Year = Var2 + year1 - 1) %>%
      rename(Age = Var3) %>%
      reshape2::dcast(Age + Var2 + Year ~ Var1) %>%
      filter(!is.na(`50%`))

    if (brood) {
      matt_q$Year <- matt_q$Year - 1 # Release year offset by 1
    }

    g <- matt_q %>%
      dplyr::filter(Age > 1) %>%
      ggplot(aes(Year, .data$`50%`, fill = factor(.data$Age), colour = factor(.data$Age))) +
      geom_line() +
      geom_ribbon(aes(ymin = `2.5%`, ymax = `97.5%`), alpha = 0.2) +
      geom_hline(data = bmatt, linetype = 2, aes(yintercept = .data$value, colour = factor(.data$Age))) +
      labs(x = ifelse(brood, "Brood Year", "Return Year"), y = "Proportion mature", colour = "Age", fill = "Age")

    if (n_r > 1) {
      g <- g +
        ggtitle(paste("Release strategy:", rs_names[r]))

    }
  }

  g
}


#' @rdname CMfigures
#' @param type Character, indicates type of variable to plot
#' @returns
#' - `CM_vul()` returns ggplot of estimated fishery vulnerability at age
#' @export
#' @importFrom dplyr rename
CM_vul <- function(report, type = c("vulPT", "vulT")) {
  type <- match.arg(type)
  vul <- sapply(report, getElement, type) # age x sim

  if (sum(vul)) {
    vul_q <- apply(vul, 1, quantile, probs = c(0.025, 0.5, 0.975)) %>%
      reshape2::melt() %>%
      rename(Age = Var2) %>%
      reshape2::dcast(list("Age", "Var1"), value.var = "value")

    g <- vul_q %>%
      ggplot(aes(.data$Age, .data$`50%`)) +
      geom_line() +
      geom_ribbon(aes(ymin = .data$`2.5%`, ymax = .data$`97.5%`), alpha = 0.2) +
      labs(x = "Age", y = ifelse(type == "vulPT", "Preterminal vulnerability", "Terminal vulnerability"))
    g
  }
}

#' @rdname CMfigures
#' @returns
#' - `CM_SRR()` returns ggplot of estimated stock-recruit relationship (density-dependent juvenile production from egg production) with average
#' relationship and realized annual values. Years correspond to return years of egg production (predicts juvenile production of the following calendar year).
#' @export
CM_SRR <- function(report, year1 = 1) {
  egg <- sapply(report, getElement, "egg") %>% apply(1, median)
  smolt <- sapply(report, function(x) x$N[-1, 1, 1]) %>% apply(1, median)
  epred <- seq(0, 1.1 * max(egg), length.out = 50)
  spred <- sapply(report, function(x) x$alpha * epred * exp(-x$beta * epred)) %>%
    apply(1, quantile, probs = c(0.025, 0.5, 0.975), na.rm = TRUE)

  year <- year1 + seq(1, length(egg)) - 1

  df <- data.frame(
    year = year,
    egg = egg,
    smolt = smolt
  )

  df_med <- data.frame(
    egg = epred,
    smolt = spred[2, ]
  )

  df_poly1 <- data.frame(
    egg = epred,
    smolt = spred[1, ]
  )

  df_poly2 <- data.frame(
    egg = rev(epred),
    smolt = rev(spred[3, ])
  )

  g <- ggplot(df, aes(.data$egg, .data$smolt)) +
    geom_point(shape = 1) +
    geom_line(data = df_med) +
    geom_polygon(data = rbind(df_poly1, df_poly2), fill = "grey", alpha = 0.5) +
    labs(x = "Egg production", y = "Smolt production") +
    expand_limits(x = 0, y = 0)

  if (requireNamespace("ggrepel", quietly = TRUE)) {
    g <- g + ggrepel::geom_text_repel(aes(label = .data$year))
  }
  g
}

.CM_prod <- function(report, d) {
  sapply(1:length(report), function(x) {
    sapply(1:d$Ldyr, function(y) {

      mo <- report[[x]]$mo[y, ]
      matt <- report[[x]]$matt[y, , d$r_matt]

      lo <- numeric(d$Nages)
      lo[1] <- 1
      for (a in 2:d$Nages) {
        lo[a] <- lo[a-1] * exp(-mo[a-1]) * (1 - matt[a-1]) # unfished juvenile survival
      }
      epro <- sum(lo * d$ssum * d$fec * matt)

      return(report[[x]]$alpha * epro)
    })
  })
}

#' @rdname CMfigures
#' @returns
#' - `CM_prod()` returns ggplot of realized productivity in the absence of fishery harvest, annual values are based on natural mortality and maturity at age
#' @export
CM_prod <- function(report, d, year1 = 1) {

  prod <- .CM_prod(report, d)

  prod_q <- apply(prod, 1, quantile, probs = c(0.025, 0.5, 0.975), na.rm = TRUE) %>%
    reshape2::melt() %>%
    mutate(Year = Var2 + year1 - 1) %>%
    reshape2::dcast(list("Year", "Var1"))

  g <- prod_q %>%
    ggplot(aes(Year, .data$`50%`)) +
    geom_line() +
    geom_ribbon(aes(ymin = `2.5%`, ymax = `97.5%`), alpha = 0.2) +
    labs(x = "Calendar Year", y = "Productivity")
  g

}

.CM_Srep <- function(report, d, type = c("spawner", "egg")) {
  type <- match.arg(type)

  sapply(1:length(report), function(x) {
    sapply(1:d$Ldyr, function(y) {

      mo <- report[[x]]$mo[y, ]
      matt <- report[[x]]$matt[y, , d$r_matt]

      lo <- numeric(d$Nages)
      lo[1] <- 1
      for (a in 2:d$Nages) {
        lo[a] <- lo[a-1] * exp(-mo[a-1]) * (1 - matt[a-1]) # unfished juvenile survival
      }
      epro <- sum(lo * d$ssum * d$fec * matt)
      spro <- sum(lo * d$ssum * matt)

      beta_s <- report[[x]]$beta * epro / spro
      alpha_s <- report[[x]]$alpha * epro

      Srep <- log(alpha_s)/beta_s

      if (type == "spawner") {
        return(Srep)
      } else {
        Erep <- Srep * epro / spro
        return(Erep)
      }
    })
  })
}

#' @rdname CMfigures
#' @returns
#' - `CM_Srep()` returns ggplot of realized spawner or egg production at replacement, annual values are based on natural mortality and maturity at age
#' @export
CM_Srep <- function(report, d, year1 = 1, type = c("spawner", "egg")) {
  type <- match.arg(type)

  Srep <- .CM_Srep(report, d, type)

  Srep_q <- apply(Srep, 1, quantile, probs = c(0.025, 0.5, 0.975), na.rm = TRUE) %>%
    reshape2::melt() %>%
    mutate(Year = Var2 + year1 - 1) %>%
    reshape2::dcast(list("Year", "Var1"))

  g <- Srep_q %>%
    ggplot(aes(Year, .data$`50%`)) +
    geom_line() +
    geom_ribbon(aes(ymin = `2.5%`, ymax = `97.5%`), alpha = 0.2) +
    labs(x = "Calendar Year", y = ifelse(type == "spawner", expression(S[rep]), expression(E[rep])))

  g
}


.CM_SMSY <- function(report, d, type = c("spawner", "egg")) {
  type <- match.arg(type)

  sapply(1:length(report), function(x) {
    sapply(1:d$Ldyr, function(y) {

      mo <- report[[x]]$mo[y, ]
      matt <- report[[x]]$matt[y, , d$r_matt]
      vulT <- report[[x]]$vulT
      FT <- report[[x]]$FT

      lo <- numeric(d$Nages)
      lo[1] <- 1
      for (a in 2:d$Nages) {
        lo[a] <- lo[a-1] * exp(-mo[a-1]) * (1 - matt[a-1]) # unfished juvenile survival
      }
      epro <- sum(lo * d$ssum * d$fec * matt)
      alpha_s <- report[[x]]$alpha * epro

      if (!sum(FT) && type == "spawner") {
        spro <- sum(lo * d$ssum * matt)
        beta_s <- report[[x]]$beta * epro / spro
        if (alpha_s > 1) {
          val <- calc_Smsy_Ricker(log(alpha_s), beta_s)
        } else {
          val <- 0
        }
      } else {
        vulPT <- report[[x]]$vulPT
        if (!sum(FT)) vulT[] <- 1
        SRRpars <- data.frame("kappa" = alpha_s, "Emax" = 1/report[[x]]$beta, "phi" = epro, "SRrel" = "Ricker")
        ref <- calc_MSY(Mjuv = c(mo, length(mo)), fec = d$fec, p_female = d$ssum, rel_F = c(0, 1), vulPT = vulPT, vulT = vulT, p_mature = matt,
                        s_enroute = 1, SRRpars = SRRpars)

        if (type == "spawner") {
          val <- ref["Spawners_MSY"]
        } else {
          val <- ref["Egg_MSY"]
        }
      }
      as.numeric(val)
    })
  })

}

CM_SMSY <- function(report, d, year1 = 1, type = c("spawner", "egg")) {
  type <- match.arg(type)

  SMSY <- .CM_SMSY(report, d, type)

  SMSY_q <- apply(SMSY, 1, quantile, probs = c(0.025, 0.5, 0.975), na.rm = TRUE) %>%
    reshape2::melt() %>%
    mutate(Year = Var2 + year1 - 1) %>%
    reshape2::dcast(list("Year", "Var1"))

  g <- SMSY_q %>%
    ggplot(aes(Year, .data$`50%`)) +
    geom_line() +
    geom_ribbon(aes(ymin = `2.5%`, ymax = `97.5%`), alpha = 0.2) +
    labs(x = "Calendar Year", y = ifelse(type == "spawner", expression(S[MSY]), expression(E[MSY])))

  g
}

.CM_Sgen <- function(report, d) {

  sapply(1:length(report), function(x) {
    sapply(1:d$Ldyr, function(y) {

      mo <- report[[x]]$mo[y, ]
      matt <- report[[x]]$matt[y, , d$r_matt]
      vulT <- report[[x]]$vulT
      FT <- report[[x]]$FT

      lo <- numeric(d$Nages)
      lo[1] <- 1
      for (a in 2:d$Nages) {
        lo[a] <- lo[a-1] * exp(-mo[a-1]) * (1 - matt[a-1]) # unfished juvenile survival
      }
      epro <- sum(lo * d$ssum * d$fec * matt)
      spro <- sum(lo * d$ssum * matt)
      alpha_s <- report[[x]]$alpha * epro
      beta_s <- report[[x]]$beta * epro / spro

      if (!sum(FT)) {
        if (alpha_s > 1) {
          val <- calc_Sgen_Ricker(log(alpha_s), beta_s)
        } else {
          val <- 0
        }
      } else {
        vulPT <- report[[x]]$vulPT
        SRRpars <- data.frame("kappa" = alpha_s, "Smax" = 1/beta_s, "Emax" = 1/report[[x]]$beta, "phi" = epro, "tau" = spro, "SRrel" = "Ricker")
        ref <- calc_MSY(Mjuv = c(mo, length(mo)), fec = d$fec, p_female = d$ssum, rel_F = c(0, 1), vulPT = vulPT, vulT = vulT, p_mature = matt,
                        s_enroute = 1, SRRpars = SRRpars)
        SMSY <- ref["Spawners_MSY"]

        val <- calc_Sgen(Mjuv = c(mo, length(mo)), fec = d$fec, p_female = d$ssum, rel_F = c(0, 1), vulPT = vulPT, vulT = vulT, p_mature = matt,
                         s_enroute = 1, SRRpars = SRRpars, SMSY = SMSY)
      }
      as.numeric(val)
    })
  })

}

CM_Sgen <- function(report, d, year1 = 1) {
  Sgen <- .CM_Sgen(report, d)

  Sgen_q <- apply(Sgen, 1, quantile, probs = c(0.025, 0.5, 0.975), na.rm = TRUE) %>%
    reshape2::melt() %>%
    mutate(Year = Var2 + year1 - 1) %>%
    reshape2::dcast(list("Year", "Var1"))

  g <- Sgen_q %>%
    ggplot(aes(Year, .data$`50%`)) +
    geom_line() +
    geom_ribbon(aes(ymin = `2.5%`, ymax = `97.5%`), alpha = 0.2) +
    labs(x = "Year", y = expression(S[gen]))

  g
}


.CM_statevarage <- function(report, year1 = 1, ci = TRUE, var, ylab, xlab = "Year", scales = "free_y") {
  arr <- sapply(report, getElement, var, simplify = "array")

  if (sum(arr, na.rm = TRUE)) {
    if (length(dim(arr)) == 4) { # Include NO/HO dimension

      df <- arr %>%
        apply(1:3, quantile, probs = c(0.025, 0.5, 0.975), na.rm = TRUE) %>%
        reshape2::melt() %>%
        mutate(Year = Var2 + year1 - 1, Origin = ifelse(Var4 == 1, "Natural", "Hatchery")) %>%
        mutate(Age = paste("Age", Var3)) %>%
        reshape2::dcast(Year + Age + Origin ~ Var1, value.var = "value")

      g <- ggplot(df, aes(.data$Year, .data$`50%`, fill = .data$Origin, colour = .data$Origin)) +
        geom_line() +
        facet_wrap(vars(Age), scales = scales) +
        labs(x = xlab, y = ylab) +
        expand_limits(y = 0)

    } else {

      df <- arr %>%
        apply(1:2, quantile, probs = c(0.025, 0.5, 0.975), na.rm = TRUE) %>%
        reshape2::melt() %>%
        mutate(Year = Var2 + year1 - 1) %>%
        mutate(Age = paste("Age", Var3)) %>%
        reshape2::dcast(Year + Age ~ Var1, value.var = "value")

      g <- ggplot(df, aes(.data$Year, .data$`50%`)) +
        geom_line() +
        facet_wrap(vars(Age), scales = scales) +
        labs(x = xlab, y = ylab) +
        expand_limits(y = 0)
    }

    if (ci) g <- g + geom_ribbon(aes(ymin = .data$`2.5%`, ymax = .data$`97.5%`), alpha = 0.2)

    return(g)
  }
  invisible()
}

CM_ts_origin <- function(report, year1 = 1, ci = TRUE, var = "Spawners", ylab = var, xlab = "Year") {
  var <- match.arg(var)
  if (var == "Spawners") {
    arr <- sapply(report, getElement, "syear", simplify = "array") %>%
      apply(c(1, 3, 4), sum)
  }

  df <- arr %>%
    apply(1:2, quantile, probs = c(0.025, 0.5, 0.975), na.rm = TRUE) %>%
    reshape2::melt() %>%
    mutate(Year = Var2 + year1 - 1, Origin = ifelse(Var3 == 1, "Natural", "Hatchery")) %>%
    reshape2::dcast(Year + Origin ~ Var1, value.var = "value")

  g <- ggplot(df, aes(.data$Year, .data$`50%`, colour = .data$Origin, fill = .data$Origin)) +
    geom_line() +
    labs(x = xlab, y = ylab) +
    expand_limits(y = 0)

  if (ci) g <- g + geom_ribbon(aes(ymin = .data$`2.5%`, ymax = .data$`97.5%`), alpha = 0.2)
  g
}


#' @rdname CMfigures
#' @param ci Logical whether to show posterior intervals in addition to posterior median
#' @returns
#' - `CM_M()` returns ggplot of estimated natural mortality time series by age (marine stage)
#' @export
CM_M <- function(report, year1 = 1, ci = TRUE) {
  df <- sapply(report, getElement, "mo", simplify = "array") %>%
    apply(1:2, quantile, probs = c(0.025, 0.5, 0.975)) %>%
    reshape2::melt() %>%
    mutate(Year = Var2 + year1 - 1) %>%
    mutate(Age = paste("Age", Var3)) %>%
    reshape2::dcast(Year + Age ~ Var1, value.var = "value")

  g <- ggplot(df, aes(.data$Year, .data$`50%`)) +
    geom_line() +
    facet_wrap(vars(Age), scales = "free_y") +
    labs(x = "Calendar Year", y = "Natural mortality") +
    expand_limits(y = 0)

  if (ci) g <- g + geom_ribbon(aes(ymin = .data$`2.5%`, ymax = .data$`97.5%`), fill = alpha("grey", 0.5))
  g
}


#' @rdname CMfigures
#' @param surv Logical, whether to plot survival (values between 0 - 1) or instantaneous mortality rates
#' @returns
#' - `CM_Megg()` returns ggplot of egg-juvenile mortality time series
#' @export
CM_Megg <- function(report, year1 = 1, ci = TRUE, surv = FALSE) {
  megg <- sapply(report, getElement, 'megg')

  if (surv) megg <- exp(-megg)

  df <- megg %>%
    apply(1, quantile, probs = c(0.025, 0.5, 0.975)) %>%
    reshape2::melt() %>%
    mutate(Year = Var2 + year1 - 1) %>%
    reshape2::dcast(list("Year", "Var1"), value.var = "value")

  g <- ggplot(df, aes(.data$Year, .data$`50%`)) +
    geom_line() +
    labs(x = "Brood Year", y = ifelse(surv, "Egg-smolt survival", "Egg-smolt mortality")) +
    expand_limits(y = 0)

  if (ci) g <- g + geom_ribbon(aes(ymin = .data$`2.5%`, ymax = .data$`97.5%`), fill = alpha("grey", 0.5))
  g
}


#' @rdname CMfigures
#' @returns
#' - `CM_Njuv()` returns ggplot of juvenile abundance
#' @export
CM_Njuv <- function(report, year1 = 1, ci = TRUE) {
  .CM_statevarage(report, year1, ci, "N", "Juvenile abundance", xlab = "Calendar Year") +
    theme(legend.position = "bottom")
}

#' @rdname CMfigures
#' @returns
#' - `CM_recr()` returns ggplot of recruitment (mature return)
#' @export
CM_recr <- function(report, year1 = 1, ci = TRUE) {
  .CM_statevarage(report, year1, ci, "recr", "Recruitment", xlab = "Return Year") +
    theme(legend.position = "bottom")
}

#' @rdname CMfigures
#' @returns
#' - `CM_esc()` returns ggplot of escapement (after terminal harvest)
#' @export
CM_esc <- function(report, year1 = 1, ci = TRUE) {
  .CM_statevarage(report, year1, ci, "escyear", "Escapement", xlab = "Return Year") +
    theme(legend.position = "bottom")
}

#' @rdname CMfigures
#' @returns
#' - `CM_F()` returns ggplot of instantaneous fishing mortality
#' @export
CM_F <- function(report, PT = TRUE, year1 = 1, ci = TRUE) {
  .CM_ts(
    report, year1, ci,
    var = ifelse(PT, "FPT", "FT"),
    ylab = ifelse(PT, "Preterminal fishing mortality", "Terminal fishing mortality"),
    xlab = "Calendar Year"
  )
}

.CM_ts <- function(report, year1 = 1, ci = TRUE, var, ylab, xlab = "Year") {
  ts <- sapply(report, getElement, var) %>%
    apply(1, quantile, probs = c(0.025, 0.5, 0.975), na.rm = TRUE) %>%
    reshape2::melt() %>%
    mutate(Year = Var2 + year1 - 1) %>%
    reshape2::dcast(list("Year", "Var1"), value.var = "value")

  if (sum(ts$`50%`, na.rm = TRUE)) {
    g <- ggplot(ts, aes(.data$Year, .data$`50%`)) +
      geom_line() +
      labs(x = xlab, y = ylab) +
      expand_limits(y = 0)

    if (ci) g <- g + geom_ribbon(aes(ymin = .data$`2.5%`, ymax = .data$`97.5%`), fill = alpha("grey", 0.5))
    return(g)
  }
  invisible()
}

#' @rdname CMfigures
#' @returns
#' - `CM_surv()` returns ggplot of natural survival (converting from instantaneous units of natural mortality)
#' @export
CM_surv <- function(report, year1 = 1, ci = TRUE) {
  df <- exp(-sapply(report, getElement, "mo", simplify = "array")) %>%
    apply(1:2, quantile, probs = c(0.025, 0.5, 0.975)) %>%
    reshape2::melt() %>%
    mutate(Year = Var2 + year1 - 1) %>%
    mutate(Age = paste("Age", Var3)) %>%
    reshape2::dcast(Year + Age ~ Var1, value.var = "value")

  g <- ggplot(df, aes(.data$Year, .data$`50%`)) +
    geom_line() +
    facet_wrap(vars(Age), scales = "free_y") +
    labs(x = "Calendar Year", y = "Natural survival") +
    expand_limits(y = 0)

  if (ci) g <- g + geom_ribbon(aes(ymin = .data$`2.5%`, ymax = .data$`97.5%`), fill = alpha("grey", 0.5))
  g
}


#' @rdname CMfigures
#' @returns
#' - `CM_wt()` returns ggplot of annual deviations in egg-juvenile mortality from the Ricker function
#' @export
CM_wt <- function(stanfit, year1 = 1, ci = TRUE) {

  wt <- try(rstan::extract(stanfit, "wt")$wt, silent = TRUE)

  if (!is.character(wt)) {
    ts <- wt %>%
      apply(2, quantile, probs = c(0.025, 0.5, 0.975)) %>%
      reshape2::melt() %>%
      mutate(Year = Var2 + year1 - 1) %>%
      reshape2::dcast(list("Year", "Var1"), value.var = "value")

    g <- ggplot(ts, aes(.data$Year, .data$`50%`)) +
      geom_line() +
      geom_point() +
      geom_hline(yintercept = 0, linetype = 2) +
      labs(x = "Brood Year", y = "Egg mortality deviation")

    if (ci) g <- g + geom_ribbon(aes(ymin = .data$`2.5%`, ymax = .data$`97.5%`), fill = alpha("grey", 0.5))
    g
  }

}

#' @rdname CMfigures
#' @returns
#' - `CM_surv2()` returns ggplot of annual survival to age 2, which includes age-1 mortality (marine life stage) for both natural and hatchery origin fish.
#' Hatchery fish experience additional mortality specified by release mortality.
#' @importFrom dplyr n
#' @export
CM_surv2 <- function(report, year1 = 1, ci = TRUE, ylab = "Survival to age 2") {

  NO <- exp(-sapply(report, function(x) x$mo[, 1])) %>%
    apply(1, quantile, probs = c(0.025, 0.5, 0.975)) %>%
    t() %>%
    as.data.frame() %>%
    mutate(type = "Natural", Year = year1 + 1:n() - 1)

  HO <- exp(-sapply(report, getElement, "moplot")) %>%
    apply(1, quantile, probs = c(0.025, 0.5, 0.975)) %>%
    t() %>%
    as.data.frame() %>%
    mutate(type = "Hatchery", Year = year1 + 1:n() - 1)

  df <- rbind(NO, HO) %>%
    mutate(BroodYear = .data$Year - 1)

  g <- ggplot(df, aes(.data$BroodYear, .data$`50%`)) +
    geom_line(aes(colour = .data$type)) +
    labs(x = "Brood Year", y = ylab, fill = "Origin", colour = "Origin") +
    expand_limits(y = 0) +
    theme(legend.position = "bottom")

  if (ci) g <- g + geom_ribbon(aes(ymin = .data$`2.5%`, ymax = .data$`97.5%`, fill = .data$type), alpha = 0.25)
  g
}

#' @rdname CMfigures
#' @returns
#' - `CM_wt()` returns ggplot of annual deviations in age 1 natural mortality (first year in marine life stage, deviations from time series average)
#' @export
CM_wto <- function(stanfit, year1 = 1, ci = TRUE) {

  wto <- try(rstan::extract(stanfit, "wto")$wto, silent = TRUE)

  if (!is.character(wto)) {
    ts <- wto %>%
      apply(2, quantile, probs = c(0.025, 0.5, 0.975)) %>%
      reshape2::melt() %>%
      mutate(Year = Var2 + year1 - 1, BroodYear = Year - 1) %>%
      reshape2::dcast(list("BroodYear", "Var1"), value.var = "value")

    g <- ggplot(ts, aes(.data$BroodYear, .data$`50%`)) +
      geom_line() +
      geom_point() +
      geom_hline(yintercept = 0, linetype = 2) +
      labs(x = "Brood Year", y = "Age 1 mortality deviation")

    if (ci) g <- g + geom_ribbon(aes(ymin = .data$`2.5%`, ymax = .data$`97.5%`), fill = alpha("grey", 0.5))
    g
  }

}

#CM_fitness <- function(report, year) {
#  .CM_ts(
#    report, year1, ci,
#    var = "fitness",
#    ylab = "Fitness"
#  )
#}

# Convert calendar year matrix to brood year matrix
CY2BY <- function(x) {

  nt <- nrow(x)
  na <- ncol(x)

  xbrood <- matrix(NA_real_, nt, na)
  for (t in 1:nt) {
    for (a in 1:na) {
      if (t+a-1 <= nt) {
        xbrood[t, a] <- x[t+a-1, a]
      }
    }
  }
  return(xbrood)
}

# Calculate adult equivalent
calc_AEQ <- function(report, brood = TRUE) {

  nt <- dim(report$matt)[1]
  na <- dim(report$matt)[2]
  n_r <- dim(report$matt)[3]

  # Always by brood year
  matt <- sapply(1:n_r, function(r) CY2BY(report$matt[, , r]), simplify = "array")
  M_BY <- CY2BY(report$mo)
  surv <- exp(-M_BY)

  AEQ <- array(NA_real_, dim(matt))
  AEQ[, na, ] <- 1
  for (t in seq(nt, 2) - 1) {
    for (a in seq(na, 2) - 1) {
      AEQ[t, a, ] <- matt[t, a, ] + (1 - matt[t, a, ]) * surv[t, a] * AEQ[t, a+1, ]
    }
  }

  return(AEQ)
}


#' @rdname CMfigures
#' @param at_age Logical, whether to make figure by individual age
#' @returns
#' - `CM_ER()` returns ggplot of exploitation rate either by individual age or aggregate values using adult equivalents
#' @export
CM_ER <- function(report, brood = TRUE, type = c("PT", "T", "all"), year1 = 1, ci = TRUE, at_age = TRUE, r = 1) {

  type <- match.arg(type)

  if (at_age) {

    ER_list <- lapply(report, function(i) {
      name <- ifelse(type == "PT", "survPT", "survT")
      CYER <- 1 - i[[name]]
      if (brood) {
        ER <- CY2BY(CYER)
      } else {
        ER <- CYER
      }
      return(list(ER = ER))
    })

    g <- .CM_statevarage(
      ER_list,
      year1 = if (brood) year1 - 1 else year1,
      ci,
      "ER",
      ylab = switch(
        type,
        "PT" = "Preterminal exploitation rate",
        "T" = "Terminal exploitation rate"
      ),
      xlab = ifelse(brood, "Brood Year", "Calendar Year"),
      scales = "fixed"
    )

  } else {

    ER_list <- lapply(report, function(i) {

      esc <- apply(i$escyear, 1:2, sum)
      morts_PT <- apply(i$cyearPT, 1:2, sum)
      morts_T <- apply(i$cyearT, 1:2, sum)

      AEQ_PT <- calc_AEQ(i)[, , r] # Always by release year
      AEQ_T <- array(1, dim(esc))

      if (brood) {
        esc <- CY2BY(esc)
        morts_PT <- CY2BY(morts_PT)
        morts_T <- CY2BY(morts_T)

        denom <- rowSums(morts_PT * AEQ_PT + morts_T * AEQ_T + esc)

      } else {
        AEQ_PT2 <- array(NA_real_, dim(esc)) # Re-index to align with calendar year
        nt <- nrow(AEQ_PT2)
        na <- ncol(AEQ_PT2)
        for (t in 1:nt) {
          for (a in 1:na) if (t-a+1>0) AEQ_PT2[t, a] <- AEQ_PT[t-a+1, a]
        }
        denom <- rowSums(morts_PT * AEQ_PT2 + morts_T * AEQ_T + esc)
      }

      if (type == "PT") {
        num <- rowSums(morts_PT * AEQ_PT)
      } else if (type == "T") {
        num <- rowSums(morts_T * AEQ_T)
      } else {
        num <- rowSums(morts_PT * AEQ_PT + morts_T * AEQ_T)
      }

      list(ER = num/denom)
    })

    g <- .CM_ts(
      ER_list,
      year1 = if (brood) year1 - 1 else year1,
      ci,
      var = "ER",
      xlab = ifelse(brood, "Brood Year", "Calendar Year"),
      ylab = switch(
        type,
        "PT" = "Preterminal exploitation rate",
        "T" = "Terminal exploitation rate",
        "all" = "Total exploitation rate"
      )
    )
  }

  g
}

#' @rdname CMfigures
#' @returns
#' - `CM_CWT_ER()` returns ggplot of CWT exploitation rate (by release strategy)
#' @export
#' @importFrom dplyr left_join
CM_CWT_ER <- function(report, brood = TRUE, type = c("PT", "T", "all"), year1 = 1, ci = TRUE, rs_names) {

  type <- match.arg(type)

  ER <- sapply(report, function(i) {

    if (brood) {
      esc <- i$ebrood
      morts_PT <- i$cbroodPT
      morts_T <- i$cbroodT
    } else {
      esc <- i$ecwt
      morts_PT <- i$ccwtPT
      morts_T <- i$ccwtT
    }

    AEQ_PT <- calc_AEQ(i, brood = brood)
    AEQ_T <- array(1, dim(esc))

    denom <- apply(morts_PT * AEQ_PT + morts_T * AEQ_T + esc, c(1, 3), sum) # year x rs

    if (type == "PT") {
      num <- apply(morts_PT * AEQ_PT, c(1, 3), sum)
    } else if (type == "T") {
      num <- apply(morts_T * AEQ_T, c(1, 3), sum)
    } else {
      num <- apply(morts_PT * AEQ_PT + morts_T * AEQ_T, c(1, 3), sum)
    }

    return(num/denom)
  }, simplify = "array")

  if (missing(rs_names)) rs_names <- seq(1, dim(ER)[2])
  n_r <- length(rs_names)

  ts <- apply(ER, 1:2, quantile, probs = c(0.025, 0.5, 0.975), na.rm = TRUE) %>%
    reshape2::melt() %>%
    mutate(Year = Var2 + year1 - 1, `Release Strategy` = factor(rs_names[Var3], rs_names)) %>%
    reshape2::dcast(Year + `Release Strategy` ~ Var1, value.var = "value")

  if (sum(ts$`50%`, na.rm = TRUE)) {

    has_rel <- sapply(1:n_r, function(r) {
      rowSums(CY2BY(report[[1]]$ecwt[, , r]), na.rm = TRUE) > 0
    }, simplify = "array") %>%
      as.data.frame()
    colnames(has_rel) <- rs_names
    has_rel$Year <- year1 + seq(1, nrow(has_rel)) - 1

    has_rel <- has_rel %>%
      reshape2::melt(id.var = "Year", variable.name = "Release Strategy") %>%
      filter(value == TRUE)

    g <- left_join(has_rel, ts, by = c("Year", "Release Strategy")) %>%
      ggplot(aes(.data$Year, .data$`50%`, colour = .data$`Release Strategy`)) +
      geom_line() +
      geom_point() +
      labs(x = ifelse(brood, "Brood year", "Return year"),
           y = switch(
             type,
             "PT" = "Preterminal exploitation rate",
             "T" = "Terminal exploitation rate",
             "all" = "Total exploitation rate"
           )) +
      expand_limits(y = 0) +
      theme(legend.position = "bottom")

    if (ci) {
      g <- g +
        geom_ribbon(aes(ymin = .data$`2.5%`, ymax = .data$`97.5%`, fill = `Release Strategy`),
                    alpha = ifelse(n_r > 1, 0.25, 0.5))
    }

    if (n_r == 1) {
      g <- g +
        guides(colour = "none", fill = "none") +
        scale_colour_manual(values = "black") +
        scale_fill_manual(values = "grey")
    }
  }

  g
}


#' @rdname CMfigures
#' @param x Matrix of covariates by year x covariate
#' @param names Character of covariate names
#' @param b Matrix of fixed effect coefficients by simulation x covariate. If missing only the covariates (`x`) are plotted,
#' otherwise, the dot product `sum(x * b)` is calculated by individual simulation and quantiles are plotted
#' @param ylab Character y axis label
#' @returns
#' - `CM_covariate()` returns ggplot of mortality covariates
#' @export
#' @importFrom dplyr left_join
CM_covariate <- function(x, names, year1 = 1, b, ylab = "Covariate") {

  if (sum(x)) {

    if (!is.matrix(x)) x <- matrix(x, ncol = 1)
    if (missing(names)) names <- paste("Covariate", 1:ncol(x))

    if (missing(b)) { # Plot covariate

      g <- structure(x, dimnames = list(Year = 1:nrow(x) + year1 - 1, Covariate = names)) %>%
        reshape2::melt() %>%
        ggplot(aes(.data$Year, .data$value, colour = .data$Covariate)) +
        geom_line() +
        geom_point() +
        labs(x = "Calendar Year", y = ylab, colour = NULL) +
        expand_limits(y = 0) +
        theme(legend.position = "bottom")

    } else { # Plot M = covariate x coefficient

      if (!is.matrix(b)) b <- matrix(b, ncol = 1) # simulation x cov
      nsim <- nrow(b)
      ncov <- ncol(b)
      nt <- nrow(x)

      Mcov <- array(NA_real_, c(nsim, ncov, nt))

      i <- expand.grid(
        x = 1:nsim,
        j = 1:ncov,
        t = 1:nt
      ) %>% as.matrix()

      i_b <- i[, c("x", "j")]
      i_x <- i[, c("t", "j")]

      Mcov[i] <- b[i_b] * x[i_x]

      g <- Mcov %>%
        apply(2:3, quantile, c(0.025, 0.5, 0.975)) %>%
        reshape2::melt() %>%
        mutate(Covariate = names[Var2], Year = Var3 + year1 - 1) %>%
        reshape2::dcast(Year + Covariate ~ Var1, value.var = "value") %>%
        ggplot(aes(.data$Year, .data$`50%`, colour = .data$Covariate, fill = .data$Covariate)) +
        geom_line() +
        geom_ribbon(aes(ymin = .data$`2.5%`, ymax = .data$`97.5%`), alpha = 0.2) +
        labs(x = "Calendar Year", y = ylab, colour = NULL, fill = NULL) +
        expand_limits(y = 0) +
        theme(legend.position = "bottom")
    }

    return(g)

  }

}

#' Conditioning model markdown report
#'
#' Generate a markdown report to plot time series and MCMC posteriors of estimates from the conditioning model. See [get_report()] for the
#' various plotting functions used in the report.
#'
#' @param stanfit Output from [sample_CM()]
#' @param year Optional vector of calendar years
#' @param cov1_names Optional character vector for names of covariates that predict age-1 natural mortality
#' @param cov_names Optional character vector for names of covariates that predict age-2+ natural mortality
#' @param rs_names Optional character vector for names of hatchery release strategies
#' @param name Optional character string for the model name to include in the report, e.g., model run number
#' @param filename Character string for the name of the markdown and HTML files
#' @param dir The directory in which the markdown and HTML files will be saved.
#' @param open_file Logical, whether the HTML document is opened after it is rendered
#' @param render_args List of arguments to pass to [rmarkdown::render()]
#' @param ... Additional arguments (not used)
#' @details Report excludes MCMC values from warmup iterations
#' @returns Returns invisibly the output of [rmarkdown::render()], typically the path of the output file
#' @seealso [fit_CM()] [get_report()]
#' @export
report_CM <- function(stanfit, year, cov1_names, cov_names, rs_names,
                      name, filename = "CM", dir = tempdir(), open_file = TRUE, render_args = list(), ...) {

  if (!requireNamespace("ggrepel", quietly = TRUE)) {
    warning("Install ggrepel package to label years for stock-recruit figure.")
  }

  report <- get_report(stanfit, inc_warmup = FALSE)
  fit <- stanfit@.MISC$CMfit
  d <- get_CMdata(fit)

  if (missing(year)) year <- 1:d$Ldyr
  if (missing(name)) name <- "CM fit"
  year1 <- year[1]

  ####### Function arguments for rmarkdown::render
  rmd <- system.file("include", "CMreport.Rmd", package = "salmonMSE") %>% readLines()
  rmd_split <- split(rmd, 1:length(rmd))

  name_ind <- grep("NAME", rmd)
  rmd_split[[name_ind]] <- paste("#", name, "{.tabset}")

  filename_rmd <- paste0(filename, ".Rmd")

  render_args$input <- file.path(dir, filename_rmd)
  if (is.null(render_args$quiet)) render_args$quiet <- TRUE

  # Generate markdown report
  if (!dir.exists(dir)) {
    message("Creating directory: ", dir)
    dir.create(dir)
  }
  write(unlist(rmd_split), file = file.path(dir, filename_rmd))

  # Rendering markdown file
  message("Rendering markdown file: ", file.path(dir, filename_rmd))
  output_filename <- do.call(rmarkdown::render, render_args)
  message("Rendered file: ", output_filename)

  if (open_file) browseURL(output_filename)
  invisible(output_filename)
}

## Internal functions to predict either:
# (b) smolt hatchery production
# (c) smolt natural production
calc_broodtake <- function(NOR_escapement, HOR_escapement, stray, brood_import, ptarget_NOB, pmax_NOB, phatchery, egg_target, p_female,
                           fec_brood, s_prespawn, m) {

  if (is.na(phatchery)) {
    HO_avail <- cbind(
      HOR_escapement,
      stray,
      brood_import
    )
  } else {
    HO_avail <- cbind(
      phatchery * HOR_escapement,
      stray,
      brood_import
    )
  }

  if (sum(HO_avail)) {

    # Optimization for pNOB target, but can fail for two reasons:
    # (1) not enough unmarked fish
    # (2) more than enough but too many unmarked hatchery fish but can't solve for ptarget_NOB
    opt <- try(
      uniroot(.broodtake_func, c(.Machine$double.eps, pmax_NOB),
              tol = .Machine$double.eps,
              NOR_escapement = NOR_escapement, HOR_escapement = HOR_escapement, stray = stray, brood_import = brood_import,
              phatchery = phatchery, p_female = p_female,
              fec = fec_brood, gamma = 1, egg_target = egg_target, s_prespawn = s_prespawn, ptarget_NOB = ptarget_NOB, m = m),
      silent = TRUE
    )

    # Assume fail for (1), then set ptake_marked = pmax_NOB
    if (is.character(opt)) {
      ptake_unmarked <- pmax_NOB
    } else {
      ptake_unmarked <- opt$root
    }

    output <- .broodtake_func(
      ptake_unmarked, NOR_escapement = NOR_escapement, HOR_escapement = HOR_escapement, stray = stray, brood_import = brood_import,
      phatchery = phatchery, p_female = p_female,
      fec = fec_brood, gamma = 1, egg_target = egg_target, s_prespawn = s_prespawn, ptarget_NOB = ptarget_NOB, m = m, opt = FALSE
    )

    egg_total <- output$egg_NOB + output$egg_HOB_unmarked + output$egg_HOB_marked

    # Ensure hatchery egg production doesn't exceed target
    # If it does, then optimization failed for reason (2), in which case, we use unmarked fish only to reach production target
    if (egg_total/egg_target > 1.01) {
      unmarked_opt <- try(
        uniroot(.egg_func, c(.Machine$double.eps, pmax_NOB),
                tol = .Machine$double.eps,
                N = NOR_escapement + (1 - m) * HOR_escapement,
                gamma = 1, fec = fec_brood, p_female = p_female,
                s_prespawn = s_prespawn, val = egg_target),
        silent = TRUE
      )
      if (is.character(unmarked_opt)) {
        ptake_unmarked <- pmax_NOB
      } else {
        ptake_unmarked <- unmarked_opt$root
      }
      output <- .broodtake_func(
        ptake_unmarked, NOR_escapement = NOR_escapement, HOR_escapement = HOR_escapement, stray = stray, brood_import = brood_import,
        phatchery = phatchery, p_female = p_female,
        fec = fec_brood, gamma = 1, egg_target = egg_target, s_prespawn = s_prespawn, ptarget_NOB = ptarget_NOB, m = m, opt = FALSE
      )
    }

    NOB <- output$NOB
    HOB <- output$HOB_unmarked + output$HOB_marked
    HOB_import <- output$HOB_import
    pNOB <- output$pNOB

  } else {

    NOBopt <- try(
      uniroot(.egg_func, c(1e-8, pmax_NOB), N = NOR_escapement, gamma = 1, fec = fec_brood, p_female = p_female,
              s_prespawn = s_prespawn, val = egg_target),
      silent = TRUE
    )
    if (is.character(NOBopt)) {
      ptake_NOB <- pmax_NOB
    } else {
      ptake_NOB <- NOBopt$root
    }
    NOB <- ptake_NOB * NOR_escapement
    egg_NOB <- sum(NOB * s_prespawn * fec_brood * p_female)
    HOB_zero <- array(0, dim(HOR_escapement))
    HOB_import <- rep(0, nrow(NOB))
    pNOB <- 1

    # Check that output is the same as .broodtake_func()
    output <- list(
      egg_NOB = egg_NOB,
      egg_HOB_unmarked = 0,
      egg_HOB_marked = 0,
      ptake_unmarked = ptake_NOB,
      ptake_marked = 0,
      pNOB = 1,
      NOB = NOB,
      HOB_unmarked = HOB_zero,
      HOB_marked = HOB_zero,
      HOB_import = rep(0, nrow(NOB)),
      HOB_stray = HOB_zero
    )
  }

  return(output)
}

calc_spawners <- function(broodtake, escapement_NOS, escapement_HOS, stray, phatchery, premove_HOS, premove_NOS, m) {
  if (is.null(premove_HOS)) premove_HOS <- 0
  if (is.null(premove_NOS)) premove_NOS <- 0
  if (is.null(m)) m <- 0

  spawners <- list()

  NO <- escapement_NOS - broodtake$NOB
  #spawners$NOS <- escapement_NOS - broodtake$NOB

  if (sum(stray)) {
    spawners$HOS_stray <- stray - broodtake$HOB_stray # Assume unmarked so not subject to premove_HOS!
  } else {
    spawners$HOS_stray <- array(0, dim(escapement_HOS))
  }

  if (sum(escapement_HOS)) {
    if (is.na(phatchery)) {
      HO <- escapement_HOS - broodtake$HOB_marked - broodtake$HOB_unmarked
    } else {
      HO <- escapement_HOS * (1 - phatchery)
    }
    if (is.function(premove_HOS)) {
      p <- premove_HOS(NO, HO, m)
    } else if (is.numeric(premove_HOS)) {
      p <- premove_HOS * m
    } else {
      stop("Error with premove_HOS")
    }
    HOS_local <- HO * (1 - p)
    spawners$HO_remove <- HO * p
  } else {
    HOS_local <- spawners$HO_remove <- HO <- array(0, dim(escapement_HOS))
  }
  spawners$HOS <- HOS_local + spawners$HOS_stray

  if (sum(escapement_NOS)) {

    if (is.function(premove_NOS)) {
      pp <- premove_NOS(NO, HO, m)
    } else if (is.numeric(premove_NOS)) {
      pp <- premove_NOS * (1 - m)
    } else {
      stop("Error with premove_NOS")
    }

    spawners$NOS <- NO * (1 - pp)
    spawners$NO_remove <- NO * pp
  } else {
    spawners$NOS <- spawners$NO_remove <- array(0, dim(escapement_NOS))
  }

  return(spawners)
}


.egg_func <- function(ptake, N, gamma = 1, fec, p_female, s_prespawn, val = 0) {
  sum(ptake * gamma * N * s_prespawn * fec * p_female) - val
}

calc_broodtake_custom <- function(f_brood, NOR_escapement, HOR_escapement, stray, p_female, fec, s_prespawn, m,
                                  egg_target) {

  brood <- f_brood(NOR_escapement, HOR_escapement, stray, m)
  brood$egg_NOB <- sum(brood$NOB * s_prespawn * fec * p_female)
  brood$egg_HOB_unmarked <- sum((brood$HOB_unmarked + brood$HOB_stray) * s_prespawn * fec * p_female)
  brood$egg_HOB_marked <- sum(brood$HOB_marked * s_prespawn * fec * p_female)
  brood$total_egg <- brood$egg_NOB + brood$egg_HOB_unmarked + brood$egg_HOB_marked

  if (brood$total_egg/egg_target > 1.001) {
    brood_init <- cbind(brood$NOB, brood$HOB_marked, brood$HOB_unmarked, brood$HOB_stray)
    opt <- uniroot(.egg_func, c(1e-8, 1), N = brood_init, fec = fec,
                   p_female = p_female, s_prespawn = s_prespawn, val = egg_target)
    p_adjust <- opt$root

  } else {
    p_adjust <- 1
  }

  NOB <- p_adjust * brood$NOB
  HOB_marked <- p_adjust * brood$HOB_marked
  HOB_unmarked <- p_adjust * brood$HOB_unmarked
  HOB_stray <- p_adjust * brood$HOB_stray

  egg_NOB <- sum(NOB * s_prespawn * fec * p_female)
  egg_HOB_unmarked <- sum((HOB_unmarked + HOB_stray) * s_prespawn * fec * p_female)
  egg_HOB_marked <- sum(HOB_marked * s_prespawn * fec * p_female)

  pNOB <- sum(fec * NOB)/sum(fec * NOB, fec * HOB_unmarked, fec * HOB_marked, fec * HOB_stray)

  # Check output is the same as .broodtake_func()
  output <- list(
    egg_NOB = egg_NOB,
    egg_HOB_unmarked = egg_HOB_unmarked,
    egg_HOB_marked = egg_HOB_marked,
    ptake_unmarked = sum(NOB, HOB_unmarked, HOB_stray)/sum(NOR_escapement, HOR_escapement, stray),
    ptake_marked = sum(HOB_marked)/sum(NOR_escapement, HOR_escapement, stray),
    pNOB = pNOB,
    NOB = NOB,
    HOB_unmarked = HOB_unmarked,
    HOB_marked = HOB_marked,
    HOB_import = rep(0, nrow(HOR_escapement)),
    HOB_stray = HOB_stray
  )
  return(output)
}

.broodtake_func <- function(ptake_unmarked, NOR_escapement, HOR_escapement, stray, brood_import, phatchery, p_female, fec, gamma,
                            egg_target, s_prespawn, ptarget_NOB, m = 1, opt = TRUE) {

  NOB <- ptake_unmarked * NOR_escapement

  if (is.na(phatchery)) {
    HOB_unmarked <- ptake_unmarked * (1 - m) * HOR_escapement
    HO_avail_marked <- cbind(
      m * HOR_escapement,
      brood_import
    )
  } else {
    HOB_unmarked <- ptake_unmarked * phatchery * (1 - m) * HOR_escapement
    HO_avail_marked <- cbind(
      m * phatchery * HOR_escapement,
      brood_import
    )
  }
  HOB_stray <- ptake_unmarked * stray

  egg_NOB <- sum(NOB * s_prespawn * fec * p_female)
  egg_HOB_unmarked <- sum((HOB_unmarked + HOB_stray) * s_prespawn * fec * p_female)

  egg_HOB_marked <- egg_target - egg_NOB - egg_HOB_unmarked

  if (egg_HOB_marked < 0 || !sum(HO_avail_marked)) {
    ptake_marked <- 0
  } else {
    # solve for the required HOB marked
    HOBopt <- try(
      uniroot(.egg_func, c(1e-8, 1), N = HO_avail_marked, gamma = gamma, fec = fec, p_female = p_female,
              s_prespawn = s_prespawn, val = egg_HOB_marked),
      silent = TRUE
    )
    if (is.character(HOBopt)) {
      ptake_marked <- 1
    } else {
      ptake_marked <- HOBopt$root
    }
  }

  egg_HOB_marked_actual <- .egg_func(
    ptake_marked, N = HO_avail_marked, gamma = gamma, fec = fec, p_female = p_female, s_prespawn = s_prespawn
  )
  HOB_marked <- ptake_marked * HO_avail_marked[, 1:ncol(HOR_escapement), drop = FALSE]
  HOB_import <- ptake_marked * HO_avail_marked[, ncol(HO_avail_marked)]

  if (opt) {
    p_unmarked <- sum(NOB, HOB_unmarked, HOB_stray)/sum(NOB, HOB_unmarked, HOB_marked, HOB_import, HOB_stray)
    obj <- log(p_unmarked) - log(ptarget_NOB)   # Objective function, get close to zero, if not stay positive
    return(obj)
  } else {

    pNOB <- sum(fec * NOB)/sum(fec * NOB, fec * HOB_unmarked, fec * HOB_marked, fec * HOB_stray, fec * HOB_import)
    output <- list(
      egg_NOB = egg_NOB,
      egg_HOB_unmarked = egg_HOB_unmarked,
      egg_HOB_marked = egg_HOB_marked_actual,
      ptake_unmarked = ptake_unmarked,
      ptake_marked = ptake_marked,
      pNOB = pNOB,
      NOB = NOB,
      HOB_unmarked = HOB_unmarked,
      HOB_marked = HOB_marked,
      HOB_import = HOB_import,
      HOB_stray = HOB_stray
    )
    return(output)
  }
}

calc_yearling <- function(egg_target, s_yearling, s_subyearling, p_yearling, p_subyearling) {

  if (sum(p_yearling) == 1 || sum(p_yearling) == 0) {
    ans <- .yearling_func(
      p_egg_yearling = sum(p_yearling), egg_target = egg_target, s_yearling = s_yearling, s_subyearling = s_subyearling, opt = FALSE
    )
  } else {
    opt <- uniroot(
      .yearling_func, interval = c(1e-8, 0.9999),
      egg_target = egg_target, s_yearling = s_yearling, s_subyearling = s_subyearling, p_yearling = sum(p_yearling)
    )
    ans <- .yearling_func(
      p_yearling = opt$root, egg_target = egg_target, s_yearling = s_yearling, s_subyearling = s_subyearling, opt = FALSE
    )
  }

  yearling <- ans[1] * p_yearling/sum(p_yearling)
  subyearling <- ans[2] * p_subyearling/sum(p_subyearling)

  yearling[is.na(yearling)] <- 0
  subyearling[is.na(subyearling)] <- 0

  list(yearling = yearling, subyearling = subyearling)
}

.yearling_func <- function(p_egg_yearling, egg_target, s_yearling, s_subyearling, p_yearling, opt = TRUE) {
  egg_yearling <- p_egg_yearling * egg_target
  egg_subyearling <- egg_target - egg_yearling
  yearling <- egg_yearling * s_yearling
  subyearling <- egg_subyearling * s_subyearling

  if (opt) {
    frac <- yearling/(yearling + subyearling)
    return(frac - p_yearling)
  } else {
    return(c(yearling, subyearling))
  }
}

#' Smolt production
#'
#' Calculate smolt production from base stock-recruit parameters and fitness loss
#'
#' @param N1 Egg production for the density-independent component of the stock-recruit relationship. Can be the number of spawners if `phi = 1`
#' and `Smax` is in units of spawners.
#' @param N2 Egg production for the density-dependent component of the stock-recruit relationship (only used if `per_recruit = FALSE`)
#' @param kappa Base productivity parameter
#' @param capacity Base capacity parameter if `SRrel = "BH"`
#' @param Smax Base Smax parameter if `SRrel = "Ricker"`
#' @param phi Unfished egg per smolt (`1/phi` corresponds to the one-to-one adult/spawner replacement line)
#' @param tau Unfished spawner per smolt
#' @param fitness_loss Survival term to reduce smolt production due to fitness, between 0-1
#' @param SRrel Character for the stock-recruit function
#' @param per_recruit Logical, whether N1 is a per recruit quantity (TRUE) or in absolute numbers (FALSE)
#' @return Numeric
#' @export
calc_smolt <- function(N1, N2 = N1, kappa, capacity, Smax, phi = 1, tau = 1, fitness_loss = 1,
                       SRrel = c("BH", "Ricker"), per_recruit = FALSE) {
  SRrel <- match.arg(SRrel)

  a <- kappa/phi * fitness_loss
  if (SRrel == "BH") {
    b <- a/(capacity * fitness_loss)

    if (per_recruit) {
      smolt <- (a * N1 - 1)/b/N1
    } else {
      smolt <- a * N1 / (1 + b * N2)
    }
  } else {
    Smax_prime <- Smax * fitness_loss
    Emax <- Smax_prime * phi/tau
    b <- 1/Emax

    if (per_recruit) {
      smolt <- log(a * N1)/b/N1
    } else {
      smolt <- a * N1 * exp(-b * N2)
    }
  }
  return(smolt)
}

# pbar Vector length 2. Pbar value from the previous generation.
# First value is for the natural population and the second value for the hatchery
calc_Ford_fitness <- function(pNOB, pHOS, pbar_prev, theta, fitness_variance, phenotype_variance, A, fitness_floor, heritability) {

  pbar <- numeric(2)

  pbar[1] <- local({
    qHOS <- 1 - pHOS

    aa <- (pbar_prev[1] * fitness_variance + theta[1] * phenotype_variance)/(fitness_variance + phenotype_variance)
    bb <- qHOS * (pbar_prev[1] + (aa - pbar_prev[1]) * heritability)
    cc <- (pbar_prev[2] * fitness_variance + theta[1] * phenotype_variance)/(fitness_variance + phenotype_variance)
    dd <- pHOS * (pbar_prev[2] + (cc - pbar_prev[2]) * heritability)

    bb + dd
  })

  pbar[2] <- local({
    qNOB <- 1 - pNOB

    aa <- (pbar_prev[2] * fitness_variance + theta[2] * phenotype_variance)/(fitness_variance + phenotype_variance)
    bb <- qNOB * (pbar_prev[2] + (aa - pbar_prev[2]) * heritability)
    cc <- (pbar_prev[1] * fitness_variance + theta[2] * phenotype_variance)/(fitness_variance + phenotype_variance)
    dd <- pNOB * (pbar_prev[1] + (cc - pbar_prev[1]) * heritability)

    bb + dd
  })

  fitness <- pmax(fitness_floor, exp(-0.5 * (pbar[1] - theta[1])^2/(fitness_variance + phenotype_variance)))

  return(list(pbar = pbar, fitness = fitness))
}

## INCOMPLETE
#calc_Busack_fitness <- function(pNOS, Zpop_prev, theta) {
#
#  # Zpop1
#  pNOS <- NOS/(NOS + RRS_HOS * HOS_total)
#
#  HOS_seg <- SAR_ratio * strays_total
#  HOS_int <- HOS_total - HOS_seg
#
#  pHOS_seg <- RRS_HOS * HOS_seg/(NOS + RRS_HOS * HOS_total)
#  pHOS_int <- RRS_HOS * HOS_int/(NOS + RRS_HOS * HOS_total)
#
#  Zpop[1, g] <- A * pNOS * (Zpop[1, g-1] - theta[1]) +
#    pHOS_int * (Zpop[2, g-1] - theta[1]) + pHOS_seg * (Zpop[3, g-1] - theta[1]) + theta[1]
#
#  pNOB1 <- brood_NOB/(brood_NOB + brood_HOB + brood_import)
#  pHOBint1 <- brood_HOB/(brood_NOB + brood_HOB + brood_import)
#  pHOBseg1 <- brood_import/(brood_NOB + brood_HOB + brood_import)
#
#  Zpop[2, g] <- A * pNOB1 * (Zpop[1, g-1] - theta[2]) +
#    pHOBint1 * (Zpop[2, g-1] - theta[2]) + pHOBseg1 * (Zpop[3, g-1] - theta[2]) + theta[2]
#
#  pNOB2 <- 0 #BV60/(BV60+BW60+BX60)
#  pHOBint2 <- max(1, export) #export #
#  pHOBseg2 <- 0 #brood_import/(brood_NOB + brood_HOB + brood_import)
#
#  #$I$49*(CE59*(CH59-$F$42)+CF59*(CI59-$F$42)+CG59*(CJ59-$F$42))+$F$42
#  Zpop[3, g] <- A * pNOB2 * (Zpop[1, g-1] - theta[3]) +
#    pHOBint2 * (Zpop[2, g-1] - theta[3]) + pHOBseg2 * (Zpop[3, g-1] - theta[3]) + theta[3]
#
#  fitness <- abs(Zpop[1, g] - mean(theta[-1]))/abs(theta[1] - mean(theta[-1]))
#
#  list(Zpop = Zpop, fitness = fitness)
#}


catch_fn <- function(return_size, u, surv_passage = c(1, 1, 1, 1), nfishery = 4) {

  catch <- vector(length = nfishery)
  N <- vector(length = nfishery + 1)
  N[1] <- return_size

  for(i in 1:nfishery) {
    catch[i] <- N[i] * surv_passage[i] * u[i]
    N[i+1] <- N[i] * surv_passage[i] * (1 - u[i])
  }

  return(list(catch = catch, N = N))
}


# Withler et al. 2018, page 27
#' @rdname calc_pwild_age
#' @examples
#'
#' calc_pwild(0.9, 0.4, 0.8)
#' @export
calc_pwild <- function(pHOS_cur, pHOS_prev, gamma) {
  num <- (1 - pHOS_prev)^2
  denom <- num + 2 * gamma * pHOS_prev * (1 - pHOS_prev) + gamma * gamma * pHOS_prev^2
  (1 - pHOS_cur) * num/denom
}

#' Example objects from salmonMSE
#'
#' @description
#' S4 objects containing operating model inputs to run the projection.
#' @format An object of class [salmonMSE::SOM-class].
#' @source See salmonMSE [website](https://docs.salmonmse.com/articles/example.html)
#'
#' @aliases multi_SOM
#' @examples
#' data(simple_SOM)
#'
#' data(multi_SOM)
"simple_SOM"

#' @rdname simple_SOM
"multi_SOM"

rmd_natural_production <- function(s, sname) {

  c(
    paste("##", sname, "{.tabset}"),
    "\n",
    "### Natural production {.tabset}",
    "\n",
    "#### Parameters",
    "\n",
    "```{r, fig.cap=\"Histogram of kappa (productivity, recruits/spawner).\"}",
    "SOM <- SMSE@Misc$SOM",
    paste0("use_habitat <- SOM@Habitat[[", s, "]]@use_habitat"),
    paste0("SRrel <- SOM@Bio[[", s, "]]@SRrel"),
    "",
    paste0("if (!use_habitat) plot_SOM(SOM@Bio[[", s, "]], var = \"kappa\", xlab = \"Productivity\")"),
    "```",
    "\n",
    "```{r, fig.cap=\"Histogram of Smax (spawner abundance that maximizes smolt production).\"}",
    paste0("if (!use_habitat && SRrel == \"Ricker\") plot_SOM(SOM@Bio[[", s, "]], var = \"Smax\", xlab = expression(S[max]))"),
    "```",
    "\n",
    "```{r, fig.cap=\"Histogram of smolt capacity from egg production (Beverton-Holt asymptote).\"}",
    paste0("if (!use_habitat && SRrel == \"BH\") plot_SOM(SOM@Bio[[", s, "]], var = \"capacity\", xlab = \"Smolt capacity\")"),
    "```",
    "\n",
    "```{r, fig.cap=\"Smolt production (density-dependent stock-recruit relationship). The dotted vertical line denotes the median egg production at unfished one to one replacement.\"}",
    paste0("if (!use_habitat) plot_SRR(SOM, s = ", s, ", surv = FALSE, check = FALSE, quant = TRUE)"),
    "```",
    "\n",
    "```{r, fig.cap=\"Egg to smolt survival from the density-dependent stock-recruit relationship. The dotted vertical line denotes the median egg production at unfished replacement.\"}",
    paste0("if (!use_habitat) plot_SRR(SOM, s = ", s, ", surv = TRUE, check = FALSE, quant = TRUE)"),
    "```",
    "```{r, fig.cap=\"Juvenile instantaneous natural mortality rate of natural origin fish (median and 95 percent interval across simulations).\"}",
    paste0("if (SOM@Bio[[", s, "]]@n_g > 1) {"),
    paste0("  plot_Mjuv_LHG(SOM@Bio[[", s, "]]@Mjuv_NOS[, , SOM@proyears, ])"),
    "} else {",
    paste0("  plot_SOM(SOM@Bio[[", s, "]], \"Mjuv_NOS\", type = \"age\", nsim = SOM@nsim, maxage = SOM@Bio[[", s, "]]@maxage-1, proyears = SOM@proyears, ylab = \"Juvenile natural mortality rate\")"),
    "}",
    "```",
    "",
    "```{r, fig.cap=\"Juvenile natural survival of natural origin fish (median and 95 percent interval across simulations).\"}",
    paste0("if (SOM@Bio[[", s, "]]@n_g > 1) {"),
    paste0("  plot_Mjuv_LHG(SOM@Bio[[", s, "]]@Mjuv_NOS[, , SOM@proyears, ], surv = TRUE, ylab = \"Juvenile survival\")"),
    "} else {",
    paste0("  plot_SOM(SOM@Bio[[", s, "]], \"Mjuv_NOS\", type = \"age\", nsim = SOM@nsim, maxage = SOM@Bio[[", s, "]]@maxage-1, proyears = SOM@proyears, surv = TRUE, ylab = \"Juvenile natural survival\")"),
    "}",
    "```",
    "",
    "```{r, fig.cap=\"Proportion mature at age (median and 95 percent interval across simulations).\"}",
    paste0("plot_SOM(SOM@Bio[[", s, "]], \"p_mature\", type = \"age\", nsim = SOM@nsim, maxage = SOM@Bio[[", s, "]]@maxage, proyears = SOM@proyears, ylab = \"Proportion mature\")"),
    "```",
    "",
    "```{r, fig.cap=\"Fecundity at age (median and 95 percent interval across simulations).\"}",
    paste0("plot_SOM(SOM@Bio[[", s, "]], \"fec\", type = \"age\", nsim = SOM@nsim, maxage = SOM@Bio[[", s, "]]@maxage, proyears = SOM@proyears, ylab = \"Fecundity\")"),
    "```",
    "\n",
    "#### Time series",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Natural origin spawners time series (annual median and 95 percent interval).\"}",
    paste0("plot_statevar_ts(SMSE, var = \"NOS\", s = ", s, ", quant = TRUE)"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Natural origin spawners, histogram in last projection year.\"}",
    paste0("plot_statevar_hist(SMSE, var = \"NOS\", s = ", s, ")"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Hatchery origin spawners time series (annual median and 95 percent interval).\"}",
    paste0("plot_statevar_ts(SMSE, var = \"HOS\", s = ", s, ", quant = TRUE)"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Hatchery origin spawners, histogram in last projection year.\"}",
    paste0("plot_statevar_hist(SMSE, var = \"HOS\", s = ", s, ")"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Proportion of hatchery spawners (annual median and 95 percent interval).\"}",
    paste0("plot_statevar_ts(SMSE, var = \"pHOS_census\", s = ", s, ", quant = TRUE, ylab = \"P(HOS)\")"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Proportion of hatchery spawners, histogram in last projection year.\"}",
    paste0("plot_statevar_hist(SMSE, var = \"pHOS_census\", s = ", s, ", xlab = \"P(HOS)\")"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Proportion of wild spawners (annual median and 95 percent interval).\"}",
    paste0("plot_statevar_ts(SMSE, var = \"p_wild\", s = ", s, ", quant = TRUE, ylab = \"P(WILD)\")"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Proportion of wild spawners, histogram in last projection year.\"}",
    paste0("plot_statevar_hist(SMSE, var = \"p_wild\", s = ", s, ", xlab = \"P(WILD)\")"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Spawner to escapement ratio, natural origin (annual median and 95 percent interval). Losses include en-route mortality and broodtake.\"}",
    paste0("plot_statevar_ts(SMSE, var = \"pNOSesc\", s = ", s, ", quant = TRUE, ylab = \"Spawner/escapement (natural)\")"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Spawner to escapement ratio, natural origin, histogram in last projection year.\"}",
    paste0("plot_statevar_hist(SMSE, var = \"pNOSesc\", s = ", s, ", xlab = \"Spawner/escapement (natural)\")"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Spawner to escapement ratio, hatchery origin (annual median and 95 percent interval). Losses include en-route mortality, broodtake, and removal from spawning grounds.\"}",
    paste0("plot_statevar_ts(SMSE, var = \"pHOSesc\", s = ", s, ", quant = TRUE, ylab = \"Spawner/escapement (hatchery)\")"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Spawner to escapement ratio, hatchery origin, histogram in last projection year.\"}",
    paste0("plot_statevar_hist(SMSE, var = \"pHOSesc\", s = ", s, ", xlab = \"Spawner/escapement (hatchery)\")"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Smolt production (annual median and 95 percent interval).\"}",
    paste0("plot_statevar_ts(SMSE, var = \"Smolt\", s = ", s, ", quant = TRUE, ylab = \"Smolt production\")"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Smolt production, histogram in last projection year.\"}",
    paste0("plot_statevar_hist(SMSE, var = \"Smolt\", s = ", s, ", xlab = \"Smolt production\")"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Egg to smolt survival (annual median and 95 percent interval).\"}",
    paste0("plot_statevar_ts(SMSE, var = \"ESS\", s = ", s, ", quant = TRUE, ylab = \"Egg-smolt survival\")"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Egg to smolt survival, histogram in last projection year.\"}",
    paste0("plot_statevar_hist(SMSE, var = \"ESS\", s = ", s, ", xlab = \"Egg-smolt survival\")"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    ""
  )

}

rmd_hatchery_production <- function(s, sname) {
  c(
    "### Hatchery production {.tabset}",
    "",
    "#### Parameters",
    "",
    "```{r results=\"asis\"}",
    paste0("knitr::kable(make_hatchery_table(SMSE, s = ", s, "), row.names = FALSE, caption = \"Hatchery enhancement settings.\")"),
    "```",
    "",
    "```{r results=\"asis\"}",
    paste0("knitr::kable(make_fitness_table(SMSE, s = ", s, "), row.names = FALSE, caption = \"Fitness parameters.\")"),
    "```",
    "",
    "```{r, fig.cap=\"Juvenile instantaneous natural mortality rate of hatchery origin fish (median and 95 percent interval across simulations).\"}",
    paste0("if (SOM@Hatchery[[", s, "]]@n_r > 1) {"),
    paste0("  plot_Mjuv_RS(SOM@Hatchery[[", s, "]]@Mjuv_HOS[, , SOM@proyears, ])"),
    "} else {",
    paste0("  plot_SOM(SOM@Hatchery[[", s, "]], \"Mjuv_HOS\", type = \"age\", nsim = SOM@nsim, maxage = SOM@Bio[[", s, "]]@maxage-1, proyears = SOM@proyears, ylab = \"Juvenile natural mortality rate\")"),
    "}",
    "```",
    "",
    "```{r, fig.cap=\"Juvenile natural survival of hatchery origin fish (median and 95 percent interval across simulations).\"}",
    paste0("if (SOM@Hatchery[[", s, "]]@n_r > 1) {"),
    paste0("  plot_Mjuv_RS(SOM@Hatchery[[", s, "]]@Mjuv_HOS[, , SOM@proyears, ], surv = TRUE)"),
    "} else {",
    paste0("  plot_SOM(SOM@Hatchery[[", s, "]], \"Mjuv_HOS\", type = \"age\", nsim = SOM@nsim, maxage = SOM@Bio[[", s, "]]@maxage-1, proyears = SOM@proyears, surv = TRUE, ylab = \"Juvenile natural survival\")"),
    "}",
    "```",
    "",

    "```{r, fig.cap=\"Maturity of hatchery origin fish (median and 95 percent interval across simulations).\"}",
    paste0("if (SOM@Hatchery[[", s, "]]@n_r > 1) {"),
    paste0("  plot_Mjuv_RS(SOM@Hatchery[[", s, "]]@p_mature_HOS[, , SOM@proyears, ], ylab = \"Proportion mature\")"),
    "} else {",
    paste0("  plot_SOM(SOM@Hatchery[[", s, "]], \"p_mature_HOS\", type = \"age\", nsim = SOM@nsim, maxage = SOM@Bio[[", s, "]]@maxage, proyears = SOM@proyears, ylab = \"Proportion mature\")"),
    "}",
    "```",
    "",
    "```{r, fig.cap=\"Broodtake fecundity.\"}",
    paste0("plot_SOM(SOM@Hatchery[[", s, "]], \"fec_brood\", type = \"age\", maxage = SOM@Bio[[", s, "]]@maxage, nsim = SOM@nsim, proyears = SOM@proyears, ylab = \"Brood fecundity\")"),
    "```",
    "\n",
    "#### Time series",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Natural origin broodtake (NOB) time series (annual median and 95 percent interval).\"}",
    paste0("plot_statevar_ts(SMSE, var = \"NOB\", s = ", s, ", quant = TRUE, ylab = \"NOB\")"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Natural origin broodtake (NOB), histogram in last projection year.\"}",
    paste0("plot_statevar_hist(SMSE, var = \"NOB\", s = ", s, ", xlab = \"NOB\")"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Local hatchery origin broodtake (HOB) time series (annual median and 95 percent interval).\"}",
    paste0("plot_statevar_ts(SMSE, var = \"HOB\", s = ", s, ", quant = TRUE, ylab = \"Local HOB\")"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Local hatchery origin broodtake (HOB), histogram in last projection year.\"}",
    paste0("plot_statevar_hist(SMSE, var = \"HOB\", s = ", s, ", xlab = \"Local HOB\")"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Imported hatchery origin broodtake (HOB) time series (annual median and 95 percent interval).\"}",
    paste0("plot_statevar_ts(SMSE, var = \"HOB_import\", s = ", s, ", quant = TRUE, ylab = \"Imported HOB\")"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Imported hatchery origin broodtake (HOB), histogram in last projection year.\"}",
    paste0("plot_statevar_hist(SMSE, var = \"HOB_import\", s = ", s, ", xlab = \"Imported HOB\")"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Proportion natural origin broodtake NOB/(NOB + HOB) time series (annual median and 95 percent interval).\"}",
    paste0("plot_statevar_ts(SMSE, var = \"pNOB\", s = ", s, ", quant = TRUE, ylab = \"P(NOB)\")"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Proportion natural origin broodtake, histogram in last projection year.\"}",
    paste0("plot_statevar_hist(SMSE, var = \"pNOB\", s = ", s, ", xlab = \"P(NOB)\")"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Proportion broodtake (ratio of total broodtake to total escapement) time series (annual median and 95 percent interval).\"}",
    paste0("plot_statevar_ts(SMSE, var = \"pbrood\", s = ", s, ", quant = TRUE, ylab = \"P(brood)\")"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Proportion broodtake, histogram in last projection year.\"}",
    paste0("plot_statevar_hist(SMSE, var = \"pbrood\", s = ", s, ", xlab = \"P(brood)\")"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Hatchery smolt releases time series (annual median and 95 percent interval).\"}",
    paste0("plot_statevar_ts(SMSE, var = \"Smolt_Rel\", s = ", s, ", quant = TRUE, ylab = \"Smolt releases\")"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Hatchery smolt releases, histogram in last projection year.\"}",
    paste0("plot_statevar_hist(SMSE, var = \"Smolt_Rel\", s = ", s, ", xlab = \"Smolt releases\")"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Fitness (of spawners to natural environment) time series (annual median and 95 percent interval).\"}",
    "SMSEf <- SMSE",
    "SMSEf@fitness <- array(SMSE@fitness[, , 1, ], c(SMSE@nsim, SMSE@nstocks, SMSE@proyears))",
    paste0("plot_statevar_ts(SMSEf, var = \"fitness\", s = ", s, ", quant = TRUE, ylab = \"Fitness\")"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Fitness (of spawners to natural environment), histogram in last projection year.\"}",
    paste0("plot_statevar_hist(SMSEf, var = \"fitness\", s = ", s, ", xlab = \"Fitness\")"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"PNI time series (annual median and 95 percent interval).\"}",
    paste0("plot_statevar_ts(SMSE, var = \"PNI\", s = ", s, ", quant = TRUE)"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"PNI, histogram in last projection year.\"}",
    paste0("plot_statevar_hist(SMSE, var = \"PNI\", s = ", s, ")"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    ""
  )
}

rmd_fishery <- function(s, sname) {
  c(
    "### Fishery {.tabset}",
    "\n",
    "#### Parameters",
    "\n",
    "```{r results=\"asis\"}",
    paste0("knitr::kable(make_harvest_table(SMSE, s = ", s, "), row.names = FALSE, caption = \"Harvest parameters.\")"),
    "```",
    "\n",
    "```{r, fig.cap=\"Vulnerability at age for the preterminal fishery (median and 95 percent interval across simulations).\"}",
    paste0("plot_SOM(SOM@Harvest[[", s, "]], \"vulPT\", type = \"age\", nsim = SOM@nsim, maxage = SOM@Bio[[", s, "]]@maxage, ylab = \"Preterminal vulnerability\")"),
    "```",
    "\n",
    "```{r, fig.cap=\"Vulnerability at age for the terminal fishery (median and 95 percent interval across simulations).\"}",
    paste0("plot_SOM(SOM@Harvest[[", s, "]], \"vulT\", type = \"age\", nsim = SOM@nsim, maxage = SOM@Bio[[", s, "]]@maxage, ylab = \"Terminal vulnerability\")"),
    "```",
    "\n",
    "#### Time series",
    "\n",
    "```{r, fig.cap=\"Kept catch in the preterminal (PT) and terminal (T) fishery (annual medians across simulations).\"}",
    paste0("plot_fishery(SMSE, s = ", s, ", type = \"catch\")"),
    "```",
    "\n",
    "```{r, fig.cap=\"Harvest rate (from kept catch and abundance) in the preterminal (PT) and terminal (T) fishery (annual medians across simulations).\"}",
    paste0("plot_fishery(SMSE, s = ", s, ", type = \"harvest\")"),
    "```",
    "\n",
    "```{r, fig.cap=\"Exploitation rate (from kept catch and dead discards and abundance) in the preterminal (PT) and terminal (T) fishery (annual medians across simulations).\"}",
    paste0("plot_fishery(SMSE, s = ", s, ", type = \"ex\")"),
    "```",
    "\n",
    "```{r, fig.cap=\"NOS/SMSY time series (annual median and 95 percent interval).\"}",
    paste0("plot_statevar_ts(SMSE, var = \"NOS/SMSY\", s = ", s, ", quant = TRUE, ylab = expression(NOS/S[MSY]))"),
    "```",
    "\n",
    "```{r, fig.cap=\"NOS/SMSY, histogram in last projection year.\"}",
    paste0("plot_statevar_hist(SMSE, var = \"NOS/SMSY\", s = ", s, ", xlab = expression(NOS/S[MSY]))"),
    "```",
    "\n",
    "```{r, fig.cap=\"Kobe figure (annual medians) with exploitation rate for the preterminal fishery.\"}",
    paste0("plot_Kobe(SMSE, s = ", s, ", type = \"PT\")"),
    "```",
    "\n",
    "```{r, fig.cap=\"Kobe figure (annual medians) with exploitation rate for the terminal fishery.\"}",
    paste0("plot_Kobe(SMSE, s = ", s, ", type = \"T\")"),
    "```",
    "\n",
    "### Comparison",
    "\n",
    "```{r, fig.cap=\"Composition of spawners (annual medians across simulations).\"}",
    paste0("plot_spawners(SMSE, s = ", s, ", prop = FALSE)"),
    "```",
    "\n",
    "```{r, fig.cap=\"Composition of spawners (proportions from annual medians across simulations).\"}",
    paste0("plot_spawners(SMSE, s = ", s, ", prop = TRUE)"),
    "```",
    "\n",
    "```{r, fig.cap=\"Ratio of spawners and broodstock to escapement (proportions from annual medians across simulations).\"}",
    paste0("plot_escapement(SMSE, s = ", s, ")"),
    "```",
    "\n",
    "```{r, fig.cap=\"Fitness and other metrics related to hatchery production (annual medians across simulations).\"}",
    paste0("plot_fitness(SMSE, s = ", s, ")"),
    "```",
    "\n"
  )
}

rmd_lhg <- function(s, sname) {
  c(
    "#### Life history groups",
    "\n",
    "```{r, fig.cap=\"Smolt production by life history group (median across simulations).\"}",
    paste0("plot_LHG(SMSE, var = \"Smolt\", type = \"abs\", s = ", s, ")"),
    "```",
    "\n",
    "```{r, fig.cap=\"Natural origin spawners by life history group (median across simulations).\"}",
    paste0("plot_LHG(SMSE, var = \"NOS\", type = \"abs\", s = ", s, ")"),
    "```",
    "\n",
    "```{r, fig.cap=\"Proportion of life history groups at the smolt life stage.\"}",
    paste0("plot_LHG(SMSE, var = \"Smolt\", s = ", s, ")"),
    "```",
    "\n",
    "```{r, fig.cap=\"Proportion of life history groups of natural spawners.\"}",
    paste0("plot_LHG(SMSE, var = \"NOS\", s = ", s, ")"),
    "```",
    "\n"
  )
}

rmd_rs <- function(s, sname) {
  c(
    "#### Release strategies",
    "\n",
    "```{r, fig.cap=\"Hatchery releases at smolt life stage by release strategy (median across simulations).\"}",
    paste0("plot_RS(SMSE, var = \"Smolt\", ylab = \"Hatchery smolts\", type = \"abs\", s = ", s, ")"),
    "```",
    "\n",
    "```{r, fig.cap=\"Hatchery origin spawners by release strategy  (median across simulations).\"}",
    paste0("plot_RS(SMSE, var = \"HOS\", type = \"abs\", s = ", s, ")"),
    "```",
    "\n",
    "```{r, fig.cap=\"Proportion of release strategy at the smolt life stage.\"}",
    paste0("plot_RS(SMSE, var = \"Smolt\", ylab = \"Proportion hatchery smolts\", s = ", s, ")"),
    "```",
    "\n",
    "```{r, fig.cap=\"Proportion of release strategy of hatchery spawners.\"}",
    paste0("plot_RS(SMSE, var = \"HOS\", s = ", s, ")"),
    "```",
    "\n"
  )
}

make_rmd_stock <- function(s, sname, n_g = 1, n_r = 1) {

  natural_production <- rmd_natural_production(s, sname)
  if (n_g > 1) {
    lhg <- rmd_lhg(s, sname)
  } else {
    lhg <- NULL
  }

  hatchery_production <- rmd_hatchery_production(s, sname)
  if (n_r > 1) {
    rs <- rmd_rs(s, sname)
  } else {
    rs <- NULL
  }

  fishery <- rmd_fishery(s, sname)

  c(natural_production, lhg, hatchery_production, rs, fishery)
}

make_rmd_stock_comparison <- function() {

  c(
    "## Among-population comparison",
    "\n",
    "```{r, fig.cap=\"Stray matrix of hatchery origin fish at the recruitment stage.\"}",
    "plot_stray(SMSE@Misc$SOM@stray)",
    "```",
    "\n",
    "```{r, fig.cap=\"Natural origin spawners time series (annual medians) across population.\"}",
    "plot_statevar_ts(SMSE, var = \"NOS\", s = 1:SMSE@nstocks)",
    "```",
    "\n",
    "```{r, fig.cap=\"Hatchery origin spawners time series (annual medians) across population.\"}",
    "plot_statevar_ts(SMSE, var = \"HOS\", s = 1:SMSE@nstocks)",
    "```",
    "\n",
    "```{r, fig.cap=\"Proportion of hatchery spawners (annual medians) across populations..\"}",
    "plot_statevar_ts(SMSE, var = \"pHOScensus\", s = 1:SMSE@nstocks, ylab = \"P(HOS)\")",
    "```",
    "\n",
    "```{r, fig.cap=\"Proportion of wild spawners (annual medians) across populations.\"}",
    "plot_statevar_ts(SMSE, var = \"p_wild\", s = 1:SMSE@nstocks, ylab = \"P(WILD)\")",
    "```",
    "\n",
    "```{r, fig.cap=\"Natural smolt production (annual medians) across populations.\"}",
    "plot_statevar_ts(SMSE, var = \"Smolt\", s = 1:SMSE@nstocks, ylab = \"Natural smolt production\")",
    "```",
    "\n",
    "```{r, fig.cap=\"Hatchery smolt releases (annual medians) across populations.\"}",
    "plot_statevar_ts(SMSE, var = \"Smolt_Rel\", s = 1:SMSE@nstocks, ylab = \"Hatchery smolt releases\")",
    "```",
    "\n",
    "```{r, fig.cap=\"Fitness (of spawners to natural environment) (annual medians) across populations.\"}",
    "plot_statevar_ts(SMSEf, var = \"fitness\", s = 1:SMSE@nstocks, ylab = \"Fitness\")",
    "```",
    "\n",
    "```{r, fig.cap=\"PNI (proportionate natural influence) (annual medians) across populations.\"}",
    "plot_statevar_ts(SMSE, var = \"PNI\", s = 1:SMSE@nstocks)",
    "```",
    "\n"
  )
}


make_rmd_compare <- function(s, sname) {

  nat <- c(paste("##", sname, "{.tabset}"),
    "### Natural production",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Natural origin spawners time series (annual median and 95 percent interval).\"}",
    paste0("compare_statevar_ts(SMSE_list, var = \"NOS\", s = ", s, ", quant = TRUE, names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Natural origin spawners, histogram in last projection year.\"}",
    paste0("compare_statevar_hist(SMSE_list, var = \"NOS\", s = ", s, ", names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Hatchery origin spawners time series (annual median and 95 percent interval).\"}",
    paste0("compare_statevar_ts(SMSE_list, var = \"HOS\", s = ", s, ", quant = TRUE, names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Hatchery origin spawners, histogram in last projection year.\"}",
    paste0("compare_statevar_hist(SMSE_list, var = \"HOS\", s = ", s, ", names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Proportion of hatchery spawners (annual median and 95 percent interval).\"}",
    paste0("compare_statevar_ts(SMSE_list, var = \"pHOS_census\", s = ", s, ", quant = TRUE, ylab = \"P(HOS)\", names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Proportion of hatchery spawners, histogram in last projection year.\"}",
    paste0("compare_statevar_hist(SMSE_list, var = \"pHOS_census\", s = ", s, ", xlab = \"P(HOS)\", names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Proportion of wild spawners (annual median and 95 percent interval).\"}",
    paste0("compare_statevar_ts(SMSE_list, var = \"p_wild\", s = ", s, ", quant = TRUE, ylab = \"P(WILD)\", names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Proportion of wild spawners, histogram in last projection year.\"}",
    paste0("compare_statevar_hist(SMSE_list, var = \"p_wild\", s = ", s, ", xlab = \"P(WILD)\", names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Spawner to escapement ratio, natural origin (annual median and 95 percent interval). Losses include en-route mortality and broodtake.\"}",
    paste0("compare_statevar_ts(SMSE_list, var = \"pNOSesc\", s = ", s, ", quant = TRUE, ylab = \"Spawner/escapement (natural)\", names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Spawner to escapement ratio, natural origin, histogram in last projection year.\"}",
    paste0("compare_statevar_hist(SMSE_list, var = \"pNOSesc\", s = ", s, ", xlab = \"Spawner/escapement (natural)\", names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Spawner to escapement ratio, hatchery origin (annual median and 95 percent interval). Losses include en-route mortality, broodtake, and removal from spawning grounds.\"}",
    paste0("compare_statevar_ts(SMSE_list, var = \"pHOSesc\", s = ", s, ", quant = TRUE, ylab = \"Spawner/escapement (hatchery)\", names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Spawner to escapement ratio, hatchery origin, histogram in last projection year.\"}",
    paste0("compare_statevar_hist(SMSE_list, var = \"pHOSesc\", s = ", s, ", xlab = \"Spawner/escapement (hatchery)\", names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Smolt production (annual median and 95 percent interval).\"}",
    paste0("compare_statevar_ts(SMSE_list, var = \"Smolt\", s = ", s, ", quant = TRUE, ylab = \"Smolt production\", names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Smolt production, histogram in last projection year.\"}",
    paste0("compare_statevar_hist(SMSE_list, var = \"Smolt\", s = ", s, ", xlab = \"Smolt production\", names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    ""
  )

  hat <- c(
    "### Hatchery production",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Natural origin broodtake (NOB) time series (annual median and 95 percent interval).\"}",
    paste0("compare_statevar_ts(SMSE_list, var = \"NOB\", s = ", s, ", quant = TRUE, ylab = \"NOB\", names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Natural origin broodtake (NOB), histogram in last projection year.\"}",
    paste0("compare_statevar_hist(SMSE_list, var = \"NOB\", s = ", s, ", xlab = \"NOB\", names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Local hatchery origin broodtake (HOB) time series (annual median and 95 percent interval).\"}",
    paste0("compare_statevar_ts(SMSE_list, var = \"HOB\", s = ", s, ", quant = TRUE, ylab = \"Local HOB\", names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Local hatchery origin broodtake (HOB), histogram in last projection year.\"}",
    paste0("compare_statevar_hist(SMSE_list, var = \"HOB\", s = ", s, ", xlab = \"Local HOB\", names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Imported hatchery origin broodtake (HOB) time series (annual median and 95 percent interval).\"}",
    paste0("compare_statevar_ts(SMSE_list, var = \"HOB_import\", s = ", s, ", quant = TRUE, ylab = \"Imported HOB\", names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Imported hatchery origin broodtake (HOB), histogram in last projection year.\"}",
    paste0("compare_statevar_hist(SMSE_list, var = \"HOB_import\", s = ", s, ", xlab = \"Imported HOB\", names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Proportion natural origin broodtake NOB/(NOB + HOB) time series (annual median and 95 percent interval).\"}",
    paste0("compare_statevar_ts(SMSE_list, var = \"pNOB\", s = ", s, ", quant = TRUE, ylab = \"P(NOB)\", names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Proportion natural origin broodtake, histogram in last projection year.\"}",
    paste0("compare_statevar_hist(SMSE_list, var = \"pNOB\", s = ", s, ", xlab = \"P(NOB)\", names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Proportion broodtake (ratio of total broodtake to total escapement) time series (annual median and 95 percent interval).\"}",
    paste0("compare_statevar_ts(SMSE_list, var = \"pbrood\", s = ", s, ", quant = TRUE, ylab = \"P(brood)\", names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Proportion broodtake, histogram in last projection year.\"}",
    paste0("compare_statevar_hist(SMSE_list, var = \"pbrood\", s = ", s, ", xlab = \"P(brood)\", names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Hatchery smolt releases time series (annual median and 95 percent interval).\"}",
    paste0("compare_statevar_ts(SMSE_list, var = \"Smolt_Rel\", s = ", s, ", quant = TRUE, ylab = \"Smolt releases\", names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Hatchery smolt releases, histogram in last projection year.\"}",
    paste0("compare_statevar_hist(SMSE_list, var = \"Smolt_Rel\", s = ", s, ", xlab = \"Smolt releases\", names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Fitness (of spawners to natural environment) time series (annual median and 95 percent interval).\"}",
    "SMSEf_list <- lapply(SMSE_list, function(SMSE) {",
    "  SMSE@fitness <- array(SMSE@fitness[, , 1, ], c(SMSE@nsim, SMSE@nstocks, SMSE@proyears))",
    "  return(SMSE)",
    "})",
    paste0("compare_statevar_ts(SMSEf_list, var = \"fitness\", s = ", s, ", quant = TRUE, ylab = \"Fitness\", names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"Fitness (of spawners to natural environment), histogram in last projection year.\"}",
    paste0("compare_statevar_hist(SMSEf_list, var = \"fitness\", s = ", s, ", xlab = \"Fitness\", names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    "",
    "<div style=\"display: flex;\">",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"PNI time series (annual median and 95 percent interval).\"}",
    paste0("compare_statevar_ts(SMSE_list, var = \"PNI\", s = ", s, ", quant = TRUE, names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "<div style=\"width: 50%;\">",
    "",
    "```{r, fig.cap=\"PNI, histogram in last projection year.\"}",
    paste0("compare_statevar_hist(SMSE_list, var = \"PNI\", s = ", s, ", names = names, col_vec = col_vec)"),
    "```",
    "",
    "</div>",
    "",
    "</div>",
    ""
  )

  c(nat, hat)

}

#' @rdname salmonMSE-int
#' @param MMSE Object of class [MSEtool::MMSE-class] returned from MSEtool
#' @param Harvest_MMP Optional harvest function created by [make_Harvest_MMP()]
#' @param N Data frame of natural origin abundance at age saved in the [salmonMSE_env] environment during the simulation
#' @param stateN Data frame of natural origin state variables saved in the [salmonMSE_env] environment during the simulation
#' @param Ford Data frame of phenotypic trait values saved in the [salmonMSE_env] environment during the simulation
#' @param H Data frame of hatchery origin abundance at age saved in the [salmonMSE_env] environment during the simulation
#' @param stateH Data frame of hatchery origin state variables saved in the [salmonMSE_env] environment during the simulation
#' @return
#' `MMSE2SMSE`: \linkS4class{SMSE} object
#' @export
MMSE2SMSE <- function(MMSE, SOM, Harvest_MMP, N, stateN, Ford, H, stateH) {
  ns <- length(SOM@Bio) # Number of stocks
  nage <- SOM@Bio[[1]]@maxage
  nyears_real <- 2

  # Declare arrays
  Njuv_NOS <- Njuv_HOS <- Escapement_NOS <- Escapement_HOS <- array(0, c(SOM@nsim, ns, nage, SOM@proyears))
  Return_NOS <- Return_HOS <- array(0, c(SOM@nsim, ns, nage, SOM@proyears))
  NOB <- HOB <- HOB_stray <- HOB_import <- array(0, c(SOM@nsim, ns, SOM@proyears))
  KPT_NOS <- KT_NOS <- KPT_HOS <- KT_HOS <- array(0, c(SOM@nsim, ns, SOM@proyears))
  DPT_NOS <- DT_NOS <- DPT_HOS <- DT_HOS <- array(0, c(SOM@nsim, ns, SOM@proyears))
  UPT_NOS <- UT_NOS <- UPT_HOS <- UT_HOS <- array(0, c(SOM@nsim, ns, nage, SOM@proyears))
  ExPT_NOS <- ExT_NOS <- ExPT_HOS <- ExT_HOS <- array(0, c(SOM@nsim, ns, nage, SOM@proyears))

  Egg_NOS <- Egg_HOS <- array(0, c(SOM@nsim, ns, SOM@proyears))
  Fry_NOS <- Fry_HOS <- array(0, c(SOM@nsim, ns, SOM@proyears))
  Smolt_NOS <- Smolt_HOS <- Smolt_Rel <- array(0, c(SOM@nsim, ns, SOM@proyears))

  NOS <- HOS <- HOS_stray <- HOS_effective <- array(0, c(SOM@nsim, ns, nage, SOM@proyears))
  fitness <- array(NA_real_, c(SOM@nsim, ns, 2, SOM@proyears))

  pNOB <- pHOS_census <- pHOS_effective <- PNI <- p_wild <- array(NA_real_, c(SOM@nsim, ns, SOM@proyears))

  Mjuv_loss <- array(0, c(SOM@nsim, ns, nage, SOM@proyears))

  LHG <- RS <- vector("list", length = ns)

  # openMSE year and age indices
  t1 <- seq(1, 2 * SOM@proyears, 2)
  t2 <- seq(2, 2 * SOM@proyears, 2)

  a1 <- seq(1, 2 * nage + 1, 2)
  a2 <- seq(2, 2 * nage + 1, 2)

  a_imm <- a1[-length(a1)]
  a_return <- a2
  a_esc <- a1[-1]

  f <- 1 # Fleet
  mp <- 1 # Only MP run per OM

  pindex <- make_stock_index(SOM)

  # Loop over s
  for (s in 1:ns) {

    # NOS state variables from MMSE object (length n_g)
    p_NOS_imm <- pindex$p[pindex$s == s & pindex$origin == "natural" & pindex$stage == "juvenile"]
    p_NOS_return <- pindex$p[pindex$s == s & pindex$origin == "natural" & pindex$stage == "recruitment"] # MSEtool population index for returning NOS
    p_NOS_escapement <- pindex$p[pindex$s == s & pindex$origin == "natural" & pindex$stage == "escapement"] # NOS escapement

    t_spawnOM <- t1 # OM years with spawning
    if (t_spawnOM[1] == 1) {
      t_spawnOM <- t_spawnOM[-1] # This is actually the spawning from the last historical year (spool-up), ignore
    }

    # Real years of spawning
    y_spawn <- seq(1, length(unique(N$t)))
    y_progeny <- y_spawn + 1
    y_progeny <- y_progeny[y_progeny <= SOM@proyears]
    y_spawnOM <- 0.5 * (t_spawnOM - 1)

    Njuv_NOS[, s, , ] <- apply(MMSE@N[, p_NOS_imm, a_imm, mp, t1, , drop = FALSE], c(1, 3, 5), sum)
    Return_NOS[, s, , ] <- apply(MMSE@N[, p_NOS_return, a_return, mp, t2, , drop = FALSE], c(1, 3, 5), sum)
    Escapement_NOS[, s, , y_spawnOM] <- apply(MMSE@N[, p_NOS_escapement, a_esc, mp, t_spawnOM, , drop = FALSE], c(1, 3, 5), sum)

    # Kept catch
    KPT_NOS[, s, ] <- apply(MMSE@Catch[, p_NOS_imm, f, mp, t1, drop = FALSE], c(1, 5), sum)
    KT_NOS[, s, ] <- apply(MMSE@Catch[, p_NOS_return, f, mp, t2, drop = FALSE], c(1, 5), sum)

    # Total discards (live + dead)
    DPT_NOS[, s, ] <- apply(MMSE@Removals[, p_NOS_imm, f, mp, t1, drop = FALSE], c(1, 5), sum) - KPT_NOS[, s, ]
    DT_NOS[, s, ] <- apply(MMSE@Removals[, p_NOS_return, f, mp, t2, drop = FALSE], c(1, 5), sum) - KT_NOS[, s, ]

    # Exploitation rate from kept + dead discards (DD)
    vulPT <- array(SOM@Harvest[[s]]@vulPT, c(SOM@nsim, nage, length(t1)))
    DDPT_NOS <- SOM@Harvest[[s]]@release_mort[1] * DPT_NOS[, s, ]
    ExPT_NOS[, s, , ] <- local({
      Ka_NOS <- sapply(1:length(p_NOS_imm), function(i) { # sim x age x year
        FM <- MMSE@FM[, p_NOS_imm[i], f, mp, t1]
        Fa <- sapply(1:length(t1), function(t) vulPT[, , t] * FM[, t], simplify = "array")
        N <- apply(MMSE@N[, p_NOS_imm[i], a_imm, mp, t1, ], 1:3, sum)
        (1 - exp(-Fa)) * N
      }, simplify = "array") %>%
        apply(1:3, sum)
      Ka_NOS/Njuv_NOS[, s, , ]
    })
    ExPT_NOS[is.na(ExPT_NOS)] <- 0

    vulT <- array(SOM@Harvest[[s]]@vulT, c(SOM@nsim, nage, length(t2)))
    DDT_NOS <- SOM@Harvest[[s]]@release_mort[2] * DT_NOS[, s, ]
    ExT_NOS[, s, , ] <- local({
      Ka_NOS <- sapply(1:length(p_NOS_return), function(i) { # sim x age x year
        FM <- MMSE@FM[, p_NOS_return[i], f, mp, t2]
        Fa <- sapply(1:length(t2), function(t) vulT[, , t] * FM[, t], simplify = "array")
        N <- apply(MMSE@N[, p_NOS_return[i], a_return, mp, t2, ], 1:3, sum)
        (1 - exp(-Fa)) * N
      }, simplify = "array") %>%
        apply(1:3, sum)
      Ka_NOS/Return_NOS[, s, , ]
    })
    ExT_NOS[is.na(ExT_NOS)] <- 0

    # Harvest rate from kept catch
    if (all(!DDPT_NOS)) UPT_NOS[, s, , ] <- ExPT_NOS[, s, , ]
    if (all(!DDT_NOS)) UT_NOS[, s, , ] <- ExT_NOS[, s, , ]

    # Escapement in last projection year (not calculated in openMSE)
    Escapement_NOS[, s, , SOM@proyears] <- Return_NOS[, s, , SOM@proyears] * (1 - ExT_NOS[, s, , SOM@proyears])

    do_hatchery <- sum(SOM@Hatchery[[s]]@n_subyearling > 0, SOM@Hatchery[[s]]@n_yearling) > 0
    has_strays <- any(SOM@stray[-s, s] > 0) || sum(SOM@Hatchery[[s]]@stray_external)
    n_g <- length(unique(pindex$g[pindex$s == s & pindex$origin == "natural"]))
    n_r <- length(unique(pindex$r[pindex$s == s & pindex$origin == "hatchery"]))

    s_enroute <- SOM@Bio[[s]]@s_enroute
    use_smolt_func <- TRUE

    if (use_smolt_func) {
      # Sum across LHG
      NOS[, s, , y_spawn] <- get_salmonMSE_agevar(N, var = "NOS", s)
      Egg_NOS[, s, y_spawn] <- get_salmonMSE_var(stateN, var = "Egg_NOS", s)

      Fry_NOS[, s, y_progeny] <- get_salmonMSE_var(stateN, var = "Fry_NOS", s)[, 1:length(y_progeny)]
      Smolt_NOS[, s, y_progeny] <- get_salmonMSE_var(stateN, var = "smolt_NOS", s)[, 1:length(y_progeny)]

    } else {
      # If no hatchery, the NOS escapement is also the NOS, Egg_NOS is the spawning output
      NOS[, s, , ] <- Escapement_NOS[, s, , ]
      Egg_NOS[, s, y_spawnOM] <- apply(MMSE@SSB[, p_NOS_escapement, mp, t_spawnOM, drop = FALSE], c(1, 4), sum) # -1 from 1-year lag
      Fry_NOS[, s, seq(2, SOM@proyears)] <- Egg_NOS[, s, seq(2, SOM@proyears) - 1]

      a_smolt <- 1
      Smolt_NOS[, s, ] <- Njuv_NOS[, s, a_smolt, ]
    }

    if (do_hatchery || has_strays) {

      # HOS state variables from MMSE object
      p_HOS_imm <- pindex$p[pindex$s == s & pindex$origin == "hatchery" & pindex$stage == "juvenile"]
      p_HOS_return <- pindex$p[pindex$s == s & pindex$origin == "hatchery" & pindex$stage == "recruitment"] # MSEtool population index for returning HOS
      p_HOS_escapement <- pindex$p[pindex$s == s & pindex$origin == "hatchery" & pindex$stage == "escapement"] # HOS escapement

      Njuv_HOS[, s, , ] <- apply(MMSE@N[, p_HOS_imm, a_imm, mp, t1, , drop = FALSE], c(1, 3, 5), sum)
      Return_HOS[, s, , ] <- apply(MMSE@N[, p_HOS_return, a_return, mp, t2, , drop = FALSE], c(1, 3, 5), sum)
      Escapement_HOS[, s, , y_spawnOM] <- apply(MMSE@N[, p_HOS_escapement, a_esc, mp, t_spawnOM, , drop = FALSE], c(1, 3, 5), sum)

      # Kept catch
      KPT_HOS[, s, ] <- apply(MMSE@Catch[, p_HOS_imm, f, mp, t1, drop = FALSE], c(1, 5), sum)
      KT_HOS[, s, ] <- apply(MMSE@Catch[, p_HOS_return, f, mp, t2, drop = FALSE], c(1, 5), sum)

      # Total discards (live + dead)
      DPT_HOS[, s, ] <- apply(MMSE@Removals[, p_HOS_imm, f, mp, t1, drop = FALSE], c(1, 5), sum) - KPT_HOS[, s, ]
      DT_HOS[, s, ] <- apply(MMSE@Removals[, p_HOS_return, f, mp, t2, drop = FALSE], c(1, 5), sum) - KT_HOS[, s, ]

      # Exploitation rate from kept + dead discards (DD)
      DDPT_HOS <- SOM@Harvest[[s]]@release_mort[1] * DPT_HOS[, s, ]
      ExPT_HOS[, s, , ] <- local({
        Ka_HOS <- sapply(1:length(p_HOS_imm), function(i) { # sim x age x year
          FM <- MMSE@FM[, p_HOS_imm[i], f, mp, t1]
          Fa <- sapply(1:length(t1), function(t) vulPT[, , t] * FM[, t], simplify = "array")
          N <- apply(MMSE@N[, p_HOS_imm[i], a_imm, mp, t1, ], 1:3, sum)
          (1 - exp(-Fa)) * N
        }, simplify = "array") %>%
          apply(1:3, sum)
        Ka_HOS/Njuv_HOS[, s, , ]
      })
      ExPT_HOS[is.na(ExPT_HOS)] <- 0

      DDT_HOS <- SOM@Harvest[[s]]@release_mort[2] * DT_HOS[, s, ]
      ExT_HOS[, s, , ] <- local({
        Ka_HOS <- sapply(1:length(p_HOS_return), function(i) { # sim x age x year
          FM <- MMSE@FM[, p_HOS_return[i], f, mp, t2]
          Fa <- sapply(1:length(t2), function(t) vulT[, , t] * FM[, t], simplify = "array")
          N <- apply(MMSE@N[, p_HOS_return[i], a_return, mp, t2, ], 1:3, sum)
          (1 - exp(-Fa)) * N
        }, simplify = "array") %>%
          apply(1:3, sum)
        Ka_HOS/Return_HOS[, s, , ]
      })
      ExT_HOS[is.na(ExT_HOS)] <- 0

      # Harvest rate from kept catch
      if (all(!DDPT_HOS)) {
        UPT_HOS[, s, , ] <- ExPT_HOS[, s, , ]
      } else {
        stop("Dead discards of hatchery fish found")
      }

      if (all(!DDT_HOS)) {
        UT_HOS[, s, , ] <- ExT_HOS[, s, , ]
      } else {
        stop("Dead discards of hatchery fish found")
      }

      # Escapement in last projection year (not calculated in openMSE)
      Escapement_HOS[, s, , SOM@proyears] <- Return_HOS[, s, , SOM@proyears] * (1 - ExT_HOS[, s, , SOM@proyears])

      # NOS + HOS state variables from salmonMSE
      ngen <- length(unique(salmonMSE_env$N$t))
      if (length(y_spawn) != ngen) warning("Number of generations in salmonMSE state variables does not match generations in openMSE")

      # Sum across RS
      HOS[, s, , y_spawn] <- get_salmonMSE_agevar(H, var = "HOS", s)
      HOS_stray[, s, , y_spawn] <- get_salmonMSE_agevar(H, var = "HOS_stray", s)
      HOS_effective[, s, , y_spawn] <- get_salmonMSE_agevar(H, var = "HOS_effective", s)

      Egg_HOS[, s, y_spawn] <- get_salmonMSE_var(stateH, var = "Egg_HOS", s)
      Fry_HOS[, s, y_progeny] <- get_salmonMSE_var(stateN, var = "Fry_HOS", s)[, 1:length(y_progeny)]
      Smolt_HOS[, ns, y_progeny] <- get_salmonMSE_var(stateN, var = "smolt_HOS", s)[, 1:length(y_progeny)]

      # Broodtake & fitness
      NOB[, s, y_spawn] <- get_salmonMSE_var(N, var = "NOB", s)
      HOB[, s, y_spawn] <- get_salmonMSE_var(H, var = "HOB", s)
      HOB_stray[, s, y_spawn] <- get_salmonMSE_var(H, var = "HOB_stray", s)
      HOB_import[, s, y_spawn] <- get_salmonMSE_var(H, var = "HOB_import", s)

      fitness[, s, 1, y_progeny] <- local({
        ff <- filter(Ford, .data$type == "natural", .data$t > 2 * nyears_real) %>%
          get_salmonMSE_var(var = "fitness", s)
        ff[, 1:length(y_progeny)]
      })
      fitness[, s, 2, y_progeny] <- local({
        ff <- filter(Ford, .data$type == "hatchery", .data$t > 2 * nyears_real) %>%
          get_salmonMSE_var(var = "fitness", s)
        ff[, 1:length(y_progeny)]
      })

      # Convert fitness to NA for empty cohorts
      y_empty <- colSums(Smolt_NOS[, s, ] + Smolt_Rel[, s, ]) == 0
      fitness[, s, , y_empty] <- NA

      # Smolt releases and SAR loss from openMSE
      a_smolt <- 1

      #smolt_rel_openmse <- apply(MMSE@N[, p_HOS_imm, a_smolt, mp, y_spawnOM, , drop = FALSE], c(1, 5), sum)
      smolt_rel_salmonmse <- get_salmonMSE_var(stateH, var = "smolt_rel", s) # debugging purposes
      Smolt_Rel[, s, y_progeny] <- smolt_rel_salmonmse[, 1:length(y_progeny)]

      if (!is.null(MMSE@Misc$MICE$M_ageArray)) {
        Mjuv_loss[, s, , ] <- MMSE@Misc$MICE$M_ageArray[, p_NOS_imm[1], a2, mp, t2] # Report first LHG only
      }

      pNOB[, s, y_spawn] <- get_salmonMSE_var(stateN, var = "pNOB", s, FUN = unique)
      pHOS_effective[, s, y_spawn] <- get_salmonMSE_var(stateN, var = "pHOSeff", s, FUN = unique)
      pHOS_census[, s, y_spawn] <- get_salmonMSE_var(stateN, var = "pHOScensus", s, FUN = unique)

      # Withler et al. 2018, page 17, 21
      h2 <- SOM@Hatchery[[s]]@heritability
      fitness_variance <- SOM@Hatchery[[s]]@fitness_variance

      # Exact PNI, see HSRG 2009, Appendix C, Eq. 33, Columbia River Hatchery Reform System-Wide Report
      PNI[, s, y_spawn] <- ifelse(
        pNOB[, s, y_spawn] > 0,
        pNOB[, s, y_spawn]/(pNOB[, s, y_spawn] + pHOS_effective[, s, y_spawn]),
        h2/(h2 + (1 - h2 + fitness_variance) * pHOS_effective[, s, y_spawn])
      )

      #PNI[, s, ] <- (h2 + (1 - h2 + fitness_variance) * pNOB[, s, ])/(h2 + (1 - h2 + fitness_variance) * (pHOS_effective[, s, ] + pNOB[, s, ]))

      p_wild[, s, ] <- calc_pwild_age(NOS[, s, , ], HOS[, s, , ], SOM@Bio[[s]]@fec[, , seq(1, SOM@proyears)], SOM@Hatchery[[s]]@gamma)

      if (n_r > 1) {
        Smolt_r <- array(NA_real_, c(SOM@nsim, n_r, SOM@proyears))
        Esc_r <- Return_r <- HOS_r <- array(NA_real_, c(SOM@nsim, n_r, nage, SOM@proyears))

        x <- a <- r <- t <- NULL

        a_smolt <- 1
        Smolt_r[] <- apply(MMSE@N[, p_HOS_imm, a_smolt, mp, t1, , drop = FALSE], c(1, 2, 5), sum)
        Return_r[] <- apply(MMSE@N[, p_HOS_return, a_return, mp, t2, , drop = FALSE], c(1, 2, 3, 5), sum)
        for (r in 1:n_r) Esc_r[, r, , ] <- Return_r[, r, , ] * (1 - ExT_HOS[, s, , ])
        HOS_r[, , , y_spawn] <- dplyr::filter(H, .data$s == .env$s) %>%
          summarise(value = sum(.data$HOS), .by = c(x, r, a, t)) %>%
          reshape2::acast(list("x", "r", "a", "t"), value.var = "value")

        RS[[s]] <- list(
          Smolt = Smolt_r,
          Esc = Esc_r,
          HOS = HOS_r
        )
      }

    } else {
      PNI[, s, y_spawn] <- p_wild[, s, y_spawn] <- 1
      pHOS_census[, s, y_spawn] <- pHOS_effective[, s, y_spawn] <- 0
    }

    if (n_g > 1) {
      Egg_g <- Fry_g <- Smolt_g <- array(NA_real_, c(SOM@nsim, n_g, SOM@proyears))
      Esc_g <- Return_g <- NOS_g <- array(NA_real_, c(SOM@nsim, n_g, nage, SOM@proyears))

      x <- a <- g <- t <- NULL
      Egg_g[, , y_spawn] <- dplyr::filter(stateN, .data$s == .env$s) %>%
        summarise(value = sum(.data$Egg_NOS), .by = c(x, g, t)) %>%
        reshape2::acast(list("x", "g", "t"), value.var = "value")

      Fry_g[, , y_spawn + 1] <- dplyr::filter(stateN, .data$s == .env$s) %>%
        summarise(value = sum(.data$Fry_NOS), .by = c(x, g, t)) %>%
        reshape2::acast(list("x", "g", "t"), value.var = "value")

      a_smolt <- 1
      Smolt_g[] <- apply(MMSE@N[, p_NOS_imm, a_smolt, mp, t1, , drop = FALSE], c(1, 2, 5), sum)
      Return_g[] <- apply(MMSE@N[, p_NOS_return, a_return, mp, t2, , drop = FALSE], c(1, 2, 3, 5), sum)
      for (g in 1:n_g) Esc_g[, g, , ] <- Return_g[, g, , ] * (1 - ExT_NOS[, s, , ])
      NOS_g[, , , y_spawn] <- dplyr::filter(N, .data$s == .env$s) %>%
        summarise(value = sum(.data$NOS), .by = c(x, g, a, t)) %>%
        reshape2::acast(list("x", "g", "a", "t"), value.var = "value")

      LHG[[s]] <- list(
        Egg = Egg_g,
        Fry = Fry_g,
        Smolt = Smolt_g,
        Esc = Esc_g,
        NOS = NOS_g
      )
    }
  }

  SMSE <- new(
    "SMSE",
    Name = "salmon MSE results",
    #nyears = SOM@nyears,
    proyears = SOM@proyears,
    nsim = SOM@nsim,
    nstocks = ns,
    Snames = sapply(1:ns, function(s) if (length(SOM@Bio[[s]]@Name)) SOM@Bio[[s]]@Name else paste("Population", s)),
    Egg_NOS = Egg_NOS,
    Egg_HOS = Egg_HOS,
    Fry_NOS = Fry_NOS,
    Fry_HOS = Fry_HOS,
    Smolt_NOS = Smolt_NOS,
    Smolt_HOS = Smolt_HOS,
    Smolt_Rel = Smolt_Rel,
    Njuv_NOS = Njuv_NOS,
    Njuv_HOS = Njuv_HOS,
    Return_NOS = Return_NOS,
    Return_HOS = Return_HOS,
    Escapement_NOS = Escapement_NOS,
    Escapement_HOS = Escapement_HOS,
    NOB = NOB,
    HOB = HOB,
    HOB_stray = HOB_stray,
    HOB_import = HOB_import,
    NOS = NOS,
    HOS = HOS,
    HOS_stray = HOS_stray,
    HOS_effective = HOS_effective,
    KPT_NOS = KPT_NOS,
    KT_NOS = KT_NOS,
    KPT_HOS = KPT_HOS,
    KT_HOS = KT_HOS,
    DPT_NOS = DPT_NOS,
    DT_NOS = DT_NOS,
    DPT_HOS = DPT_HOS,
    DT_HOS = DT_HOS,
    UPT_NOS = UPT_NOS,
    UT_NOS = UT_NOS,
    UPT_HOS = UPT_HOS,
    UT_HOS = UT_HOS,
    ExPT_NOS = ExPT_NOS,
    ExT_NOS = ExT_NOS,
    ExPT_HOS = ExPT_HOS,
    ExT_HOS = ExT_HOS,
    fitness = fitness,
    pNOB = pNOB,
    pHOS_census = pHOS_census,
    pHOS_effective = pHOS_effective,
    PNI = PNI,
    p_wild = p_wild,
    Mjuv_loss = Mjuv_loss
  )
  if (!missing(Harvest_MMP)) SMSE@Misc$Harvest_MMP <- Harvest_MMP
  if (!missing(Ford) && nrow(Ford)) SMSE@Misc$Ford <- salmonMSE_env$Ford

  SMSE@Misc$SOM <- SOM
  SMSE@Misc$Ref <- calc_ref(SOM, check = FALSE)
  SMSE@Misc$LHG <- LHG
  SMSE@Misc$RS <- RS

  return(SMSE)
}


#' @importFrom dplyr filter summarise
#' @importFrom reshape2 acast
get_salmonMSE_var <- function(d, var = "Egg_NOS", s = 1, FUN = function(x) sum(x, na.rm = TRUE)) {
  x <- t <- NULL
  dplyr::filter(d, .data$s %in% .env$s) %>%
    summarise(value = FUN(.data[[var]]), .by = c(x, t)) %>%
    reshape2::acast(list("x", "t"), value.var = "value")
}

get_salmonMSE_agevar <- function(d, var = "Egg_NOS", s = 1, FUN = function(x) sum(x, na.rm = TRUE)) {
  x <- a <- t <- NULL
  dplyr::filter(d, .data$s %in% .env$s) %>%
    summarise(value = FUN(.data[[var]]), .by = c(x, a, t)) %>%
    reshape2::acast(list("x", "a", "t"), value.var = "value")
}


#' Proportion wild spawners
#'
#' @description Calculate the proportion of wild spawners from a time series of spawners
#' - `calc_pwild()` is the simple calculation based on the proportion of hatchery spawners
#' - `calc_pwild_age()` performs the calculation weighted by age class fecundity
#'
#' @param NOS_a Array `[nsim, maxage, years]` for natural spawners
#' @param HOS_a Array `[nsim, maxage, years]` for hatchery spawners
#' @param fec Array `[nsim, maxage, years]` for age class fecundity
#' @param gamma Numeric, reduced reproductive success of hatchery spawners
#' @param pHOS_cur Numeric, proportion of hatchery spawners in current generation
#' @param pHOS_prev Numeric, proportion of hatchery spawners in previous generation
#' @return `calc_pwild_age()` a matrix of pWILD by simulation and year. `calc_pwild()` returns a numeric
#' @keywords internal
calc_pwild_age <- function(NOS_a, HOS_a, fec, gamma) {

  nsim <- dim(NOS_a)[1]
  maxage <- dim(NOS_a)[2]
  proyears <- dim(NOS_a)[3]

  prob <- array(NA_real_, c(nsim, proyears))

  for (y in 1:proyears) {
    if (sum(NOS_a[, , y], HOS_a[, , y])) {
      pNOS_y <- matrix(1, nsim, maxage)
      pnatural_b <- phetero_b <- phatch_b <- matrix(0, nsim, maxage)

      pNOS_y[] <- NOS_a[, , y]/rowSums(NOS_a[, , y] + HOS_a[, , y])

      for (a in 1:maxage) {
        pNOS_b <- matrix(1, nsim, maxage)
        pHOS_b <- matrix(0, nsim, maxage)

        b <- y - a # brood_year
        if (b > 0) {
          pNOS_b[] <- NOS_a[, , b]/rowSums(fec[, , b] * NOS_a[, , b] + fec[, , b] * HOS_a[, , b])
          pHOS_b[] <- HOS_a[, , b]/rowSums(fec[, , b] * NOS_a[, , b] + fec[, , b] * HOS_a[, , b])
        }
        pnatural_b[, a] <- rowSums(pNOS_b, na.rm = TRUE)^2
        phetero_b[, a] <- 2 * gamma * rowSums(pNOS_b, na.rm = TRUE) * rowSums(pHOS_b, na.rm = TRUE)
        phatch_b[, a] <- gamma^2 * rowSums(pHOS_b, na.rm = TRUE)^2
      }
      denom <- pnatural_b + phetero_b + phatch_b

      prob[, y] <- rowSums(pNOS_y * pnatural_b/denom, na.rm = TRUE)
    }
  }

  return(prob)
}

#' All-H Analyzer
#'
#' Wrapper function for an implementation of All-H Analyzer ([AHA](https://www.streamnet.org/home/data-maps/hatchery-reform/hsrg-tools/)) in R. Can be used to compare outputs between AHA
#' and salmonMSE.
#'
#' @param SOM An object of class \linkS4class{SOM}
#' @param ngen Integer, the number of generations for which to run the simulation
#' @param silent Logical, indicates whether to silence messages to the R console
#' @return A named list containing vectors of state variables (by simulation, population, and generation). See \linkS4class{SMSE} object description.
#'
#' @references Hatchery Scientific Review Group. 2020. All-H Analyzer Tool Guide and Documentation. May 2020.
#' @export
AHA <- function(SOM, ngen = 100, silent = FALSE) {
  SOM <- check_SOM(SOM)
  ns <- length(SOM@Bio)

  output_s <- list()

  for (s in 1:ns) {

    if (!silent) message("Running AHA for population ", s)

    if (sum(SOM@Harvest[[s]]@u_preterminal)) {
      warning("Pre-terminal fishing is not modeled in AHA. Setting SOM@Harvest[[", s, "]]@u_preterminal = 0")
      SOM@Harvest[[s]]@u_preterminal <- 0
    }

    if (SOM@Bio[[s]]@SRrel != "BH") {
      warning("Only Beverton-Holt smolt production is used in AHA. Setting SOM@Bio[[", s, "]]@SRrel = \"BH\"")
      SOM@Bio[[s]]@SRrel <- "BH"
    }

    if (!length(SOM@Bio[[s]]@capacity)) {
      stop("Need to specify SOM@Bio[[", s, "]]@capacity")
    }

    if (SOM@Hatchery[[s]]@fitness_type[2] != "none") {
      warning("Only fitness dynamics in the natural environment will be modeled.")
    }

    if (SOM@Hatchery[[s]]@pmax_esc < 1) {
      warning("SOM@Hatchery[[", s, "]]@pmax_esc = ", SOM@Hatchery@pmax_esc,
              ". AHA compatibility is maintained only when SOM@Hatchery@pmax_esc = 1")
    }

    if (SOM@Hatchery[[s]]@m < 1) {
      warning("SOM@Hatchery[[", s, "]]@m = ", SOM@Hatchery@m, ". AHA compatibility is maintained only when mark rate m = 1 for non-selective broodstock.")
    }

    age_mature <- SOM@Bio[[s]]@p_mature[1, , 1] > 0
    age_mature <- which(age_mature)[1]
    message("Age of maturity assumed to be: ", age_mature)

    fec <- SOM@Bio[[s]]@fec[1, age_mature, 1]
    SOM@Bio[[s]]@fec <- fec
    message("Fecundity of spawners assumed to be: ", fec)

    p_female_mean <- mean(SOM@Bio[[s]]@p_female)
    SOM@Bio[[s]]@p_female <- p_female_mean
    message("Fecundity of spawners assumed to be: ", p_female_mean)

    do_hatchery <- sum(SOM@Hatchery[[s]]@n_subyearling, SOM@Hatchery[[s]]@n_yearling) > 0
    if (do_hatchery) {
      fec_brood <- SOM@Hatchery[[s]]@fec_brood[1, age_mature, 1]
      message("Fecundity of broodtake assumed to be: ", fec_brood)

      if (is.na(SOM@Hatchery[[s]]@phatchery)) stop("Need value for SOM@Hatchery[[", s, "]]@phatchery")
    } else {
      fec_brood <- 0
    }
    SOM@Hatchery[[s]]@fec_brood <- fec_brood

    message("SAR calculated from survival from Mjuv to age ", age_mature)

    if (SOM@Bio[[s]]@n_g > 1) {
      warning("Multiple life history groups detected: SOM@Bio[[", s, "]]@n_g > 1. Using parameters for group 1")
    }
    if (SOM@Hatchery[[s]]@n_r > 1) {
      warning("Multiple hatchery release groups detected: SOM@Hatchery[[", s, "]]@n_r > 1. Using parameters for group 1")
    }

    surv_NOS <- surv_HOS <- matrix(1, SOM@nsim, age_mature)
    for (a in 2:age_mature) {
      surv_NOS[, a] <- surv_NOS[, a-1] * exp(-SOM@Bio[[s]]@Mjuv_NOS[, a-1, 1, 1])
      if (do_hatchery) {
        surv_HOS[, a] <- surv_HOS[, a-1] * exp(-SOM@Hatchery[[s]]@Mjuv_HOS[, a-1, 1, 1])
      }
    }

    if (SOM@Harvest[[s]]@MSF_T) {
      .F <- get_F(u = SOM@Harvest[[s]]@u_terminal, M = 1e-8, ret = SOM@Hatchery[[s]]@m, release_mort = SOM@Harvest[[s]]@release_mort[2])
      Frel <- (1 - SOM@Hatchery[[s]]@m) * SOM@Harvest[[s]]@release_mort[2] * .F
      Fret <- SOM@Hatchery[[s]]@m * .F
      u_NOR <- 1 - exp(-Frel)
      u_HOR <- 1 - exp(-Frel - Fret)
      message(
        "Mark-selective fishing for terminal fishery detected. For AHA, setting harvest rate of natural and hatchery fish = ",
        round(u_NOR, 3), " and ", round(u_HOR, 3), ", respectively. Catch represents kept catch + dead releases."
      )
    }

    output_sim <- lapply(1:SOM@nsim, .AHA_wrapper, s = s, SOM = SOM, ngen = ngen, SAR_NOS = surv_NOS[, age_mature], SAR_HOS = surv_HOS[, age_mature])
    var_report <- names(output_sim[[1]])

    output_s[[s]] <- lapply(var_report, function(i) {
      sapply(output_sim, getElement, i) %>% t()
    }) %>%
      structure(names = var_report)

  }

  output <- lapply(var_report, function(i) {
    mat <- sapply(output_s, getElement, i, simplify = "array") %>%
      aperm(c(1, 3, 2))
  }) %>%
    structure(names = var_report)

  return(output)
}


.AHA_wrapper <- function(x, s, SOM, ngen, SAR_NOS, SAR_HOS, silent = FALSE) {

  Bio <- SOM@Bio[[s]]
  Habitat <- SOM@Habitat[[s]]
  Hatchery <- SOM@Hatchery[[s]]
  Harvest <- SOM@Harvest[[s]]

  if (SOM@Harvest[[s]]@MSF_T) {
    .F <- get_F(u = Harvest@u_terminal, M = 1e-8, ret = Hatchery@m, release_mort = Harvest@release_mort[2])
    Frel <- (1 - Hatchery@m) * Harvest@release_mort[2] * .F
    Fret <- Hatchery@m * .F
    u_NOR <- 1 - exp(-Frel)
    u_HOR <- 1 - exp(-Frel - Fret)
  } else {
    u_NOR <- u_HOR <- Harvest@u_terminal
  }

  output <- .AHA(
    prod_adult = Bio@kappa[x],
    capacity_adult = Bio@capacity[x] * SAR_NOS[x],
    fec_spawn = sum(Bio@fec),
    p_female = Bio@p_female,
    surv_ocean = SAR_NOS[x],
    surv_passage_juv = 1,
    surv_passage_adult = Bio@s_enroute,
    u_HOR = c(u_HOR, 0, 0, 0),
    u_NOR = c(u_NOR, 0, 0, 0),
    surv_pre_spawn = Hatchery@s_prespawn,
    fec_brood = sum(Hatchery@fec_brood),
    surv_egg_smolt = Hatchery@s_egg_smolt,
    surv_egg_subyearling = Hatchery@s_egg_subyearling,
    capacity_spawn_em = 1e12,
    capacity_smolt_adult = 1e12,
    surv_adult_return_of_yearling = SAR_HOS[x],
    surv_adult_return_of_subyearling = SAR_HOS[x],
    RRS_HOS = Hatchery@gamma,
    p_weir_efficiency = Hatchery@premove_HOS,
    p_return_hatchery = Hatchery@phatchery,
    n_release_yearling = sum(Hatchery@n_yearling),
    n_release_subyearling = sum(Hatchery@n_subyearling),
    brood_import = 0,
    brood_export = 0,
    p_NOR_brood_max = Hatchery@pmax_NOB,
    p_HOS_goal = 0, # Not used
    p_NOB_goal = Hatchery@ptarget_NOB,
    SAR_vary = "none",
    fitness_type = Hatchery@fitness_type[1], # Fitness
    theta = Hatchery@theta,
    rel_loss_egg = Hatchery@rel_loss[1],
    rel_loss_fry = Hatchery@rel_loss[2],
    rel_loss_smolt = Hatchery@rel_loss[3],
    Zpop_start = c(Hatchery@zbar_start[x, 1, ], 100),
    phenotype_variance = Hatchery@phenotype_variance,
    selection_strength = sqrt(Hatchery@fitness_variance/Hatchery@phenotype_variance),
    heritability = Hatchery@heritability[x],
    fitness_floor = Hatchery@fitness_floor,
    strays_total = 0,
    ngen = ngen
  )

  var_out <- c(
    "NOR" = "Return_NOS",
    "HOR" = "Return_HOS",
    "harvest_NOR" = "KT_NOS",
    "harvest_HOR" = "KT_HOS",
    "escapement_NOR" = "Escapement_NOS",
    "escapement_HOR" = "Escapement_HOS",
    "NOB" = "NOB",
    "HOB" = "HOB",
    "NOS" = "NOS",
    "HOS" = "HOS",
    "HOS_effective" = "HOS_effective",
    "fry_NOS" = "Egg_NOS",
    "fry_HOS" = "Egg_HOS",
    "fry_HOR" = "Egg_Rel",
    "smolt_NOS" = "Smolt_NOS",
    "smolt_HOS" = "Smolt_HOS",
    "smolt_HOR" = "Smolt_Rel",
    "fitness" = "fitness",
    "SAR_loss" = "SAR_loss",
    "pNOB" = "pNOB",
    "pHOSeff" = "pHOSeff",
    "alpha" = "alpha",
    "beta" = "beta"
  )

  out <- lapply(names(var_out), function(x) {
    sapply(output, getElement, x)
  }) %>%
    structure(names = var_out)
  out$PNI <- out$pNOB/(out$pNOB + out$pHOSeff)
  out$p_wild <- sapply(2:length(out$pHOSeff), function(g) {
    calc_pwild(out$pHOSeff[g], out$pHOSeff[g-1], Hatchery@gamma)
  })
  out$Ref <- output[[1]][["Ref"]]

  return(out)
}

.AHA <- function(prod_adult = 2.7,                     # AHA habitat fields
                 capacity_adult = 17250,
                 #capacity_smolt = 1.917e6,
                 fec_spawn = 5040,
                 p_female = 0.49,
                 surv_ocean = 0.01,                    # Ocean dynamics
                 surv_passage_juv = 1,
                 surv_passage_adult = 1,
                 u_HOR = 1e-2 * c(3.8, 2.5, 0, 14),   # Harvest dynamics
                 u_NOR = 1e-2 * c(3.8, 2.5, 0, 14),
                 surv_pre_spawn = 1,                   ## Survival prior to spawning
                 fec_brood = 5040,                       # In-hatchery
                 surv_egg_smolt = 1e-6,
                 surv_egg_subyearling = 0.92,
                 capacity_spawn_em = 1e12,                 ## Hard wired hatchery settings
                 capacity_smolt_adult = 1e12,
                 surv_adult_return_of_yearling = 1e-4,       # Post-hatchery
                 surv_adult_return_of_subyearling = 8.5e-3,
                 RRS_HOS = 0.8,
                 p_weir_efficiency = 0,
                 p_return_hatchery = 0.04,
                 n_release_yearling = 0,
                 n_release_subyearling = 2e6,
                 brood_import = 0,
                 brood_export = 0,
                 p_NOR_brood_max = 0.7,
                 p_HOS_goal = 0,
                 p_NOB_goal = 0.51,
                 SAR_vary = c("none", "PDO", "random"),
                 fitness_type = c("Ford", "Busack", "none"), # Fitness
                 theta = c(100, 80, 70),
                 rel_loss_egg = 0.5,
                 rel_loss_fry = 0.4,
                 rel_loss_smolt = 0.1,
                 Zpop_start = c(93.1, 92, 90),
                 phenotype_variance = 10,
                 selection_strength = 3,
                 heritability = 0.5,
                 fitness_floor = 0.5,
                 strays_total = 0,
                 ngen = 100) {

  SAR_vary <- match.arg(SAR_vary)
  stopifnot(SAR_vary == "none")

  fitness_type <- match.arg(fitness_type)
  if (fitness_type == "Busack") stop("Busack fitness calcs not completed yet")
  # Habitat section
  prod_spawn_em <- fec_spawn * p_female

  #habitat_h8 <- 0.009 # See Excel spreadsheet
  habitat_h8 <- surv_ocean
  surv_smolt_adult_obs <- surv_passage_juv * surv_ocean * surv_passage_adult # SAR, survival from release to adult return (22)
  surv_smolt_adult_base <- ifelse(habitat_h8 > 0, habitat_h8, surv_smolt_adult_obs) # (23)

  prod_smolt_adult <- surv_smolt_adult_base # (13)
  prod_em_smolt <- prod_adult/prod_spawn_em/prod_smolt_adult # (11)

  capacity_em_smolt <- local({ # (12)
    denom <- 1/capacity_adult-1/capacity_smolt_adult
    1/prod_smolt_adult/denom
  })

  # Reporting variables (5,6)
  adjust_prod_adult <- prod_adult * surv_smolt_adult_obs / surv_smolt_adult_base
  adjust_capacity_adult <- prod_em_smolt * surv_smolt_adult_obs/
    (1/capacity_spawn_em + prod_em_smolt/capacity_em_smolt + prod_em_smolt * surv_smolt_adult_obs/capacity_smolt_adult)

  # Ref pt (15-18)
  Neq <- adjust_capacity_adult * (1 - 1/adjust_prod_adult)
  MSY <- ((sqrt(adjust_prod_adult)-1)^2) * adjust_capacity_adult / adjust_prod_adult
  SMSY <- (sqrt(adjust_prod_adult) - 1) * adjust_capacity_adult /adjust_prod_adult
  umsy <- MSY/(MSY + SMSY)

  # (41-42)
  surv_adult_return_of_yearling_applied <- surv_adult_return_of_yearling *
    surv_smolt_adult_base/surv_smolt_adult_obs
  surv_adult_return_of_subyearling_applied <- surv_adult_return_of_subyearling *
    surv_smolt_adult_base/surv_smolt_adult_obs

  # (49-50)
  p_yearling <- n_release_yearling/(n_release_yearling + n_release_subyearling)
  p_subyearling <- 1 - p_yearling

  # (52)
  brood_local <- n_release_yearling/(surv_pre_spawn * p_female * fec_brood * surv_egg_smolt) +
    n_release_subyearling/(surv_pre_spawn * p_female * fec_brood * surv_egg_subyearling) - brood_import

  #SAR_rel (#56) Relative survival of hatchery origin fish after release from hatchery
  surv_adult_smolt_rel <- (surv_adult_return_of_yearling_applied * p_yearling +
                             surv_adult_return_of_subyearling_applied * p_subyearling)/surv_smolt_adult_base

  # Calc fitness variables (80,81)
  omega <- sqrt(phenotype_variance) * selection_strength
  omega2 <- omega * omega
  A <- 1 - heritability * phenotype_variance/(omega2 + phenotype_variance)

  egg_per_spawner <- fec_brood * surv_pre_spawn * p_female # E
  surv_egg_release <- surv_egg_subyearling * p_subyearling + surv_egg_smolt * p_yearling # F

  # BC
  surv_adult_return_apply <- numeric(ngen)
  if (SAR_vary == "none") {
    surv_adult_return_apply[] <- surv_ocean
  } else if (SAR_vary == "random") {
    #surv_random <- MIN(LOGINV(RAND(),LN(BB59),MAX(LN(2*$I$17+1)/2,0.0000000001)),0.99999)
    #surv_adult_return_apply[] <- surv_random
  } else { #PDO
    #surv_adult_return_apply[] <- surv_ocean * SAR_V8
  }

  # Geometric mean BC55
  geomean <- function(x) exp(mean(log(x)))
  surv_adult_return_geo <- geomean(surv_adult_return_apply[1:ngen])
  #surv_adult_return_geo <- geomean(surv_adult_return_apply[21:100])

  # G (hatchery survival from release to adulthood)
  surv_release_adult <- surv_adult_return_apply * surv_adult_smolt_rel * surv_passage_juv

  ###### AHA loop
  AHA_loop <- list()

  # Fitness calculation
  pbar <- matrix(NA, 2, ngen)

  for(g in 1:ngen) {

    if (g == 1) {
      HOR_total <- NOR_total <- 1e3
      #HOR_total <- 0
      #phi0 <- surv_smolt_adult_base * p_female * fec_spawn
      #SRRpars <- calc_SRRpars(prod_spawn_em * prod_em_smolt, capacity_em_smolt, fec_spawn, p_female)
      #R0 <- MSEtool::R0conv(SRRpars[1], SRRpars[2], phi0)
      #NOR_total <- R0 * phi0
    } else {
      pbar[, g-1] <- AHA_loop[[g-1]]$pbar
      HOR_total <- AHA_loop[[g-1]]$adult_HOR
      NOR_total <- AHA_loop[[g-1]]$adult_HOS + AHA_loop[[g-1]]$adult_NOS
    }

    AHA_loop[[g]] <- .AHA_loop(
      g,
      HOR_total, NOR_total,
      surv_passage_adult, surv_passage_juv,
      u_HOR, u_NOR,
      p_NOB_goal,
      brood_local, brood_import,
      p_NOR_brood_max,
      p_return_hatchery,
      surv_adult_return_apply[g], SAR_ratio = surv_adult_return_apply[g]/surv_adult_return_geo,
      p_weir_efficiency,
      strays_total,
      RRS_HOS,
      p_HOS_goal,
      fitness_type, Zpop_start, theta, omega2, A, phenotype_variance, fitness_floor, heritability,
      pbar_prev = if (g == 1) NA else pbar[, g-1],
      rel_loss_egg, rel_loss_fry, rel_loss_smolt,
      prod_spawn_em, prod_em_smolt, prod_smolt_adult,
      capacity_spawn_em, capacity_em_smolt, capacity_smolt_adult,
      egg_per_spawner, surv_egg_release, surv_release_adult[g]
    )
  }

  # Reference points in the first entry in the loop
  AHA_loop[[1]]$Ref <- c(Neq = Neq, MSY = MSY, SMSY = SMSY, umsy = umsy)

  return(AHA_loop)
}


.AHA_loop <- function(g = 1, HOR_total = 1e3, NOR_total = 1e3,
                      surv_passage_adult, surv_passage_juv,
                      u_HOR, u_NOR, p_NOB_goal, brood_local, brood_import,
                      p_NOR_brood_max, p_return_hatchery,
                      surv_adult_return_apply,
                      SAR_ratio,
                      p_weir_efficiency,
                      strays_total,
                      RRS_HOS,
                      p_HOS_goal,
                      fitness_type,
                      Zpop_start,
                      theta, omega2, A, phenotype_variance, fitness_floor, heritability, pbar_prev,
                      rel_loss_egg, rel_loss_fry, rel_loss_smolt,
                      prod_spawn_em, prod_em_smolt, prod_smolt_adult,
                      capacity_spawn_em, capacity_em_smolt, capacity_smolt_adult,
                      egg_per_spawner, surv_egg_release, surv_release_adult, ...) {

  # Fishery harvest
  nfishery <- 4
  do_harvest_NOR <- catch_fn(NOR_total, u_NOR, surv_passage = c(1, 1, 1, surv_passage_adult), nfishery = nfishery)
  do_harvest_HOR <- catch_fn(HOR_total, u_HOR, surv_passage = c(1, 1, 1, surv_passage_adult), nfishery = nfishery)

  harvest_NOR <- sum(do_harvest_NOR$catch)
  harvest_HOR <- sum(do_harvest_HOR$catch)

  #AHA is always programmed so that NOR escapement is a minimum of 1.
  #NOB is limited by the max % of the NOR run specified in AHA
  escapement_NOR <- do_harvest_NOR$N[nfishery + 1]
  escapement_HOR <- do_harvest_HOR$N[nfishery + 1]

  brood_NOB <- ifelse(escapement_NOR > 1,
                      min(p_NOB_goal * brood_local, p_NOR_brood_max * (escapement_NOR - 1)),
                      0)

  brood_HOB <- local({
    aa <- min(brood_NOB/p_NOB_goal, brood_local, escapement_HOR)
    bb <- max(brood_local, aa)
    cc <- min(brood_local, escapement_HOR)
    dd <- ifelse(p_NOB_goal > 0, bb - brood_NOB, cc)
    ee <- max(0, dd)

    min(ee, p_return_hatchery * escapement_HOR)
  })

  total_brood <- brood_NOB + brood_HOB + brood_import

  NOS <- escapement_NOR - brood_NOB

  HOS_total <- max(0, escapement_HOR * (1 - p_return_hatchery)) * (1 - p_weir_efficiency) + SAR_ratio * strays_total
  HOS_effective <- max(0, escapement_HOR * (1 - p_return_hatchery)) * (1 - p_weir_efficiency) * RRS_HOS + SAR_ratio * strays_total

  #HOS_goal <- min(HOS_effective, NOS * p_HOS_goal/(1 - p_HOS_goal))
  #HOS_surplus <- HOS_total - HOS_goal

  total_escapement <- HOS_total + NOS
  total_spawners <- HOS_effective + NOS

  pNOB <- ifelse(total_brood > 0, brood_NOB/total_brood,
                 ifelse(brood_import > 0, 0, 1))
  pNOS <- NOS/total_spawners

  # Fitness this generation
  if (fitness_type == "none") {

    fitness <- 1

  } else if (g == 1) {

    if (fitness_type == "Ford") {

      pbar <- Zpop_start[1:2]
      fitness <- local({
        x <- exp(-0.5 * (pbar[1] - theta[1])^2/(omega2 + phenotype_variance))
        max(fitness_floor, x)
      })

    } else { # Busack

      Zpop <- Zpop_start
      fitness <- abs(Zpop[1, g] - mean(theta[-1]))/abs(theta[1] - mean(theta[-1]))

    }

  } else if (fitness_type == "Ford") {

    do_fitness <- calc_Ford_fitness(pNOB, pHOS = 1 - pNOS, pbar_prev,
                                    theta, omega2, phenotype_variance, A, fitness_floor, heritability)
    pbar <- do_fitness$pbar
    fitness <- do_fitness$fitness

  } else { # Busack - incomplete
    #do_fitness <- calc_Busack_fitness()
    #Zpop <- do_fitness$Zpop
    #fitness <- do_fitness$fitness
  }

  # AQ,AR,AS
  fitness_egg <- fitness^rel_loss_egg
  fitness_fry <- fitness^rel_loss_fry
  fitness_smolt <- fitness^rel_loss_smolt

  # AT, AU, AV, AW
  fitprod_spawn_em <- prod_spawn_em * fitness_egg
  fitprod_em_smolt <- prod_em_smolt * fitness_fry
  fitprod_smolt_adult <- surv_passage_juv * surv_adult_return_apply * fitness_smolt
  fitprod_cumulative <- fitprod_spawn_em * fitprod_em_smolt * fitprod_smolt_adult

  # AX, AY, AZ, BA
  fitcap_spawn_em <- capacity_spawn_em * fitness_egg
  fitcap_em_smolt <- capacity_em_smolt * fitness_fry
  fitcap_smolt_adult <- capacity_smolt_adult * fitness_smolt
  fitcap_cumulative <-
    fitprod_cumulative/(fitprod_spawn_em/fitcap_spawn_em+fitprod_spawn_em*
                         fitprod_em_smolt/fitcap_em_smolt+fitprod_spawn_em*
                         fitprod_em_smolt*fitprod_smolt_adult/fitcap_smolt_adult)

  ## Next generation - natural production
  # AI, AJ - offspring from hatchery origin spawners + natural origin spawners
  fry_HOS <- calc_SRR(HOS_effective, total_spawners, fitprod_spawn_em, fitcap_spawn_em)
  fry_NOS <- calc_SRR(NOS, total_spawners, fitprod_spawn_em, fitcap_spawn_em)
  total_fry <- fry_HOS + fry_NOS

  smolt_HOS <- calc_SRR(fry_HOS, total_fry, fitprod_em_smolt, fitcap_em_smolt)
  smolt_NOS <- calc_SRR(fry_NOS, total_fry, fitprod_em_smolt, fitcap_em_smolt)
  total_smolt <- smolt_HOS + smolt_NOS

  SRRpar <- calc_SRRpars(fitprod_em_smolt, fitcap_em_smolt)

  ## Hatchery origin release
  fry_HOR <- total_brood * egg_per_spawner
  smolt_HOR <- total_brood * egg_per_spawner * surv_egg_release

  # Next generation return
  adult_HOS <- calc_SRR(smolt_HOS, total_smolt + smolt_HOR, fitprod_smolt_adult, fitcap_smolt_adult)
  adult_NOS <- calc_SRR(smolt_NOS, total_smolt + smolt_HOR, fitprod_smolt_adult, fitcap_smolt_adult)

  adult_HOR <- local({
    smolt_temp <- smolt_HOR * surv_release_adult
    smolt_temp2 <- smolt_temp + total_smolt * surv_adult_return_apply * surv_passage_juv
    smolt_temp/(1 + smolt_temp2/capacity_smolt_adult)
  })

  out <- list(
    NOR = NOR_total, # Natural origin return (function input)
    HOR = HOR_total, # Hatchery origin return (function input)
    harvest_NOR = harvest_NOR,
    harvest_HOR = harvest_HOR,
    escapement_NOR = escapement_NOR,
    escapement_HOR = escapement_HOR,
    NOB = brood_NOB,
    HOB = brood_HOB,
    NOS = NOS,
    HOS = HOS_total,
    HOS_effective = HOS_effective,
    fitness = fitness,
    fry_NOS = fry_NOS, # Next generation
    fry_HOS = fry_HOS,
    fry_HOR = fry_HOR, # Hatchery origin release
    smolt_NOS = smolt_NOS,
    smolt_HOS = smolt_HOS,
    smolt_HOR = smolt_HOR,
    adult_HOS = adult_HOS, # Actually the return
    adult_NOS = adult_NOS,
    adult_HOR = adult_HOR, # Hatchery origin return
    SAR_loss = fitprod_smolt_adult,
    pNOB = pNOB,
    pHOSeff = 1 - pNOS,
    alpha = SRRpar[1],
    beta = SRRpar[2]
  )

  if (fitness_type == "Ford") out$pbar <- pbar

  return(out)
}

#' @title Compare scenarios in markdown
#'
#' @description Generate a markdown report for multiple model runs to compare scenarios. 3-5 is likely the ideal number of scenarios for comparison.
#'
#' @param SMSE_list List of \linkS4class{SMSE} objects
#' @param names Character vector `length(SMSE_list)` to label individual model runs
#' @param col_vec Character vector `length(SMSE_list)` for custom colour schemes for comparing across model scenarios in figures
#' @param filename Character string for the name of the markdown and HTML files.
#' @param dir The directory in which the markdown and HTML files will be saved.
#' @param open_file Logical, whether the HTML document is opened after it is rendered.
#' @param render_args List of arguments to pass to [rmarkdown::render()].
#' @param ... Additional arguments (not used)
#' @importFrom utils browseURL
#' @seealso [report()]
#' @return Returns invisibly the output of [rmarkdown::render()], typically the path of the output file
#' @export
compare <- function(SMSE_list, names, col_vec,
                    filename = "SMSEcompare", dir = tempdir(), open_file = TRUE, render_args = list(), ...) {

  class_check <- sapply(SMSE_list, inherits, "SMSE")
  if (!all(class_check)) stop("Not all objects in SMSE_list are SMSE objects")

  if (missing(names)) names <- paste("Scenario", 1:length(SMSE_list))
  if (missing(col_vec)) {
    col_vec <- hcl.colors(length(SMSE_list), palette = "Dark 3", alpha = 1)
    shade_vec <- hcl.colors(length(SMSE_list), palette = "Dark 3", alpha = 0.25)
  } else {
    if (requireNamespace("scales", quietly = TRUE)) {
      shade_vec <- scales::alpha(col_vec, 0.25)
    } else {
      shade_vec <- col_vec
    }
  }

  ####### Function arguments for rmarkdown::render
  rmd <- system.file("include", "SMSEcompare.Rmd", package = "salmonMSE") %>% readLines()
  rmd_split <- split(rmd, 1:length(rmd))

  stock_ind <- grep("ADD RMD BY STOCK", rmd)
  rmd_split[[stock_ind]] <- Map(make_rmd_compare, s = 1:SMSE_list[[1]]@nstocks, sname = SMSE_list[[1]]@Snames) %>% unlist()

  filename_rmd <- paste0(filename, ".Rmd")

  render_args$input <- file.path(dir, filename_rmd)
  if (is.null(render_args$quiet)) render_args$quiet <- TRUE

  # Generate markdown report
  if (!dir.exists(dir)) {
    message("Creating directory: ", dir)
    dir.create(dir)
  }
  write(unlist(rmd_split), file = file.path(dir, filename_rmd))

  # Rendering markdown file
  message("Rendering markdown file: ", file.path(dir, filename_rmd))
  output_filename <- do.call(rmarkdown::render, render_args)
  message("Rendered file: ", output_filename)

  if (open_file) browseURL(output_filename)
  invisible(output_filename)
}

#' Compare simulation runs
#'
#' Create figures that compare results across two dimensions
#'
#' - [compare_spawners()] generates a time series of the composition of spawners
#' - [compare_fitness()] generates a time series of metrics (fitness, PNI, pHOS, and pWILD) related to hatchery production
#' - [compare_escapement()] generates a time series of the proportion of spawners and broodtake to escapement
#'
#' @param SMSE_list A list of SMSE objects returned by [salmonMSE()]
#' @param Design A data frame with two columns that describes the factorial design of the simulations. Used to label the figure.
#' Rows correspond to each object in `SMSE_list`. There two columns are variables against which to plot the result. See
#' example in \url{https://docs.salmonmse.com/articles/decision-table.html}.
#' @param prop Logical, whether to plot absolute numbers over proportions
#' @param FUN Summarizing function across simulations, typically [stats::median()] or [base::mean()]
#' @returns A ggplot object
#' @export
compare_spawners <- function(SMSE_list, Design, prop = FALSE, FUN = median) {
  Sp <- lapply(1:nrow(Design), function(x) {
    plot_spawners(SMSE_list[[x]], prop = FALSE, FUN = FUN, figure = FALSE) %>%
      reshape2::melt() %>% # Metric x Year
      mutate(Design1 = Design[x, 1], Design2 = Design[x, 2])
  }) %>%
    bind_rows() %>%
    rename(Year = Var2) %>%
    dplyr::filter(value > 0)
  Sp$Var1 <- factor(Sp$Var1, levels = c("HOS", "NOS_notWILD", "WILD"))

  if (prop) {
    Sp <- mutate(Sp, p = value/sum(value), .by = c(Year, Design1, Design2))

    g <- ggplot(Sp, aes(.data$Year, .data$p, fill = .data$Var1)) +
      geom_area() +
      facet_grid(vars(Design1), vars(Design2)) +
      labs(x = "Projection Year", y = "Proportion", fill = NULL) +
      coord_cartesian(ylim = c(0, 1), expand = FALSE) +
      theme(panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            legend.position = "bottom") +
      scale_fill_brewer(palette = "PuBuGn")
  } else {
    g <- ggplot(Sp, aes(.data$Year, .data$value, fill = .data$Var1)) +
      geom_area() +
      facet_grid(vars(Design1), vars(Design2)) +
      labs(x = "Projection Year", y = "Spawners", fill = NULL) +
      coord_cartesian(expand = FALSE) +
      theme(panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            legend.position = "bottom") +
      scale_fill_brewer(palette = "PuBuGn")
  }
  g
}

#' @rdname compare_spawners
#' @export
compare_fitness <- function(SMSE_list, Design, FUN = median) {

  fitness <- lapply(1:nrow(Design), function(x) {
    plot_fitness(SMSE_list[[x]], figure = FALSE, FUN = FUN) %>%
      reshape2::melt() %>% # Metric x Year
      mutate(Design1 = Design[x, 1], Design2 = Design[x, 2])
  }) %>%
    bind_rows() %>%
    rename(Year = Var1) %>%
    dplyr::filter(value > 0)

  g <- ggplot(fitness, aes(.data$Year, .data$value, colour = .data$Var2)) +
    geom_line(linewidth = 1) +
    facet_grid(vars(Design1), vars(Design2)) +
    labs(x = "Projection Year", y = "Median", colour = "Metric") +
    coord_cartesian(ylim = c(0, 1), expand = FALSE) +
    theme(legend.position = "bottom", panel.grid.minor = element_blank()) +
    scale_colour_brewer(palette = "RdYlBu")

  g
}

#' @rdname compare_spawners
#' @export
compare_escapement <- function(SMSE_list, Design, FUN = median) {

  label_fn <- function(x) {
    switch(
      x,
      "pNOSesc" = "Spawner/Escapement (natural)",
      "pHOSesc" = "Spawner/Escapement (hatchery)",
      "pbrood" = "Broodstock/Escapement (total)"
    )
  }

  esc <- lapply(1:nrow(Design), function(x) {
    plot_escapement(SMSE_list[[x]], figure = FALSE, FUN = FUN) %>%
      reshape2::melt() %>% # Metric x Year
      mutate(Design1 = Design[x, 1], Design2 = Design[x, 2])
  }) %>%
    bind_rows() %>%
    rename(Year = Var1) %>%
    dplyr::filter(value > 0) %>%
    mutate(Var2 = sapply(as.character(Var2), label_fn))

  g <- ggplot(esc, aes(.data$Year, .data$value, colour = .data$Var2)) +
    geom_line(linewidth = 1) +
    facet_grid(vars(Design1), vars(Design2)) +
    labs(x = "Projection Year", y = "Median proportion", colour = "Metric") +
    coord_cartesian(ylim = c(0, 1), expand = FALSE) +
    guides(colour = guide_legend(ncol = 2)) +
    theme(legend.position = "bottom", panel.grid.minor = element_blank()) +
    scale_colour_brewer(palette = "Accent")

  g
}

#' Example performance metrics
#'
#' Functions that evaluate return probabilities of outcomes from the simulations.
#'
#' - `PNI50` calculates the probability that PNI exceeds 0.50 (threshold for an integrated-transition population, Withler et al. 2018)
#' - `PNI80` calculates the probability that PNI exceeds 0.80 (threshold for an integrated-wild population, Withler et al. 2018)
#' - `WILD50` calculates the probability that at least 50 percent of natural spawners are wild
#' - `SMSY85` calculates the probability that NOS/SMSY exceeds 0.85
#' - `Sgen100` calculates the probability that NOS/Sgen exceeds 1
#'
#' @param SMSE SMSE object returned by [salmonMSE()]
#' @param Ref Threshold for the performance metric, used to calculate the probability that the metric exceeds this value
#' @param Yrs Numeric vector of length 2 to indicate the year range over which to summarize performance.
#' If NULL, the performance is summarized over all projection years.
#' @references
#' Withler et al. 2018. Genetically Based Targets for Enhanced Contributions to Canadian Pacific Chinook Salmon Populations.
#' DFO Can. Sci. Advis. Sec. Res. Doc. 2018/019. xii + 88 p.
#' @export
#' @return A vector of probabilities corresponding to population
PNI50 <- function(SMSE, Ref = 0.5, Yrs = NULL) {
  if (is.null(Yrs)) Yrs <- c(1, SMSE@proyears)
  apply(SMSE@PNI[, , Yrs[1]:Yrs[2], drop = FALSE] >= Ref, 2, mean, na.rm = TRUE)
}

#' @rdname PNI50
#' @export
PNI80 <- PNI50
formals(PNI80)$Ref <- 0.8

#' @rdname PNI50
#' @export
WILD50 <- function(SMSE, Ref = 0.50, Yrs = NULL) {
  if (is.null(Yrs)) Yrs <- c(1, SMSE@proyears)
  apply(SMSE@p_wild[, , Yrs[1]:Yrs[2], drop = FALSE] >= Ref, 2, mean, na.rm = TRUE)
}

#' @rdname PNI50
#' @export
SMSY85 <- function(SMSE, Ref = 0.85, Yrs = NULL) {
  if (is.null(Yrs)) Yrs <- c(1, SMSE@proyears)
  y <- seq(Yrs[1], Yrs[2])
  NOS <- apply(SMSE@NOS[, , , y, drop = FALSE], c(1, 2, 4), sum)
  SMSY <- sapply(1:SMSE@nstocks, function(s) SMSE@Misc$Ref[[s]]["Spawners_MSY", ]) %>%
    array(c(SMSE@nsim, SMSE@nstocks, length(y)))
  ratio <- NOS/SMSY
  apply(ratio >= Ref, 2, mean, na.rm = TRUE)
}

#' @rdname PNI50
#' @export
Sgen100 <- function(SMSE, Ref = 1, Yrs = NULL) {
  if (is.null(Yrs)) Yrs <- c(1, SMSE@proyears)
  y <- seq(Yrs[1], Yrs[2])
  NOS <- apply(SMSE@NOS[, , , y, drop = FALSE], c(1, 2, 4), sum)
  Sgen <- sapply(1:SMSE@nstocks, function(s) SMSE@Misc$Ref[[s]]["Sgen", ]) %>%
    array(c(SMSE@nsim, SMSE@nstocks, length(y)))
  ratio <- NOS/Sgen
  apply(ratio >= Ref, 2, mean, na.rm = TRUE)
}

#' @name salmonMSE-int
#' @title Internal salmonMSE functions for converting operating model inputs and outputs
#'
#' @description
#' - [SOM2MOM()] converts a salmon operating model (\linkS4class{SOM}) to a multi-stock operating model ([MSEtool::MOM-class])
#' - [make_Stock()] creates the [MSEtool::Stock-class] object (openMSE) corresponding to salmon life stage
#' - [make_Fleet()] creates the [MSEtool::Fleet-class] object (openMSE) corresponding to the fishery that interacts with the various salmon life stages
#' - [multiHist2SHist()] converts the openMSE historical reconstruction into a salmon Hist object (\linkS4class{SHist})
#' - [MMSE2SMSE()] converts the openMSE projection output, along with additional state variables recorded in [salmonMSE_env], into a salmon MSE object (\linkS4class{SMSE})
#' - [make_Harvest_MMP()] creates a multi-stock management procedure for the harvest component of the operating model by
#' specifying exploitation rates through updating the formal arguments for [Harvest_MMP()]
#'
#' [salmonMSE()] is the wrapper function that coordinates the simulation and the output.
#'
#' @param SOM An object of class \linkS4class{SOM}
#' @param check Logical, whether to check the `SOM` object using [check_SOM()]
#' @export
#' @return
#' `SOM2MOM`: [MSEtool::MOM-class] object
SOM2MOM <- function(SOM, check = TRUE) {
  if (check) SOM <- check_SOM(SOM)

  MOM <- suppressMessages(new("MOM"))
  MOM@Name <- SOM@Name
  MOM@proyears <- 2 * SOM@proyears
  MOM@nsim <- SOM@nsim
  MOM@interval <- 1
  MOM@pstar <- 0.5
  MOM@maxF <- 20
  MOM@reps <- 1

  # Stock objects
  ns <- length(SOM@Bio)
  Stocks_s <- lapply(1:ns, function(s) make_Stock_objects(SOM, s = s))
  Stocks <- do.call(c, Stocks_s)
  #ns <- length(Stocks_s)
  np_s <- sapply(Stocks_s, length)
  np <- length(Stocks)
  n_g <- sapply(1:ns, function(s) SOM@Bio[[s]]@n_g)
  n_r <- sapply(1:ns, function(s) SOM@Hatchery[[s]]@n_r)

  do_hatchery <- vapply(SOM@Hatchery, function(Hatchery) sum(Hatchery@n_yearling, Hatchery@n_subyearling) > 0, logical(1))
  has_strays <- vapply(1:ns, function(s) any(SOM@stray[-s, s] > 0) || sum(SOM@Hatchery[[s]]@stray_external), logical(1))

  # Fleet objects (one per openMSE population/life stage) ----
  nf <- 1
  Fleets_s <- lapply(1:ns, function(s) make_Fleet_objects(SOM, s = s))
  Fleets <- do.call(c, Fleets_s)

  MOM@Stocks <- lapply(Stocks, getElement, "Stock")
  MOM@Fleets <- lapply(1:np, function(p) list(Fleets[[p]]$Fleet))
  MOM@Obs <- lapply(1:np, function(...) {
    lapply(1:nf, function(...) MSEtool::Generic_Obs)
  })
  MOM@Imps <- lapply(1:np, function(...) {
    lapply(1:nf, function(...) MSEtool::Perfect_Imp)
  })

  MOM@cpars <- lapply(1:np, function(p) {
    lapply(1:nf, function(...) {
      c(Stocks[[p]]$cpars_bio, Fleets[[p]]$cpars_fleet)
    })
  })

  # Combine all fisheries into a single fleet
  MOM@CatchFrac <- MOM@Allocation <- lapply(1:np, function(...) matrix(1, SOM@nsim, nf))

  # Ignore Allocation, nf = 1
  # Ignore Efactor, nf = 1
  # Ignore Complexes for now

  # Specify smolt natural production from NOS population
  pindex <- .make_stock_index(ns, n_g, n_r, np_s)
  SSBfrom <- matrix(0, np, np)
  for (s in 1:ns) {
    for (g in 1:n_g[s]) {
      # Juv NOS predicted from NOS escapement of the corresponding LHG. Note p_LHG must be full rank
      p_nat_juv <- pindex$p[pindex$s == s & pindex$g == g & pindex$origin == "natural" & pindex$stage == "juvenile"]
      p_nat_esc <- pindex$p[pindex$s == s & pindex$g == g & pindex$origin == "natural" & pindex$stage == "escapement"]
      SSBfrom[p_nat_juv, p_nat_esc] <- 1

      # Integrated hatchery: triggers SRRfun so that there is hatchery production only when there is natural escapement for all LHG
      if (do_hatchery[s]) {
        for (r in 1:n_r[s]) {
          p_hat_juv <- pindex$p[pindex$s == s & pindex$r == r & pindex$origin == "hatchery" & pindex$stage == "juvenile"]
          SSBfrom[p_hat_juv, p_nat_esc] <- 1
        }
      }
    }
  }

  # Move stocks around
  # First generate the escapement, then move juveniles to adult
  maxage_s <- vapply(SOM@Bio, slot, numeric(1), "maxage")
  nage <- 2 * maxage_s[1] + 1
  nyears_real <- 2
  nyears <- 2 * nyears_real
  proyears <- 2 * SOM@proyears

  Herm_mature <- lapply(1:ns, function(s) {

    # Maturation occurs for even age classes (age class 2, 4, 6) at the beginning of even time steps.
    prop_mature_NOS <- prop_escapement_NOS <- array(0, c(SOM@nsim, nage, nyears + proyears))
    prop_mature_NOS[, 2 * seq(1, maxage_s[s]), nyears + seq(2, proyears, 2)] <- SOM@Bio[[s]]@p_mature

    Herm_mature_NOS <- sapply(1:(nyears + proyears), function(y) {
      sapply(1:SOM@nsim, function(x) solve_Herm(prop_mature_NOS[x, , y])) # Converts from proportion mature to cumulative maturity over brood year
    }, simplify = "array") %>%
      aperm(c(2, 1, 3))

    # In reality, escapement occurs for even age classes (age class 2, 4, 6) at the end of even time steps.
    # In openMSE, we do escapement for odd age classes at the beginning of the subsequent odd time steps
    first_mature_age <- sapply(seq(1, SOM@proyears), function(y) {
      sapply(1:SOM@nsim, function(x) which(SOM@Bio[[s]]@p_mature[x, , y] > 0)[1])
    })

    prop_escapement_NOS[, 2 * seq(1, maxage_s[s]) + 1, nyears + seq(1, proyears, 2)] <- sapply(1:SOM@nsim, function(x) {
      sapply(1:SOM@proyears, function(y) ifelse(1:maxage_s[s] >= first_mature_age[x, y], 1, 0))
    }, simplify = "array") %>%
      aperm(c(3, 1, 2))

    Herm_escapement_NOS <- sapply(1:(nyears + proyears), function(y) {
      sapply(1:SOM@nsim, function(x) solve_Herm(prop_escapement_NOS[x, , y]))
    }, simplify = "array") %>%
      aperm(c(2, 1, 3))

    # For NOS
    Herm <- list()
    Herm_names <- character(0)
    for (g in 1:n_g[s]) {
      p_nat_esc <- pindex$p[pindex$s == s & pindex$g == g & pindex$origin == "natural" & pindex$stage == "escapement"]
      p_nat_rec <- pindex$p[pindex$s == s & pindex$g == g & pindex$origin == "natural" & pindex$stage == "recruitment"]
      p_nat_juv <- pindex$p[pindex$s == s & pindex$g == g & pindex$origin == "natural" & pindex$stage == "juvenile"]

      Herm <- c(Herm, list(Herm_escapement_NOS, Herm_mature_NOS))
      Herm_names <- c(Herm_names, paste0("H_", p_nat_esc, "_", p_nat_rec), paste0("H_", p_nat_rec, "_", p_nat_juv))
    }

    # HOS
    if (do_hatchery[s] || has_strays[s]) {
      for (r in 1:n_r[s]) {

        # Maturation occurs for even age classes (age class 2, 4, 6) at the beginning of even time steps.
        prop_mature_HOS <- prop_escapement_HOS <- array(0, c(SOM@nsim, nage, nyears + proyears))
        prop_mature_HOS[, 2 * seq(1, maxage_s[s]), nyears + seq(2, proyears, 2)] <- SOM@Hatchery[[s]]@p_mature_HOS[, , , r]

        Herm_mature_HOS <- sapply(1:(nyears + proyears), function(y) {
          sapply(1:SOM@nsim, function(x) solve_Herm(prop_mature_HOS[x, , y]))
        }, simplify = "array") %>%
          aperm(c(2, 1, 3))

        # In reality, escapement occurs for even age classes (age class 2, 4, 6) at the end of even time steps.
        # In openMSE, we do escapement for odd age classes at the beginning of the subsequent odd time steps
        first_mature_age <- sapply(seq(1, SOM@proyears), function(y) {
          sapply(1:SOM@nsim, function(x) which(SOM@Hatchery[[s]]@p_mature_HOS[x, , y, r] > 0)[1])
        })

        prop_escapement_HOS[, 2 * seq(1, maxage_s[s]) + 1, nyears + seq(1, proyears, 2)] <- sapply(1:SOM@nsim, function(x) {
          sapply(1:SOM@proyears, function(y) ifelse(1:maxage_s[s] >= first_mature_age[x, y], 1, 0))
        }, simplify = "array") %>%
          aperm(c(3, 1, 2))

        Herm_escapement_HOS <- sapply(1:(nyears + proyears), function(y) {
          sapply(1:SOM@nsim, function(x) solve_Herm(prop_escapement_HOS[x, , y]))
        }, simplify = "array") %>%
          aperm(c(2, 1, 3))

        p_hat_esc <- pindex$p[pindex$s == s & pindex$r == r & pindex$origin == "hatchery" & pindex$stage == "escapement"]
        p_hat_rec <- pindex$p[pindex$s == s & pindex$r == r & pindex$origin == "hatchery" & pindex$stage == "recruitment"]
        p_hat_juv <- pindex$p[pindex$s == s & pindex$r == r & pindex$origin == "hatchery" & pindex$stage == "juvenile"]

        Herm <- c(Herm, list(Herm_escapement_HOS, Herm_mature_HOS))
        Herm_names <- c(Herm_names, paste0("H_", p_hat_esc, "_", p_hat_rec), paste0("H_", p_hat_rec, "_", p_hat_juv))
      }
    }

    structure(Herm, names = Herm_names)
  })
  Herm_mature <- do.call(c, Herm_mature)

  MOM@SexPars <- list(SSBfrom = SSBfrom, Herm = Herm_mature, share_par = FALSE)

  # Rel
  # The predicted smolt production (juvenile NOS) is from the adult escapement (natural and hatchery)
  # Combines hatchery and habitat dynamics
  # Hatchery: Presence/absence of HOS (from strays or own system)
  # Habitat: Update Perr_y of juvenile NOS from adult escapement from stage specific relationships
  Rel_s <- lapply(1:ns, function(s) {
    Bio <- SOM@Bio[[s]]
    Habitat <- SOM@Habitat[[s]]
    Hatchery <- SOM@Hatchery[[s]]
    S <- Stocks_s[[s]]
    SRRpars <- S[[1]]$cpars_bio$SRR$SRRpars # Assume SRR pars are identical for all LHG

    p_LHG <- Bio@p_LHG

    pind <- pindex[pindex$s == s, ]

    # Not needed if all of the following are true: no hatchery, no strays, s_enroute = 1, n_g = 1
    # SRR pars in the OM should suffice in that case (the escapement is the spawning output)
    do_hatchery_s <- do_hatchery[s]
    has_strays_s <- has_strays[s]
    g_s <- n_g[s]
    r_s <- n_r[s]

    use_smolt_func <- TRUE

    Rel_smolt_g <- list()
    Rel_hatchrel <- list()
    Rel_SAR_NOS_g <- list()
    Rel_SAR_HOS <- list()

    if (use_smolt_func) {

      if (do_hatchery_s) {
        # Determine the total number of eggs needed from the number of yearling and subyearling releases, their survival from egg stage
        # and fecundity of broodtake (identical between natural and hatchery escapement)
        # This is a management action, cannot be stochastic
        # No imports
        egg_yearling <- ifelse(sum(Hatchery@n_yearling) > 0, sum(Hatchery@n_yearling)/Hatchery@s_egg_smolt, 0)
        egg_subyearling <- ifelse(sum(Hatchery@n_subyearling) > 0, sum(Hatchery@n_subyearling)/Hatchery@s_egg_subyearling, 0)
        egg_target <- egg_yearling + egg_subyearling

        p_yearling <- Hatchery@n_yearling/sum(Hatchery@n_yearling, Hatchery@n_subyearling) # Vector by release strategy
        p_subyearling <- Hatchery@n_subyearling/sum(Hatchery@n_yearling, Hatchery@n_subyearling) # Vector by release strategy
      } else {
        egg_target <- p_yearling <- p_subyearling <- 0
      }

      fitness_args <- list()
      hatchery_args <- list(
        egg_target = egg_target,
        premove_NOS = Hatchery@premove_NOS
      )
      habitat_args <- list(
        use_habitat = Habitat@use_habitat
      )
      stray_args <- list(
        stray_external = Hatchery@stray_external
      )

      donate_stray <- SOM@stray[s, s] < 1
      receive_stray <- any(SOM@stray[-s, s] > 0)

      if (donate_stray || receive_stray) {
        if (any(n_r > 1)) stop("Straying matrix not yet configured for multiple release strategies.")
        stray_args$stray_matrix <- SOM@stray
        stray_args$p_donor <- pindex[pindex$stage == "escapement" & pindex$origin == "hatchery", c("s", "p")]
      }

      if (habitat_args$use_habitat) {
        habitat_args$Habitat <- Habitat
        habitat_args$nyears_om <- 2 * nyears_real
      }

      if (do_hatchery_s || has_strays_s) {

        hatchery_args <- list(
          ptarget_NOB = Hatchery@ptarget_NOB,
          pmax_esc = Hatchery@pmax_esc,
          pmax_NOB = Hatchery@pmax_NOB,
          fec_brood = Hatchery@fec_brood,
          egg_target = egg_target,
          brood_import = Hatchery@brood_import,
          p_female = Bio@p_female,
          s_yearling = Hatchery@s_egg_smolt,
          s_subyearling = Hatchery@s_egg_subyearling,
          p_yearling = p_yearling,
          p_subyearling = p_subyearling,
          yearling_DD = Hatchery@yearling_DD,
          subyearling_DD = Hatchery@subyearling_DD,
          phatchery = Hatchery@phatchery,
          premove_HOS = Hatchery@premove_HOS,
          premove_NOS = Hatchery@premove_NOS,
          s_prespawn = Hatchery@s_prespawn,
          gamma = Hatchery@gamma,
          m = Hatchery@m,
          f_brood = Hatchery@f_brood
        )

        if (any(Hatchery@fitness_type == "Ford") && (do_hatchery_s || has_strays_s)) {
          fitness_args <- list(
            fitness_type = Hatchery@fitness_type,
            rel_loss = Hatchery@rel_loss,
            phenotype_variance = Hatchery@phenotype_variance,
            fitness_variance = Hatchery@fitness_variance,
            fitness_floor = Hatchery@fitness_floor,
            heritability = Hatchery@heritability,
            #zbar_start = Hatchery@zbar_start # Now assigned by salmonMSE::salmonMSE()
            theta = Hatchery@theta
          )
        }
      }

      p_nat_esc <- pind$p[pind$origin == "natural" & pind$stage == "escapement"] # length ng
      p_hat_esc <- if(do_hatchery_s || has_strays_s) {
        pind$p[pind$origin == "hatchery" & pind$stage == "escapement"]
      } else {
        NULL
      }

      for (g in seq_len(g_s)) {
        # Natural smolt production from NOS and HOS escapement (including strays), habitat, and/or en-route mortality
        p_nat_smolt <- pind$p[pind$origin == "natural" & pind$g == g & pind$stage == "juvenile"]

        Rel_smolt_g[[g]] <- makeRel_smolt(
          p_smolt = p_nat_smolt, s = s, p_natural = p_nat_esc, p_hatchery = p_hat_esc,
          p_stray = setdiff(stray_args$p_donor[, "p"], p_hat_esc),
          output = "natural", maxage = maxage_s[s],
          s_enroute = Bio@s_enroute, p_female = Bio@p_female, fec = Bio@fec,
          SRRpars = SRRpars,
          hatchery_args = hatchery_args, fitness_args = fitness_args, habitat_args = habitat_args,
          stray_args = stray_args,
          g = g, prop_LHG = p_LHG
        )

        # Marine survival of natural origin fish
        if (do_hatchery_s && Hatchery@fitness_type[1] != "none" && Hatchery@rel_loss[3] > 0) {
          Rel_SAR_NOS_g[[g]] <- makeRel_SAR(
            p_smolt = p_nat_smolt, s = s, envir = "natural",
            rel_loss = Hatchery@rel_loss[3], nyears = 2 * nyears_real,
            Mbase = S[[p_nat_smolt]]$cpars_bio$M_ageArray[, , nyears + seq(1, 2 * SOM@proyears)]
          )
        }
      }

      if (do_hatchery_s) {
        for (r in seq_len(r_s)) {
          p_hat_smolt <- pind$p[pind$origin == "hatchery" & pind$r == r & pind$stage == "juvenile"]

          # Hatchery smolt releases from NOS and HOS escapement
          Rel_hatchrel[[r]] <- makeRel_smolt(
            p_smolt = p_hat_smolt, s = s, p_natural = p_nat_esc, p_hatchery = p_hat_esc,
            p_stray = setdiff(stray_args$p_donor[, "p"], p_hat_esc),
            output = "hatchery", maxage = maxage_s[s],
            s_enroute = Bio@s_enroute, p_female = Bio@p_female, fec = Bio@fec,
            SRRpars = SRRpars,
            hatchery_args = hatchery_args, fitness_args = fitness_args, habitat_args = habitat_args,
            stray_args = stray_args,
            r = r
          )

          # Marine survival of hatchery origin fish
          if (Hatchery@fitness_type[2] != "none" && Hatchery@rel_loss[3] > 0) {
            Rel_SAR_HOS[[1]] <- makeRel_SAR(
              p_smolt = p_hat_smolt, s = s, envir = "hatchery",
              rel_loss = Hatchery@rel_loss[3], nyears = 2 * nyears_real,
              Mbase = S[[p_hat_smolt]]$cpars_bio$M_ageArray[, , 2 * nyears + seq(1, 2 * SOM@proyears)]
            )
          }
        }

      }
    }
    Rel <- c(Rel_smolt_g, Rel_SAR_NOS_g, Rel_SAR_HOS, Rel_hatchrel)

    return(Rel)
  })

  MOM@Rel <- do.call(c, Rel_s)
  MOM@cpars$control <- list(HistRel = FALSE, HermEq = FALSE)

  return(MOM)
}

NAor0 <- function(x) !length(x) || is.na(x) || !x

#' Check inputs to SOM object
#'
#' Ensures that the slots in the [salmonMSE::SOM-class] object have the correct dimensions. Function will
#' update some slots to their full dimensions.
#'
#' @param SOM [salmonMSE::SOM-class] object
#' @param silent Logical, whether to report progress in console
#' @returns Updated [salmonMSE::SOM-class] object with full dimensions in various slots
#' @examples
#' SOM_checked <- check_SOM(simple_SOM, silent = TRUE)
#'
#' @export
check_SOM <- function(SOM, silent = FALSE) {

  ### Base slots ----
  if (!length(SOM@nsim)) stop("Need SOM@nsim")
  #if (!length(SOM@nyears)) stop("Need SOM@nyears")
  if (!length(SOM@proyears)) stop("Need SOM@proyears")
  if (!length(SOM@seed)) stop("Need SOM@seed")
  nsim <- SOM@nsim
  if (nsim < 2) stop("Need SOM@nsim > 1")
  proyears <- SOM@proyears

  # Convert sub class objects to lists
  obj <- c("Bio", "Habitat", "Hatchery", "Harvest", "Historical")
  for (i in obj) {
    if (inherits(slot(SOM, i), i)) slot(SOM, i) <- list(slot(SOM, i))
  }

  # Loop over populations
  ns <- length(SOM@Bio)

  # Check straying
  if (!length(SOM@stray)) SOM@stray <- diag(ns)

  for (s in 1:ns) {
    if (!silent && ns > 1) message("Checking parameters for population ", s)

    ### Check Bio ----
    Bio <- SOM@Bio[[s]]

    Bio <- check_numeric(Bio, "maxage")
    maxage <- Bio@maxage

    Bio <- check_numeric(Bio, "n_g", default = 1)
    Bio <- check_numeric(Bio, "p_LHG", size = Bio@n_g, default = rep(1/Bio@n_g, Bio@n_g))

    Bio <- check_maxage2array(Bio, "p_mature", maxage, nsim, proyears)
    if (length(Bio@p_female) == 1) Bio@p_female <- rep(Bio@p_female, maxage)
    Bio <- check_numeric(Bio, "p_female", size = maxage, default = rep(0.5, maxage))
    Bio <- check_numeric(Bio, "s_enroute", default = 1)
    Bio <- check_maxage2array(Bio, "fec", maxage, nsim, proyears)

    ### Check Habitat (initial) ----
    Habitat <- SOM@Habitat[[s]]
    Habitat <- check_numeric(Habitat, "use_habitat", default = FALSE)

    if (length(dim(Bio@Mjuv_NOS)) == 3 && Bio@n_g == 1) {
      Bio <- check_maxage2array(Bio, "Mjuv_NOS", maxage-1, nsim, proyears)
      Bio@Mjuv_NOS <- array(Bio@Mjuv_NOS, c(dim(Bio@Mjuv_NOS), Bio@n_g))
    }
    Bio <- check_maxage2garray(Bio, "Mjuv_NOS", maxage-1, nsim, proyears, Bio@n_g)

    if (!Habitat@use_habitat) {
      # SRR pars
      slot(Bio, "SRrel") <- match.arg(slot(Bio, "SRrel"), choices = c("BH", "Ricker"))
      Bio <- check_numeric2nsim(Bio, "kappa", nsim)

      if (Bio@SRrel == "BH") {
        Bio <- check_numeric2nsim(Bio, "capacity", nsim)
      } else {
        Bio <- check_numeric2nsim(Bio, "Smax", nsim)
      }

      if (!length(Bio@phi) && length(Bio@tau)) {
        stop("Bio@tau is specified but not Bio@phi. Should specify both or none.")
      }
      if (length(Bio@phi) && !length(Bio@tau)) {
        stop("Bio@phi is specified but not Bio@tau. Should specify both or none.")
      }

      if (!length(Bio@phi)) {
        Bio@phi <- sapply(1:SOM@nsim, function(x) {
          calc_phi(
            Mjuv = matrix(Bio@Mjuv_NOS[x, , 1, ], Bio@maxage-1, Bio@n_g),
            p_mature = matrix(Bio@p_mature[x, , 1], Bio@maxage, Bio@n_g),
            p_female = Bio@p_female,
            fec = matrix(Bio@fec[x, , 1], Bio@maxage, Bio@n_g),
            s_enroute = Bio@s_enroute,
            n_g = Bio@n_g,
            p_LHG = Bio@p_LHG
          )
        })
      }
      if (!length(Bio@tau)) {
        Bio@tau <- sapply(1:SOM@nsim, function(x) {
          calc_phi(
            Mjuv = matrix(Bio@Mjuv_NOS[x, , 1, ], Bio@maxage-1, Bio@n_g),
            p_mature = matrix(Bio@p_mature[x, , 1], Bio@maxage, Bio@n_g),
            p_female = Bio@p_female,
            #fec = matrix(Bio@fec[x, , 1], Bio@maxage, Bio@n_g),
            s_enroute = Bio@s_enroute,
            n_g = Bio@n_g,
            p_LHG = Bio@p_LHG,
            output = "spawner"
          )
        })
      }
      Bio <- check_numeric2nsim(Bio, "phi", nsim)
      Bio <- check_numeric2nsim(Bio, "tau", nsim)
    } else {
      Habitat <- check_numeric(Habitat, "prespawn_rel", default = "BH")
      Habitat <- check_numeric(Habitat, "prespawn_prod", default = 1)
      Habitat <- check_numeric(Habitat, "prespawn_capacity", default = Inf)

      Habitat <- check_numeric(Habitat, "egg_rel", default = "BH")
      Habitat <- check_numeric(Habitat, "egg_prod", default = 1)
      Habitat <- check_numeric(Habitat, "egg_capacity", default = Inf)

      Habitat <- check_numeric(Habitat, "fry_rel", default = "BH")
      Habitat <- check_numeric(Habitat, "fry_prod", default = 0.4)
      Habitat <- check_numeric(Habitat, "fry_capacity", default = Inf)
      if (!length(Habitat@fry_sdev)) {
        Habitat@fry_sdev <- matrix(1, nsim, proyears)
      }
      Habitat <- check_maxage2matrix(Habitat, "fry_sdev", proyears, nsim)

      Habitat <- check_numeric(Habitat, "smolt_rel", default = "BH")
      Habitat <- check_numeric(Habitat, "smolt_prod", default = 1)
      Habitat <- check_numeric(Habitat, "smolt_capacity", default = Inf)
      if (!length(Habitat@smolt_sdev)) {
        Habitat@smolt_sdev <- matrix(1, nsim, proyears)
      }
      Habitat <- check_maxage2matrix(Habitat, "smolt_sdev", proyears, nsim)
    }

    ### Check Hatchery ----
    Hatchery <- SOM@Hatchery[[s]]

    Hatchery <- check_numeric(Hatchery, "n_r", default = 1)
    Hatchery <- check_numeric(Hatchery, "n_yearling", size = Hatchery@n_r, default = rep(0, Hatchery@n_r))
    Hatchery <- check_numeric(Hatchery, "n_subyearling", size = Hatchery@n_r, default = rep(0, Hatchery@n_r))
    if (!length(Hatchery@yearling_DD)) Hatchery@yearling_DD <- FALSE
    if (!length(Hatchery@subyearling_DD)) Hatchery@subyearling_DD <- FALSE

    do_hatchery <- sum(Hatchery@n_yearling, Hatchery@n_subyearling) > 0

    if (!length(Hatchery@stray_external)) Hatchery@stray_external <- matrix(0, maxage, Hatchery@n_r)
    Hatchery <- check_maxage2matrix(Hatchery, "stray_external", Hatchery@n_r, maxage)

    has_strays <- any(SOM@stray[-s, s] > 0) || sum(Hatchery@stray_external)

    if (do_hatchery || has_strays) {
      Hatchery <- check_numeric(Hatchery, "s_prespawn", default = 1)
      Hatchery <- check_numeric(Hatchery, "s_egg_smolt", default = 1)
      Hatchery <- check_numeric(Hatchery, "s_egg_subyearling", default = 1)

      if (length(dim(Hatchery@Mjuv_HOS)) == 3 && Hatchery@n_r == 1) {
        Hatchery <- check_maxage2array(Hatchery, "Mjuv_HOS", maxage-1, nsim, proyears)
        Hatchery@Mjuv_HOS <- array(Hatchery@Mjuv_HOS, c(dim(Hatchery@Mjuv_HOS), 1))
      }
      Hatchery <- check_maxage2garray(Hatchery, "Mjuv_HOS", maxage-1, nsim, proyears, Hatchery@n_r)

      if (!length(Hatchery@p_mature_HOS)) {
        Hatchery@p_mature_HOS <- array(Bio@p_mature, c(nsim, maxage, proyears, Hatchery@n_r))
      }
      if (length(dim(Hatchery@p_mature_HOS)) == 3 && Hatchery@n_r == 1) {
        Hatchery <- check_maxage2array(Hatchery, "p_mature_HOS", maxage, nsim, proyears)
        Hatchery@p_mature_HOS <- array(Hatchery@p_mature_HOS, c(nsim, maxage, proyears, 1))
      }
      Hatchery <- check_maxage2garray(Hatchery, "p_mature_HOS", maxage, nsim, proyears, Hatchery@n_r)
      Hatchery <- check_numeric(Hatchery, "gamma", default = 1)

      Hatchery <- check_numeric(Hatchery, "m", default = 0)

      Hatchery <- check_numeric(Hatchery, "brood_import", size = maxage, default = rep(0, maxage))
      Hatchery <- check_numeric(Hatchery, "pmax_esc", default = 0.75)
      Hatchery <- check_numeric(Hatchery, "pmax_NOB", default = 1)
      Hatchery <- check_numeric(Hatchery, "ptarget_NOB", default = 0.9)

      if (!length(Hatchery@fec_brood)) Hatchery@fec_brood <- Bio@fec
      Hatchery <- check_maxage2array(Hatchery, "fec_brood", maxage, nsim, proyears)

      Hatchery <- check_numeric(Hatchery, "phatchery", default = NA)

      if (!is.function(Hatchery@premove_HOS) && is.numeric(Hatchery@premove_HOS)) {
        Hatchery <- check_numeric(Hatchery, "premove_HOS", default = 0)
      }
      if (!is.function(Hatchery@premove_NOS) && is.numeric(Hatchery@premove_NOS)) {
        Hatchery <- check_numeric(Hatchery, "premove_NOS", default = 0)
      }

      Hatchery <- check_numeric(Hatchery, "fitness_type", size = 2)
      if (any(Hatchery@fitness_type == "Ford")) {
        Hatchery <- check_numeric(Hatchery, "theta", size = 2)
        Hatchery <- check_numeric(Hatchery, "rel_loss", size = 3)

        if (!is.array(Hatchery@zbar_start)) {
          Hatchery <- check_numeric(Hatchery, "zbar_start", size = 2)
          Hatchery@zbar_start <- array(Hatchery@zbar_start, c(2, nsim, maxage)) %>%
            aperm(c(2, 3, 1))
        } else {
          Hatchery <- check_array(Hatchery, "zbar_start", c(nsim, maxage, 2))
        }

        Hatchery <- check_numeric(Hatchery, "fitness_variance", default = 100)
        Hatchery <- check_numeric(Hatchery, "phenotype_variance", default = 10)

        if (length(Hatchery@heritability) == 1) {
          Hatchery@heritability <- rep(Hatchery@heritability, nsim)
        }
        Hatchery <- check_numeric(Hatchery, "heritability", size = nsim, default = rep(0.5, nsim))
        Hatchery <- check_numeric(Hatchery, "fitness_floor", default = 0.5)
      }
    } else {
      Hatchery <- check_numeric(Hatchery, "m", default = 0)
      Hatchery <- check_numeric(Hatchery, "premove_HOS", default = 0)
      Hatchery <- check_numeric(Hatchery, "premove_NOS", default = 0)
      Hatchery <- check_numeric(Hatchery, "gamma", default = 1)
    }

    ### Check for PNI_wild calculation ----
    if (!do_hatchery && sum(Hatchery@stray_external)) {
      if (!length(Hatchery@heritability)) {
        stop("Need hatchery heritability parameter for PNI calculation (no hatchery, presence of external strays)")
      }
      if (!length(Hatchery@phenotype_variance)) {
        stop("Need hatchery phenotype_variance parameter for PNI calculation (no hatchery, presence of external strays)")
      }
      if (!length(Hatchery@fitness_variance)) {
        stop("Need hatchery fitness_variance parameter for PNI calculation (no hatchery, presence of external strays)")
      }
    }

    ### Check Harvest ----
    Harvest <- SOM@Harvest[[s]]

    Harvest <- check_numeric(Harvest, "type_PT", default = "u")
    Harvest <- check_numeric(Harvest, "type_T", default = "u")

    Harvest@type_PT <- match.arg(Harvest@type_PT, choices = c("u", "catch"))
    Harvest@type_T <- match.arg(Harvest@type_T, choices = c("u", "catch"))

    if (Harvest@type_PT == "u") {
      Harvest <- check_numeric(Harvest, "u_preterminal", default = 0)
      Harvest <- check_numeric(Harvest, "K_PT", default = NA_real_)
    } else {
      Harvest <- check_numeric(Harvest, "u_preterminal", default = NA_real_)
      Harvest <- check_numeric(Harvest, "K_PT", default = 0)
    }

    if (Harvest@type_T == "u") {
      Harvest <- check_numeric(Harvest, "u_terminal", default = 0)
      Harvest <- check_numeric(Harvest, "K_T", default = NA_real_)
    } else {
      Harvest <- check_numeric(Harvest, "u_terminal", default = NA_real_)
      Harvest <- check_numeric(Harvest, "K_T", default = 0)
    }

    Harvest <- check_numeric(Harvest, "MSF_PT", default = FALSE)
    Harvest <- check_numeric(Harvest, "MSF_T", default = FALSE)

    Harvest <- check_numeric(Harvest, "release_mort", size = 2, default = c(0, 0))

    if (!length(Harvest@vulPT)) {
      if (NAor0(Harvest@u_preterminal > 0) || NAor0(Harvest@K_PT)) {
        Harvest@vulPT <- matrix(1, nsim, maxage)
      }
    }
    if (!length(Harvest@vulT)) {
      if (NAor0(Harvest@u_terminal > 0) || NAor0(Harvest@K_T)) {
        Harvest@vulT <- matrix(1, nsim, maxage)
      }
    }

    Harvest <- check_maxage2matrix(Harvest, "vulPT", maxage, nsim)
    Harvest <- check_maxage2matrix(Harvest, "vulT", maxage, nsim)

    ### Check Historical (initial) ----
    Historical <- SOM@Historical[[s]]

    Njuv_NOS <- Njuv_HOS <- NA_real_

    if (!length(Historical@InitNjuv_NOS)) {
      Njuv_NOS <- 1000
    } else if (length(Historical@InitNjuv_NOS) == 1) {
      Njuv_NOS <- Historical@InitNjuv_NOS
    }

    if (!length(Historical@InitNjuv_NOS)) {
      Njuv_HOS <- 1000
    } else if (length(Historical@InitNjuv_HOS) == 1) {
      Njuv_HOS <- Historical@InitNjuv_HOS
    }

    if (!is.array(Historical@InitNjuv_NOS)) {
      Historical@InitNjuv_NOS <- array(0, c(nsim, maxage, Bio@n_g))
      Historical@InitNjuv_NOS[, maxage, ] <- Njuv_NOS/Bio@n_g
    }
    if (!is.array(Historical@InitNjuv_HOS)) {
      Historical@InitNjuv_HOS <- array(0, c(nsim, maxage, Hatchery@n_r))
      Historical@InitNjuv_HOS[, maxage, ] <- Njuv_HOS/Hatchery@n_r
    }
    Historical <- check_array(Historical, "InitNjuv_NOS", dims = c(nsim, maxage, Bio@n_g))
    Historical <- check_array(Historical, "InitNjuv_HOS", dims = c(nsim, maxage, Hatchery@n_r))

    SOM@Bio[[s]] <- Bio
    SOM@Habitat[[s]] <- Habitat
    SOM@Hatchery[[s]] <- Hatchery
    SOM@Harvest[[s]] <- Harvest
    SOM@Historical[[s]] <- Historical
  }

  # Check all maxage is identical
  maxage_s <- unique(vapply(SOM@Bio, slot, numeric(1), "maxage"))
  if (length(maxage_s) > 1) stop("Max age should be identical for all populations")

  return(SOM)
}

check_numeric <- function(object, name, size = 1, default) {
  object_name <- as.character(substitute(object))

  if (!length(slot(object, name))) {
    if (!missing(default)) {
      slot(object, name) <- default
    } else {
      stop(paste0("Need ", object_name, "@", name, " (numeric)"))
    }
  } else if (length(slot(object, name)) != size) {
    stop(paste0("Length of ", object_name, "@", name, " needs to be ", size))
  }

  return(invisible(object))
}

check_maxage <- function(object, name, maxage) {
  object_name <- as.character(substitute(object))

  if (length(slot(object, name)) != maxage) {
    stop(paste0("Need ", object_name, "@", name, " (maxage vector)"))
  }
  return(invisible(object))
}

check_maxage2matrix <- function(object, name, maxage, nsim) {
  object_name <- as.character(substitute(object))

  if (!is.array(slot(object, name))) {
    if (length(slot(object, name)) != maxage) {
      stop(paste0(object_name, "@", name, " needs to be length maxage (", maxage, ")"))
    }
    slot(object, name) <- matrix(slot(object, name), nsim, maxage, byrow = TRUE)
  }

  dim_i <- dim(slot(object, name))
  dim_check <- length(dim_i) == 2 && all(dim(slot(object, name)) == c(nsim, maxage))
  if (!dim_check) {
    stop(
      paste0(object_name, "@", name, " must be a matrix with dimension ",
             paste(c(nsim, maxage), collapse = ", "))
    )
  }
  return(invisible(object))
}

check_maxage2array <- function(object, name, maxage, nsim, years) {
  object_name <- as.character(substitute(object))

  if (!is.array(slot(object, name))) {
    if (length(slot(object, name)) != maxage) {
      stop(paste0(object_name, "@", name, " needs to be length maxage (", maxage, ")"))
    }
    slot(object, name) <- array(slot(object, name), c(maxage, nsim, years)) %>%
      aperm(c(2, 1, 3))
  }

  dim_i <- dim(slot(object, name))
  dim_check <- length(dim_i) == 3 && all(dim(slot(object, name)) == c(nsim, maxage, years))
  if (!dim_check) {
    stop(
      paste0(object_name, "@", name, " must be an array with dimension ",
             paste(c(nsim, maxage, years), collapse = ", "))
    )
  }
  return(invisible(object))
}

check_maxage2garray <- function(object, name, maxage, nsim, years, n_g) {
  object_name <- as.character(substitute(object))

  if (!is.array(slot(object, name))) {
    if (length(slot(object, name)) != maxage) {
      stop(paste0(object_name, "@", name, " needs to be length maxage (", maxage, ")"))
    }
    slot(object, name) <- array(slot(object, name), c(maxage, nsim, years, n_g)) %>%
      aperm(c(2, 1, 3, 4))
  }

  dim_i <- dim(slot(object, name))
  dim_check <- length(dim_i) == 4 && all(dim(slot(object, name)) == c(nsim, maxage, years, n_g))
  if (!dim_check) {
    stop(
      paste0(object_name, "@", name, " must be an array with dimension ",
             paste(c(nsim, maxage, years, n_g), collapse = ", "))
    )
  }
  return(invisible(object))
}

check_array <- function(object, name, dims) {
  object_name <- as.character(substitute(object))

  dim_i <- dim(slot(object, name))

  dim_check <- length(dim_i) == length(dims) && all(dim(slot(object, name)) == dims)
  if (!dim_check) {
    stop(
      paste0(object_name, "@", name, " must be an array with dimension ",
             paste(dims, collapse = ", "))
    )
  }
  return(invisible(object))
}

check_numeric2nsim <- function(object, name, nsim) {
  object_name <- as.character(substitute(object))

  if (!length(slot(object, name))) {
    stop(paste0("Need ", object_name, "@", name))
  }

  if (length(slot(object, name)) == 1) slot(object, name) <- rep(slot(object, name), nsim)
  if (length(slot(object, name)) != nsim) {
    stop(paste0(object_name, "@", name, " needs to length nsim (", nsim, ")"))
  }
  return(invisible(object))
}


solve_Herm <- function(h, p1 = 0) {
  p <- numeric(length(h))
  p[1] <- p1

  for (a in 2:length(p)) p[a] <- 1 - (1 - h[a])*(1 - p[a-1])
  #all(!MSEtool:::hrate(p) - h)
  return(p)
}

#' Run salmonMSE
#'
#' @description
#' `salmonMSE()` runs a salmon management strategy evaluation through the following steps:
#' - Converts a salmon operating model (\linkS4class{SOM}) to a multi-stock operating model ([MSEtool::MOM-class]) via [SOM2MOM()]
#' - Creates a harvest management procedure specifying the harvest control rule
#' - Generates the historical reconstruction of the state variables
#' - Runs projection (if `Hist = FALSE`)
#' - Converts the openMSE output, along with additional state variables recorded in [salmonMSE_env], into a salmon MSE object (SMSE) via [MMSE2SMSE()]
#' @param SOM An object of class \linkS4class{SOM}
#' @param Hist Logical, whether to stop the function stop after historical simulations?
#' @param silent Logical, whether to report progress in console
#' @param trace Logical, whether to report additional messages from openMSE
#' @param convert Logical, whether to convert the output into a salmon MSE (SHist or SMSE, depending on `Hist`) object
#' @return
#' If `Hist = TRUE`: if `convert = TRUE`, a \linkS4class{SHist} object or if `convert = FALSE`, a multiHist object (list).
#'
#' If `Hist = FALSE`: if `convert = TRUE`, a \linkS4class{SMSE} object or if `convert = FALSE`, a [MSEtool::MMSE-class] object.
#' @examples
#' \dontrun{
#' SMSE <- salmonMSE(simple_SOM)
#' }
#'
#' @export
salmonMSE <- function(SOM, Hist = FALSE, silent = FALSE, trace = FALSE, convert = TRUE) {

  if (!silent) message("Converting salmon operating model to MOM..")

  SOM <- check_SOM(SOM, silent = silent)
  MOM <- SOM2MOM(SOM, check = FALSE)

  HMMP <- make_Harvest_MMP(SOM, check = FALSE)

  salmonMSE_env$Ford <- data.frame()
  salmonMSE_env$N <- data.frame()
  salmonMSE_env$stateN <- data.frame()
  salmonMSE_env$H <- data.frame()
  salmonMSE_env$stateH <- data.frame()

  if (!silent) message("Generating historical dynamics..")

  H <- MSEtool::SimulateMOM(MOM, parallel = FALSE, silent = !trace)

  # Add numbers at age
  H <- initialize_population(H, SOM)

  if (Hist) {
    if (!silent) message("Returning historical simulations..")

    if (convert) {
      if (!silent) message("Converting to salmon Hist object..")
      SHist <- multiHist2SHist(H, SOM, check = FALSE)
      return(SHist)
    } else {
      return(H)
    }
  }

  if (!silent) message("Running forward projections..")

  # Initialize zbar in data frame
  salmonMSE_env$Ford <- initialize_zbar(SOM)

  M <- MSEtool::ProjectMOM(
    H, MPs = "HMMP", parallel = FALSE, silent = !trace, checkMPs = FALSE,
    dropHist = TRUE, extended = FALSE
  )
  M@multiHist <- H

  if (convert) {
    if (!silent) message("Converting to salmon MSE object..")

    calculate_last_projection_year(SOM, MOM, M)

    SMSE <- MMSE2SMSE(M, SOM, HMMP, N = salmonMSE_env$N, salmonMSE_env$stateN, Ford = salmonMSE_env$Ford, salmonMSE_env$H, salmonMSE_env$stateH)
    SMSE@Misc$SHist <- multiHist2SHist(H, SOM, check = FALSE)

    return(SMSE)

  } else {
    return(M)
  }

}

initialize_zbar <- function(SOM) {
  pindex <- make_stock_index(SOM)
  ns <- length(SOM@Bio)

  zbar_df <- lapply(1:ns, function(s) {
    do_hatchery <- sum(SOM@Hatchery[[s]]@n_subyearling, SOM@Hatchery[[s]]@n_yearling) > 0
    has_strays <- any(SOM@stray[-s, s] > 0) || sum(SOM@Hatchery[[s]]@stray_external)

    if ((has_strays || do_hatchery) && any(SOM@Hatchery[[s]]@fitness_type == "Ford")) {

      zbar_start <- reshape2::melt(SOM@Hatchery[[s]]@zbar_start)
      nyears_real <- 2
      df <- data.frame(
        x = zbar_start$Var1,
        s = s,
        t = 2 * (nyears_real + 1 - zbar_start$Var2),  # Even time steps relative to first projection year (remember MICE predicts Perr_y for next time step)
        type = ifelse(zbar_start$Var3 == 1, "natural", "hatchery"),
        zbar = zbar_start$value
      )

      fitness_type <- SOM@Hatchery[[s]]@fitness_type
      theta <- SOM@Hatchery[[s]]@theta

      fitness_variance <- SOM@Hatchery[[s]]@fitness_variance
      phenotype_variance <- SOM@Hatchery[[s]]@phenotype_variance

      fitness_floor <- SOM@Hatchery[[s]]@fitness_floor

      df$fitness <- sapply(1:nrow(df), function(i) {
        theta <- switch(df$type[i], "natural" = theta[1], "hatchery" = theta[2])
        fitness_fn <- switch(df$type[i], "natural" = fitness_type[1], "hatchery" = fitness_type[2])

        switch(
          fitness_fn,
          "Ford" = calc_fitness(df$zbar[i], theta, fitness_variance, phenotype_variance, fitness_floor),
          "none" = 1
        )
      })

      return(df)

    } else {
      return(data.frame())
    }
  })

  return(bind_rows(zbar_df))
}


initialize_population <- function(H, SOM) {
  pindex <- make_stock_index(SOM)
  ns <- length(SOM@Bio)
  nyears <- 4
  nareas <- 2

  for (s in 1:ns) {

    Bio <- SOM@Bio[[s]]
    Hatchery <- SOM@Hatchery[[s]]
    a_juv <- seq(1, 2 * (Bio@maxage-1), 2) + 1 # Age classes in second semester of last historical year, leading into first projection year

    do_hatchery <- sum(SOM@Hatchery[[s]]@n_yearling, SOM@Hatchery[[s]]@n_subyearling) > 0
    has_strays <- any(SOM@stray[-s, s] > 0) || sum(SOM@Hatchery[[s]]@stray_external)

    for (g in 1:Bio@n_g) {

      # Add juvenile population (for real age class 2, 3, ...)
      p_NOjuv <- pindex$p[pindex$s == s & pindex$g == g &
                            pindex$stage == "juvenile" &
                            pindex$origin == "natural"]
      H[[p_NOjuv]][[1]]@AtAge$Number[, a_juv, nyears, ] <-
        array(SOM@Historical[[s]]@InitNjuv_NOS[, -1, g]/nareas, c(SOM@nsim, Bio@maxage-1, nareas))

      # Add arbitrary 100 natural-origin spawners in first openMSE year which corresponds to last projection year in real salmon dynamics
      # Will not show up in the results
      p_NOesc <- pindex$p[pindex$s == s & pindex$g == g &
                            pindex$stage == "escapement" &
                            pindex$origin == "natural"]
      H[[p_NOesc]][[1]]@AtAge$Number[, 2 * Bio@maxage, nyears, ] <- array(100/nareas, c(SOM@nsim, 1, nareas))

      # Update Perr_y parameter to parameterize age 1 abundance in first openMSE projection year
      Perr_new <- sapply(1:SOM@nsim, function(x) {
        Egg_openMSE <- sum(100 * Bio@p_female[Bio@maxage] * Bio@fec[x, Bio@maxage, 1])
        SRRpars <- H[[p_NOjuv]][[1]]@SampPars$Stock$SRRpars[[x]]

        Pred_N1 <- H[[p_NOjuv]][[1]]@SampPars$Stock$SRRfun(Egg_openMSE, SRRpars)
        Perr <- SOM@Historical[[s]]@InitNjuv_NOS[x, 1, g]/Pred_N1
        return(Perr)
      })
      nyears <- 4 # Hard coded in
      H[[p_NOjuv]][[1]]@SampPars$Stock$Perr_y[, 2 * Bio@maxage + nyears + 1] <- Perr_new
    }

    if (do_hatchery || has_strays) {
      for (r in 1:Hatchery@n_r) {

        # Add juvenile population
        p_HOjuv <- pindex$p[pindex$s == s & pindex$r == r &
                              pindex$stage == "juvenile" &
                              pindex$origin == "hatchery"]
        H[[p_HOjuv]][[1]]@AtAge$Number[, a_juv, nyears, ] <-
          array(SOM@Historical[[s]]@InitNjuv_HOS[, -1, r]/nareas, c(SOM@nsim, Bio@maxage-1, nareas))

        # Add arbitrary 100 hatchery-origin spawners in first openMSE year which corresponds to last projection year in real salmon dynamics
        # Will not show up in the results
        p_HOesc <- pindex$p[pindex$s == s & pindex$r == r &
                              pindex$stage == "escapement" &
                              pindex$origin == "hatchery"]
        H[[p_HOesc]][[1]]@AtAge$Number[, 2 * Bio@maxage, nyears, ] <- array(100/nareas, c(SOM@nsim, 1, nareas))

        # Update Perr_y parameter to parameterize age 1 abundance in first openMSE projection year
        H[[p_HOjuv]][[1]]@SampPars$Stock$Perr_y[, 2 * Bio@maxage + nyears + 1] <- SOM@Historical[[s]]@InitNjuv_HOS[, 1, r]
      }
    }
  }

  return(H)
}



calculate_last_projection_year <- function(SOM, MOM, MMSE) {

  # Run this to get last escapement
  SMSE_prelim <- MMSE2SMSE(MMSE, SOM,
                           N = salmonMSE_env$N, stateN = salmonMSE_env$stateN, Ford = salmonMSE_env$Ford,
                           H = salmonMSE_env$H, stateH = salmonMSE_env$stateH)

  # Run additional calculations for spawners, brood, and egg production in last projection year
  pindex <- make_stock_index(SOM)

  Rel <- MOM@Rel
  ns <- length(SOM)

  for (i in 1:length(Rel)) {
    terms_i <- grepl("Nage_", Rel[[i]]$terms)
    if (any(terms_i)) {

      p <- Rel[[i]]$terms[grepl("Nage_", Rel[[i]]$terms)] %>%
        strsplit("_") %>%
        sapply(getElement, 2) %>%
        as.numeric()

      p_natural <- p[Rel[[i]]$natural_origin]
      s_natural <- unique(pindex$s[pindex$p %in% p_natural])
      if (length(p_natural) > 1) {
        p_NOR <- pindex$p[pindex$s %in% s_natural & pindex$stage == "recruitment" & pindex$origin == "natural"]
      }

      p_hatchery <- p[!Rel[[i]]$natural_origin]
      if (length(p_hatchery)) {
        s_hatchery <- unique(pindex$s[pindex$p %in% p_hatchery])
        if (length(p_hatchery) > 1) {
          p_HOR <- pindex$p[pindex$s %in% s_hatchery & pindex$stage == "recruitment" & pindex$origin == "hatchery"]
        }
      }

      p_stray <- p[Rel[[i]]$stray]
      if (length(p_stray)) {
        s_stray <- sapply(p_stray, function(p) pindex$s[pindex$p == p])
        p_stray_return <- sapply(p_stray, function(p) {
          s <- pindex$s[pindex$p == p]
          r <- pindex$r[pindex$p == p]
          pindex$p[pindex$s == s & pindex$r == r & pindex$stage == "recruitment" & pindex$origin == "hatchery"]
        })
      }

      maxage <- SOM@Bio[[1]]@maxage
      proyears <- MMSE@proyears

      run_func <- sapply(1:SOM@nsim, function(x) {
        if (length(p_natural) == 1) {
          Nage_NOS <- matrix(
            SMSE_prelim@Escapement_NOS[x, s_natural, , SOM@proyears],
            length(s_natural), maxage
          ) %>% t()
        } else {
          a <- seq(2, 2 * maxage, 2)
          Return_NOS <- apply(MMSE@N[x, p_NOR, a, 1, proyears, ], 1:2, sum)
          FT <- outer(MMSE@FM[x, p_NOR, 1, 1, proyears], SOM@Harvest[[s_natural]]@vulT[x, ])
          Nage_NOS <- matrix(Return_NOS * exp(-FT), length(p_natural), maxage) %>% t()
        }

        if (length(p_hatchery)) {
          if (length(p_hatchery) == 1) {
            Nage_HOS <- matrix(
              SMSE_prelim@Escapement_HOS[x, s_hatchery, , SOM@proyears],
              length(p_hatchery), maxage
            ) %>% t()
          } else {
            a <- seq(2, 2 * maxage, 2)
            Return_HOS <- apply(MMSE@N[x, p_HOR, a, 1, proyears, ], 1:2, sum)
            FT <- outer(MMSE@FM[x, p_HOR, 1, 1, proyears], SOM@Harvest[[s_hatchery]]@vulT[x, ])
            Nage_HOS <- matrix(Return_HOS * exp(-FT), length(p_hatchery), maxage) %>% t()
          }
        } else {
          Nage_HOS <- matrix(0, maxage, 1)
        }

        if (length(p_stray)) {
          a <- seq(2, 2 * maxage, 2)
          Return_stray <- apply(MMSE@N[x, p_stray_return, a, 1, proyears, , drop = FALSE], 2:3, sum)
          vulT <- sapply(SOM@Harvest[s_stray], function(i) i@vulT[x, ])
          FM <- MMSE@FM[x, p_stray_return, 1, 1, proyears]
          FT <- FM * t(vulT)
          Nage_stray <- matrix(Return_stray * exp(-FT), length(s_stray), maxage) %>% t()
        } else {
          Nage_stray <- array(0, dim(Nage_HOS))
        }

        Rel[[i]]$func(
          Nage_NOS = Nage_NOS, Nage_HOS = Nage_HOS, Nage_stray = Nage_stray,
          x = x, y = MMSE@nyears + MMSE@proyears
        )
      })
    }
  }
  invisible()
}

#' @rdname salmonMSE-int
#' @param multiHist Class multiHist object returned from MSEtool
#' @return
#' `multiHist2SHist`: \linkS4class{SHist} object
#' @export
multiHist2SHist <- function(multiHist, SOM, check = TRUE) {

  if (check) SOM <- check_SOM(SOM)

  ns <- length(SOM@Bio) # Number of stocks
  nage <- SOM@Bio[[1]]@maxage
  nyears_real <- 2

  # Declare arrays
  Njuv_NOS <- Njuv_HOS <- Escapement_NOS <- Escapement_HOS <- array(0, c(SOM@nsim, ns, nage, nyears_real))
  Return_NOS <- Return_HOS <- array(0, c(SOM@nsim, ns, nage, nyears_real))
  KPT_NOS <- KT_NOS <- KPT_HOS <- KT_HOS <- array(0, c(SOM@nsim, ns, nyears_real))
  DPT_NOS <- DT_NOS <- DPT_HOS <- DT_HOS <- array(0, c(SOM@nsim, ns, nyears_real))
  UPT_NOS <- UT_NOS <- UPT_HOS <- UT_HOS <- array(0, c(SOM@nsim, ns, nyears_real))
  ExPT_NOS <- ExT_NOS <- ExPT_HOS <- ExT_HOS <- array(0, c(SOM@nsim, ns, nyears_real))

  Egg_NOS <- Egg_HOS <- array(0, c(SOM@nsim, ns, nyears_real))
  Smolt <- Smolt_Rel <- array(0, c(SOM@nsim, ns, nyears_real))

  NOS <- HOS <- HOS_effective <- array(0, c(SOM@nsim, ns, nyears_real))

  # openMSE year and age indices
  t1 <- seq(1, 2 * nyears_real, 2)
  t2 <- seq(2, 2 * nyears_real, 2)

  a1 <- seq(1, 2 * nage + 1, 2)
  a2 <- seq(2, 2 * nage + 1, 2)

  a_imm <- a1[-length(a1)]
  a_return <- a2
  a_esc <- a1[-1]
  #age_escapement <- age_bio + 1 # MSEtool age class

  f <- 1 # Fleet

  pindex <- make_stock_index(SOM)

  for (s in 1:ns) {

    # NOS state variables from MMSE object
    p_NOS_imm <- pindex$p[pindex$s == s & pindex$origin == "natural" & pindex$stage == "juvenile"]
    p_NOS_return <- pindex$p[pindex$s == s & pindex$origin == "natural" & pindex$stage == "recruitment"] # MSEtool population index for returning NOS
    p_NOS_escapement <- pindex$p[pindex$s == s & pindex$origin == "natural" & pindex$stage == "escapement"] # NOS escapement

    y_spawnOM <- which(rowSums(multiHist[[p_NOS_escapement[1]]][[f]]@TSdata$SBiomass[1, , ]) > 0)

    # Reported spawning is delayed by one year
    t1_sp <- t1[-1]
    t2_sp <- t2[-1]
    y_spawn <- 0.5 * (y_spawnOM - 1)

    Njuv_NOS[, s, , ] <- sapply(p_NOS_imm, function(p) {
      apply(multiHist[[p]][[f]]@AtAge$Number[, a_imm, t1, ], 1:3, sum)
    }, simplify = "array") %>%
      apply(1:3, sum)

    Return_NOS[, s, , ] <- sapply(p_NOS_return, function(p) {
      apply(multiHist[[p]][[f]]@AtAge$Number[, a_return, t2, ], 1:3, sum)
    }, simplify = "array") %>%
      apply(1:3, sum)

    Escapement_NOS[, s, , 1:length(t1_sp)] <- sapply(p_NOS_escapement, function(p) {
      apply(multiHist[[p]][[f]]@AtAge$Number[, a_esc, t1_sp, , drop = FALSE], 1:3, sum)
    }, simplify = "array") %>%
      apply(1:3, sum)

    # Kept catch
    KPT_NOS[, s, ] <- sapply(p_NOS_imm, function(p) {
      apply(multiHist[[p]][[f]]@TSdata$Landings[, t1, ], 1:2, sum)
    }, simplify = "array") %>%
      apply(1:2, sum)

    KT_NOS[, s, ] <- sapply(p_NOS_return, function(p) {
      apply(multiHist[[p]][[f]]@TSdata$Landings[, t2, ], 1:2, sum)
    }, simplify = "array") %>%
      apply(1:2, sum)

    # Total discards (live + dead)
    DPT_NOS[, s, ] <- sapply(p_NOS_imm, function(p) {
      apply(multiHist[[p]][[f]]@TSdata$Discards[, t1, ], 1:2, sum)
    }, simplify = "array") %>%
      apply(1:2, sum)

    DT_NOS[, s, ] <- sapply(p_NOS_return, function(p) {
      apply(multiHist[[p]][[f]]@TSdata$Discards[, t2, ], 1:2, sum)
    }, simplify = "array") %>%
      apply(1:2, sum)

    # Harvest rate from kept catch
    vulPT <- array(SOM@Harvest[[s]]@vulPT, c(nage, SOM@nsim, length(t1))) %>% aperm(c(2, 1, 3))
    vulNOS_imm <- apply(vulPT * Njuv_NOS[, s, , ], c(1, 3), sum)
    UPT_NOS[, s, ] <- KPT_NOS[, s, ]/vulNOS_imm
    UPT_NOS[is.na(UPT_NOS)] <- 0

    vulT <- array(SOM@Harvest[[s]]@vulT, c(nage, SOM@nsim, length(t2))) %>% aperm(c(2, 1, 3))
    vulNOS_ret <- apply(vulT * Return_NOS[, s, , ], c(1, 3), sum)
    UT_NOS[, s, ] <- KT_NOS[, s, ]/vulNOS_ret
    UT_NOS[is.na(UT_NOS)] <- 0

    # Exploitation rate from kept + dead discards (DD)
    DDPT_NOS <- SOM@Harvest[[s]]@release_mort[1] * DPT_NOS[, s, ]
    ExPT_NOS[, s, ] <- (KPT_NOS[, s, ] + DDPT_NOS)/vulNOS_imm
    ExPT_NOS[is.na(ExPT_NOS)] <- 0

    DDT_NOS <- SOM@Harvest[[s]]@release_mort[2] * DT_NOS[, s, ]
    ExT_NOS[, s, ] <- (KT_NOS[, s, ] + DDT_NOS)/vulNOS_ret
    ExT_NOS[is.na(ExT_NOS)] <- 0

    Egg_NOS[, s, 1:length(t1_sp)] <- sapply(p_NOS_escapement, function(p) {
      apply(multiHist[[p]][[f]]@TSdata$SBiomass[, t1_sp, , drop = FALSE], 1:2, sum)
    }, simplify = "array") %>%
      apply(1:2, sum)

    Smolt[, s, ] <- sapply(p_NOS_imm, function(p) {
      apply(multiHist[[p]][[f]]@AtAge$Number[, 1, t1, ], 1:2, sum)
    }, simplify = "array") %>%
      apply(1:2, sum)

    do_hatchery <- sum(SOM@Hatchery[[s]]@n_subyearling, SOM@Hatchery[[s]]@n_yearling) > 0

    if (do_hatchery) {

      # HOS state variables from MMSE object
      p_HOS_imm <- pindex$p[pindex$s == s & pindex$origin == "hatchery" & pindex$stage == "juvenile"]
      p_HOS_return <- pindex$p[pindex$s == s & pindex$origin == "hatchery" & pindex$stage == "recruitment"] # MSEtool population index for returning HOS
      p_HOS_escapement <- pindex$p[pindex$s == s & pindex$origin == "hatchery" & pindex$stage == "escapement"] # HOS escapement

      Njuv_HOS[, s, , ] <- sapply(p_HOS_imm, function(p) {
        apply(multiHist[[p]][[f]]@AtAge$Number[, a_imm, t1, ], 1:3, sum)
      }, simplify = "array") %>%
        apply(1:3, sum)

      Return_HOS[, s, , ] <- sapply(p_HOS_return, function(p) {
        apply(multiHist[[p]][[f]]@AtAge$Number[, a_return, t2, ], 1:3, sum)
      }, simplify = "array") %>%
        apply(1:3, sum)

      Escapement_HOS[, s, , 1:length(t1_sp)] <- sapply(p_HOS_escapement, function(p) {
        apply(multiHist[[p]][[f]]@AtAge$Number[, a_esc, t1_sp, , drop = FALSE], 1:3, sum)
      }, simplify = "array") %>%
        apply(1:3, sum)

      # Kept catch
      KPT_HOS[, s, ] <- sapply(p_HOS_imm, function(p) {
        apply(multiHist[[p]][[f]]@TSdata$Landings[, t1, ], 1:2, sum)
      }, simplify = "array") %>%
        apply(1:2, sum)

      KT_HOS[, s, ] <- sapply(p_HOS_return, function(p) {
        apply(multiHist[[p]][[f]]@TSdata$Landings[, t2, ], 1:2, sum)
      }, simplify = "array") %>%
        apply(1:2, sum)

      # Total discards (live + dead)
      DPT_HOS[, s, ] <- sapply(p_HOS_imm, function(p) {
        apply(multiHist[[p]][[f]]@TSdata$Discards[, t1, ], 1:2, sum)
      }, simplify = "array") %>%
        apply(1:2, sum)

      DT_HOS[, s, ] <- sapply(p_HOS_return, function(p) {
        apply(multiHist[[p]][[f]]@TSdata$Discards[, t2, ], 1:2, sum)
      }, simplify = "array") %>%
        apply(1:2, sum)

      # Harvest rate from kept catch
      vulHOS_imm <- apply(vulPT * Njuv_HOS[, s, , ], c(1, 3), sum)
      UPT_HOS[, s, ] <- KPT_HOS[, s, ]/vulHOS_imm
      UPT_HOS[is.na(UPT_HOS)] <- 0

      vulHOS_ret <- apply(vulT * Return_HOS[, s, , ], c(1, 3), sum)
      UT_HOS[, s, ] <- KT_HOS[, s, ]/vulHOS_ret
      UT_HOS[is.na(UT_HOS)] <- 0

      # Exploitation rate from kept + dead discards (DD)
      DDPT_HOS <- SOM@Harvest[[s]]@release_mort[1] * DPT_HOS[, s, ]
      ExPT_HOS[, s, ] <- (KPT_HOS[, s, ] + DDPT_HOS)/vulHOS_imm
      ExPT_HOS[is.na(ExPT_HOS)] <- 0

      DDT_HOS <- SOM@Harvest[[s]]@release_mort[2] * DT_HOS[, s, ]
      ExT_HOS[, s, ] <- (KT_HOS[, s, ] + DDT_HOS)/vulHOS_ret
      ExT_HOS[is.na(ExT_HOS)] <- 0

      Egg_HOS[, s, 1:length(t1_sp)] <- sapply(p_HOS_escapement, function(p) {
        apply(multiHist[[p]][[f]]@TSdata$SBiomass[, t1_sp, , drop = FALSE], 1:2, sum)
      }, simplify = "array") %>%
        apply(1:2, sum)
      Smolt_Rel[, s, ] <- sapply(p_HOS_imm, function(p) {
        apply(multiHist[[p]][[f]]@AtAge$Number[, 1, t1, ], 1:2, sum)
      }, simplify = "array") %>%
        apply(1:2, sum)
    }

  }

  SHist <- new(
    "SHist",
    Name = SOM@Name,
    nyears = nyears_real,
    nsim = SOM@nsim,
    nstocks = ns,
    Snames = sapply(1:ns, function(s) if (length(SOM@Bio[[s]]@Name)) SOM@Bio[[s]]@Name else paste("Population", s)),
    Egg_NOS = Egg_NOS,
    Egg_HOS = Egg_HOS,
    Smolt = Smolt,
    Smolt_Rel = Smolt_Rel,
    Njuv_NOS = Njuv_NOS,
    Njuv_HOS = Njuv_HOS,
    Return_NOS = Return_NOS,
    Return_HOS = Return_HOS,
    Escapement_NOS = Escapement_NOS,
    Escapement_HOS = Escapement_HOS,
    NOS = NOS,
    HOS = HOS,
    HOS_effective = HOS_effective,
    KPT_NOS = KPT_NOS,
    KT_NOS = KT_NOS,
    KPT_HOS = KPT_HOS,
    KT_HOS = KT_HOS,
    DPT_NOS = DPT_NOS,
    DT_NOS = DT_NOS,
    DPT_HOS = DPT_HOS,
    DT_HOS = DT_HOS,
    UPT_NOS = UPT_NOS,
    UT_NOS = UT_NOS,
    UPT_HOS = UPT_HOS,
    UT_HOS = UT_HOS,
    ExPT_NOS = ExPT_NOS,
    ExT_NOS = ExT_NOS,
    ExPT_HOS = ExPT_HOS,
    ExT_HOS = ExT_HOS
  )

  return(SHist)
}

make_hatchery_table <- function(SMSE, s = 1) {
  hatch_settings <- c("n_yearling", "n_subyearling", "pmax_esc", "pmax_NOB", "premove_HOS", "premove_NOS",
                      "s_prespawn", "s_egg_smolt", "s_egg_subyearling", "ptarget_NOB",
                      "phatchery", "m")

  hatch_extra <- data.frame(
    Definition = c("Maximum imported brood (assumed hatchery origin)"),
    Slot = c("brood_import")
  )

  hatch_table <- rbind(
    glossary[match(hatch_settings, glossary$Slot), c("Definition", "Slot")],
    hatch_extra
  )

  hatch_table$Value <- sapply(hatch_table$Slot, function(i) {
    SOM <- SMSE@Misc$SOM
    v <- slot(SOM@Hatchery[[s]], i)
    if (!length(v)) {
      v <- NA
    } else if (i == "brood_import") {
      v <- sum(v)
    } else if (is.function(v)) {
      v <- "User function"
    }
    return(format(v))
  })

  names(hatch_table)[2] <- "Parameter"
  return(hatch_table)
}


make_fitness_table <- function(SMSE, s = 1) {

  fitness_settings <- c("gamma", "phenotype_variance", "fitness_variance", "heritability", "theta")

  fitness_extra <- data.frame(
    Definition = c("Fitness function for the natural and hatchery environments", "Minimum population fitness value"),
    Slot = c("fitness_type", "fitness_floor")
  )
  fitness_table <- rbind(
    fitness_extra,
    glossary[match(fitness_settings, glossary$Slot), c("Definition", "Slot")]
  )
  fitness_table$Value <- sapply(fitness_table$Slot, function(i) {
    SOM <- SMSE@Misc$SOM
    v <- slot(SOM@Hatchery[[s]], i)
    if (!length(v)) v <- NA
    if (length(v) > 1) {
      if (i == "heritability") {
        v <- paste("Mean =", mean(v) %>% round(3))
      } else {
        v <- paste(v, sep = ",")
      }
    }
    return(format(v))
  })
  names(fitness_table)[2] <- "Parameter"
  return(fitness_table)
}


make_harvest_table <- function(SMSE, s = 1) {

  harvest_par <- c("u_preterminal", "u_terminal", "m")

  harvest_extra <- data.frame(
    Definition = c("Mark-selective fishing (preterminal fishery)",
                   "Mark-selective fishing (terminal fishery)",
                   "Mortality from catch and release (proportion, in preterminal and terminal fishery, respectively)"),
    Slot = c("MSF_PT", "MSF_T", "release_mort")
  )
  harvest_table <- rbind(
    glossary[match(harvest_par, glossary$Slot), c("Definition", "Slot")],
    harvest_extra
  )
  harvest_table$Value <- sapply(harvest_table$Slot, function(i) {
    SOM <- SMSE@Misc$SOM
    if (i == "m") {
      v <- slot(SOM@Hatchery[[s]], i)
    } else {
      v <- slot(SOM@Harvest[[s]], i)
    }
    if (!length(v)) v <- NA
    if (length(v) > 1) {
      v <- paste(v, sep = ",")
    }
    return(format(v))
  })
  names(harvest_table)[2] <- "Parameter"
  return(harvest_table)
}


CMpar_key <- data.frame(
  Parameter = c("log_cr", "log_so", "moadd", "wt", "wto", "wt_sd", "wto_sd",
                "logit_matt", "sd_matt",
                "log_fanomalyPT", "log_FbasePT", "fanomalyPT_sd",
                "log_fanomalyT", "log_FbaseT", "fanomalyT_sd",
                "lnE_sd", "b1", "b", "logit_vulPT", "logit_vulT"),

  Description = c("Log productivity at initial maturity and natural mortality",
                  "Log spawners at replacement (at initial maturity and natural mortality)",
                  "Additional age 1 natural mortality (M)",
                  "Annual lognormal deviation in egg-smolt mortality",
                  "Annual lognormal deviation in age 1 natural mortality",
                  "Prior standard deviation in egg-smolt mortality deviates",
                  "Prior standard deviation in annual age 1 M deviates",
                  "Logit annual maturity at age",
                  "Prior standard deviation in annual maturity at age",
                  "Annual lognormal deviation in pre-terminal instantaneous fishing mortality (from specified trend)",
                  "Lognormal scaling parameter for pre-terminal fishing mortality (from specified trend)",
                  "Prior standard deviation in pre-terminal fishing mortality deviates",
                  "Annual lognormal deviation in terminal instantaneous fishing mortality (from specified trend)",
                  "Lognormal scaling parameter for terminal fishing mortality (from specified trend)",
                  "Prior standard deviation in terminal fishing mortality deviates",
                  "Lognormal standard deviation in total escapement (observation error)",
                  "Linear coefficients for age 1 mortality covariates",
                  "Linear coefficients for age 2+ mortality covariates",
                  "Logit preterminal fishery vulnerability at age",
                  "Logit terminal fishery vulnerability at age")
)

make_CM_table <- function(fit) {
  model_names <- unique(names(fit$obj$par))
  CMpar_key[match(model_names, CMpar_key$Parameter), ]
}

#' Estimation function for conditioning model
#'
#' Population dynamics model of an age structured salmon population. Used with RTMB to estimate historical reconstruction from data.
#'
#' @param p List of parameter variables. See [fit_CM()].
#' @param d List of data variables. See [fit_CM()].
#' @keywords internal
#' @return Numeric, objective function value (log-posterior)
#' @author Q. Huynh with Stan code provided by J. Korman and C. Walters
#' @export
CM_int <- function(p, d) {

  # Dispatch method for AD variables ----
  is_ad <- any(sapply(p, inherits, "advector"))
  if (is_ad) {
    `[<-` <- RTMB::ADoverload("[<-")
  }

  # Transformed data ----
  tiny <- 1e-6
  moaddcwt <- -log(d$hatchsurv)
  logobsesc <- log(d$obsescape)

  # Vulnerability (= 0 for age 1, = 1 for oldest age)
  if (is_ad) {
    vulPT <- vulT <- advector(rep(0, d$Nages))
  } else {
    vulPT <- vulT <- numeric(d$Nages)
  }
  vulPT[seq(2, d$Nages-1)] <- plogis(p$logit_vulPT)
  vulPT[d$Nages] <- 1
  vulT[seq(2, d$Nages-1)] <- plogis(p$logit_vulT)
  vulT[d$Nages] <- 1

  # Initial conditions for F
  if (is.character(d$finitPT) && d$finitPT == "estimate") {
    finitPT <- exp(p$log_finitPT)
  } else {
    finitPT <- d$finitPT
  }

  if (is.character(d$finitT) && d$finitT == "estimate") {
    finitT <- exp(p$log_finitT)
  } else {
    finitT <- d$finitT
  }

  lo <- numeric(d$Nages)
  if (is_ad) {
    lhist <- advector(rep(0, d$Nages))
  } else {
    lhist <- numeric(d$Nages)
  }

  lo[1] <- 1
  lhist[1] <- 1
  for (a in 2:d$Nages) {
    lo[a] <- lo[a-1] * exp(-d$mobase[a-1]) * (1 - d$bmatt[a-1]) # unfished juvenile survival
    lhist[a] <- lhist[a-1] * exp(-d$mobase[a-1] - vulPT[a] * finitPT) * (1 - d$bmatt[a-1]) # historical juvenile survival
  }

  epro <- sum(lo * d$ssum * d$fec * d$bmatt)                     # unfished egg production per smolt (recruit, pr)
  spro <- sum(lo * d$ssum * d$bmatt)                             # unfished female spawner per smolt

  memax <- -log(1.0/epro) # unfished M from egg to smolt

  # Transformed parameters ----
  so <- exp(p$log_so)
  ro <- so/spro                 # unfished recruitment ro
  eo <- ro * epro               # unfished total egg production
  mden <- p$log_cr/eo           # Ricker b parameter for egg-smolt relationship
  memin <- memax - p$log_cr     # minimum egg-smolt M at low population density

  alpha <- exp(-memin)
  beta <- mden

  spawnhist <- d$spawn_init
  sprhist <- sum(lhist * d$ssum * exp(-vulPT * finitPT) * d$bmatt * exp(-vulT * finitT))   # historical female spawners per recruit (pr)
  rhist <- spawnhist/sprhist # historical recruitment

  mo <- matrix(0, d$Ldyr, d$Nages-1) # annual ocean M by age

  # predicted survival rate from fishing
  # predicted catch at age vector
  # predicted spawners
  # predicted cwt catches for year
  # predicted cwt spawners for the year
  survPT <- survT <- matrix(NA_real_, d$Ldyr, d$Nages)
  cyearPT <- cyearT <- recr <- syear <- escyear <- array(NA_real_, c(d$Ldyr, d$Nages, 2))
  ccwtPT <- ccwtT <- ecwt <- array(NA_real_, c(d$Ldyr, d$Nages, d$n_r))

  spawners <- catchPT <- catchT <- egg <- megg <- logpredesc <- moplot <- FPT <- FT <- numeric(d$Ldyr)
  matt <- array(0, c(d$Ldyr, d$Nages, d$n_r))
  matt[, d$Nages, ] <- 1

  N <- array(0, c(d$Ldyr+1, d$Nages, 2)) # Array slice 1 = natural origin fish, 2 = hatchery fish
  Ncwt <- array(0, c(d$Ldyr+1, d$Nages, d$n_r))

  # Initialize mean phenotype, fitness, etc.
  brood <- array(NA_real_, c(d$Ldyr, d$Nages, 2))
  pNOB <- pHOSeff <- pHOScensus <- numeric(d$Ldyr)

  if (d$fitness) {
    zbar <- matrix(0, d$Ldyr, 2)
    fitness <- matrix(1, d$Ldyr, 2)
  }

  # Initialize N ----
  N[1, , 1] <- rhist * (1 - d$pHOS_init) * lhist               # initial numbers at age year 1
  N[1, , 2] <- rhist * d$pHOS_init * lhist
  N[1, 1, 2] <- d$hatchsurv * d$hatchrelease[1] # initial age 1 numbers for hatchery release in year 1
  if (d$lht==2) {  #in case spring run type where age of ocean entry=2, not 1
    N[2, 1, ] <- N[1, 1, ]
  }

  # Initialize F
  FPT[] <- exp(p$log_FbasePT) * d$RelRegFPT * exp(p$log_fanomalyPT)
  FT[] <- exp(p$log_FbaseT) * d$RelRegFT * exp(p$log_fanomalyT)

  # Loop over years ----
  for (t in 1:d$Ldyr) {
    # First year ocean M
    if (length(d[["covariate1"]])) {
      mo[t, 1] <- d$mobase[1] + p$moadd + p[["wto"]][t] + sum(d[["covariate1"]][t, ] * p[["b1"]])
    } else {
      mo[t, 1] <- d$mobase[1] + p$moadd + p[["wto"]][t]
    }

    # M's for older ages
    if (length(d[["covariate"]])) {
      mo[t, 2:(d$Nages-1)] <- d$mobase[2:(d$Nages-1)] + sum(d[["covariate"]][t, ] * p[["b"]])
    } else {
      mo[t, 2:(d$Nages-1)] <- d$mobase[2:(d$Nages-1)]
    }

    Ncwt[t, 1, ] <- d$cwtrelease[t, ] * d$hatchsurv

    matt[t, 2:(d$Nages-1), 1] <- plogis(p$logit_matt[t, ])
    if (d$n_r > 1) {
      for (r in 2:d$n_r) matt[t, 2:(d$Nages-1), r] <- plogis(p$logit_matt[t, ] + p$matt_offset[, r-1])
    }

    # set age-specific survival rates through fishing
    survPT[t, ] <- exp(-vulPT * FPT[t])
    survT[t, ] <- exp(-vulT * FT[t])

    # predict preterminal catch at age for the year
    cyearPT[t, , ] <- N[t, , ] * (1 - survPT[t, ])
    ccwtPT[t, , ] <- Ncwt[t, , ] * (1 - survPT[t, ])

    # predict recruitment at age for the year
    recr[t, , ] <- N[t, , ] * survPT[t, ] * matt[t, , d$r_matt]

    # predict terminal catch at age for the year
    cyearT[t, , ] <- recr[t, , ] * (1 - survT[t, ])
    ccwtT[t, , ] <- Ncwt[t, , ] * survPT[t, ] * matt[t, , ] * (1 - survT[t, ])

    # predict escapement at age for the year
    escyear[t, , ] <- recr[t, , ] * survT[t, ]
    ecwt[t, , ] <- Ncwt[t, , ] * survPT[t, ] * matt[t, , ] * survT[t, ]

    # predict spawners at age for the year
    syear[t, , ] <- d$propwildspawn[t] * escyear[t, , ] * d$s_enroute

    # predict egg production for the year
    egg[t] <- sum(d$ssum * d$fec * (syear[t, , 1] + d$gamma * syear[t, , 2]))

    # Assume broodtake is equal between NOB and HOB
    # predict pHOS, pNOB
    brood[t, , ] <- (1 - d$propwildspawn[t]) * escyear[t, , ]

    pNOB[t] <- sum(d$fec * brood[t, , 1])/sum(d$fec * (brood[t, , 1] + brood[t, , 2]))
    pHOSeff[t] <- sum(d$fec * d$gamma * syear[t, , 2])/
      sum(d$fec * (syear[t, , 1] + d$gamma * syear[t, , 2]))
    pHOScensus[t] <- sum(d$fec * syear[t, , 2])/sum(d$fec * (syear[t, , 1] + syear[t, , 2]))

    if (d$fitness) {
      # Retrieve zbar and fitness by brood year
      zbar_brood <- matrix(0, d$Nages, 2)
      for (a in 1:d$Nages) {
        tt <- t - a
        if (tt > 0) {
          zbar_brood[a, ] <- zbar[tt, ]
          if (a < d$Nages) {
            surv_fitness <- exp(-mo[t, a]) * fitness[tt, 1]^d$rel_loss[3]
            mo[t, a] <- -log(surv_fitness)
          }
        } else {
          zbar_brood[a, ] <- d$zbar_start
        }
      }
      zbar[t, ] <- calc_zbar(
        syear[t, , 1], d$gamma * syear[t, , 2], brood[t, , 1], brood[t, , 2],
        d$fec, d$fec, zbar_brood,
        d$fitness_variance, d$theta, d$phenotype_variance, d$heritability
      )
      fitness[t, ] <- exp(-0.5 * (zbar[t, ] - d$theta)^2/(d$fitness_variance + d$phenotype_variance)) # fitness_floor not used here!
    }

    # survive fish over the year, removing maturing fish that will spawn
    N[t+1, 2:d$Nages, ] <- N[t, 2:d$Nages - 1, ] * survPT[t, 2:d$Nages - 1] * exp(-mo[t, 2:d$Nages - 1]) * (1 - matt[t, 2:d$Nages - 1, d$r_matt])
    Ncwt[t+1, 2:d$Nages, ] <- Ncwt[t, 2:d$Nages - 1, ] * survPT[t, 2:d$Nages - 1] * exp(-mo[t, 2:d$Nages - 1]) * (1 - matt[t, 2:d$Nages - 1, ])

    if (d$fitness) {
      egg_fitness <- egg[t] * fitness[t, 1]^d$rel_loss[1]
      alpha_fitness <- alpha * fitness[t, 1]^d$rel_loss[2]
      beta_fitness <- beta/fitness[t, 1]^d$rel_loss[2]

      megg[t] <- memin + mden * egg_fitness + p[["wt"]][t]
      N[t + d$lht, 1, 1] <- alpha_fitness * egg_fitness * exp(-beta_fitness * egg_fitness) * exp(-p[["wt"]][t])
    } else {
      megg[t] <- memin + mden * egg[t] + p[["wt"]][t]
      N[t + d$lht, 1, 1] <- alpha * egg[t] * exp(-beta * egg[t]) * exp(-p[["wt"]][t])
    }
    N[t + d$lht, 1, 2] <- d$hatchsurv * d$hatchrelease[t+1]

    # total spawners and escapement
    spawners[t] <- sum(syear[t, , ])
    logpredesc[t] <- log(sum(escyear[t, , ]) + tiny)

    catchPT[t] <- sum(cyearPT[t, , ]) # add up total catch for the year
    catchT[t] <- sum(cyearT[t, , ]) # add up total catch for the year
    moplot[t] <- mo[t, 1] + moaddcwt # for plotting first year ocean M, including hatchery survival
  }

  # Convert predicted cwt catches and escapement from calendar year to brood year for likelihoods
  # data are in brood year format
  cbroodPT <- cbroodT <- ebrood <- array(tiny, c(d$Ldyr, d$Nages, d$n_r))

  pHOScensus_brood <- numeric(d$Ldyr)
  NOS_brood <- CY2BY(syear[, , 1])
  HOS_brood <- CY2BY(syear[, , 2])

  for (t in 1:(d$Ldyr)) {
    for (a in 1:d$Nages) {
      if (t+a-1 <= d$Ldyr) {
        cbroodPT[t, a, ] <- ccwtPT[t+a-1, a, ]/d$cwtExp + tiny
        cbroodT[t, a, ] <- ccwtT[t+a-1, a, ]/d$cwtExp + tiny
        ebrood[t, a, ] <- ecwt[t+a-1, a, ]/d$cwtExp + tiny
      }
    }
    pHOScensus_brood[t] <- sum(d$fec * HOS_brood[t, ])/sum(d$fec * (HOS_brood[t, ] + NOS_brood[t, ]))
  }

  # Log prior for parameters
  logprior_so <- dnorm(p$log_so, d$so_mu, d$so_sd, log = TRUE) # prior on so in log space as it is poorly determined from data
  logprior_wt <- dnorm(p[["wt"]], 0, p$wt_sd, log = TRUE)
  logprior_wto <- dnorm(p[["wto"]], 0, p$wto_sd, log = TRUE)

  if (sum(d$cwtcatPT)) {
    logprior_fanomPT <- dnorm(p$log_fanomalyPT, 0, p$fanomalyPT_sd, log = TRUE)
    logprior_fanomPT_sd <- dgamma(p$fanomalyPT_sd, 2, scale = 0.2, log = TRUE)
  } else if (is_ad) {
    logprior_fanomPT <- logprior_fanomPT_sd <- advector(0)
  } else {
    logprior_fanomPT <- logprior_fanomPT_sd <- 0
  }
  if (sum(d$cwtcatT)) {
    logprior_fanomT <- dnorm(p$log_fanomalyT, 0, p$fanomalyT_sd, log = TRUE)
    logprior_fanomT_sd <- dgamma(p$fanomalyT_sd, 2, scale = 0.2, log = TRUE)
  } else if (is_ad) {
    logprior_fanomT <- logprior_fanomT_sd <- advector(0)
  } else {
    logprior_fanomT <- logprior_fanomT_sd <- 0
  }

  # Log prior for vulnerability
  logit_bvulPT <- qlogis(squeeze(d$bvulPT[2:(d$Nages-1)]))
  logit_bvulT <- qlogis(squeeze(d$bvulT[2:(d$Nages-1)]))

  if (sum(d$bvulPT)) {
    logprior_vulPT <- dnorm(p$logit_vulPT, logit_bvulPT, 1.75, log = TRUE)
  } else if (is_ad) {
    logprior_vulPT <- advector(0)
  } else {
    logprior_vulPT <- 0
  }

  if (sum(d$bvulT)) {
    logprior_vulT <- dnorm(p$logit_vulT, logit_bvulT, 1.75, log = TRUE)
  } else if (is_ad) {
    logprior_vulT <- advector(0)
  } else {
    logprior_vulT <- 0
  }

  #convert base maturity rates to logit for calculation of time-varying rates
  #no time variation for first and last age
  logit_bmatt <- qlogis(d$bmatt[2:(d$Nages-1)])
  logprior_matt <- sapply(2:(d$Nages - 1), function(a) {
    dnorm(p$logit_matt[, a-1], logit_bmatt[a-1], p$sd_matt[a-1], log = TRUE)
  })

  logprior <- sum(
    logprior_so, logprior_wt, logprior_wto,
    logprior_fanomPT, logprior_fanomT, logprior_matt,
    logprior_vulPT, logprior_vulT
  )

  # Log prior for hyperparameters ----
  # Gamma mean=shape/rate, variance=shape/rate^2 (scale = 1/rate)
  # 2, 5 has mean of 0.4 and cv of 0.71 (95% CI = 0.03-1.0)
  logprior <- logprior +
    sum(
      dgamma(p$sd_matt, 2, scale = 0.2, log = TRUE),
      dgamma(p$wt_sd, 2, scale = 0.2, log = TRUE),
      dgamma(p$wto_sd, 2, scale = 0.2, log = TRUE),
      dgamma(p$lnE_sd, 2, scale = 0.2, log = TRUE)
    ) +
    logprior_fanomPT_sd + logprior_fanomT_sd

  # Log likelihood
  loglike_esc <- dnorm(logobsesc, logpredesc, p$lnE_sd, log = TRUE)
  loglike_esc[is.na(d$obsescape)] <- 0
  loglike_cwtesc <- dpois(d$cwtesc, ebrood, log = TRUE)

  if (sum(d$cwtcatPT)) {
    loglike_cwtcatPT <- dpois(d$cwtcatPT, cbroodPT, log = TRUE)
  } else if (is_ad) {
    loglike_cwtcatPT <- advector(0)
  } else {
    loglike_cwtcatPT <- 0
  }
  if (sum(d$cwtcatT)) {
    loglike_cwtcatT <- dpois(d$cwtcatT, cbroodT, log = TRUE)
  } else if (is_ad) {
    loglike_cwtcatT <- advector(0)
  } else {
    loglike_cwtcatT <- 0
  }

  if (!is.null(d$obs_pHOS) && sum(d$obs_pHOS, na.rm = TRUE)) {
    loglike_pHOS <- dnorm(qlogis(d$obs_pHOS), qlogis(squeeze(pHOScensus_brood)), d$pHOS_sd, log = TRUE)
    loglike_pHOS[is.na(d$obs_pHOS)] <- 0
    loglike_pHOS[d$Ldyr - seq(1, d$Nages - 1) + 1] <- 0
  } else if (is_ad) {
    loglike_pHOS <- advector(0)
  } else {
    loglike_pHOS <- 0
  }

  loglike <- sum(loglike_esc, loglike_cwtcatPT, loglike_cwtcatT, loglike_cwtesc, loglike_pHOS)

  # Objective function
  fn <- -1 * (logprior + loglike)

  # Report variables
  REPORT(so)
  REPORT(eo)
  REPORT(ro)
  REPORT(mden)
  REPORT(memax)
  REPORT(memin)
  REPORT(alpha)
  REPORT(beta)

  REPORT(survPT)
  REPORT(survT)
  REPORT(cyearPT)
  REPORT(cyearT)
  REPORT(recr)
  REPORT(syear)
  REPORT(escyear)
  REPORT(ccwtPT)
  REPORT(ccwtT)
  REPORT(ecwt)

  REPORT(spawners)
  REPORT(logpredesc)
  REPORT(catchPT)
  REPORT(catchT)
  REPORT(egg)
  REPORT(megg)
  REPORT(mo)
  REPORT(moplot)
  REPORT(FPT)
  REPORT(FT)
  REPORT(vulPT)
  REPORT(vulT)

  REPORT(matt)
  REPORT(N)
  REPORT(Ncwt)

  REPORT(cbroodPT)
  REPORT(cbroodT)
  REPORT(ebrood)

  REPORT(epro)
  REPORT(spro)

  REPORT(brood)
  REPORT(pNOB)
  REPORT(pHOSeff)
  REPORT(pHOScensus)
  REPORT(pHOScensus_brood)

  if (d$fitness) {
    REPORT(fitness)
    REPORT(zbar)
  }

  REPORT(loglike_esc)
  REPORT(loglike_cwtcatPT)
  REPORT(loglike_cwtcatT)
  REPORT(loglike_cwtesc)
  REPORT(loglike_pHOS)

  REPORT(loglike)
  REPORT(logprior)
  REPORT(fn)

  return(fn)
}


# Make list of starting values
make_CMpars <- function(p, d) {
  if (is.null(p$log_cr)) p$log_cr <- 3
  if (is.null(p$log_so)) p$log_so <- log(3 * max(d$obsescape))
  if (is.null(p$moadd)) p$moadd <- 0
  if (is.null(p[["wt"]])) p[["wt"]] <- rep(0, d$Ldyr)
  if (is.null(p[["wto"]])) p[["wto"]] <- rep(0, d$Ldyr)
  if (is.null(p$log_fanomalyPT)) p$log_fanomalyPT <- rep(0, d$Ldyr)
  if (is.null(p$log_fanomalyT)) p$log_fanomalyT <- rep(0, d$Ldyr)
  if (is.null(p$lnE_sd)) p$lnE_sd <- 0.1

  if (is.null(p$log_FbasePT)) p$log_FbasePT <- log(0.1)
  if (is.null(p$log_FbaseT)) p$log_FbaseT <- log(0.1)
  if (is.null(p$logit_vulPT)) p$logit_vulPT <- qlogis(squeeze(d$bvulPT[-c(1, d$Nages)]))
  if (is.null(p$logit_vulT)) p$logit_vulT <- qlogis(squeeze(d$bvulT[-c(1, d$Nages)]))

  if (is.null(p$logit_matt)) p$logit_matt <- matrix(qlogis(d$bmatt[-c(1, d$Nages)]), d$Ldyr, d$Nages - 2, byrow = TRUE)
  if (is.null(p$sd_matt)) p$sd_matt <- rep(0.5, d$Nages - 2)
  if (is.null(p$matt_offset) && d$n_r > 1) {
    p$matt_offset <- array(0, c(d$Nages - 2, d$n_r - 1))
  }

  if (is.null(p[["wt_sd"]])) p[["wt_sd"]] <- 1
  if (is.null(p[["wto_sd"]])) p[["wto_sd"]] <- 1
  if (is.null(p$fanomalyPT_sd)) p$fanomalyPT_sd <- 1
  if (is.null(p$fanomalyT_sd)) p$fanomalyT_sd <- 1

  if (is.null(p[["b1"]])) {
    if (length(d[["covariate1"]])) {
      p[["b1"]] <- rep(0, ncol(d[["covariate1"]]))
    } else {
      p[["b1"]] <- 0
    }
  }

  if (is.null(p[["b"]])) {
    if (length(d[["covariate"]])) {
      p[["b"]] <- rep(0, ncol(d[["covariate"]]))
    } else {
      p[["b"]] <- 0
    }
  }

  if (is.null(p[["log_finitPT"]])) p[["log_finitPT"]] <- log(0.1)
  if (is.null(p[["log_finitT"]])) p[["log_finitT"]] <- log(0.1)

  return(p)
}

#' @importFrom stats na.omit
check_data <- function(data) {

  if (is.null(data$Nages)) stop("data$Nages not found")
  if (is.null(data$Ldyr)) stop("data$Ldyr not found")
  if (is.null(data$lht)) data$lht <- 1

  if (is.null(data$n_r)) data$n_r <- 1

  if (is.null(data$cwtrelease)) stop("data$cwtrelease should be a matrix Ldyr x n_r")

  if (data$n_r == 1 && is.null(dim(data$cwtrelease))) {
    if (length(data$cwtrelease) != data$Ldyr) {
      stop("data$cwtrelease should be a matrix Ldyr x n_r")
    }
    data$cwtrelease <- matrix(data$cwtrelease, data$Ldyr, data$n_r)
  }
  if (!is.matrix(data$cwtrelease)) {
    stop("data$cwtrelease should be a matrix Ldyr x n_r")
  }

  if (is.null(data$cwtesc) || is.matrix(data$cwtesc) || !is.array(data$cwtesc)) {
    stop("data$cwtesc should be an array: Ldyr (by brood year) x Nages x n_r")
  }

  if (is.null(data$cwtcatPT)) {
    data$cwtcatPT <- array(0, c(data$Ldyr, data$Nages, data$n_r))
  } else if (is.matrix(data$cwtcatPT) || !is.array(data$cwtcatPT)) {
    stop("data$cwtcatPT should be an array: Ldyr (by brood year) x Nages x n_r")
  }

  if (is.null(data$cwtcatT)) {
    data$cwtcatT <- array(0, c(data$Ldyr, data$Nages, data$n_r))
  } else if (is.matrix(data$cwtcatT) || !is.array(data$cwtcatT)) {
    stop("data$cwtcatT should be an array: Ldyr (by brood year) x Nages x n_r")
  }

  FPT <- sum(data$cwtcatPT)
  FT <- sum(data$cwtcatT)
  if (!FPT && !FT) stop("No CWT catch found")

  if (FPT) {
    if (is.null(data$bvulPT) || length(data$bvulPT) != data$Nages) {
      stop("data$bvulPT should be a vector length Nages")
    }
    if (is.null(data$RelRegFPT)) data$RelRegFPT <- rep(1, data$Ldyr)
    if (length(data$RelRegFPT) != data$Ldyr) stop("data$RelRegFPT should be a vector length Ldyr")
  } else {
    data$bvulPT <- rep(0, data$Nages)
    data$RelRegFPT <- rep(0, data$Ldyr)
  }

  if (FT) {
    if (is.null(data$bvulT) || length(data$bvulT) != data$Nages) {
      stop("data$bvulT should be a vector length Nages")
    }
    if (is.null(data$RelRegFT)) data$RelRegFT <- rep(1, data$Ldyr)
    if (length(data$RelRegFT) != data$Ldyr) stop("data$RelRegFT should be a vector length Ldyr")
  } else {
    data$bvulT <- rep(0, data$Nages)
    data$RelRegFT <- rep(0, data$Ldyr)
  }

  if (is.null(data$bmatt) || length(data$bmatt) != data$Nages) {
    stop("data$bmatt should be a vector length Nages")
  }

  if (is.null(data$mobase) || length(data$mobase) != data$Nages) {
    stop("data$mobase should be a vector length Nages")
  }

  if (!is.null(data[["covariate1"]])) {
    if (!is.matrix(data[["covariate1"]])) stop("data$covariate1 should be a matrix. See help('fit-CM') for dimensions")
  }
  if (!is.null(data[["covariate"]])) {
    if (!is.matrix(data[["covariate"]])) stop("data$covariate should be a matrix. See help('fit-CM') for dimensions")
  }

  if (is.null(data$hatchsurv)) data$hatchsurv <- 1
  if (is.null(data$gamma)) data$gamma <- 1
  if (is.null(data$ssum)) stop("data$ssum (proportion female spawners) should be between 0-1")

  if (is.null(data$fec) || length(data$fec) != data$Nages) {
    stop("data$fec should be a vector length Nages")
  }

  if (is.null(data$r_matt)) data$r_matt <- 1L

  if (is.null(data$obsescape) || length(data$obsescape) != data$Ldyr) {
    stop("data$obsescape should be a vector length Ldyr")
  }

  if (is.null(data$propwildspawn) || length(data$propwildspawn) != data$Ldyr) {
    stop("data$propwildspawn should be a vector length Ldyr")
  }

  if (is.null(data$hatchrelease) || length(data$hatchrelease) != data$Ldyr + 1) {
    #stop("data$hatchrelease should be a vector length Ldyr+1")
    data$hatchrelease <- rep(0, data$Ldyr + 1)
  }

  if (!is.null(data$obs_pHOS) && length(data$obs_pHOS) != data$Ldyr) {
    stop("data$obs_pHOS should be a vector length Ldyr")
  }
  if (!is.null(data$obs_pHOS) && is.null(data$pHOS_sd)) data$pHOS_sd <- 1

  if (is.null(data$cwtExp)) data$cwtExp <- 1
  if (data$cwtExp < 1) warning("CWT expansion factor in data object (cwtExp) < 1. Are you sure?")

  if (is.null(data$s_enroute)) data$s_enroute <- 1

  if (is.null(data$so_mu)) data$so_mu <- log(3 * max(data$obsescape, na.rm = TRUE))
  if (is.null(data$so_sd)) data$so_sd <- 0.5

  if (is.null(data$finitPT)) data$finitPT <- 0
  if (is.null(data$finitT)) data$finitT <- 0
  if (is.null(data$pHOS_init)) data$pHOS_init <- 0

  if (is.null(data$spawn_init)) data$spawn_init <- na.omit(data$obsescape)[1]
  if (is.null(data$fitness)) data$fitness <- FALSE

  if (data$fitness) {
    if (is.null(data$theta)) data$theta <- c(100, 80)
    if (is.null(data$rel_loss)) data$rel_loss <- c(0.3, 0.2, 0.5)
    if (is.null(data$zbar_start)) data$zbar_start <- c(100, 100)
    if (is.null(data$fitness_variance)) data$fitness_variance <- 100
    if (is.null(data$phenotype_variance)) data$phenotype_variance <- 10
    if (is.null(data$heritability)) data$heritability <- 0.5
    if (is.null(data$fitness_floor)) data$fitness_floor <- 0.5
  }

  return(data)
}


make_map <- function(map = list(), p, d) {

  FPT <- sum(d$cwtcatPT)
  FT <- sum(d$cwtcatT)

  if (!FPT) {
    map$log_FbasePT <- factor(NA)
    map$log_fanomalyPT <- factor(rep(NA, d$Ldyr))
    map$fanomalyPT_sd <- factor(NA)
    map$logit_vulPT <- factor(rep(NA, d$Nages - 2))
  }

  if (!FT) {
    map$log_FbaseT <- factor(NA)
    map$log_fanomalyT <- factor(rep(NA, d$Ldyr))
    map$fanomalyT_sd <- factor(NA)
    map$logit_vulT <- factor(rep(NA, d$Nages - 2))
  }

  if (!length(d[["covariate1"]])) map[["b1"]] <- factor(NA)
  if (!length(d[["covariate"]])) map[["b"]] <- factor(NA)

  if (is.numeric(d$finitPT)) map[["log_finitPT"]] <- factor(NA)
  if (is.numeric(d$finitT)) map[["log_finitT"]] <- factor(NA)
  return(map)
}

make_bounds <- function(par_names, data, lower = list(), upper = list()) {

  .lower <- structure(rep(-Inf, length(par_names)), names = par_names)
  .upper <- structure(rep(Inf, length(par_names)), names = par_names)

  # Add important defaults first
  if ("log_cr" %in% names(.lower)) .lower["log_cr"] <- 1e-8
  if ("moadd" %in% names(.lower)) .lower["moadd"] <- 1e-8
  #if ("FbasePT" %in% names(.lower)) .lower["FbasePT"] <- 1e-8
  #if ("FbaseT" %in% names(.lower)) .lower["FbaseT"] <- 1e-8

  if ("lnE_sd" %in% names(.lower)) .lower["lnE_sd"] <- 1e-8
  if ("wt_sd" %in% names(.lower)) .lower["wt_sd"] <- 1e-8
  if ("wto_sd" %in% names(.lower)) .lower["wto_sd"] <- 1e-8
  if ("fanomalyPT_sd" %in% names(.lower)) .lower["fanomalyPT_sd"] <- 1e-8
  if ("fanomalyT_sd" %in% names(.lower)) .lower["fanomalyT_sd"] <- 1e-8

  if ("sd_matt" %in% names(.lower)) .lower[names(.lower) == "sd_matt"] <- 1e-8

  #if ("wt" %in% names(.lower)) {
  #  .lower[names(.lower) == "wt"] <- -3
  #  .upper[names(.upper) == "wt"] <- 3
  #}
  #if ("wto" %in% names(.lower)) {
  #  .lower[names(.lower) == "wto"] <- -3
  #  .upper[names(.upper) == "wto"] <- 3
  #}
  #if ("fanomalyPT" %in% names(.lower)) {
  #  .lower[names(.lower) == "fanomalyPT"] <- -3
  #  .upper[names(.upper) == "fanomalyPT"] <- 3
  #}
  #if ("fanomalyT" %in% names(.lower)) {
  #  .lower[names(.lower) == "fanomalyT"] <- -3
  #  .upper[names(.upper) == "fanomalyT"] <- 3
  #}

  # Override with user-defined bounds
  for (i in unique(par_names)) {
    if (!is.null(lower[[i]])) .lower[names(.lower) == i] <- lower[[i]]
    if (!is.null(upper[[i]])) .upper[names(.upper) == i] <- upper[[i]]
  }

  list(lower = .lower, upper = .upper)
}

#' @rdname CMfigures
#' @param sims Optional integer vector for subset of MCMC iterations
#' @returns
#' - `get_report()` returns the list of state variables by individual MCMC samples
#' @export
get_report <- function(stanfit, sims, inc_warmup = FALSE) {
  if (!requireNamespace("rstan", quietly = TRUE)) stop("rstan package is needed.")

  pars <- rstan::extract(stanfit)
  if (missing(sims)) {
    sims <- seq_len(length(pars[["lp__"]]))

    if (!inc_warmup) {
      it <- seq(1, stanfit@sim$iter, by = stanfit@sim$thin)
      sims <- sims[it > stanfit@sim$warmup]
    }
  }

  pars_samp <- lapply(pars[names(pars) != "lp__"], function(x) {
    if (is.matrix(x)) x[sims, , drop = FALSE] else x[sims]
  })

  fit <- stanfit@.MISC$CMfit
  if (is.null(fit)) stop("CM fitted object not found in stanfit@.MISC$CMfit")

  # Update data object
  d <- get_CMdata(fit)
  dnew <- check_data(d)
  if (!identical(d, dnew)) {
    env_func <- attr(fit$obj$env$data, "func") %>% environment()
    assign("data", dnew, envir = env_func)
  }

  report <- lapply(1:length(sims), function(i) {
    par_x <- lapply(pars_samp, function(x) {
      if (is.matrix(x)) x[i, ] else x[i]
    })
    fit$obj$report(unlist(par_x))
  })

  return(report)
}

#' @rdname CMfigures
#' @param fit Output from `[fit_CM()]`
#' @returns
#' - `get_CMdata()` returns the list of data variables used in the conditioning model
#' @export
get_CMdata <- function(fit) {
  func <- attr(fit$obj$env$data, "func")
  get("data", envir = environment(func), inherits = FALSE)
}


#' @importFrom abind abind
expand_array <- function(x, proyears) {
  xlast <- x[, , dim(x)[3]]
  xproj <- array(xlast, c(dim(x)[1:2], proyears))
  abind::abind(x, xproj, along = 3) %>%
    structure(dimnames = NULL)
}

squeeze <- function(x) (1 - .Machine$double.eps) * (x - 0.5) + 0.5

calc_zbar <- function(NOS, HOS_effective, NOB, HOB, fec, fec_brood,
                      zbar_prev, fitness_variance, theta, phenotype_variance, heritability) {

  pNOS <- NOS * fec/sum(fec * (NOS + HOS_effective))
  if (!sum(NOS)) pNOS[] <- 0

  pHOSeff <- HOS_effective * fec/sum(fec * (NOS + HOS_effective))
  if (!sum(HOS_effective)) pHOSeff[] <- 0

  pNOB <- NOB * fec_brood/sum(fec_brood * (NOB + HOB))
  if (!sum(NOB)) pNOB[] <- 0
  pHOB <- HOB * fec_brood/sum(fec_brood * (NOB + HOB))
  if (!sum(HOB)) pHOB[] <- 0

  # Trait value after selection
  zprime_natural <- (zbar_prev * fitness_variance + theta[1] * phenotype_variance)/(fitness_variance + phenotype_variance)
  zprime_hatchery <- (zbar_prev * fitness_variance + theta[2] * phenotype_variance)/(fitness_variance + phenotype_variance)

  # Change in trait value in the next generation, indexed by environment and brood year
  znext_natural <- zbar_prev + (zprime_natural - zbar_prev) * heritability
  znext_hatchery <- zbar_prev + (zprime_hatchery - zbar_prev) * heritability

  # Realized change after weighting by environment, brood year, and age class fecundity
  z_natural <- sum(znext_natural[, 1] * pNOS, znext_natural[, 2] * pHOSeff)
  z_hatchery <- sum(znext_hatchery[, 1] * pNOB, znext_hatchery[, 2] * pHOB)

  c(z_natural, z_hatchery)
}


calc_fitness <- function(zbar, theta, fitness_variance, phenotype_variance, fitness_floor) {
  W <- exp(-0.5 * (zbar - theta)^2/(fitness_variance + phenotype_variance))
  pmax(W, fitness_floor)
}

#' Reference points
#'
#' @description
#' Calculate MSY and Sgen reference points for the operating model. Uses the biological parameters (maturity, natural mortality)
#' in the last year of the projection.
#' - `calc_MSY()` calculates the MSY reference points from a set of biological and fishery parameters
#' - `calc_Sgen()` calculates the Sgen, the spawner abundance that would reach the spawner abundance at MSY after one generation without fishing
#' - `calc_ref()` is a wrapper function that calculates MSY and Sgen for an operating model
#'
#' @param SOM An object of class [salmonMSE::SOM-class]
#' @param rel_F Numeric length 2, indicates the relative effort in the preterminal and terminal fisheries, with a maximum value of 1.
#' The default is `c(0, 1)` which indicates a yield calculation with only the terminal fishery.
#' @param check Logical, whether to check the SOM object using [check_SOM()]
#' @param maximize Character, whether the MSY calculation is the optimum that maximizes catch (`"MSY"`) or excess recruitment (`"MER"`). The two
#' methods should be equivalent when `rel_F = c(0, 1)`.
#' @returns
#' - `calc_MSY` returns a vector of various state variables (catch, exploitation rate, egg production, spawners) at MSY
#' - `calc_Sgen` returns a numeric
#' - `calc_ref` returns a list by stock, each containing a matrix of MSY state variables and Sgen by simulation
#' @seealso [calc_Smsy_Ricker()]
#' @importFrom stats optimize
#' @examples
#' ref_simple <- calc_ref(simple_SOM)
#' ref_multi <- calc_ref(multi_SOM)
#'
#' @export
calc_ref <- function(SOM, rel_F, check = TRUE, maximize = c("MSY", "MER")) {
  maximize <- match.arg(maximize)

  if (check) SOM <- check_SOM(SOM)

  ns <- length(SOM@Bio)

  y <- SOM@proyears

  if (missing(rel_F)) rel_F <- c(0, 1)
  rel_F_s <- lapply(1:ns, function(...) rel_F)

  ref_s <- lapply(1:ns, function(s) {

    if (SOM@Habitat[[s]]@use_habitat) return(matrix(0, 0, 0))

    SRR <- make_SRR(SOM@Bio[[s]])

    Mjuv_NOS <- SOM@Bio[[s]]@Mjuv_NOS
    p_female <- SOM@Bio[[s]]@p_female
    vulPT <- SOM@Harvest[[s]]@vulPT
    vulT <- SOM@Harvest[[s]]@vulT
    maxage <- SOM@Bio[[s]]@maxage
    s_enroute <- SOM@Bio[[s]]@s_enroute
    n_g <- SOM@Bio[[s]]@n_g
    p_LHG <- SOM@Bio[[s]]@p_LHG


    val <- sapply(1:SOM@nsim, function(x) {
      fec <- matrix(SOM@Bio[[s]]@fec[x, , y], maxage, n_g)
      Mjuv <- matrix(Mjuv_NOS[x, , y, ], maxage-1, n_g)
      p_mature <- matrix(SOM@Bio[[s]]@p_mature[x, , y], maxage, n_g)

      ref <- calc_MSY(Mjuv, fec, p_female, rel_F_s[[s]], vulPT[x, ], vulT[x, ], p_mature, s_enroute, n_g, p_LHG,
                      SRR$SRRpars[x, ], maximize)

      if (any(ref < 0, na.rm = TRUE)) {

        ref[!is.na(ref)] <- 0
        ref["Sgen"] <- 0

      } else {
        Sgen <- calc_Sgen(Mjuv, fec, p_female, rel_F_s[[s]], vulPT[x, ], vulT[x, ], p_mature, s_enroute, n_g, p_LHG,
                          SRR$SRRpars[x, ], SMSY = ref["Spawners_MSY"])
        ref["Sgen"] <- Sgen
      }
      return(ref)
    })

    return(val)
  })

  return(ref_s)
}

#' @rdname calc_ref
#' @param Mjuv Numeric `maxage` for juvenile natural mortality. Can be a matrix `[maxage, n_g]`.
#' @param fec Numeric `maxage` for fecundity. Can be a matrix `[maxage, n_g]`.
#' @param p_female Numeric for proportion female spawners
#' @param vulPT Numeric `maxage` for preterminal vulnerability at age
#' @param vulT Numeric `maxage` for terminal vulnerability at age
#' @param p_mature Numeric `maxage` for maturity proportions at age. Can be a matrix `[maxage, n_g]`.
#' @param s_enroute Numeric for en-route survival of escapement to spawning grounds
#' @param n_g Integer, number of life history groups within a cohort
#' @param p_LHG Numeric `n_g` for proportion of the total egg production assigned to each life history group within a cohort
#' @param SRRpars Data frame, one row, that contains the stock recruit parameters that predicts density-dependent survival at the egg-smolt life stage
#' @param F_search Numeric, length 2 for the range of F values to search for the instantaneous fishing mortality that produces MSY
#' @export
calc_MSY <- function(Mjuv, fec, p_female, rel_F, vulPT, vulT, p_mature, s_enroute, n_g = 1, p_LHG = 1,
                     SRRpars, maximize = c("MSY", "MER"), F_search = c(1e-8, 5)) {
  maximize <- match.arg(maximize)

  if (!is.matrix(Mjuv)) Mjuv <- matrix(Mjuv, length(Mjuv), n_g)
  if (!is.matrix(p_mature)) p_mature <- matrix(p_mature, length(p_mature), n_g)
  if (!is.matrix(fec)) fec <- matrix(fec, length(fec), n_g)

  if (missing(p_LHG)) p_LHG <- rep(1/n_g, n_g)

  opt <- optimize(
    .calc_eq, interval = F_search,
    Mjuv = Mjuv,
    fec = fec,
    p_female = p_female, rel_F = rel_F,
    vulPT = vulPT, vulT = vulT,
    p_mature = p_mature,
    s_enroute = s_enroute,
    n_g = n_g,
    p_LHG = p_LHG,
    SRRpars = SRRpars,
    maximize = maximize,
    maximum = TRUE
  )

  ref <- .calc_eq(
    .F = opt$maximum,
    Mjuv = Mjuv, fec = fec, p_female = p_female, rel_F = rel_F,
    vulPT = vulPT, vulT = vulT, p_mature = p_mature, s_enroute = s_enroute,
    n_g = n_g, p_LHG = p_LHG,
    SRRpars = SRRpars, maximize = maximize, opt = FALSE
  ) %>% unlist()

  names(ref) <- paste0(names(ref), "_MSY")
  if (ref["KPT_MSY"] == 0) ref["UPT_MSY"] <- 0
  if (ref["KT_MSY"] == 0) ref["UT_MSY"] <- 0

  return(ref)
}

#' @rdname calc_ref
#' @param SMSY Numeric, spawning abundance at MSY
#' @param nyears Integer, number of years to project the population with no fishing to reach `SMSY`. Default
#' is the minimum age of maturity.
#' @export
calc_Sgen <- function(Mjuv, fec, p_female, rel_F, vulPT, vulT, p_mature, s_enroute, n_g = 1, p_LHG = 1,
                      SRRpars, SMSY, F_search = c(1e-8, 100), nyears) {

  if (!is.matrix(Mjuv)) Mjuv <- matrix(Mjuv, length(Mjuv), n_g)
  if (!is.matrix(p_mature)) p_mature <- matrix(p_mature, length(p_mature), n_g)
  if (!is.matrix(fec)) fec <- matrix(fec, length(fec), n_g)

  if (missing(p_LHG)) p_LHG <- rep(1/n_g, n_g)
  if (missing(nyears)) nyears <- which(rowSums(p_mature) > 0)[1]

  opt_Sgen <- try(
    uniroot(
      .calc_Sgen, interval = F_search,
      Mjuv = Mjuv, fec = fec, p_female = p_female, rel_F = rel_F,
      vulPT = vulPT, vulT = vulT, p_mature = p_mature, s_enroute = s_enroute,
      n_g = n_g, p_LHG = p_LHG,
      SRRpars = SRRpars, SMSY = SMSY, nyears = nyears
    ),
    silent = TRUE
  )

  if (is.character(opt_Sgen)) {
    Sgen <- NA
  } else {
    Sgen_out <- .calc_Sgen(
      .F = opt_Sgen$root,
      Mjuv = Mjuv, fec = fec, p_female = p_female, rel_F = rel_F,
      vulPT = vulPT, vulT = vulT, p_mature = p_mature, s_enroute = s_enroute,
      n_g = n_g, p_LHG = p_LHG,
      SRRpars = SRRpars, SMSY = SMSY, nyears = nyears, opt = FALSE
    )
    Sgen <- sum(Sgen_out$Sgen$Spawners)
    attr(Sgen, "Sgen") <- Sgen_out$Sgen
    attr(Sgen, "proj") <- Sgen_out$proj
  }

  return(Sgen)
}

.calc_Sgen <- function(.F, Mjuv, fec, p_female, gamma = 1, rel_F, vulPT, vulT, p_mature, s_enroute = 1,
                       n_g, p_LHG, SRRpars, popt = 1, SMSY, nyears, opt = TRUE) {

  # Equilibrium quantities at .F
  Sgen_ref <- .calc_eq(
    .F, Mjuv, fec, p_female, gamma, rel_F, vulPT, vulT, p_mature, s_enroute, n_g, p_LHG,
    SRRpars, opt = FALSE, aggregate_age = FALSE
  )

  if (Sgen_ref$Egg <= 0) {
    Sgen_plusone <- 0
  } else {
    maxage <- length(fec)
    if (missing(nyears)) nyears <- maxage

    Njuv <- Return <- Spawner <- array(0, c(maxage, nyears, n_g))
    Egg <- rep(0, nyears)
    Njuv[, 1, ] <- Sgen_ref$Njuv

    for (y in 1:nyears) {
      Return[, y, ] <- Njuv[, y, ] * p_mature * s_enroute
      Spawner[, y, ] <- Return[, y, ] * s_enroute

      Egg[y] <- sum(Spawner[, y, ] * gamma * p_female * fec) # sum across LHG

      if (y < nyears) {
        Smolt <- calc_smolt(
          Egg[y],
          kappa = SRRpars["kappa"], capacity = SRRpars["capacity"], Smax = SRRpars["Smax"], phi = SRRpars["phi"],
          tau = SRRpars["tau"], fitness_loss = 1, SRrel = as.character(SRRpars["SRrel"])
        ) %>%
          as.numeric()
        Njuv[1, y+1, ] <- p_LHG * Smolt
        Njuv[2:maxage, y+1, ] <- Njuv[2:maxage - 1, y, ] * exp(-Mjuv[2:maxage - 1, ]) * (1 - p_mature[2:maxage - 1, ])
      }
    }

    Sgen_plusone <- sum(Spawner[, nyears, ])
  }

  if (opt) {
    return(Sgen_plusone - popt * SMSY)
  } else {
    return(list(Sgen = Sgen_ref, proj = list(Njuv = Njuv, Return = Return, Spawner = Spawner, Egg = Egg)))
  }
}


.calc_eq <- function(.F, Mjuv, fec, p_female, gamma = 1, rel_F, vulPT, vulT, p_mature, s_enroute = 1, n_g, p_LHG,
                     SRRpars, maximize = c("MSY", "MER"), opt = TRUE, aggregate_age = TRUE) {

  maximize <- match.arg(maximize)

  if (maximize == "MER") rel_F <- c(0, 1)

  FPT <- vulPT * .F * rel_F[1]
  FT <- vulT * .F * rel_F[2]

  surv_juv <- sapply(1:n_g, function(g) p_LHG[g] * calc_survival(Mjuv[, g] + FPT[-length(FPT)], p_mature[, g])) # First semester due to exploitation
  surv_return <- surv_juv * p_mature
  surv_esc <- surv_return * exp(-FT)
  surv_spawn <- surv_esc * s_enroute

  EPR <- sum(surv_spawn * gamma * p_female * fec) # Egg per smolt at F

  Smolt <- calc_smolt(
    EPR,
    kappa = SRRpars["kappa"], capacity = SRRpars["capacity"], Smax = SRRpars["Smax"], phi = SRRpars["phi"],
    tau = SRRpars["tau"], fitness_loss = 1, SRrel = as.character(SRRpars["SRrel"]), per_recruit = TRUE
  ) %>%
    as.numeric()

  Njuv <- Smolt * surv_juv
  Return <- Smolt * surv_return

  KPT <- Njuv * (1 - exp(-FPT))
  KT <- Return * (1 - exp(-FT))

  Spawners <- Smolt * surv_spawn

  if (opt) {

    if (maximize == "MSY") {
      return(sum(KPT, KT))
    } else {
      return(sum(Return) - sum(Spawners))
    }


  } else {

    output <- list(
      KPT = KPT,
      KT = KT,
      FPT = max(FPT),
      FT = max(FT),
      UPT = max(1 - exp(-FPT)),
      UT = max(1 - exp(-FT)),
      Smolt = Smolt,
      Njuv = Njuv,
      Return = Return,
      Escapement = Smolt * surv_esc,
      Spawners = Spawners,
      Egg = Smolt * EPR
    )

    if (aggregate_age) output <- lapply(output, sum)

    return(output)
  }
}


#' Ricker reference points
#'
#' Compute reference points (Umsy, Smsy, and Sgen) from Ricker stock-recruit function
#' based on Scheuerell (2016).
#'
#' @param loga Numeric, alpha parameter (returns per spawner) in the Ricker function: \eqn{R=S\exp(\log(a)-bS)}
#' where `S` is the number of spawners and `R` is the return
#' @param b Numeric, beta parameter
#'
#' @importFrom gsl lambert_W0
#'
#' @returns All three functions return a numeric
#' @references Scheuerell, M.D. 2016. An explicit solution for
#' calculating optimum spawning stock size from Ricker’s stock recruitment model. PeerJ 4:e1623. \doi{10.7717/peerj.1623}
#' @seealso [calc_ref()]
#' @examples
#' a <- 3
#' Smax <- 100
#' b <- 1/Smax
#'
#' calc_Smsy_Ricker(log(a), b)
#' calc_Umsy_Ricker(log(a))
#' calc_Sgen_Ricker(log(a), b)
#' @export
calc_Smsy_Ricker <- function(loga,b) {
  #gives the min Ricker log-likelihood
  Smsy <- (1 - gsl::lambert_W0(exp(1 - loga))) /b

  return(Smsy)
}

#' @rdname calc_Smsy_Ricker
#' @export
calc_Umsy_Ricker <- function(loga) {
  #gives the min Ricker log-likelihood
  umsy <- (1 - gsl::lambert_W0(exp(1 - loga)))

  return(umsy)
}



#' @rdname calc_Smsy_Ricker
#' @export
calc_Sgen_Ricker <- function(loga, b){
  sMSY <- ( 1 - gsl::lambert_W0 (exp ( 1 - loga) ) ) / b
  a <- exp(loga)

  return(-1/b*gsl::lambert_W0(-b*sMSY/a))
}

#' Predict smolt production
#'
#' @description Internal function that predicts the natural origin or hatchery origin smolt production from the
#' escapement, and saves state variables to [salmonMSE::salmonMSE_env].
#' - `smolt_func()` is the population dynamics function
#' - `makeRel_smolt()` generates a list for openMSE to use in the simulations
#'
#' @param Nage_NOS Matrix `[n_age, n_g]` of natural escapement from openMSE. `n_g` is the number of life history groups (sub groups within cohorts)
#' that contribute to spawning.
#' @param Nage_HOS Matrix `[n_age, n_r]` of hatchery escapement from openMSE. `n_r` is the number of release strategies (sub groups within cohorts)
#' that contribute to spawning.
#' @param Nage_stray Matrix `[nage, n_stray]` of hatchery escapement from donor populations that will stray to the recipient population. `stray_args$stray_matrix`
#' determines the proportions of the donor escapement that will move to the recipient population.
#' @param x Integer, simulation number from openMSE
#' @param y Integer, simulation year (including historical years)
#' @param output Character, whether to predict the natural origin or hatchery origin smolt production
#' @param s_enroute Numeric, en route survival of the escapement to the spawning grounds
#' @param p_female Vector `[maxage]`, proportion female for calculating the egg production.
#' @param fec Array `[nsim, maxage, nyears+proyears]`. The fecundity at age schedule of spawners
#' @param SRRpars Data frame containing stock recruit parameters for natural smolt production from egg production.
#' Column names include: SRrel, kappa, capacity, Smax, phi
#' @param hatchery_args Named list containing various arguments controlling broodtake and hatchery production. See details below.
#' @param fitness_args Named list containing various arguments controlling population fitness from hatchery production.
#' Names include: fitness_type, omega2, theta, fitness_variance, heritability, zbar_start, fitness_floor, rel_loss
#' @param habitat_args Named list with arguments sontrolling freshwater life stage survival.
#' Names include: use_habitat (logical), Habitat (\linkS4class{Habitat} object)
#' @param stray_args Named list with arguments controlling strays in and out of the local population
#' Names include:
#' @param s Integer, salmonMSE population index. Used to report variables to [salmonMSE::salmonMSE_env].
#' @section hatchery_args:
#' Hatchery control parameters are included in a named list with the following arguments:
#'
#' - `egg_target` Numeric, target egg production for hatchery. Set to zero for no hatchery production.
#' - `ptarget_NOB` Numeric, the target proportion of the natural origin broodtake relative to the overall broodtake
#' - `pmax_NOB` Numeric, the maximum proportion of the natural origin escapement to be used as broodtake
#' - `fec_brood` Array `[nsim, maxage, nyears+proyears]`. the fecundity at age schedule of broodtake to calculate the total hatchery egg production
#' - `s_yearling` Numeric, the survival of eggs to the smolt life stage (for yearling release)
#' - `s_subyearling` Numeric. the survival of eggs to subyearling life stage (for subyearling release)
#' - `p_yearling` Numeric, the proportion of annual releases at the yearling life stage (vs. subyearling)
#' - `phatchery` Numeric, the proportion of the hatchery origin escapement that return to the hatchery,
#' for example, by removal from spawning grounds or swim-in facilities. These fish are available for broodtake.
#' - `premove_HOS` Numeric or function, the proportion of the hatchery origin fish to be removed from the spawning
#' grounds (in order to ensure a high proportion of NOS or for an in-river fishery). These fish are not available for broodtake.
#' For example, a value less than one can represent imperfect implementation of weir removal.
#' - `premove_NOS` Numeric or function, the proportion of the natural origin fish to be removed from the spawning
#' grounds, for example, an in-river fishery.
#' - `s_prespawn` Numeric, the survival of broodtake prior to egg production. `1 - s_prespawn` is the proportion of fish
#' not used for hatchery purposes, e.g., mortality or other resesarch purposes. Used to back-calculate the broodtake.
#' - `gamma` Numeric, the relative reproductive success of hatchery origin spawners (relative to natural origin spawners)
#' - `m` Numeric, mark rate for selective broodtake
#' @returns
#' - `smolt_func()` returns a numeric for the ratio of the realized smolt production vs. the hypothetical value if there were no
#' hatchery, en route mortality, or habitat improvement
#' - `makeRel_smolt()` returns a list that is passed to openMSE as a inter-population relationship
#' @keywords internal
smolt_func <- function(Nage_NOS, Nage_HOS, Nage_stray, x = -1, y, output = c("natural", "hatchery"),
                       s_enroute, p_female, fec, SRRpars, # Spawning (natural production)
                       hatchery_args, fitness_args, habitat_args, stray_args, s, g, prop_LHG, r) {

  if (y %% 2 != 0 || x < 0) return(0) # Odd time steps
  nyears <- 4
  yy <- max(floor((nyears-4)/2), 1)
  fec_t <- fec[x, , yy]
  if (!is.null(hatchery_args$fec_brood)) fec_brood_t <- hatchery_args$fec_brood[x, , yy]

  output <- match.arg(output)
  Nage_NOS[is.na(Nage_NOS)] <- 0

  # Hatchery fish
  if (missing(Nage_HOS)) Nage_HOS <- matrix(0, nrow(Nage_NOS), 1)
  Nage_HOS[is.na(Nage_HOS)] <- 0

  # Add incoming strays (external and movement from donor populations)
  stray_external <- stray_args$stray_external
  if (!missing(Nage_stray) && sum(Nage_stray)) {
    s_donor <- setdiff(stray_args$p_donor$s, s)
    for (ss in s_donor) {
      if (ss != s) {
        p <- stray_args$p_donor$p[stray_args$p_donor$s == ss]
        stray_external <- stray_external + stray_args$stray_matrix[ss, s] * Nage_stray[, paste0("Nage_", p)]
      }
    }
    # Remove donor strays
    Nage_HOS <- stray_args$stray_matrix[s, s] * Nage_HOS
  }

  # En-route mortality
  Nage_NOS_enroute <- Nage_NOS * s_enroute
  Nage_HOS_enroute <- Nage_HOS * s_enroute
  stray_external_enroute <- stray_external * s_enroute

  # Broodtake
  custom_brood_rule <- is.function(hatchery_args$f_brood) && !is.null(formals(hatchery_args$f_brood))
  if (hatchery_args$egg_target > 0 && sum(Nage_NOS, Nage_HOS)) {

    if (!custom_brood_rule) { # AHA approach
      Nage_NOS_avail_brood <- Nage_NOS_enroute * hatchery_args$pmax_esc
      Nage_HOS_avail_brood <- Nage_HOS_enroute * hatchery_args$pmax_esc
      broodtake <- calc_broodtake(
        NOR_escapement = Nage_NOS_avail_brood,
        HOR_escapement = Nage_HOS_avail_brood,
        stray_external_enroute,
        hatchery_args$brood_import,
        hatchery_args$ptarget_NOB,
        hatchery_args$pmax_NOB,
        hatchery_args$phatchery,
        hatchery_args$egg_target,
        hatchery_args$p_female,
        fec_brood_t,
        hatchery_args$s_prespawn,
        hatchery_args$m
      )
    } else { # Custom brood approach
      broodtake <- calc_broodtake_custom(
        f_brood = hatchery_args$f_brood,
        NOR_escapement = Nage_NOS_enroute,
        HOR_escapement = Nage_HOS_enroute,
        stray_external_enroute,
        hatchery_args$p_female,
        fec_brood_t,
        hatchery_args$s_prespawn,
        hatchery_args$m,
        hatchery_args$egg_target
      )
    }

    egg_NOB <- broodtake$egg_NOB
    egg_HOB <- broodtake$egg_HOB_unmarked + broodtake$egg_HOB_marked
    pNOB <- broodtake$pNOB

    hatchery_production <- calc_yearling(
      egg_NOB + egg_HOB,
      hatchery_args$s_yearling,
      hatchery_args$s_subyearling,
      hatchery_args$p_yearling,
      hatchery_args$p_subyearling
    )

    # List of preliminary spawners and in-river removals after broodtake
    spawners <- calc_spawners(
      broodtake, Nage_NOS_enroute, Nage_HOS_enroute, stray_external_enroute,
      ifelse(custom_brood_rule, NA, hatchery_args$phatchery),
      hatchery_args$premove_HOS, hatchery_args$premove_NOS, hatchery_args$m
    )

  } else {

    # Check for same format as list returned by .broodtake_func
    broodtake <- list(
      egg_NOB = 0,
      egg_HOB_unmarked = 0,
      egg_HOB_marked = 0,
      ptake_unmarked = 0,
      ptake_marked = 0,
      pNOB = 0,
      NOB = array(0, dim(Nage_NOS)),
      HOB_unmarked = array(0, dim(Nage_HOS)),
      HOB_marked = array(0, dim(Nage_HOS)),
      HOB_import = rep(0, nrow(Nage_NOS)),
      HOB_stray = array(0, dim(Nage_HOS))
    )
    egg_NOB <- egg_HOB <- pNOB <- 0

    hatchery_production <- list(
      yearling = rep(0, ncol(Nage_HOS)),
      subyearling = rep(0, ncol(Nage_HOS))
    )

    # Preliminary spawners after broodtake
    spawners <- calc_spawners(
      broodtake, Nage_NOS_enroute, Nage_HOS_enroute, stray_external_enroute,
      NA, # phatchery
      hatchery_args$premove_HOS, hatchery_args$premove_NOS, hatchery_args$m
    )

  }

  # Add habitat density-dependence on pre-spawn mortality
  if (habitat_args$use_habitat) {

    Habitat <- habitat_args$Habitat

    # By parental life cycle group x age
    NOS <- calc_SRR(
      spawners$NOS, sum(spawners$NOS, spawners$HOS),
      p = Habitat@prespawn_prod, capacity = Habitat@prespawn_capacity,
      type = Habitat@prespawn_rel
    )

    # By parental release strategy x age
    HOS <- calc_SRR(
      spawners$HOS, sum(spawners$NOS, spawners$HOS),
      p = Habitat@prespawn_prod, capacity = Habitat@prespawn_capacity,
      type = Habitat@prespawn_rel
    )

  } else {
    NOS <- spawners$NOS
    HOS <- spawners$HOS # Includes local and strays!
  }
  HOS_effective <- HOS * hatchery_args$gamma
  if (!sum(HOS) && is.null(hatchery_args$gamma)) HOS_effective <- HOS

  # Spawners weighted by fecundity
  pHOSeff <- sum(fec_t * HOS_effective)/sum(fec_t * NOS, fec_t * HOS_effective)
  pHOScensus <- sum(fec_t * HOS)/sum(fec_t * NOS, fec_t * HOS)

  # Natural egg production in the absence of fitness effects by life cycle group (NOS) and release strategy (HOS)
  Egg_NOS <- colSums(NOS * p_female * fec_t)
  Egg_HOS <- colSums(HOS_effective * p_female * fec_t)

  total_egg <- sum(Egg_NOS, Egg_HOS)

  # Fitness calculations
  if (x > 0 && nrow(salmonMSE_env$Ford) && any(fitness_args$fitness_type == "Ford")) {

    # Get zbar from salmonMSE_env
    zbar_prev <- filter(salmonMSE_env$Ford, .data$x == .env$x, .data$s == .env$s)

    if (nrow(zbar_prev)) {
      maxage <- nrow(NOS)
      zbar_brood <- matrix(0, maxage, 2) # Column 1 = natural environment, 2 = hatchery environment

      # Trait value by brood year
      for (a in 1:maxage) {
        if (sum(NOS[a, ]) || sum(broodtake$NOB[a, ])) {
          zbar1 <- dplyr::filter(zbar_prev, .data$t == .env$y - 2 * .env$a, .data$type == "natural") %>%
            pull(.data$zbar)
          if (!length(zbar1)) stop("Cannot find zbar") # zbar1 <- fitness_args$zbar_start[1]
          zbar_brood[a, 1] <- zbar1
        }
        if (sum(HOS_effective[a, ]) || sum(broodtake$HOB[a, ]) || sum(broodtake$HOB_import[a])) {
          zbar2 <- dplyr::filter(zbar_prev, .data$t == .env$y - 2 * .env$a, .data$type == "hatchery") %>%
            pull(.data$zbar)
          if (!length(zbar2)) stop("Cannot find zbar") # zbar2 <- fitness_args$zbar_start[2]
          zbar_brood[a, 2] <- zbar2
        }
      }

      zbar <- calc_zbar(
        rowSums(NOS), rowSums(HOS_effective), rowSums(broodtake$NOB),
        rowSums(broodtake$HOB_marked + broodtake$HOB_unmarked) + broodtake$HOB_import,
        fec_t, fec_brood_t, zbar_brood,
        fitness_args$fitness_variance, fitness_args$theta, fitness_args$phenotype_variance, fitness_args$heritability[x]
      )
    } else {
      zbar <- fitness_args$zbar_start
    }

    # Fitness in the natural and hatchery environments
    fitness <- rep(1, 2)

    if (sum(Nage_NOS, Nage_HOS)) {
      for (i in 1:2) {
        if (fitness_args$fitness_type[i] == "Ford") {
          fitness[i] <- calc_fitness(
            zbar[i], fitness_args$theta[i], fitness_args$fitness_variance,
            fitness_args$phenotype_variance, fitness_args$fitness_floor
          )
        }
      }
    }

    fitness_loss <- outer(fitness, fitness_args$rel_loss, "^")
  } else {
    zbar <- rep(100, 2)
    fitness <- c(1, 1)
    fitness_loss <- matrix(1, 2, 3)
  }

  # Egg production from egg output, then fry production (stage-specific density-dependent relationship as habitat variables)
  if (habitat_args$use_habitat) {
    t <- max(1, 0.5 * (y - habitat_args$nyears_om))
    Habitat <- habitat_args$Habitat

    ### Incubation survival
    # By parental life cycle group
    Egg_prod_NOS <- calc_SRR(
      Egg_NOS, sum(Egg_HOS, Egg_NOS),
      p = Habitat@egg_prod, capacity = Habitat@egg_capacity,
      type = Habitat@egg_rel
    )
    # By parental release strategy
    Egg_prod_HOS <- calc_SRR(
      Egg_HOS, sum(Egg_HOS, Egg_NOS),
      p = Habitat@egg_prod, capacity = Habitat@egg_capacity,
      type = Habitat@egg_rel
    )

    # By parental life cycle group
    Fry_NOS <- calc_SRR(
      Egg_prod_NOS, sum(Egg_prod_NOS, Egg_prod_HOS),
      p = Habitat@fry_prod * fitness_loss[1, 1], capacity = Habitat@fry_capacity * fitness_loss[1, 1],
      type = Habitat@fry_rel
    ) * Habitat@fry_sdev[x, t]

    # By parental release strategy
    Fry_HOS <- calc_SRR(
      Egg_prod_HOS, sum(Egg_prod_NOS, Egg_prod_HOS),
      p = Habitat@fry_prod * fitness_loss[1, 1], capacity = Habitat@fry_capacity * fitness_loss[1, 1],
      type = Habitat@fry_rel
    ) * Habitat@fry_sdev[x, t]

  } else {
    # Egg and fry production from egg output (by parental life cycle group/release strategy)
    Egg_prod_NOS <- Egg_NOS
    Egg_prod_HOS <- Egg_HOS

    Fry_NOS <- fitness_loss[1, 1] * Egg_prod_NOS
    Fry_HOS <- fitness_loss[1, 1] * Egg_prod_HOS
  }

  # Hatchery production after fitness loss by release strategy (next generation)
  yearling <- hatchery_production$yearling * fitness_loss[2, 1] * fitness_loss[2, 2]
  subyearling <- hatchery_production$subyearling * fitness_loss[2, 1]

  # Juveniles in freshwater environment together that experience density-dependent survival
  total_fry_DD <- sum(Fry_NOS, Fry_HOS)
  if (sum(yearling) && hatchery_args$yearling_DD) total_fry_DD <- sum(total_fry_DD, yearling)
  if (sum(subyearling) && hatchery_args$subyearling_DD) total_fry_DD <- sum(total_fry_DD, subyearling)

  # Smolt production
  SRrel <- match.arg(SRRpars[x, "SRrel"], choices = c("BH", "Ricker"))

  # Hatchery releases in competition with natural-origin juveniles with fitness effects (by release strategy)
  if (sum(yearling) && hatchery_args$yearling_DD) {
    if (habitat_args$use_habitat) {
      smolt_yearling <- calc_SRR(
        yearling, total_fry_DD,
        p = Habitat@smolt_prod * fitness_loss[2, 2], capacity = Habitat@smolt_capacity * fitness_loss[2, 2],
        type = Habitat@smolt_rel
      )
    } else {
      smolt_yearling <- calc_smolt(
        yearling, total_fry_DD,
        SRRpars[x, "kappa"], SRRpars[x, "capacity"], SRRpars[x, "Smax"], SRRpars[x, "phi"], SRRpars[x, "tau"],
        fitness_loss[2, 1] * fitness_loss[2, 2],
        SRrel
      )
    }
  } else {
    smolt_yearling <- yearling
  }

  if (sum(subyearling) && hatchery_args$subyearling_DD) {
    if (habitat_args$use_habitat) {
      smolt_subyearling <- calc_SRR(
        subyearling, total_fry_DD,
        p = Habitat@smolt_prod * fitness_loss[2, 2], capacity = Habitat@smolt_capacity * fitness_loss[2, 2],
        type = Habitat@smolt_rel
      )
    } else {
      smolt_subyearling <- calc_smolt(
        subyearling, total_fry_DD,
        SRRpars[x, "kappa"], SRRpars[x, "capacity"], SRRpars[x, "Smax"], SRRpars[x, "phi"], SRRpars[x, "tau"],
        fitness_loss[2, 1] * fitness_loss[2, 2],
        SRrel
      )
    }
  } else {
    smolt_subyearling <- subyearling
  }

  # Total outmigrating hatchery releases
  smolt_rel <- smolt_yearling + smolt_subyearling
  if (output == "hatchery") return(smolt_rel[r])

  # Natural fry production for life history group g (next generation)
  Fry_NOS_g <- sum(Fry_NOS) * prop_LHG
  Fry_HOS_g <- sum(Fry_HOS) * prop_LHG

  # Natural smolt production for life history group g (next generation)
  if (habitat_args$use_habitat) {
    smolt_NOS <- calc_SRR(
      Fry_NOS_g, total_fry_DD,
      p = Habitat@smolt_prod * fitness_loss[1, 2], capacity = Habitat@smolt_capacity * fitness_loss[1, 2],
      type = Habitat@smolt_rel
    ) * Habitat@smolt_sdev[x, t]
    smolt_HOS <- calc_SRR(
      Fry_HOS_g, total_fry_DD,
      p = Habitat@smolt_prod * fitness_loss[1, 2], capacity = Habitat@smolt_capacity * fitness_loss[1, 2],
      type = Habitat@smolt_rel
    ) * Habitat@smolt_sdev[x, t]
  } else {
    smolt_NOS <- calc_smolt(
      Fry_NOS_g, total_fry_DD,
      SRRpars[x, "kappa"], SRRpars[x, "capacity"], SRRpars[x, "Smax"], SRRpars[x, "phi"], SRRpars[x, "tau"],
      fitness_loss[1, 2],
      SRrel
    )
    smolt_HOS <- calc_smolt(
      Fry_HOS_g, total_fry_DD,
      SRRpars[x, "kappa"], SRRpars[x, "capacity"], SRRpars[x, "Smax"], SRRpars[x, "phi"], SRRpars[x, "tau"],
      fitness_loss[1, 2],
      SRrel
    )
  }

  # Predicted egg production naively calculated by openMSE
  # if there were no hatchery production, habitat improvement, enroute mortality, or multiple LHG
  Egg_openMSE <- sum(Nage_NOS[, g] * p_female * fec_t)

  # Predicted juveniles by openMSE
  if (habitat_args$use_habitat) {
    smolt_NOS_SRR <- 1
  } else {
    smolt_NOS_SRR <- calc_smolt(
      Egg_openMSE, Egg_openMSE,
      SRRpars[x, "kappa"], SRRpars[x, "capacity"], SRRpars[x, "Smax"], SRRpars[x, "phi"], SRRpars[x, "tau"],
      SRrel = SRrel
    )
  }

  # Perr_y is an adjustment to the predicted juveniles (as a result of everything happening in this function,
  # including brood, en-route mortality)
  #
  # MICE parameter (natural production, Perr = process error for recruitment of new cohort)
  # Updated here and passed to openMSE operating model so that brood year juveniles in marine life stage
  # in openMSE is equal to numbers predicted in this function (smolt_NOS + smolt_HOS)
  Perr_y <- (smolt_NOS[g] + smolt_HOS[g])/smolt_NOS_SRR
  if (!smolt_NOS_SRR) Perr_y[] <- 0

  # Write all state variables to salmonMSE_env
  if (output == "natural" && g == 1) {
    #no_hatchery <- hatchery_args$egg_target == 0
    #if (no_hatchery) r <- 1

    nr <- ncol(Nage_HOS)

    df_H <- lapply(1:nr, function(r) {
      data.frame(
        x = x,
        s = s,
        r = r,
        t = y, # Even time steps (remember MICE predicts Perr_y for next time step)
        a = 1:nrow(Nage_HOS),
        Esc_HOS = Nage_HOS[, r],
        HOB = broodtake$HOB_unmarked[, r] + broodtake$HOB_marked[, r] + broodtake$HOB_stray[, r],
        HOB_stray = broodtake$HOB_stray[, r],
        HOS = HOS[, r],
        HOS_stray = spawners$HOS_stray[, r],
        HOS_effective = HOS_effective[, r],
        HOB_import = if (r == 1) broodtake$HOB_import else rep(NA, nrow(Nage_HOS)),
        HOS_remove = spawners$HO_remove[, r]
      )
    }) %>%
      bind_rows()
    salmonMSE_env$H <- rbind(salmonMSE_env$H, df_H)

    df_stateH <- lapply(1:nr, function(r) {
      data.frame(
        x = x,
        s = s,
        r = r,
        t = y, # Even time steps (remember MICE predicts Perr_y for next time step)
        Egg_HOS = Egg_HOS[r], # Spawning output by RS r of this generation
        yearling = yearling[r],
        subyearling = subyearling[r],
        smolt_rel = smolt_rel[r]
      )
    }) %>%
      bind_rows()
    salmonMSE_env$stateH <- rbind(salmonMSE_env$stateH, df_stateH)

    ng <- ncol(Nage_NOS)
    df_N <- lapply(1:ng, function(g) {
      data.frame(
        x = x,
        s = s,
        g = g,
        t = y, # Even time steps (remember MICE predicts Perr_y for next time step)
        a = 1:nrow(Nage_NOS),
        Esc_NOS = Nage_NOS[, g],
        NOB = broodtake$NOB[, g],
        NOS = NOS[, g],
        NOS_remove = spawners$NO_remove[, g]
      )
    }) %>%
      bind_rows()
    salmonMSE_env$N <- rbind(salmonMSE_env$N, df_N)

    df_stateN <- lapply(1:ng, function(g) {
      data.frame(
        x = x,
        s = s,
        g = g,
        t = y, # Even time steps (remember MICE predicts Perr_y for next time step)
        Egg_NOS = Egg_NOS[g], # Egg production by LHG g of this generation
        Fry_NOS = Fry_NOS_g[g],  # Fry production assigned to LHG g for next generation
        Fry_HOS = Fry_HOS_g[g],
        smolt_NOS = smolt_NOS[g],
        smolt_HOS = smolt_HOS[g],
        pNOB = as.numeric(pNOB),
        pHOSeff = as.numeric(pHOSeff),
        pHOScensus = as.numeric(pHOScensus),
        Perr_y = Perr_y
      )
    }) %>%
      bind_rows()
    salmonMSE_env$stateN <- rbind(salmonMSE_env$stateN, df_stateN)

    # Save zbar
    df_Ford <- data.frame(
      x = x,
      s = s,
      t = y, # Even time steps (remember MICE predicts Perr_y for next time step)
      type = c("natural", "hatchery"),
      zbar = zbar,
      fitness = fitness
    )
    salmonMSE_env$Ford <- rbind(salmonMSE_env$Ford, df_Ford)
  }

  if (is.na(Perr_y)) stop("Perr_y is NA. Check calculations in smolt_func()")

  return(Perr_y)
}

#' @rdname smolt_func
#' @param p_smolt Integer, the population index for the smolt production in the openMSE model, corresponding to `output`
#' @param p_natural Integer vector, the population index for the natural origin escapement in the openMSE model. Can be more than one
#' if spawning is from multiple life history groups
#' @param p_hatchery Integer vector, the population index for the hatchery origin escapement in the openMSE model. Can be more than one
#' if there are multiple release strategies. Set to `NULL` for no hatchery production
#' @param p_stray Integer vector, population index for the hatchery strays in multi-system models
#' @param g Integer for the life history group of natural origin fish to pass the parameter back to openMSE (if `output = "natural"`)
#' @param prop_LHG Numeric vector, proportion of the egg production assign to life history groups (sums up to one) (only used if `output = "natural"`)
#' @param r Integer for the release strategy of hatchery origin fish to pass the parameter back to openMSE (if `output = "hatchery"`)
makeRel_smolt <- function(p_smolt = 1, s = 1, p_natural, p_hatchery = NULL,
                          p_stray = NULL,
                          maxage,
                          output = c("natural", "hatchery"), s_enroute,
                          p_female, fec, SRRpars,  # Spawning (natural production)
                          hatchery_args, fitness_args, habitat_args, stray_args, g, prop_LHG, r) {

  output <- match.arg(output)

  .smolt_func <- smolt_func

  formals(.smolt_func)$output <- output
  formals(.smolt_func)$s_enroute <- s_enroute
  formals(.smolt_func)$p_female <- p_female
  formals(.smolt_func)$fec <- fec

  formals(.smolt_func)$SRRpars <- SRRpars

  formals(.smolt_func)$hatchery_args <- hatchery_args
  formals(.smolt_func)$fitness_args <- fitness_args
  formals(.smolt_func)$habitat_args <- habitat_args
  formals(.smolt_func)$stray_args <- stray_args
  formals(.smolt_func)$s <- s

  if (output == "natural") {
    formals(.smolt_func)$g <- g
    formals(.smolt_func)$prop_LHG <- prop_LHG
  }
  if (output == "hatchery") formals(.smolt_func)$r <- r

  N_natural <- matrix(1, maxage, length(p_natural))

  if (!is.null(p_hatchery)) { # Need p_hatchery to use p_stray
    N_hatchery <- matrix(0, maxage, length(p_hatchery))
    N_stray <- matrix(0, maxage, length(p_stray))

    Perr_y <- .smolt_func(N_natural, N_hatchery, x = -1, y = 1)
    model <- c(Perr_y = Perr_y, colSums(N_natural), colSums(N_hatchery), colSums(N_stray), x = -1, y = 1)
    input <- paste0("Nage_", c(p_natural, p_hatchery, p_stray))

    natural_origin <- c(rep(TRUE, length(p_natural)), rep(FALSE, length(c(p_hatchery, p_stray))))
    stray <- c(rep(FALSE, length(c(p_natural, p_hatchery))), rep(TRUE, length(p_stray)))
  } else {
    Perr_y <- .smolt_func(N_natural, x = -1, y = 1)
    model <- c(Perr_y = Perr_y, colSums(N_natural), x = -1, y = 1)
    input <- paste0("Nage_", p_natural)

    natural_origin <- rep(TRUE, length(p_natural))
    stray <- rep(FALSE, length(p_natural))
  }

  response <- paste0("Perr_y_", p_smolt)
  terms <- c(response, input, "x", "y")

  out <- list(
    func = .smolt_func,
    model = structure(model, names = terms),
    fitted.values = Perr_y,
    CV = 0,
    terms = terms,
    lag = "next",
    type = "Recruitment deviation",
    Rel = switch(output,
                 "natural" = "Smolt natural production from escapement",
                 "hatchery" = "Smolt hatchery releases from escapement"),
    natural_origin = natural_origin,
    stray = stray
  )
  structure(out, class = "RelSmolt")
}

#' @importFrom stats predict
#' @export
predict.RelSmolt <- function(object, newdata, ...) {

  if (missing(newdata)) return(object$fitted.values)

  vars <- names(newdata)

  vars_Rel <- names(object$model)[-1]
  Nage <- vars_Rel[grepl("Nage", vars_Rel)]
  Nage_NOS <- Nage[object$natural_origin]
  Nage_HOS <- Nage[!object$natural_origin & !object$stray]
  if (any(object$stray)) Nage_stray <- Nage[object$stray]

  do_hatchery <- length(Nage_HOS) > 0

  val <- sapply(1:nrow(newdata), function(i) {
    Esc_NOS <- sapply(Nage_NOS, function(j) as.numeric(newdata[i, grepl(j, vars)])) # matrix n_age x ng
    n_age <- nrow(Esc_NOS)
    a <- seq(3, n_age, 2)
    if (sum(Esc_NOS[!a, ])) stop("Internal salmonMSE error: there is natural escapement in even age classes")

    if (do_hatchery) {
      Esc_HOS <- sapply(Nage_HOS, function(j) as.numeric(newdata[i, grepl(j, vars)]))
      if (sum(Esc_HOS[!a, ])) stop("Internal salmonMSE error: there is hatchery escapement in even age classes")
      if (any(object$stray)) {
        Esc_stray <- sapply(Nage_stray, function(j) as.numeric(newdata[i, grepl(j, vars)]))
        if (sum(Esc_stray[!a, ])) stop("Internal salmonMSE error: there are hatchery strays in even age classes")
      } else {
        Esc_stray <- matrix(0, n_age, 1)
      }
    } else {
      Esc_HOS <- Esc_stray <- matrix(0, n_age, 1)
    }
    x <- newdata[i, "x"]
    y <- newdata[i, "y"]
    object$func(Nage_NOS = Esc_NOS[a, , drop = FALSE], Nage_HOS = Esc_HOS[a, , drop = FALSE],
                Nage_stray = Esc_stray[a, , drop = FALSE], x = x, y = y)
  })

  return(val)

}

#' @importFrom stats simulate
#' @export
simulate.RelSmolt <- function(object, nsim = 1, seed = 1, ...) {

  set.seed(seed)
  stdev <- sqrt(log(1 + object$CV^2))
  do_sim <- object$fitted.values * rlnorm(nsim * length(object$fitted.values), -0.5 * stdev^2, stdev)
  val <- matrix(do_sim, length(object$fitted.values), nsim) %>% as.data.frame() %>%
    structure(names = paste0("sim_", 1:nsim), seed = seed)
  val
}

#' Update natural mortality of juveniles
#'
#' @description Internal function that updates juvenile natural mortality in the marine environement due
#' to fitness
#' - `SAR_fitness()` calculates the new natural mortality value
#' - `makeRel_SAR` generates a list for openMSE to use in the simulations
#' @param x Integer, simulation number from openMSE
#' @param y Integer, simulation year (including historical years)
#' @param envir Character, whether to obtain the fitness value for the natural or hatchery environment.
#' @param rel_loss Numeric, the loss exponent for the juveniles
#' @param s Integer, the salmonMSE population index. Used to search for the fitness value
#' @param nyears Integer, the number of historical years in the operating model
#' @param Mbase Array `[nsim, n_age, proyears]` the base natural mortality value in the openMSE operating model.
#' @returns
#' - `smolt_func()` returns a numeric for the ratio of the realized smolt production vs. the hypothetical value if there were no
#' hatchery, en route mortality, or habitat improvement
#' - `makeRel_smolt` returns a list that is passed to openMSE as a inter-population relationship
#' @keywords internal
SAR_fitness <- function(x = -1, y = 1,
                        envir = c("natural", "hatchery"),
                        rel_loss = 1, s = 1,
                        nyears, Mbase) {

  envir <- match.arg(envir)

  even_time_step <- !y %% 2
  if (even_time_step) {
    n_age <- dim(Mbase)[2]
    fitness_a <- rep(1, n_age)
    for (a in seq(2, n_age, 2)) {
      if (nrow(salmonMSE_env$Ford) && x > 0) {
        fitness_y <- dplyr::filter(
          salmonMSE_env$Ford,
          .data$x == .env$x, .data$s == .env$s, .data$t == y - a, .data$type == envir
        ) %>%
          pull(.data$fitness)

        if (length(fitness_y)) fitness_a[a] <- fitness_y
      }
    }

    fitness_loss <- fitness_a^rel_loss
    M <- Mbase[x, , y - nyears]
    SAR_base <- SAR_loss <- exp(-M)
    SAR_loss[M > .Machine$double.eps] <- SAR_base[M > .Machine$double.eps] * fitness_loss[M > .Machine$double.eps]
    M_loss <- -log(SAR_loss)
    if (any(is.na(M_loss))) stop("M_loss has NA. Check calculations in SAR_fitness()")
    return(M_loss)
  } else {
    return(Mbase[x, , y - nyears])
  }
}

#' @rdname SAR_fitness
#' @param p_smolt Integer, the population index for the juvenile population in the openMSE model
#' @keywords internal
makeRel_SAR <- function(p_smolt = 1, s = 1, envir = c("natural", "hatchery"),
                        rel_loss, nyears, Mbase) {

  envir <- match.arg(envir)

  .SAR_fitness <- SAR_fitness

  formals(.SAR_fitness)$s <- s
  formals(.SAR_fitness)$rel_loss <- rel_loss
  formals(.SAR_fitness)$envir <- envir
  formals(.SAR_fitness)$nyears <- nyears
  formals(.SAR_fitness)$Mbase <- Mbase

  model <- data.frame(M = 1, N = seq(0, 10), x = -1, y = 1)

  response <- paste0("M_", p_smolt)
  input <- paste0("N_", p_smolt)
  terms <- c(response, input, "x", "y")

  out <- list(
    func = .SAR_fitness,
    model = structure(model, names = terms),
    fitted.values = model$M,
    CV = 0,
    terms = terms,
    type = "Natural mortality",
    Rel = "Marine survival reduced by fitness",
    age = seq(0, dim(Mbase)[2] - 1)
  )
  structure(out, class = "SARfitness")
}

#' @export
predict.SARfitness <- function(object, newdata, ...) {
  if (missing(newdata)) return(object$fitted.values)
  val <- sapply(1:nrow(newdata), function(i) object$func(x = newdata[i, "x"], y = newdata[i, "y"]))
  return(val)
}

#' @export
simulate.SARfitness <- simulate.RelSmolt

#' @rdname salmonMSE-int
#' @param NOS Logical, whether the Stock or Fleet object corresponds to natural origin or hatchery origin fish
#' @param s Integer, the population integer for which to create the Stock or Fleet object
#' @param g Integer, the life history group for which to create the Stock object. Not relevant if `NOS = FALSE`
#' @param r Integer, the hatchery release group for which to create the Stock object. Not relevant if `NOS = TRUE`
#' @param stage Character indicating the corresponding salmon life stage of the Stock or Fleet object
#' @return
#' `make_Stock`: List containing a [MSEtool::Stock-class] object and accompanying custom parameters list
#' @export
make_Stock <- function(SOM, s = 1, g = 1, r = 1, NOS = TRUE, stage = c("immature", "return", "escapement")) {
  stage <- match.arg(stage)

  Bio <- SOM@Bio[[s]]
  Habitat <- SOM@Habitat[[s]]
  Harvest <- SOM@Harvest[[s]]
  Hatchery <- SOM@Hatchery[[s]]
  Historical <- SOM@Historical[[s]]

  #### Stock object ----
  Stock <- new("Stock")
  cpars_bio <- list()

  Stock@Name <- paste("Population", s, ifelse(NOS, "NOS:", "HOS:"), stage)

  # Fish spawn at the beginning of the terminal age + 1
  n_age <- 2 * Bio@maxage + 1
  Stock@maxage <- n_age - 1
  nyears_real <- 2
  nyears <- 2 * nyears_real
  proyears <- 2 * SOM@proyears

  # Time steps for first and second half of the year
  t1 <- seq(1, nyears, 2)
  t2 <- seq(2, nyears, 2)

  t1_proj <- nyears + seq(1, proyears, 2)
  t2_proj <- nyears + seq(2, proyears, 2)

  all_t1 <- seq(1, nyears + proyears, 2)
  all_t2 <- seq(2, nyears + proyears, 2)

  a2 <- seq(2, n_age, 2)

  # Arbitrary placeholder, survival to be modeled in cpars
  Stock@M <- Stock@Msd <- c(0, 0)

  # Stock-recruit relationship (custom option)
  Stock@SRrel <- 3
  Stock@h <- rep(0.99, 2) # Arbitrary placeholder, not used
  Stock@R0 <- 1 # Arbitrary placeholder, not used

  # Custom SRR
  if (NOS && !Habitat@use_habitat) {
    cpars_bio$SRR <- make_SRR(Bio)
  } else {
    SRRpars <- data.frame(x = 1:SOM@nsim)
    SRRfun <- function(SB, SRRpars) if(sum(SB, na.rm = TRUE)) rep(1, length(SB)) else 0
    relRfun <- function(SSBpR, SRRpars) return(1)
    SPRcrashfun <- function(SSBpR0, SRRpars) return(0)
    cpars_bio$SRR <- list(SRRfun = SRRfun, SRRpars = SRRpars, relRfun = relRfun, SPRcrashfun = SPRcrashfun)
  }

  # Ignore depletion setup (use cpars$qs = 1, and Eff)
  Stock@D <- c(0, 0)

  # Rec devs to be updated in MICE rel
  Stock@Perr <- Stock@AC <- c(0, 0)

  # Ignore size dynamics
  Stock@Linf <- c(1, 1)
  Stock@K <- c(0.1, 0.1)
  Stock@t0 <- c(0, 0)
  Stock@LenCV <- c(0.05, 0.05)
  Stock@Linfsd <- Stock@Ksd <-
    Stock@L50 <- Stock@L50_95 <- c(0, 0)
  Stock@a <- Stock@b <- 1

  # Ignore area dynamics
  Stock@Size_area_1 <- Stock@Frac_area_1 <- Stock@Prob_staying <- c(0.5, 0.5)

  # Proportion of releases that die (only if there is discarding from mark-selective fishing)
  if (stage == "immature") {
    Stock@Fdisc <- rep(Harvest@release_mort[1], 2)
  } else {
    Stock@Fdisc <- rep(Harvest@release_mort[2], 2)
  }

  # Custom pars
  # openMSE catch is in biomass so we need to set weight at age = 1 so that catch is in numbers, i.e., pieces
  cpars_bio$Wt_age <- array(1, c(SOM@nsim, n_age, nyears + proyears))
  cpars_bio$Wa <- cpars_bio$Wb <- 1 # Arbitrary placeholders, avoids lm fitting in MSEtool::SampleStockPars()

  # Spawning at the last age class, solely for necessary but irrelevant per recruit calculations
  # Proportion maturity that activates return phase is actually modeled by Herm feature
  # Unfished spawners per recruit must be identical between immature and mature NOS population
  cpars_bio$Mat_age <- array(0, c(SOM@nsim, n_age, nyears + proyears))
  if (stage == "escapement") { # All escapement are mature and will spawn in the beginning of the first half of the year
    cpars_bio$Mat_age[, , all_t1] <- 1
  } else {
    cpars_bio$Mat_age[, n_age, ] <- 1 # No spawning, n_age should be an empty age class
  }

  # Ocean survival occurs in the second time step of the year
  cpars_bio$M_ageArray <- array(0, c(SOM@nsim, n_age, nyears + proyears))
  if (stage == "immature") {
    if (NOS) {
      cpars_bio$M_ageArray[, a2[-length(a2)], t2_proj] <- Bio@Mjuv_NOS[, , , g]
    } else {
      cpars_bio$M_ageArray[, a2[-length(a2)], t2_proj] <- Hatchery@Mjuv_HOS[, , , r]
    }
  } else if (stage == "return") {
    cpars_bio$M_ageArray[, n_age, ] <- 0.1 # Return does not experience M, n_age should be an empty age class
  } else if (stage == "escapement") {
    cpars_bio$M_ageArray[, , nyears + seq(1, proyears)] <- 1e8  # Escapement either removed (broodtake, etc.) or spawns at beginning of year and then dies
  }
  cpars_bio$M_ageArray[cpars_bio$M_ageArray < .Machine$double.eps] <- .Machine$double.eps

  # Unfished spawners per recruit must be identical between immature and mature NOS populations (?)
  cpars_bio$Fec_age <- array(0, c(SOM@nsim, n_age, nyears + proyears))
  if (stage == "escapement") {
    age_fec <- seq(3, n_age, 2) # We have to shift the age up once integer age class due to delay in the
    p_female <- array(Bio@p_female, c(Bio@maxage, SOM@nsim, SOM@proyears)) %>% aperm(c(2, 1, 3))
    if (NOS) {
      cpars_bio$Fec_age[, age_fec, t1_proj] <- p_female * Bio@fec
    } else {
      cpars_bio$Fec_age[, age_fec, t1_proj] <- p_female * Hatchery@gamma * Bio@fec
    }
  } else {
    cpars_bio$Fec_age[, n_age, ] <- 1
  }

  #cpars_bio$spawn_time_frac <- rep(0, SOM@nsim) # Spawn at the beginning of the year, i.e., same time step when age-1 smolts appear

  # No plus group - salmon are terminal spawners
  cpars_bio$plusgroup <- 0L

  # Generate recruitment devs from Historical object. Specify future rec dev = 1, updated by hatchery Rel
  cpars_bio$Perr_y <- matrix(0, SOM@nsim, Stock@maxage + nyears + proyears)
  #if (stage == "immature" && NOS) cpars_bio$Perr_y[, Stock@maxage + all_t1] <- 1
  if (stage == "immature" && NOS) cpars_bio$Perr_y[, Stock@maxage + t1_proj] <- 1

  # Escapement needs an initial abundance = 1 in order to generate the smolt production in the first historical year
  #if (stage == "escapement" && NOS) cpars_bio$Perr_y[, Stock@maxage + 1] <- 1

  # Main historical ts, get deviations in fecundity (escapement) and Perr_y (immature)
  #if (stage %in% c("immature", "escapement")) {
  #  pind <- ifelse(NOS, 1, 2)
  #  if (NOS) {
  #    HistPars <- lapply(1:SOM@nsim, function(x) {
  #      calc_Perr_main(
  #        Njuv = Historical@HistNjuv_NOS[x, , , g],
  #        Fjuv = outer(Harvest@vulPT[x, ], Historical@HistFPT[x, , pind]),
  #        Fterm = outer(Harvest@vulT[x, ], Historical@HistFT[x, , pind]),
  #        p_mature = Bio@p_mature[x, , ],
  #        nyears = SOM@nyears,
  #        HistSpawner = if (is.array(Historical@HistSpawner_NOS) && length(Historical@HistSpawner_NOS)) Historical@HistSpawner_NOS[x, , , g] else NULL,
  #        fec = Bio@fec[x, , seq(1, SOM@nyears)],
  #        p_female = Bio@p_female,
  #        gamma = 1,
  #        NOS = TRUE,
  #        SRRfun = cpars_bio$SRR$SRRfun,
  #        SRRpars = cpars_bio$SRR$SRRpars[x, ]
  #      )
  #    })
  #  } else {
  #    HistPars <- lapply(1:SOM@nsim, function(x) {
  #      calc_Perr_main(
  #        Njuv = Historical@HistNjuv_HOS[x, , , r],
  #        Fjuv = outer(Harvest@vulPT[x, ], Historical@HistFPT[x, , pind]),
  #        Fterm = outer(Harvest@vulT[x, ], Historical@HistFT[x, , pind]),
  #        p_mature = Hatchery@p_mature_HOS[x, , , r],
  #        nyears = SOM@nyears,
  #        HistSpawner = if (is.array(Historical@HistSpawner_HOS) && length(Historical@HistSpawner_HOS)) Historical@HistSpawner_HOS[x, , , r] else NULL,
  #        fec = Bio@fec[x, , seq(1, SOM@nyears)],
  #        p_female = Bio@p_female,
  #        gamma = Hatchery@gamma,
  #        NOS = FALSE
  #      )
  #    })
  #  }
#
  #  # Specify rec devs
  #  if (stage == "immature") {
  #    # Initial abundance vector. With custom SRR, we set R0 = 1!
  #    survOM <- sapply(1:SOM@nsim, function(x) {
  #      calc_survival(cpars_bio$M_ageArray[x, , 1], p_mature = rep(0, Stock@maxage))
  #    }) # maxage x nsim
  #    NPR <- t(survOM)
  #    age_init <- seq(1, Stock@maxage, 2)[-1]
  #    if (NOS) {
  #      Ninit <- Historical@HistNjuv_NOS[, -1, 1, g]
  #    } else {
  #      Ninit <- Historical@HistNjuv_HOS[, -1, 1, r]
  #    }
  #    Perr_init <- Ninit/NPR[, rev(age_init)]
  #    cpars_bio$Perr_y[, rev(age_init)] <- Perr_init
#
  #    Perr_main <- sapply(HistPars, getElement, "dev")
  #    cpars_bio$Perr_y[, Stock@maxage + c(t1, max(t1) + 2)] <- t(Perr_main)
#
  #  } else if (stage == "escapement") {
  #    # Specify fecundity adjustment to match escapement to spawners
  #    fec_adjust <- replicate(length(age_fec), sapply(HistPars, getElement, "fec_ratio")) %>%
  #      aperm(c(2, 3, 1))
  #    cpars_bio$Fec_age[, age_fec, t1 + 2] <- cpars_bio$Fec_age[, age_fec, t1 + 2] * fec_adjust
  #  }
  #}

  cpars_bio$AC <- rep(0, SOM@nsim)

  return(list(Stock = Stock, cpars_bio = cpars_bio))
}

#' @rdname salmonMSE-int
#' @return
#' `make_Stock`: List containing a [MSEtool::Fleet-class] object and accompanying custom parameters list
#' @export
make_Fleet <- function(SOM, s, NOS = TRUE, stage = c("immature", "return", "escapement")) {
  stage <- match.arg(stage)

  Bio <- SOM@Bio[[s]]
  Harvest <- SOM@Harvest[[s]]
  Hatchery <- SOM@Hatchery[[s]]
  Historical <- SOM@Historical[[s]]

  Fleet <- new("Fleet")
  Fleet@Name <- switch(
    stage,
    "immature" = "Preterminal fishery",
    "return" = "Terminal fishery",
    "escapement" = "Escapement placeholder"
  )

  n_age <- 2 * Bio@maxage + 1
  maxage <- n_age - 1
  nyears_real <- 2
  nyears <- 2 * nyears_real
  proyears <- 2 * SOM@proyears

  Fleet@nyears <- nyears
  Fleet@CurrentYr <- nyears
  Fleet@EffYears <- 1:Fleet@nyears

  # Time step for the first and second half of the year
  t1 <- seq(1, nyears, 2)
  t2 <- seq(2, nyears, 2)

  a1 <- seq(1, 2 * Bio@maxage, 2)
  a2 <- seq(2, n_age, 2)

  Fleet@EffLower <- c(0, 0.1)
  Fleet@EffUpper <- c(0, 0.1)

  # No change in fishing efficiency, projected F proportional to effort
  Fleet@Esd <- Fleet@qinc <- Fleet@qcv <- c(0, 0)

  # Fishery selectivity - placeholder, will use V slot
  Fleet@L5 <- c(0.7, 0.7)
  Fleet@LFS <- c(0.71, 0.71)
  Fleet@Vmaxlen <- c(1, 1)

  Fleet@LR5 <- Fleet@LFR <- c(-100, -100)
  Fleet@Rmaxlen <- c(1, 1)
  Fleet@DR <- c(0, 0) # Discard rate
  Fleet@Spat_targ <- c(1, 1)
  Fleet@MPA <- FALSE

  Fleet@isRel <- FALSE

  cpars_fleet <- list()
  cpars_fleet$qs <- rep(1, SOM@nsim)

  cpars_fleet$Find <- matrix(0, SOM@nsim, Fleet@nyears) # Escapement experiences zero F
  if (stage %in% c("immature", "return")) { # FinF must be greater than zero
    cpars_fleet$Find[, nyears] <- 1e-4
  }

  cpars_fleet$V <- array(0, c(SOM@nsim, n_age, nyears + proyears))
  if (stage == "immature") {

    if (all(!Harvest@vulPT)) {
      # Define dummy selectivity to remove excessive messages about selectivity < 0.01.
      cpars_fleet$V[, n_age, ] <- 1
    } else {
      vulPT <- array(Harvest@vulPT, c(SOM@nsim, Bio@maxage, nyears + proyears))
      cpars_fleet$V[, a1, ] <- vulPT
    }

  } else if (stage == "return") {

    if (all(!Harvest@vulT)) {
      cpars_fleet$V[, n_age, ] <- 1
    } else {
      vulT <- array(Harvest@vulT, c(SOM@nsim, Bio@maxage, nyears + proyears))
      cpars_fleet$V[, a2, ] <- vulT
    }

  } else {
    cpars_fleet$V[, n_age, ] <- 1
  }

  MSF_PT <- stage == "immature" && Harvest@MSF_PT
  MSF_T <- stage == "return" && Harvest@MSF_T

  if (MSF_PT || MSF_T) {
    cpars_fleet$retA <- cpars_fleet$V
    retAproj <- cpars_fleet$retA[, , nyears + seq(1, proyears)]
    if (NOS) {
      retAproj[] <- .Machine$double.eps
    } else {
      retAproj[retAproj > 0] <- Hatchery@m
    }
    cpars_fleet$retA[, , nyears + seq(1, proyears)] <- retAproj
  }

  return(list(Fleet = Fleet, cpars_fleet = cpars_fleet))
}


calc_survival <- function(Mjuv, p_mature, maxage = length(p_mature)) {
  surv <- rep(NA_real_, maxage)
  surv[1] <- 1
  for (a in 2:maxage) {
    surv[a] <- surv[a-1] * exp(-Mjuv[a-1]) * (1 - p_mature[a-1])
  }
  return(surv)
}


#' Calculate equilibrium quantities with life history groups
#'
#' Calculate eggs/smolt or spawners/smolt based on life history parameters (survival, maturity, fecundity)
#'
#' @param Mjuv Matrix `[maxage, n_g]`, but can be a vector if `n_g = 1`. Juvenile natural mortality
#' @param p_mature Matrix `[maxage, n_g]`, but can be a vector if `n_g = 1`. Maturity at age
#' @param p_female Numeric. Proportion female
#' @param fec Matrix `[maxage, n_g]`, but can be a vector if `n_g = 1`. Fecundity at age. Only used if `output = "egg"`
#' @param s_enroute Numeric, en-route survival of escapement to spawning grounds
#' @param n_g Integer. Number of life history groups
#' @param p_LHG Vector length `n_g` of proportion of life history groups per recruit. Default is `rep(1/n_g, n_g)`
#' @param output Character to indicate the output units, e.g., "egg" returns eggs per smolt, and "spawner" returns spawners per smolt
#' @keywords internal
#' @return Numeric, units depend on `"output"` argument
calc_phi <- function(Mjuv, p_mature, p_female, fec, s_enroute = 1, n_g = 1, p_LHG, output = c("egg", "spawner")) {
  output <- match.arg(output)

  if (n_g == 1) {
    if (!is.matrix(Mjuv)) Mjuv <- matrix(Mjuv, ncol = 1)
    if (!is.matrix(p_mature)) p_mature <- matrix(p_mature, ncol = 1)
    if (output == "egg" && !is.matrix(fec)) fec <- matrix(fec, ncol = 1)
  }
  if (missing(p_LHG)) p_LHG <- rep(1/n_g, n_g)

  surv_juv <- sapply(1:n_g, function(g) p_LHG[g] * calc_survival(Mjuv[, g], p_mature[, g])) # age x g
  Esc <- p_mature * surv_juv # escapement per smolt

  if (output == "egg") {
    x <- p_female * Esc * s_enroute * fec
  } else {
    x <- p_female * Esc * s_enroute
  }

  return(sum(x))
}

make_Stock_objects <- function(SOM, s = 1) {
  do_hatchery <- sum(SOM@Hatchery[[s]]@n_yearling, SOM@Hatchery[[s]]@n_subyearling) > 0
  has_strays <- any(SOM@stray[-s, s] > 0) || sum(SOM@Hatchery[[s]]@stray_external)

  Stocks_g <- lapply(1:SOM@Bio[[s]]@n_g, function(g) {
    S <- list()
    S[[1]] <- make_Stock(SOM, s, g = g, NOS = TRUE, stage = "immature")   # NOS_juv
    S[[2]] <- make_Stock(SOM, s, g = g, NOS = TRUE, stage = "return")     # NOS_recruitment
    S[[3]] <- make_Stock(SOM, s, g = g, NOS = TRUE, stage = "escapement") # NOS_escapement
    return(S)
  })
  Stocks_g <- do.call(c, Stocks_g)

  if (do_hatchery || has_strays) {
    Stocks_r <- lapply(1:SOM@Hatchery[[s]]@n_r, function(r) {
      S <- list()
      S[[1]] <- make_Stock(SOM, s, r = r, NOS = FALSE, stage = "immature")   # HOS_juv
      S[[2]] <- make_Stock(SOM, s, r = r, NOS = FALSE, stage = "return")     # HOS_recruitment
      S[[3]] <- make_Stock(SOM, s, r = r, NOS = FALSE, stage = "escapement") # HOS_escapement
      return(S)
    })
    Stocks_r <- do.call(c, Stocks_r)
  } else {
    Stocks_r <- list()
  }

  c(Stocks_g, Stocks_r)
}

make_Fleet_objects <- function(SOM, s = 1) {
  do_hatchery <- sum(SOM@Hatchery[[s]]@n_yearling, SOM@Hatchery[[s]]@n_subyearling) > 0
  has_strays <- any(SOM@stray[-s, s] > 0) || sum(SOM@Hatchery[[s]]@stray_external)

  Fleets_g <- lapply(1:SOM@Bio[[s]]@n_g, function(g) {
    FF <- list()
    FF[[1]] <- make_Fleet(SOM, s, NOS = TRUE, stage = "immature")   # NOS_juv
    FF[[2]] <- make_Fleet(SOM, s, NOS = TRUE, stage = "return")     # NOS_recruitment
    FF[[3]] <- make_Fleet(SOM, s, NOS = TRUE, stage = "escapement") # NOS_escapement
    return(FF)
  })
  Fleets_g <- do.call(c, Fleets_g)

  if (do_hatchery || has_strays) {
    Fleets_r <- lapply(1:SOM@Hatchery[[s]]@n_r, function(r) {
      FF <- list()
      FF[[1]] <- make_Fleet(SOM, s, NOS = FALSE, stage = "immature")   # HOS_juv
      FF[[2]] <- make_Fleet(SOM, s, NOS = FALSE, stage = "return")     # HOS_recruitment
      FF[[3]] <- make_Fleet(SOM, s, NOS = FALSE, stage = "escapement") # HOS_escapement
      return(FF)
    })
    Fleets_r <- do.call(c, Fleets_r)
  } else {
    Fleets_r <- list()
  }
  c(Fleets_g, Fleets_r)
}

make_stock_index <- function(SOM, check = FALSE) {
  if (check) SOM <- check_SOM(SOM)
  ns <- length(SOM@Bio)
  n_g <- sapply(1:ns, function(s) SOM@Bio[[s]]@n_g)
  n_r <- sapply(1:ns, function(s) SOM@Hatchery[[s]]@n_r)
  np_s <- sapply(1:ns, function(s) {
    do_hatchery <- sum(SOM@Hatchery[[s]]@n_yearling, SOM@Hatchery[[s]]@n_subyearling) > 0
    has_strays <- any(SOM@stray[-s, s] > 0) || sum(SOM@Hatchery[[s]]@stray_external)
    3 * (n_g[s] + ifelse(do_hatchery || has_strays, n_r[s], 0))
  })
  .make_stock_index(ns, n_g, n_r, np_s)
}

.make_stock_index <- function(ns, n_g, n_r, np_s) {

  dat <- lapply(1:ns, function(s) {

    g <- rep(1:n_g[s], each = 3)
    n_natural <- n_g[s] * 3
    n_hatchery <- np_s[s] - n_natural

    df_natural <- data.frame(
      s = s,
      g = g,
      r = NA,
      origin = "natural",
      stage = rep(c("juvenile", "recruitment", "escapement"), times = n_g[s])
    )

    if (n_hatchery > 0) {
      r <- rep(1:n_r[s], each = 3)
      df_hatchery <- data.frame(
        s = s,
        g = NA,
        r = r,
        origin = "hatchery",
        stage = rep(c("juvenile", "recruitment", "escapement"), times = n_r[s])
      )
    } else {
      df_hatchery <- data.frame()
    }

    rbind(df_natural, df_hatchery)

  }) %>%
    bind_rows()
  dat$p <- seq_len(nrow(dat))
  return(dat)
}

#calc_Perr_main <- function(Njuv, Fjuv, Fterm, p_mature, nyears, HistSpawner, fec, p_female, gamma = 1, NOS = TRUE,
#                           SRRfun, SRRpars) {
#
#  Recruit <- Njuv[, seq(1, nyears)] * exp(-Fjuv) * p_mature[, seq(1, nyears)]
#  SpawnerOM <- Recruit * exp(-Fterm) # Escapement
#
#  if (!length(HistSpawner)) HistSpawner <- SpawnerOM
#
#  EggOM <- colSums(SpawnerOM * gamma * fec * p_female)
#  EggHist <- colSums(HistSpawner * gamma * fec * p_female)
#
#  fec_ratio <- EggHist/EggOM
#  fec_ratio[is.na(fec_ratio)] <- 1 # No spawners
#
#  if (NOS) {
#    Smolt_pred <- SRRfun(EggHist, SRRpars)
#    Smolt_actual <- Njuv[1, ]
#    dev <- Smolt_actual/c(1, Smolt_pred) # Length nyears + 1
#  } else {
#    dev <- Njuv[1, ]
#  }
#  dev[is.na(dev)] <- 0
#  list(dev = dev, fec_ratio = fec_ratio)
#}

#' Calculate abundance from density-dependent mortality
#'
#' Calculates the abundance of survivors after applying either a Beverton-Holt or Ricker stock-recruit relationship.
#'
#' @param N1 Numeric, the initial abundance that scales the density-independent survival term
#' @param N2 Numeric, the initial abundance that scales the density-dependent survival term
#' @param p Numeric, the productivity parameter that sets the maximum survival as the initial abundance approaches zero
#' @param capacity Numeric, the capacity parameter that set the maximum survivors
#' @param type Character, the functional form of the stock-recruit relationship
#' @return
#' Numeric, the abundance of survivors
#' @details
#' The Beverton-Holt stock recruit relationship is of the following form:
#' \deqn{\textrm{Smolt} = \dfrac{\alpha N_1}{1 + \beta N_2}}
#' where \eqn{\alpha = P}, \eqn{\beta = P/C}.
#'
#' The Ricker stock recruit relationship is of the following form:
#' \deqn{\textrm{Smolt} = \alpha N_1 \exp(-\beta N_2)}
#' where \eqn{\alpha = P}, \eqn{\beta = P/(Ce)}, \eqn{e} is Euler's number.
#'
#' Productivity \eqn{P} is in terms of abundance per unit of \eqn{N_1} and \eqn{N_2}.
#'
#' The hockey stick is of the following form:
#'
#' \deqn{
#' \textrm{Smolt} =
#' \begin{cases}
#' p N_1 &, N_1 \le \frac{N_1}{N_2} \times C\\
#' \frac{N_1}{N_2} \times C &, \textrm{otherwise}
#' \end{cases}
#' }
#'
#' @seealso [calc_SRRpars()]
#' @examples
#' N1 <- 100
#' N2 <- 200
#' p <- 10
#' capacity <- 250
#'
#' calc_SRR(N1, N2, p, capacity, type = "BH")
#'
#' # Validation
#' SRRpars <- calc_SRRpars(p, capacity, type = "BH")
#' a <- SRRpars[1]
#' b <- SRRpars[2]
#' a * N1/(1 + b * N2)
#' @export
calc_SRR <- function(N1, N2 = N1, p, capacity, type = c("BH", "Ricker", "HS")) {
  type <- match.arg(type)

  pars <- calc_SRRpars(p, capacity)
  alpha <- pars[1]
  beta <- pars[2]
  if (type == "BH") {
    out <- alpha * N1 / (1 + beta * N2)
  } else if (type == "Ricker") {
    out <- alpha * N1 * exp(-beta * N2)
  } else {
    capacity_N1 <- N1/N2 * capacity
    Nstar <- capacity_N1/p
    out <- pmin(p * N1, capacity_N1)
  }
  out[is.na(out)] <- 0
  return(out)
}


#' Convert density-dependent survival parameters
#'
#' Converts from capacity/productivity parameters to alpha/beta stock-recruit parameters where productivity is in terms
#' of smolts per spawner and alpha is terms of smolts per egg.
#'
#' @param p Numeric, the productivity parameter that sets the maximum survival as the initial abundance approaches zero
#' @param capacity Numeric, the capacity parameter that set the maximum survivors
#' @param f Fecundity, the spawning output per mature female
#' @param p_female The proportion of females per spawner
#' @param type Character, the functional form of the stock-recruit relationship
#' @returns Numeric vector length 2 for alpha and beta value, respectively
#' @details
#' \deqn{\alpha = \dfrac{P}{f \times p_{female}}}
#'
#' For the Beverton-Holt stock recruit relationship:
#' \deqn{\beta = \dfrac{\alpha}{C}}
#'
#' For the Ricker stock recruit relationship:
#' \deqn{\beta = \dfrac{\alpha}{Ce}}, \eqn{e} is Euler's number.
#' @seealso [calc_SRR()]
#' @examples
#'
#' N1 <- 100
#' N2 <- 200
#' p <- 10
#' capacity <- 250
#'
#' calc_SRRpars(p, capacity, type = "BH")
#' @export
calc_SRRpars <- function(p, capacity, f = 1, p_female = 1, type = c("BH", "Ricker", "HS")) {
  type <- match.arg(type)
  if (length(p_female) > 1) stop("p_female needs to be length 1")
  alpha <- p/f/p_female
  beta <- switch(
    type,
    "BH" = alpha/capacity,
    "Ricker" = exp(-1) * alpha/capacity,
    NA
  )
  c(alpha, beta)
}

make_SRR <- function(Bio) {

  a <- Bio@kappa/Bio@phi
  if (Bio@SRrel == "BH") {
    b <- a/Bio@capacity
    SRRpars <- data.frame(
      a = a, b = b, phi = Bio@phi, SPRcrash = 1/a/Bio@phi, SRrel = Bio@SRrel, kappa = Bio@kappa,
      capacity = Bio@capacity
    )
  } else {
    Emax <- Bio@Smax * Bio@phi/Bio@tau
    b <- 1/Emax
    SRRpars <- data.frame(
      a = a, b = b, phi = Bio@phi, tau = Bio@tau, SPRcrash = 1/a/Bio@phi, SRrel = Bio@SRrel, kappa = Bio@kappa,
      Emax = Emax, Smax = Bio@Smax
    )
  }
  SRRfun <- function(SB, SRRpars) {
    if (SRRpars$SRrel == "BH") {
      SRRpars$a * SB / (1 + SRRpars$b * SB)
    } else {
      SRRpars$a * SB * exp(-SRRpars$b * SB)
    }
  }
  relRfun <- function(SSBpR, SRRpars) {
    if (SRRpars$SRrel == "BH") {
      (SRRpars$a * SSBpR - 1)/SRRpars$b/SSBpR
    } else {
      log(SRRpars$a * SSBpR)/SRRpars$b/SSBpR
    }
  }
  SPRcrashfun <- function(SSBpR0, SRRpars) SRRpars$SPRcrash
  list(SRRfun = SRRfun, SRRpars = SRRpars, relRfun = relRfun, SPRcrashfun = SPRcrashfun)
}

#' Plot core output from salmonMSE
#'
#' @description
#' Various functions that plot the state variables from salmonMSE projections:
#'
#' - `plot_statevar_ts()` produces a time series for all simulations, or with medians and 95th percentile intervals
#' - `plot_statevar_hist()` produces a histogram across all simulations for a particular year
#' - `plot_spawners()` produces a summary barplot of spawners, including NOS, HOS, and wild spawners
#' - `plot_escapement()` produces a summary figure of the proportion of spawners and broodtake to escapement
#' - `plot_fitness()` produces a summary figure of metrics (fitness, PNI, pHOS, and pWILD) related to hatchery production
#' - `plot_fishery()` produces a summary figure of metrics related to the fishery, e.g., median catch, exploitation rate or harvest rate
#'
#' @param SMSE Class \linkS4class{SMSE} object returned by [salmonMSE()]
#' @param var Character. Slot for the state variable in `SMSE` object. See `slotNames(SMSE)` for options. Additional supported options are:
#' `"ESS"` (egg-smolt survival), `"pbrood"` (broodtake to escapement ratio), `"pNOSesc"` (NOS/natural escapement), `"pHOSesc"` (HOS/hatchery escapement),
#' `Total Spawners` (NOS + HOS), `NOS/SMSY`, `S/SMSY`, and `NOS/Sgen`.
#' @param s Integer. Population index for multi-population model (e.g., `s = 1` is the first population in the model)
#' @param xlab Character. Name of time variable for the figure
#' @param figure Logical, whether to generate a figure (set to FALSE if only using the function to return the data matrix)
#' @param quant Logical, whether to plot individual simulations (FALSE) or the median with 95 percent confidence intervals (TRUE)
#' @param ylab Character. Name of the state variable for the figure
#' @param ylim Vector. Y-axis limits
#' @param agg.fun Function. Defines how to aggregate state variables that are reported by age. Typically, `sum` is used but `max` is also
#' possible for reporting apical exploitation rates.
#' @return Functions return the matrix of plotted values invisibly. Figure plotted from base graphics
#' @importFrom graphics matplot grid legend
#' @importFrom methods slot
#' @importFrom stats quantile
#' @importFrom grDevices hcl.colors
#' @seealso [plot_decision_table()] [plot_LHG()] [compare_statevar_ts()]
#' @export
plot_statevar_ts <- function(SMSE, var = "PNI", s = 1, figure = TRUE, xlab = "Projection Year", quant = FALSE, ylab = var, ylim,
                             agg.fun = sum, ...) {

  if (length(s) > 1) {
    x <- sapply(s, function(i) get_statevar(SMSE, var, i, agg.fun = agg.fun), simplify = "array")
    xplot <- apply(x, 2:3, quantile, 0.5, na.rm = TRUE)

    if (figure && any(xplot > 0, na.rm = TRUE)) {
      ind <- rowSums(xplot, na.rm = TRUE) > 0
      Year <- 1:SMSE@proyears

      if (missing(ylim)) ylim <- c(0, 1.1) * range(xplot, na.rm = TRUE)

      col <- grDevices::hcl.colors(length(s), palette = "Dark 3", alpha = 1)

      matplot(Year[ind], xplot[ind, ], type = 'o', ylim = ylim, col = col, lty = 1, pch = 1,
              xlab = "Projection Year", ylab = ylab, ...)
      legend("topleft", legend = SMSE@Snames, col = col, lty = 1, pch = 1, bty = "n")

    }

  } else {

    x <- get_statevar(SMSE, var, s, agg.fun = agg.fun)

    if (!quant) {
      xplot <- x
    } else {
      xplot <- apply(x, 2, quantile, c(0.025, 0.5, 0.975), na.rm = TRUE)
    }

    if (figure && any(xplot > 0, na.rm = TRUE)) {
      ind <- colSums(xplot, na.rm = TRUE) > 0
      Year <- 1:SMSE@proyears

      if (missing(ylim)) ylim <- c(0, 1.1) * range(xplot, na.rm = TRUE)

      if (!quant) {
        matplot(Year[ind], t(xplot[, ind]), type = 'l', col = "grey40", ylim = ylim, lty = 1,
                xlab = "Projection Year", ylab = ylab, ...)
      } else {
        matplot(Year[ind], t(xplot[, ind]), type = 'o', pch = c(NA, 1, NA), col = 1, lty = c(2, 1, 2), ylim = ylim,
                xlab = "Projection Year", ylab = ylab, ...)
      }
    }
  }


  invisible(xplot)
}

#' @name plot_statevar_ts
#' @param y Integer. Projection year for the state variable to plot the histogram. If missing, the last projection year is used.
#' @param ... Additional arguments to base plot function
#' @export
#' @importFrom graphics hist
plot_statevar_hist <- function(SMSE, var = "PNI", s = 1, y, figure = TRUE, xlab = var, ...) {
  x <- get_statevar(SMSE, var, s)

  if (missing(y)) {
    xvar <- colSums(x, na.rm = TRUE)
    if (all(!xvar)) {
      y <- length(xvar)
    } else {
      ymax <- max(which(!is.na(xvar) & xvar > 0))
      y <- ymax
    }
  }
  xplot <- x[, y]
  if (figure && any(xplot > 0, na.rm = TRUE)) hist(xplot, xlab = xlab, main = NULL, ...)
  invisible(xplot)
}

get_statevar <- function(SMSE, var, s, agg.fun = sum) {

  x <- matrix(NA, SMSE@nsim, SMSE@proyears)

  if (var %in% slotNames(SMSE)) {

    .x <- slot(SMSE, var)
    if (length(dim(.x)) == 4) {
      x[] <- apply(.x[, s, , ], c(1, 3), agg.fun)
    } else if (length(dim(.x)) == 3) {
      x[] <- .x[, s, ]
    } else {
      stop("Dimension of slot ", var, " is not 3 or 4")
    }
    return(x)
  }

  if (var == "ESS") {
    x[, seq(2, SMSE@proyears)] <- local({
      Smolt <- SMSE@Smolt_NOS[, s, ] + SMSE@Smolt_HOS[, s, ]
      Egg <- SMSE@Egg_NOS[, s, ] + SMSE@Egg_HOS[, s, ]

      Smolt[, seq(2, SMSE@proyears)]/Egg[, seq(2, SMSE@proyears) - 1]
    })
  }

  if (var == "pbrood") {
    x[] <- local({
      Esc_NOS <- apply(SMSE@Escapement_NOS[, s, , ], c(1, 3), sum)
      Esc_HOS <- apply(SMSE@Escapement_HOS[, s, , ], c(1, 3), sum)
      (SMSE@NOB[, s, ] + SMSE@HOB[, s, ])/(Esc_NOS + Esc_HOS)
    })
  }

  if (var == "pNOSesc") {
    x[] <- local({
      Esc_NOS <- apply(SMSE@Escapement_NOS[, s, , ], c(1, 3), sum)
      NOS <- apply(SMSE@NOS[, s, , ], c(1, 3), sum)
      NOS/Esc_NOS
    })
  }
  if (var == "pHOSesc") {
    x[] <- local({
      Esc_HOS <- apply(SMSE@Escapement_HOS[, s, , ], c(1, 3), sum)
      HOS <- apply(SMSE@HOS[, s, , ], c(1, 3), sum)
      HOS/Esc_HOS
    })
  }

  if (var == "Smolt") x[] <- SMSE@Smolt_NOS[, s, ] + SMSE@Smolt_HOS[, s, ]

  if (var == "Total Spawners") x[] <- apply(slot(SMSE, "NOS")[, s, , ] + slot(SMSE, "HOS")[, s, , ], c(1, 3), sum)

  if (var == "NOS/SMSY" && length(SMSE@Misc$Ref[[s]])) {
    x[] <- local({
      NOS <- apply(slot(SMSE, "NOS")[, s, , ], c(1, 3), sum)
      SMSY <- SMSE@Misc$Ref[[s]]["Spawners_MSY", ]
      NOS/SMSY
    })
  }

  if (var == "S/SMSY" && length(SMSE@Misc$Ref[[s]])) {
    x[] <- local({
      S <- apply(slot(SMSE, "NOS")[, s, , ] + slot(SMSE, "HOS")[, s, , ], c(1, 3), sum)
      SMSY <- SMSE@Misc$Ref[[s]]["Spawners_MSY", ]
      S/SMSY
    })
  }

  if (var == "NOS/Sgen" && length(SMSE@Misc$Ref[[s]])) {
    x[] <- local({
      NOS <- apply(slot(SMSE, "NOS")[, s, , ], c(1, 3), sum)
      Sgen <- SMSE@Misc$Ref[[s]]["Sgen", ]
      NOS/Sgen
    })
  }

  x[is.infinite(x)] <- NA

  return(x)
}

#' @name plot_statevar_ts
#' @param prop Logical, whether to plot proportions or absolute numbers
#' @param FUN Summarizing function across simulations, typically [median()] or [mean()]
#' @importFrom graphics barplot box
#' @importFrom stats median
#' @export
plot_spawners <- function(SMSE, s = 1, prop = TRUE, FUN = median, figure = TRUE, ylim) {
  Year <- 1:SMSE@proyears

  .HOS <- apply(SMSE@HOS[, s, , ], c(1, 3), sum)
  HOS <- apply(.HOS, 2, FUN)

  .NOS <- apply(SMSE@NOS[, s, , ], c(1, 3), sum)
  NOS <- apply(.NOS, 2, FUN)

  p_wild <- SMSE@p_wild[, s, ]
  p_wild[is.na(p_wild)] <- 0

  Spawners <- HOS + NOS
  WILD <- apply(p_wild * .NOS, 2, FUN)
  NOS_notWILD <- apply((1 - p_wild) * .NOS, 2, FUN)

  x <- rbind(WILD, NOS_notWILD, HOS)

  if (prop) {
    x <- apply(x, 2, function(i) i/sum(i))
    x[is.na(x)] <- 0
  }

  if (figure) {
    if (missing(ylim)) {
      if (prop) {
        ylim <- c(0, 1)
      } else {
        ylim <- c(0, 1.1) * range(Spawners)
      }
    }
    col <- c("#004533", "#76A6D0", "#FFF7FD") #grDevices::hcl.colors(3, palette = "PuBuGn")
    plot(Year, Spawners, xlim = range(Year) + c(-1, 0),
         xlab = "Projection Year", ylab = ifelse(prop, "Proportion", "Spawners"),
         type = "n", ylim = ylim, xaxs = "i", yaxs = "i")
    barplot(x, legend.text = rownames(x), space = 0, xlim = range(Year),
            col = col, border = "grey40", add = TRUE, xpd = FALSE)
    box()
  }

  invisible(x)
}

#' @name plot_statevar_ts
#' @importFrom graphics legend lines
#' @export
plot_fitness <- function(SMSE, s = 1, FUN = median, figure = TRUE, ylim) {
  Year <- 1:SMSE@proyears

  Fitness <- apply(SMSE@fitness[, s, 1, ], 2, FUN)
  PNI <- apply(SMSE@PNI[, s, ], 2, FUN)
  pHOSeff <- apply(SMSE@pHOS_effective[, s, ], 2, FUN)
  pWILD <- apply(SMSE@p_wild[, s, ], 2, FUN)

  x <- cbind(Fitness, PNI, pHOSeff, pWILD)

  if (figure) {
    if (missing(ylim)) ylim <- c(0, 1)
    matplot(Year, x, type = "n", xlab = "Projection Year", ylab = "Value",
            ylim = ylim)

    col <- 1:ncol(x)
    pch <- c(1, 4, 16, 18)
    for (i in 1:ncol(x)) {
      x_i <- x[, i]
      lines(Year[!is.na(x_i)], x_i[!is.na(x_i)], type = "o", col = col[i], pch = pch[i])
    }
    legend("bottomleft", legend = colnames(x), col = col, pch = pch, lwd = 1, bty = "n")
  }

  invisible(x)
}

#' @name plot_statevar_ts
#' @export
plot_escapement <- function(SMSE, s = 1, FUN = median, figure = TRUE, ylim) {

  Year <- 1:SMSE@proyears

  pNOSesc <- plot_statevar_ts(SMSE, "pNOSesc", s, figure = FALSE) %>%
    apply(2, FUN)
  pHOSesc <- plot_statevar_ts(SMSE, "pHOSesc", s, figure = FALSE) %>%
    apply(2, FUN)
  pbrood <- plot_statevar_ts(SMSE, "pbrood", s, figure = FALSE) %>%
    apply(2, FUN)

  x <- cbind(pNOSesc, pHOSesc, pbrood)

  if (any(x > 1, na.rm = TRUE)) warning("Spawners exceed escapement. Error in reporting?")

  if (figure) {
    if (missing(ylim)) ylim <- c(0, 1)
    matplot(Year, x, type = "n", xlab = "Projection Year", ylab = "Proportion",
            ylim = ylim)

    legnames <- c("Spawner/Escapement (natural)", "Spawner/Escapement (hatchery)", "Broodstock/Escapement (total)")

    col <- 1:ncol(x)
    pch <- c(1, 4, 16, 18)
    for (i in 1:ncol(x)) {
      x_i <- x[, i]
      lines(Year[!is.na(x_i)], x_i[!is.na(x_i)], type = "o", col = col[i], pch = pch[i])
    }
    legend("bottomleft", legend = legnames, col = col, pch = pch, lwd = 1, bty = "n")
  }

  invisible(x)
}


#' @name plot_statevar_ts
#' @param type Character. For `plot_fishery`, the fishery state variable to plot.
#' @export
plot_fishery <- function(SMSE, s = 1, type = c("catch", "exploit", "harvest"), FUN = median, figure = TRUE, ylim, ylab, ...) {
  type <- match.arg(type)

  var <- switch(
    type,
    "catch" = "K",
    "exploit" = "Ex",
    "harvest" = "U"
  )

  var_vec <- outer(paste0(c("PT", "T"), "_"), c("NOS", "HOS"), FUN = "paste0") %>%
    as.character()

  x <- sapply(var_vec, function(i) {
    plot_statevar_ts(SMSE, var = paste0(var, i), s = s, figure = FALSE, quant = FALSE, agg.fun = max)
  }, simplify = "array") %>%
    apply(2:3, FUN)

  if (figure) {
    Year <- 1:SMSE@proyears

    if (missing(ylim)) ylim <- c(0, 1.1) * range(x, na.rm = TRUE)

    if (missing(ylab)) {
      ylab <- switch(
        type,
        "catch" = "Kept catch",
        "exploit" = "Exploitation rate",
        "harvest" = "Harvest rate"
      )
    }

    plot(Year, NULL, type = 'n', col = "grey40", ylim = ylim, xlab = "Projection Year",
         ylab = ylab, ...)
    col <- 1:length(var_vec)

    for (i in 1:length(var_vec)) {
      x_i <- x[, i]
      ind <- x_i > 0
      lines(Year[ind], x_i[ind], type = 'o', pch = 1, col = col[i], lty = 1)
    }
    legend("topleft", legend = var_vec, col = 1:4, pch = 1, lty = 1, bty = "n")
  }

  invisible(x)
}


#' @name plot_statevar_ts
#'
#' @param xlim Vector. X-axis limits
#' @param type For `plot_Kobe`, the fishery state variable to plot. Whether to plot the exploitation rate for the terminal (T) or pre-terminal fishery (PT).
#' @export
plot_Kobe <- function(SMSE, s = 1, FUN = median, figure = TRUE, xlim, ylim,
                      xlab = expression(NOS/S[MSY]), ylab = expression(U/U[MSY]), type = c("T", "PT")) {

  type <- match.arg(type)

  if (!length(SMSE@Misc$Ref) || !length(SMSE@Misc$Ref[[s]])) return(invisible(data.frame()))

  NOS <- apply(SMSE@NOS[, s, , ], c(1, 3), sum)

  S_SMSY <- apply(NOS/SMSE@Misc$Ref[[s]]["Spawners_MSY", ], 2, FUN)
  if (type == "T") {
    Ex <- apply(SMSE@ExT_NOS[, s, , ], c(1, 3), max)
    Ex_ExMSY <- apply(Ex/SMSE@Misc$Ref[[s]]["UT_MSY", ], 2, FUN)
  } else {
    Ex <- apply(SMSE@ExPT_NOS[, s, , ], c(1, 3), max)
    Ex_ExMSY <- apply(Ex/SMSE@Misc$Ref[[s]]["UPT_MSY", ], 2, FUN)
  }

  if (figure) {
    Sind <- !is.na(S_SMSY) & is.finite(S_SMSY)
    Eind <- !is.na(Ex_ExMSY) & is.finite(Ex_ExMSY)
    ind <- Sind & Eind
    npoints <- sum(ind)

    if (npoints > 0) {
      Year <- seq(1, SMSE@proyears)
      Year_legend <- pretty(Year[ind], n = 4)
      Year_legend <- Year_legend[Year_legend %in% Year]

      cols <- grDevices::hcl.colors(length(Year), palette = "viridis", rev = FALSE)

      if (missing(xlim)) {
        xlim <- c(0, 1.1) * range(S_SMSY, na.rm = TRUE)
      }
      if (missing(ylim)) {
        ylim <- c(0, 1.1) * range(Ex_ExMSY, na.rm = TRUE)
      }

      plot(S_SMSY[ind], Ex_ExMSY[ind], type = "p", pch = 21, bg = cols[ind],
           xlab = xlab, ylab = ylab,
           xlim = xlim, ylim = ylim)
      abline(h = 1, v = 1, lty = 2, col = "grey40")
      legend("topright", legend = Year_legend, pt.bg = cols[Year %in% Year_legend], pch = 21, bty = "n", title = "Year")
    }
  }

  invisible(data.frame(S_SMSY, Ex_ExMSY))
}

#' Decision table of performance metrics
#'
#' Generates a coloured table of a performance metric across two axes, which may be a population dynamics variable
#' (e.g., productivity) or a management action (e.g., hatchery production levels or harvest strategy).
#' See example at \url{https://docs.salmonmse.com/articles/decision-table.html}. More examples below.
#' - `plot_decision_table()` is a simple figure where colour range is intended to continuously transition from pink to white to green corresponding to values of 0, 0.5, and 1, respectively.
#' - `plot_decision_table2()` is converts performance metrics values into bins and provides more user control in the colour scheme
#'
#' @param x Atomic, vector of values for the x axis (same length as z). Will be converted to factors
#' @param y Atomic, vector of values for the y axis (same length as z). Will be converted to factors
#' @param z Numeric, vector of values for the performance metric
#' @param title Character, optional title of figure
#' @param xlab Character, optional x-axis label
#' @param ylab Character, optional y-axis label
#' @param scenario Atomic, vector of faceting variables (same length as z) used to generate a grid of decision tables
#' @param ncol Integer, number of columns for decision table grid, only used if `scenario` is provided
#' @param dir Character, either "h" or "v" to describe how the grid of tables should be organized (horizontally or vertically)
#' @return ggplot object
#' @seealso [plot_statevar_ts()] [plot_tradeoff()]
#' @import ggplot2
#' @examples
#' # Simple decision table
#' results <- data.frame(
#'   PNI = c(0.7, 0.23, 0.05, 0.9, 0.85, 0.74, 0.95, 0.92, 0.9),
#'   pNOB = rep(c(0.5, 0.75, 1), each = 3),
#'   ER = rep(c(0.2, 0.3, 0.4), 3),
#'   scenario = "High productivity"
#' )
#' plot_decision_table(
#'   x = results$ER,
#'   y = results$pNOB,
#'   z = results$PNI,
#'   title = "PNI",
#'   xlab = "Exploitation rate",
#'   ylab = "pNOB target"
#' )
#'
#' # Multiple decision tables organized by scenario
#' # Continuing from above
#' results_low <- results
#' results_low$scenario <- "Low productivity"
#' results_low$PNI <- 0.5 * results$PNI
#'
#' results_all <- rbind(results, results_low)
#' plot_decision_table(
#'   x = results_all$ER,
#'   y = results_all$pNOB,
#'   z = results_all$PNI,
#'   title = "PNI",
#'   xlab = "Exploitation rate",
#'   ylab = "pNOB target",
#'   scenario = results_all$scenario
#' )
#' @export
plot_decision_table <- function(x, y, z, title, xlab, ylab, scenario, ncol = NULL, dir = "v") {
  dt <- data.frame(x = x, y = y, z = z)
  dt$txt <- format(round(dt$z, 2))

  if (!missing(scenario)) dt$scenario <- scenario

  g <- ggplot(dt, aes(factor(.data$x), factor(.data$y), fill = .data$z)) +
    geom_tile(colour = "grey20", alpha = 0.6) +
    geom_text(aes(label = .data$txt)) +
    coord_cartesian(expand = FALSE) +
    guides(fill = "none") +
    theme_bw() +
    theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank()) +
    scale_fill_gradient2(low = "deeppink", high = "green4", mid = "white", midpoint = 0.5)

  if (!missing(scenario)) {
    g <- g +
      facet_wrap(vars(.data$scenario), ncol = ncol, dir = dir) +
      theme(strip.background = element_blank())
  }

  if (!missing(xlab)) g <- g + labs(x = xlab)
  if (!missing(ylab)) g <- g + labs(y = ylab)
  if (!missing(title)) g <- g + ggtitle(title)

  return(g)
}

#' @name plot_decision_table
#' @param zlab Character, optional color legend
#' @param bin Numeric vector of bins to sort values of `z`
#' @param bin_labels Character vector for bin names for the figure
#' @param bin_col Character vector of colors for the bins in the figure
#' @param cell_border Logical, whether to add borders for each cell in the figure
#' @param add_values Logical, whether to add the values of `z` in the figure
#' @examples
#' # Example of binned decision table
#'
#' df <- expand.grid(
#'   SAR = seq(0.005, 0.03, 0.005),
#'   ER = seq(0, 0.5, 0.1)
#' )
#' df$value <- ifelse(5 * df$SAR + 0.2 > df$ER, 0.75, 0.05)
#' df$value <- ifelse(df$SAR < 0.02 & df$ER > 0.4, 0.04, df$value)
#'
#' plot_decision_table2(
#'   x = df$SAR,
#'   y = df$ER,
#'   z = df$value,
#'   xlab = "Marine survival",
#'   ylab = "Exploitation rate",
#'   zlab = "Probability\nof objective"
#' )
#' @export
plot_decision_table2 <- function(x, y, z, title, xlab, ylab, zlab, scenario, ncol = NULL, dir = "v",
                                 bin = c(0, 0.05, 0.25, 0.5, 0.75, 0.95),
                                 bin_labels = c("0-0.04", "0.05-0.24", "0.25-0.49", "0.5-0.74", "0.75-0.94", "0.95-1"),
                                 bin_col = c("purple4", "deeppink", "pink", "white", "green", "green4"),
                                 cell_border = FALSE,
                                 add_values = FALSE) {

  n_bin <- length(bin)
  if (length(bin_labels) != n_bin) stop("length(bin_labels) not equal to length(bin)")
  if (length(bin_col) != n_bin) stop("length(bin_col) not equal to length(bin)")

  dt <- data.frame(x = x, y = y, z = z)
  dt$bin_interval <- findInterval(z, bin)
  dt$bin_value <- factor(bin_col[dt$bin_interval], bin_col)
  dt$txt <- format(round(dt$z, 2))

  if (!missing(scenario)) dt$scenario <- scenario

  g <- ggplot(dt, aes(factor(.data$x), factor(.data$y), fill = .data$bin_value)) +
    geom_tile(colour = ifelse(cell_border, "grey20", NA),
              width = 1, height = 1, alpha = 0.6, show.legend = TRUE) +
    coord_cartesian(expand = FALSE) +
    theme_bw() +
    theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank()) +
    scale_fill_identity(name = if (missing(zlab)) NULL else zlab,
                        guide = guide_legend(),
                        drop = FALSE,
                        limits = bin_col,
                        labels = bin_labels) +
    expand_limits(fill = bin_col)

  if (add_values) g <- g + geom_text(aes(label = .data$txt))

  if (!missing(scenario)) {
    g <- g +
      facet_wrap(vars(.data$scenario), ncol = ncol, dir = dir) +
      theme(strip.background = element_blank())
  }

  if (!missing(xlab)) g <- g + labs(x = xlab)
  if (!missing(ylab)) g <- g + labs(y = ylab)
  if (!missing(title)) g <- g + ggtitle(title)

  return(g)
}

#' Tradeoff figure
#'
#' Generates a tradeoff figure, a comparison between two performance metrics, across two variables which may represent
#' a population dynamics variable (e.g., productivity) or a management action (e.g., hatchery production levels or harvest strategy).
#' See example at \url{https://docs.salmonmse.com/articles/decision-table.html}
#'
#' @param pm1 Numeric or matrix. A vector of values for the first performance metric on the x-axis. Alternatively, provide a three column matrix corresponding
#' to the lower bound, central tendency, and upper bound.
#' @param pm2 Numeric or matrix. A vector of values for the second performance metric on the y-axis (same length as pm1). Alternatively, provide a three column matrix corresponding
#' to the lower bound, central tendency, and upper bound.
#' @param x1 Atomic, vector of values for the first grouping variable. Various levels are represented by colours. Same length as pm1. If numeric, you may want to convert values to factors.
#' @param x2 Atomic, vector of values for the second grouping variable. Various levels are represented by shapes. Same length as pm1. If numeric, will be converted to factors.
#' @param xlab Character, optional x-axis label
#' @param ylab Character, optional y-axis label
#' @param x1lab Character, optional label for the first grouping variable
#' @param x2lab Character, optional label for the second grouping variable
#' @param scenario Atomic, vector of scenario names (same length as `pm1`, `pm2`) used to identify and create separate panels
#' @param scenario_rows Atomic, vector of scenario variables (same length as `pm1`, `pm2`) used to identify the rows in a grid of tradeoff figures. Use with `scenario_cols`
#' @param scenario_cols Atomic, vector of scenario variables (same length as `pm1`, `pm2`) used to identify the columns in a grid of tradeoff figures. Use with `scenario_cols`
#' @param ncol Integer, number of columns in the grid, only used if `scenario` is provided
#' @param dir Character, either "h" or "v" to describe how the grid of tables should be organized (horizontally or vertically) , only used if `scenario` is provided
#' @param add_letters Logical, whether to identify separate panels by letters (only used if scenarios are identified). Helpful for publication-level figures
#' @return ggplot object
#' @seealso [plot_statevar_ts()] [plot_decision_table()]
#' @import ggplot2
#' @importFrom dplyr n
#' @examples
#'
#' # Single tradeoff panel
#' results <- data.frame(
#'   PNI = c(0.7, 0.23, 0.05, 0.9, 0.85, 0.74, 0.95, 0.92, 0.9),
#'   Catch = c(10, 14, 12, 8, 7, 7, 8.2, 7.1, 6.9),
#'   pNOB = rep(c(0.5, 0.75, 1), each = 3),
#'   ER = rep(c(0.2, 0.3, 0.4), 3),
#'   scenario = "High productivity"
#' )
#'
#' plot_tradeoff(
#'   pm1 = results$PNI,
#'   pm2 = results$Catch,
#'   x1 = results$ER,
#'   x2 = results$pNOB,
#'   xlab = "PNI",
#'   ylab = "Catch",
#'   x1lab = "Exploitation\nrate",
#'   x2lab = "pNOB\ntarget"
#' )
#'
#' # Multiple panels, continuing from above code
#' results_low <- results
#' results_low$scenario <- "Low productivity"
#' results_low$PNI <- 0.5 * results$PNI
#' results_low$Catch <- c(0.9, 0.8, 0.7) * results$Catch
#'
#' results_all <- rbind(results, results_low)
#' g <- plot_tradeoff(
#'   pm1 = results_all$PNI,
#'   pm2 = results_all$Catch,
#'   x1 = results_all$ER,
#'   x2 = results_all$pNOB,
#'   xlab = "PNI",
#'   ylab = "Catch",
#'   x1lab = "Exploitation\nrate",
#'   x2lab = "pNOB\ntarget",
#'   scenario = results_all$scenario
#' )
#'
#' # Change legend with ggplot2
#' library(ggplot2)
#' g + scale_shape_manual(values = c(1, 4, 16))
#'
#'
#' @export
plot_tradeoff <- function(pm1, pm2, x1, x2, xlab, ylab, x1lab, x2lab, scenario,
                          scenario_rows, scenario_cols, ncol = NULL, dir = "v",
                          add_letters = FALSE) {

  # Make data frame
  if (missing(x1)) x1 <- 0
  if (missing(x2)) x2 <- 1
  #if (is.numeric(x1)) x1 <- factor(x1)
  if (is.numeric(x2)) x2 <- factor(x2)

  dt <- data.frame(
    pm1 = if (is.matrix(pm1)) pm1[, 2] else pm1,
    pm2 = if (is.matrix(pm1)) pm2[, 2] else pm2,
    pm1_lower = if (is.matrix(pm1)) pm1[, 1] else pm1,
    pm2_lower = if (is.matrix(pm1)) pm2[, 1] else pm2,
    pm1_upper = if (is.matrix(pm1)) pm1[, 3] else pm1,
    pm2_upper = if (is.matrix(pm1)) pm2[, 3] else pm2,
    x1 = x1,
    x2 = x2
  )

  if (!missing(scenario)) {
    dt$scenario <- scenario
  } else if (!missing(scenario_rows) && !missing(scenario_cols)) {
    dt$scenario_rows <- scenario_rows
    dt$scenario_cols <- scenario_cols
  }

  # Initial ggplot call
  g <- ggplot(dt, aes(.data$pm1, .data$pm2, colour = .data$x1, shape = .data$x2))

  # Add points, i.e., the results to figure
  g <- g +
    geom_point() +
    theme_bw() +
    theme(panel.grid = element_blank())

  # Add errorbars
  if (is.matrix(pm1)) {
    g <- g + geom_linerange(aes(xmin = .data$pm1_lower, xmax = .data$pm1_upper), linewidth = 0.25)
  }
  if (is.matrix(pm2)) {
    g <- g + geom_linerange(aes(ymin = .data$pm2_lower, ymax = .data$pm2_upper), linewidth = 0.25)
  }

  # Remove legends if needed
  if (length(unique(x1)) == 1) {
    g <- g + guides(colour = "none")
  }
  if (length(unique(x2)) == 1) {
    g <- g + guides(shape = "none")
  }

  # Do faceting
  if (!missing(scenario)) {

    g <- g +
      facet_wrap(vars(.data$scenario), ncol = ncol, dir = dir) +
      theme(strip.background = element_blank())

    if (add_letters) {
      letter_legend <- expand.grid(
        scenario = unique(dt$scenario)
      ) %>%
        mutate(text = paste0("(", letters[1:n()], ")"))

      g <- g +
        geom_text(data = letter_legend, aes(label = .data$text), inherit.aes = FALSE,
                  x = Inf, y = Inf, hjust = "inward", vjust = "inward")

    }

  } else if (!missing(scenario_rows) && !missing(scenario_cols)) {

    g <- g +
      facet_grid(vars(.data$scenario_rows), vars(.data$scenario_cols)) +
      theme(strip.background = element_blank())

    if (add_letters) {
      letter_legend <- expand.grid(
        scenario_cols = unique(dt$scenario_cols),
        scenario_rows = unique(dt$scenario_rows)
      ) %>%
        mutate(text = paste0("(", letters[1:n()], ")"))

      g <- g +
        geom_text(data = letter_legend, aes(label = .data$text), inherit.aes = FALSE,
                  x = Inf, y = Inf, hjust = "inward", vjust = "inward")
    }
  }

  # Add labels
  if (!missing(xlab)) g <- g + labs(x = xlab)
  if (!missing(ylab)) g <- g + labs(y = ylab)
  if (!missing(x1lab)) g <- g + labs(colour = x1lab)
  if (!missing(x2lab)) g <- g + labs(shape = x2lab)

  return(g)
}


#' Compare state variables from simulation runs
#'
#' @description
#' Compare outputs from multiple simulations to evaluate performance across states of nature and/or management levers (identified by colour):
#' - `compare_statevar_ts()` produces a time series for all simulations, or with medians and 95th percentile intervals
#' - `compare_statevar_hist()` produces a histogram or density plot across all simulations for a particular year
#'
#' @inheritParams plot_statevar_ts
#' @param SMSE_list List of SMSE objects for multiple model runs returned by [salmonMSE()]
#' @param names Character vector `length(SMSE_list)` to label individual model runs
#' @param col_vec Character vector `length(SMSE_list)` for custom colour schemes for comparing across model scenarios in figures
#' @returns An array invisibly. Also generates base graphics if `figure = TRUE`
#' @seealso [plot_statevar_ts()] [compare()]
#' @importFrom grDevices hcl.colors
#' @importFrom graphics polygon matlines
#' @export
compare_statevar_ts <- function(SMSE_list, var = "PNI", s = 1, figure = TRUE, xlab = "Projection Year", quant = FALSE, ylab = var,
                                ylim, agg.fun = sum,
                                names, col_vec, ...) {

  if (missing(names)) names <- paste("Scenario", 1:length(SMSE_list))
  if (missing(col_vec)) {
    col_vec <- hcl.colors(length(SMSE_list), palette = "Dark 3", alpha = 1)
    shade_vec <- hcl.colors(length(SMSE_list), palette = "Dark 3", alpha = 0.25)
  } else {
    if (requireNamespace("scales", quietly = TRUE)) {
      shade_vec <- scales::alpha(col_vec, 0.25)
    } else {
      shade_vec <- col_vec
    }
  }
  if (length(s) > 1) stop("s must be a single integer")

  # if quant = FALSE: Sim x year x scenario
  # if quant = TRUE: 3 x year x scenario
  output <- sapply(SMSE_list, plot_statevar_ts, var = var, s = s, figure = FALSE, quant = quant, agg.fun = agg.fun,
                   simplify = "array")

  if (all(is.na(output))) output[] <- 0

  if (figure) {
    if (missing(ylim)) ylim <- c(0, 1.1) * range(output, na.rm = TRUE)

    Year <- seq(1, dim(output)[2])

    matplot(Year, Year, type = "n", ylim = ylim, xlab = "Projection Year", ylab = ylab, ...)

    for (i in 1:length(SMSE_list)) {
      xplot <- output[, , i]
      if (any(xplot > 0, na.rm = TRUE)) {
        ind <- colSums(xplot, na.rm = TRUE) > 0

        if (!quant) {
          matlines(Year[ind], t(xplot[, ind]), type = 'l', col = col_vec[i], lty = 1, ...)
        } else {
          #matlines(Year[ind], t(xplot[, ind]), type = 'o', col = col_vec[i], pch = c(NA, 1, NA), lty = c(2, 1, 2), ...)
          polygon(c(Year[ind], rev(Year[ind])), c(xplot[1, ind], rev(xplot[3, ind])),
                  col = shade_vec[i], border = NA)
          lines(Year[ind], xplot[2, ind], type = 'o', col = col_vec[i], pch = 1, lty = 1, ...)
        }
      }
    }
    legend("topleft", legend = names, col = col_vec, lty = 1, pch = 1, bty = "n")
  }

  return(invisible(output))
}

#' @rdname compare_statevar_ts
#' @inheritParams plot_statevar_hist
#' @param type Character, whether to generate a density figure or histogram
#' @importFrom stats density
#' @export
compare_statevar_hist <- function(SMSE_list, var = "PNI", s = 1, y, figure = TRUE, xlab = var,
                                  names, col_vec, type = c("density", "hist"), ...) {

  type <- match.arg(type)

  if (missing(names)) names <- paste("Scenario", 1:length(SMSE_list))
  if (missing(col_vec)) {
    col_vec <- hcl.colors(length(SMSE_list), palette = "Dark 3", alpha = 1)
    shade_vec <- hcl.colors(length(SMSE_list), palette = "Dark 3", alpha = 0.25)
  } else {
    if (requireNamespace("scales", quietly = TRUE)) {
      shade_vec <- scales::alpha(col_vec, 0.25)
    } else {
      shade_vec <- col_vec
    }
  }
  if (length(s) > 1) stop("s must be a single integer")

  output <- lapply(SMSE_list, plot_statevar_hist, s = s, y = y, var = var, figure = FALSE)

  if (figure) {
    xlim <- range(unlist(output), na.rm = TRUE)
    plot_made <- FALSE

    if (type == "hist") {
      hist_all <- hist(unlist(output), plot = FALSE)
      hist_i <- lapply(output, hist, breaks = hist_all$breaks, plot = FALSE)
      ylim <- c(0, 1.1) * range(sapply(hist_i, getElement, "counts"))

      for (i in 1:length(SMSE_list)) {
        plot(hist_i[[i]], xlab = xlab, main = NULL,
             col = shade_vec[i], border = col_vec[i], add = i > 1, ylim = ylim, ...)
      }
      plot_made <- TRUE

    } else {
      output_dens <- lapply(output, function(i) try(density(i), silent = TRUE))

      is_dens <- sapply(output_dens, inherits, "density")
      xd <- sapply(output_dens[is_dens], getElement, "x")
      yd <- sapply(output_dens[is_dens], getElement, "y")
      ylim <- c(0, 1.1) * range(yd)
      col_vec2 <- col_vec[is_dens]
      shade_vec2 <- shade_vec[is_dens]

      if (any(is_dens)) {
        plot(NULL, NULL, xlim = xlim, ylim = ylim, type = "n", xlab = xlab, ylab = "Density")
        for (i in 1:sum(is_dens)) {
          polygon(c(xd[, i], rev(xd[, i])), c(yd[, i], rep(0, nrow(yd))), border = col_vec2[i], col = shade_vec2[i])
        }
        plot_made <- TRUE
      }

    }
    if (plot_made) legend("topleft", legend = names, col = col_vec, lty = 1, pch = 1, bty = "n")
  }

  return(invisible(output))
}

#' Harvest component of operating model
#'
#' A function used by openMSE to specify the fishing effort during the projections.
#' salmonMSE updates the arguments of this function from the salmon operating model.
#'
#' @param x Simulation number
#' @param DataList A nested list of [MSEtool::Data-class] objects by stock then fleet, generated by [MSEtool::multiMSE()]
#' @param reps The number of stochastic replicates to be returned by the function
#' @param u_terminal Numeric vector by population (s). Harvest rate of retained catch in the terminal fishery
#' @param u_preterminal Single numeric. Harvest rate of retained catch in the pre-terminal fishery
#' @param K_PT Single numeric. Total retained catch in the pre-terminal fishery
#' @param K_T Numeric vector by population (s). Total retained catch in the terminal fishery
#' @param type_PT Single character, containing either "catch" or "u". Indicates whether to manage by harvest rate or total catch numbers for preterminal fisheries
#' @param type_T Character vector by population (s), containing either "catch" or "u". indicates whether to manage by harvest rate or total catch numbers for terminal fisheries
#' @param MSF_PT Single logical, whether to implement mark-selective fishing for the preterminal fishery
#' @param MSF_T Logical vector by population (s), whether to implement mark-selective fishing for the terminal fishery
#' @param m Numeric vector by population (s). Mark rate of hatchery origin fish, as a proxy for fishery retention. Only used to calculate the fishing effort.
#' Retention in the operating model is specified in the [MSEtool::MOM-class] object
#' @param release_mort Matrix `[2, s]`. Release mortality of discarded fish in the pre-terminal (1st row) and terminal (2nd row) fishery. Only used
#' if `MSF_PT = TRUE` or `MSF_T[s] = TRUE`. Only used to calculate the fishing effort.
#' Release mortality in the operating model is specified in the [MSEtool::MOM-class] object
#' @param p_terminal Numeric vector. Population index (p) for the recruitment that experiences the terminal fishing mortality
#' @param p_preterminal Numeric vector. Population index (p) for immature fish that experience the pre-terminal fishing mortality
#' @param p_natural Numeric vector. Population index (p) for natural origin fish
#' @param p_hatchery Numeric vector. Population index (p) for hatchery origin fish
#' @param pkey Data frame that assigns openMSE population by life stage and origin (p) to salmonMSE population (s)
#'
#' @param ... Not used
#'
#' @return A nested list of [MSEtool::Rec-class] objects, same dimension as `DataList`
#'
#' @keywords internal
Harvest_MMP <- function(x = 1, DataList, reps = 1,
                        u_terminal, u_preterminal, K_PT, K_T,
                        type_PT = "u", type_T = "u", MSF_PT = FALSE, MSF_T = FALSE, m, release_mort,
                        p_terminal = c(2, 5), p_preterminal = c(1, 4), pkey = data.frame(p = 1:6, s = 1),
                        p_natural = 1:3, p_hatchery = 4:6, ...) {

  type_PT <- match.arg(type_PT, choices = c("u", "catch"), several.ok = TRUE)
  type_T <- match.arg(type_T, choices = c("u", "catch"), several.ok = TRUE)

  np <- length(DataList)
  nf <- length(DataList[[1]])

  y <- max(DataList[[1]][[1]]@Year) - DataList[[1]][[1]]@LHYear + 1
  nyears <- length(DataList[[1]][[1]]@Misc$FleetPars$Find[x, ])

  odd_time_step <- as.logical(y %% 2)

  multiRec <- lapply(1:np, function(p) {

    Effort <- 0 # Stays at zero unless otherwise

    Nage_p <- rowSums(DataList[[p]][[1]]@Misc$StockPars$N_P[x, , y, ], na.rm = TRUE)

    if (sum(Nage_p)) {

      #### Preterminal is mixed stock fishery ----
      if (odd_time_step && p %in% p_preterminal) {

        if (MSF_PT) {
          # MSF, solve for effort based on kept catch of all HO juveniles
          p_HOS_PT <- intersect(p_preterminal, p_hatchery) # Identify HO juveniles in openMSE (p)
          s_PT <- pkey$s[match(p_HOS_PT, pkey$p)] # Identify salmonMSE population (s) corresponding to p

          # Abundance and vulnerability
          Nage_PT <- sapply(p_HOS_PT, function(pp) rowSums(DataList[[pp]][[1]]@Misc$StockPars$N_P[x, , y, ]))
          V_PT <- sapply(p_HOS_PT, function(pp) DataList[[pp]][[1]]@Misc$FleetPars$V[x, , nyears + y])

          # Mark rate (as retention) and release mortality
          m_s <- matrix(m[s_PT], nrow(Nage_PT), ncol(Nage_PT), byrow = TRUE)
          relmort_s <- matrix(release_mort[1, s_PT], nrow(Nage_PT), ncol(Nage_PT), byrow = TRUE)
        } else {
          # No MSF, solve for effort based on kept catch of all juveniles (NO + HO)
          Nage_PT <- sapply(p_preterminal, function(pp) rowSums(DataList[[pp]][[1]]@Misc$StockPars$N_P[x, , y, ]))
          V_PT <- sapply(p_preterminal, function(pp) DataList[[pp]][[1]]@Misc$FleetPars$V[x, , nyears + y])

          m_s <- 1
          relmort_s <- 0
        }

        Effort <- get_F(
          u = u_preterminal, K = K_PT, type = type_PT,
          M = array(0, dim(Nage_PT)), N = Nage_PT,
          vul = V_PT, ret = m_s, release_mort = relmort_s
        )

      } else if (!odd_time_step && p %in% p_terminal) {

        #### Terminal fishery is a single stock fishery ----
        # Identify salmonMSE population s corresponding openMSE population p
        s_T <- pkey$s[match(p, pkey$p)]
        if (length(s_T) > 1) stop("length(s_T) > 1")

        if (MSF_T[s_T]) {
          # MSF, solve for effort based on kept catch of HO return corresponding to s
          p_HOS_T_all <- intersect(p_terminal, p_hatchery)
          p_HOS_T <- intersect(p_HOS_T_all, pkey$p[pkey$s == s_T])

          # Abundance and vulnerability
          Nage_T <- sapply(p_HOS_T, function(pp) rowSums(DataList[[pp]][[1]]@Misc$StockPars$N_P[x, , y, ]))
          V_T <- sapply(p_HOS_T, function(pp) DataList[[pp]][[1]]@Misc$FleetPars$V[x, , nyears + y])

          # Mark rate (as retention) and release mortality
          m_s <- matrix(m[s_T], nrow(Nage_T), ncol(Nage_T))
          relmort_s <- matrix(release_mort[2, s_T], nrow(Nage_T), ncol(Nage_T))

        } else {
          # No MSF, solve for effort based on kept catch of all return (NO + HO) corresponding to s
          p_T <- intersect(p_terminal, pkey$p[pkey$s == s_T])

          Nage_T <- sapply(p_T, function(pp) rowSums(DataList[[pp]][[1]]@Misc$StockPars$N_P[x, , y, ]))
          V_T <- sapply(p_T, function(pp) DataList[[pp]][[1]]@Misc$FleetPars$V[x, , nyears + y])

          m_s <- 1
          relmort_s <- 0
        }

        Effort <- get_F(
          u = u_terminal[s_T], K = K_T[s_T], type = type_T[s_T],
          M = array(0, dim(Nage_T)), N = Nage_T,
          vul = V_T, ret = m_s, release_mort = relmort_s
        )
      }
    }

    lapply(1:nf, function(f) {
      Rec <- new("Rec")
      if (p %in% c(p_terminal, p_preterminal)) {
        HistE <- DataList[[p]][[f]]@OM$FinF[x] # Last historical fishing effort
        Rec@Effort <- rep(Effort/HistE, reps)
      } else {
        Rec@Effort <- rep(0, reps)
      }
      return(Rec)
    })
  })

  return(multiRec)
}

#' @rdname salmonMSE-int
#' @return
#' `make_Harvest_MMP`: Function of class "MMP" by updating the formal arguments for [Harvest_MMP()]
#' @export
make_Harvest_MMP <- function(SOM, check = TRUE) {

  if (check) SOM <- check_SOM(SOM)

  pindex <- make_stock_index(SOM)

  f <- Harvest_MMP

  # Terminal fishery, specific to population
  formals(f)$type_T <- vapply(SOM@Harvest, slot, character(1), "type_T")
  formals(f)$u_terminal <- vapply(SOM@Harvest, slot, numeric(1), "u_terminal")
  formals(f)$K_T <- vapply(SOM@Harvest, slot, numeric(1), "K_T")
  formals(f)$MSF_T <- vapply(SOM@Harvest, slot, logical(1), "MSF_T")

  formals(f)$m <- vapply(SOM@Hatchery, slot, numeric(1), "m")
  formals(f)$release_mort <- vapply(SOM@Harvest, slot, numeric(2), "release_mort")

  # Pre-terminal fishery, shared for all populations
  formals(f)$type_PT <- SOM@Harvest[[1]]@type_PT
  formals(f)$u_preterminal <- SOM@Harvest[[1]]@u_preterminal
  formals(f)$K_PT <- SOM@Harvest[[1]]@K_PT
  formals(f)$MSF_PT <- SOM@Harvest[[1]]@MSF_PT

  formals(f)$p_preterminal <- pindex$p[pindex$stage == "juvenile"]
  formals(f)$p_terminal <- pindex$p[pindex$stage == "recruitment"]
  formals(f)$p_natural <- pindex$p[pindex$origin == "natural"]
  formals(f)$p_hatchery <- pindex$p[pindex$origin == "hatchery"]
  formals(f)$pkey <- pindex[, c("s", "p")]

  if (formals(f)$MSF_PT && !length(formals(f)$p_hatchery)) {
    stop("Mark-selective fishing is TRUE for preterminal fishery but there are no hatchery populations")
  }
  if (any(formals(f)$MSF_T)) {
    ns <- length(SOM@Bio)

    for (s in 1:ns) {
      MSF_s <- formals(f)$MSF_T[s]
      p_hatchery <- pindex[pindex$s == s & pindex$origin == "hatchery", ]

      if (MSF_s && !nrow(p_hatchery)) {
        stop("Mark-selective fishing is TRUE for terminal fishery but there is no hatchery for population ", s)
      }
    }
  }

  class(f) <- "MMP"
  return(f)
}

#' Calculate F from harvest rate
#'
#' Solves for apical instantaneous fishing mortality rate (F), proportional to fishing effort, from harvest rate (total retained catch over total abundance).
#' The apical F can be greater than the realized F, if retention < 1.
#'
#' @param u Harvest rate, between 0-1
#' @param K Catch, between 0-Inf
#' @param type Character, either `"catch"`, or `"u"`, whether to solve for catch or harvest rate, respectively
#' @param M Instantaneous natural mortality rate
#' @param N Abundance
#' @param vul Vulnerability
#' @param ret Retention rate
#' @param release_mort Release mortality as a proportion, between 0-1. Only relevant if `ret < 1`.
#' @param Fmax Maximum allowable value of F
#' @return Numeric for the apical F
#'
#' @keywords internal
get_F <- function(u = 0, K = 0, type = c("u", "catch"), M, N = 1, vul = 1, ret = 1, release_mort = 0, Fmax = 20) {
  type <- unique(type)

  type <- match.arg(type)
  Fout <- 0

  solve_u <- type == "u" && u > 0
  solve_K <- type == "catch" && K > 0

  if (solve_u || solve_K) {
    .F <- try(
      uniroot(F_solver, interval = c(0, Fmax), M = M, N = N, vul = vul, ret = ret, release_mort = release_mort, u = u, K = K, type),
      silent = TRUE
    )

    if (is.character(.F)) {
      Fout <- Fmax
    } else {
      Fout <- .F$root
    }
  }

  return(Fout)
}

F_solver <- function(.F, M, N = 1, vul = 1, ret = 1, release_mort = 0, u = 0, K = 0, type = c("u", "catch")) {
  type <- match.arg(type)

  F_ret <- vul * ret * .F
  F_rel <- vul * (1 - ret) * release_mort * .F
  Z <- F_ret + F_rel + M
  catch_ret <- F_ret/Z * (1 - exp(-Z)) * N
  catch_ret[is.na(catch_ret)] <- 0

  if (type == "u") {
    fn <- (1 - exp(-max(F_ret))) - u
  } else {
    fn <- sum(catch_ret)/K - 1
  }
  return(fn)

}

#' Plot life history groups and release strategies
#'
#' Plot the annual proportions of life history groups (natural origin fish) or release strategies (hatchery origin) at various life stages
#'
#' @param SMSE Class \linkS4class{SMSE} object returned by [salmonMSE()]
#' @param var Character. Slot for the state variables in `SMSE@Misc$LHG[[1]]` or `SMSE@Misc$RS[[1]]`.
#' @param type Character to indicate whether to plot proportion or absolute numbers
#' @param s Integer. Population index for multi-population model (e.g., `s = 1` is the first population in the model)
#' @param FUN Summarizing function across simulations, typically [median()] or [mean()]
#' @param xlab Character. Name of time variable for the figure
#' @param figure Logical, whether to generate a figure (set to FALSE if only using the function to return the data matrix)
#' @param ylab Character. Name of the state variable for the figure
#' @param name Character. Vector of names for the life history groups or release strategies
#' @param ylim Vector length 2, y-axis limits
#' @importFrom grDevices hcl.colors
#' @seealso [plot_statevar_ts()]
#' @export
#' @return Base graphics figure, barplot of distribution or total numbers by LHG or RS. Returns invisibly the matrix of plotted values
#' @importFrom graphics box
plot_LHG <- function(SMSE, var = "NOS", type = c("prop", "abs"), s = 1, FUN = median, figure = TRUE, xlab = "Projection Year",
                     ylab, name, ylim) {

  LHG <- SMSE@Misc$LHG[[s]]
  if (length(LHG)) {
    var <- match.arg(var, choices = names(LHG))
    x <- LHG[[var]]

    if (missing(name)) name <- paste("LHG", 1:dim(x)[2])
    .plot_LHG(x, var, type, FUN, figure, xlab, ylab, name, palette = "Dark 2", ylim)
  } else {
    x <- array(1, c(1, 1, SMSE@proyears))
    return(invisible(x))
  }

}

#' @rdname plot_LHG
#' @export
plot_RS <- function(SMSE, var = "HOS", type = c("prop", "abs"), s = 1, FUN = median, figure = TRUE, xlab = "Projection Year",
                    ylab, name, ylim) {

  RS <- SMSE@Misc$RS[[s]]
  if (length(RS)) {
    var <- match.arg(var, choices = names(RS))
    x <- RS[[var]]

    if (missing(name)) name <- paste("RS", 1:dim(x)[2])
    .plot_LHG(x, var, type, FUN, figure, xlab, ylab, name, palette = "Cold", ylim)
  } else {
    x <- array(1, c(1, 1, SMSE@proyears))
    return(invisible(x))
  }

}

.plot_LHG <- function(x, var, type = c("prop", "abs"), FUN, figure = TRUE, xlab = "Projection Year", ylab, name, palette = "Dark 2", ylim) {

  type <- match.arg(type)

  if (length(dim(x)) == 4) { # Sum across age
    x <- apply(x, c(1, 2, 4), sum)
  }

  if (type == "abs") {
    xplot <- apply(x, 2:3, FUN)
  } else {
    xplot <- apply(x, c(1, 3), function(i) i/sum(i)) %>% apply(c(1, 3), FUN)
  }

  if (figure && any(xplot > 0, na.rm = TRUE)) {
    ind <- colSums(xplot, na.rm = TRUE) > 0
    Year <- 1:ncol(xplot)

    col <- grDevices::hcl.colors(nrow(xplot), palette = palette)

    if (type == "abs") {
      if (missing(ylim)) ylim <- c(0, 1.1) * range(colSums(xplot, na.rm = TRUE))
      if (missing(ylab)) ylab <- var
    } else {
      if (missing(ylim)) ylim <- c(0, 1)
      if (missing(ylab)) ylab <- paste("Proportion", var)
    }
    plot(Year, NULL, xlim = range(Year) + c(-1, 0),
         xlab = "Projection Year", ylab = ylab,
         type = "n", ylim = ylim, xaxs = "i", yaxs = "i")
    barplot(xplot, legend.text = name, space = 0, xlim = range(Year),
            col = col, border = "grey40", add = TRUE, xpd = FALSE)
    box()
  }

  invisible(xplot)
}

---
title: "salmonMSE projection"
date: "`r Sys.Date()`"
output:
  html_document:
    highlight: rstudio
    df_print: paged
---

<style type="text/css">
h1 { /* Header 1 */
  font-size: 24px;
}
</style>

```{r setup, include = FALSE, echo = FALSE}
library(dplyr)
knitr::opts_chunk$set(
  collapse = TRUE, echo = FALSE, message = FALSE,
  fig.width = 6, fig.height = 4.5, out.width = "650px", comment = "#>"
)
```

# NAME {.tabset}

<!-- ADD RMD BY STOCK -->

## About

This report was generated on: `r Sys.time()`<br />
salmonMSE version `r packageVersion("salmonMSE")`<br />
`r R.version.string`

---
title: "salmonMSE conditioning model"
date: "`r Sys.Date()`"
output:
  html_document:
    highlight: rstudio
    df_print: paged
---

<style type="text/css">
h1 { /* Header 1 */
  font-size: 24px;
}
</style>

```{r setup, include = FALSE, echo = FALSE}
library(ggplot2)
library(salmonMSE)
knitr::opts_chunk$set(
  collapse = TRUE, echo = FALSE, message = FALSE,
  fig.width = 6, fig.height = 4.5, out.width = "650px", comment = "#>"
)
theme_set(theme_bw())
```

# NAME {.tabset}

## Summary

```{r}
data.frame(
  Parameter = c("Historical years", "Age classes", "MCMC iterations"),
  Value = c(d$Ldyr, d$Nages, length(report))
)
```

```{r}
df_pars <- rstan::summary(stanfit)$summary %>%
  round(3) %>%
  as.data.frame()
df_pars[, c("mean", "sd", "2.5%", "50%", "97.5%", "n_eff", "Rhat")]
```

### Legend

```{r par-key, results="asis"}
knitr::kable(make_CM_table(fit), row.names = FALSE)
```

## MCMC

```{r trace-cr, fig.cap="Wormplots of productivity parameters, scaling parameters and the log-posterior from MCMC iterations after thinning and warmup."}
pars_core <- c("cr", "log_so", "moadd", "log_FbasePT", "log_FbaseT", "lp")
stan_trace(stanfit, pars_core)
```

```{r trace-cov, fig.cap="Wormplots of coefficients for mortality covariates from MCMC iterations after thinning and warmup."}
pars_cov <- c("^b")
stan_trace(stanfit, pars_cov)
```

```{r trace-vul, fig.cap="Wormplots of fishery vulnerability at age parameters from MCMC iterations after thinning and warmup."}
pars_vul <- c("^logit_vul")
stan_trace(stanfit, pars_cov)
```

```{r trace-var, fig.cap="Wormplots of variance parameters from MCMC iterations after thinning and warmup."}
pars_var <- c("lnE_sd", "wt_sd", "wto_sd", "fanomalyPT_sd", "fanomalyT_sd", "sd_matt")
stan_trace(stanfit, pars_var)
```

```{r pairs-cr, fig.cap="Marginal posterior distributions and correlations of productivity parameters, scaling parameters and the log-posterior (from MCMC iterations after thinning and warmup)."}
pairs_panel(stanfit, pars_core)
```

```{r pairs-cov, fig.cap="Marginal posterior distributions and correlations of coefficients for mortality covariates."}
pairs_panel(stanfit, pars_cov)
```

```{r pairs-vul, fig.cap="Marginal posterior distributions and correlations for fishery vulnerability at age parameters."}
pairs_panel(stanfit, pars_vul)
```

```{r pairs-var, fig.cap="Marginal posterior distributions and correlations of variance parameters."}
pairs_panel(stanfit, pars_var)
```

```{r wt, fig.cap="Posterior estimates of egg mortality deviates from the stock-recruit relationship (medians and 95% prediction interval)."}
CM_wt(stanfit, year1)
```

```{r wto, fig.cap="Posterior estimates of age 1 mortality deviates (medians and 95% prediction interval)."}
CM_wto(stanfit, year1)
```

## Data and fits {.tabset}

### Hatchery and escapement

```{r hatch-rel, fig.cap="Total hatchery releases."}
CM_data(d$hatchrelease, c(year, max(year) + 1), ylab = "Hatchery release")
```

```{r fit-esc, fig.cap="Observed total escapement (points with dotted lines) and posterior predicted values (medians and 95% prediction interval)."}
CM_fit_esc(report, d, year)
```

```{r cov1, fig.cap="Linear covariates to natural mortality in age 1."}
CM_covariate(d[["covariate1"]], cov1_names, year1, ylab = "M covariates (age 1)")
```

```{r cov, fig.cap="Linear covariates to natural mortality in ages 2+."}
CM_covariate(d[["covariate"]], cov_names, year1, ylab = "M covariates (age 2+)")
```

### CWT

```{r cwt-rel, fig.cap="Total CWT releases."}
CM_CWTrel(d$cwtrelease, year1, rs_names)
```

```{r fit-cwt-esc, fig.cap="Observed coded wire tag escapement at age (points with dotted lines) and posterior predicted values (medians and 95% prediction interval)."}
CM_fit_CWTesc(report, d, year1, rs_names)
```

```{r fit-cwt-catpt, fig.cap="Observed coded wire tag preterminal catch at age (points with dotted lines) and posterior predicted values (medians and 95% prediction interval)."}
CM_fit_CWTcatch(report, d, PT = TRUE, year1, rs_names)
```

```{r fit-cwt-catt, fig.cap="Observed coded wire tag terminal catch at age (points with dotted lines) and posterior predicted values (medians and 95% prediction interval)."}
CM_fit_CWTcatch(report, d, PT = FALSE, year1, rs_names)
```

```{r mat-cwt-rs, fig.cap="Brood year maturity by release strategy."}
if (d$n_r > 1) {
  CM_maturity(report, d, year1, rs_names = rs_names, annual = TRUE)
}
```

## State variables {.tabset}

### Spawners/Broodtake

```{r Spawners, fig.cap="Number of spawners by origin (medians and 95% prediction interval)."}
CM_ts_origin(report, year1, var = "Spawners")
```

```{r pHOS, fig.cap="Proportion hatchery origin spawners (medians and 95% prediction interval)."}
.CM_ts(report, year1, var = "pHOScensus", ci = TRUE, ylab = "pHOScensus")
```

```{r pNOB, fig.cap="Proportion natural origin broodtake, assuming non-selective broodtake (medians and 95% prediction interval)."}
.CM_ts(report, year1, var = "pNOB", ci = TRUE, ylab = "pNOB")
```

```{r PNI, fig.cap="Proportionate natural influence."}
report <- lapply(report, function(i) {
  i$PNI <- i$pNOB/(i$pNOB + i$pHOSeff)
  return(i)
})
.CM_ts(report, year1, var = "PNI", ci = TRUE, ylab = "PNI")
```

### Egg production

```{r SRR, fig.cap="Estimated stock-recruit relationship (medians and 95% prediction interval). Points show annual smolt production (medians)."}
CM_SRR(report, year1)
```

```{r egg, fig.cap="Egg production (medians and 95% prediction interval)."}
.CM_ts(report, year1, var = "egg", ci = TRUE, ylab = "Egg production")
```

```{r smolt, fig.cap="Natural smolt production (medians and 95% prediction interval)."}
report <- lapply(report, function(i) {
  i$Smolt <- rowSums(i$N[, 1, ])
  return(i)
})
.CM_ts(report, year1, var = "Smolt", ci = TRUE, ylab = "Smolt production")
```

```{r Megg, fig.cap="Estimated egg-smolt instantaneous mortality rate (medians and 95% prediction interval)."}
CM_Megg(report, year1, ci = TRUE, surv = FALSE)
```

```{r Meggsurv, fig.cap="Estimated egg-smolt survival (medians and 95% prediction interval)."}
CM_Megg(report, year1, ci = TRUE, surv = TRUE) +
  coord_cartesian(ylim = c(0, 1))
```

### Juvenile abundance

```{r Njuv, fig.cap="Juvenile abundance at age (medians and 95% prediction interval). Natural origin age 1 fish corresponds to smolt production."}
CM_Njuv(report, year1, ci = FALSE)
```

```{r M, fig.cap="Juvenile instantaneous natural mortality at age (medians and 95% prediction interval)."}
CM_M(report, year1)
```

```{r Msurv, fig.cap="Juvenile natural survival at age (medians and 95% prediction interval)."}
CM_surv(report, year1) +
  coord_cartesian(ylim = c(0, 1))
```

```{r Mcov1, fig.cap="Contribution to natural mortality in age 1 from covariates."}
CM_covariate(d[["covariate1"]], cov1_names, year1, b = rstan::extract(stanfit, "b1")[["b1"]], ylab = "Natural mortality (age 1)")
```

```{r Mcov, fig.cap="Contribution to natural mortality in age 1 from covariates."}
CM_covariate(d[["covariate"]], cov_names, year1, b = rstan::extract(stanfit, "b")[["b"]], ylab = "Natural mortality (age 2+)")
```

### Preterminal fishery

```{r vulPT, fig.cap="Posterior estimates of preterminal vulnerability at age (medians and 95% prediction interval)."}
CM_vul(report, type = "vulPT")
```

```{r FPT, fig.cap="Preterminal instantaneous fishing mortality (medians and 95% prediction interval)."}
CM_F(report, PT = TRUE, year1)
```

```{r CYER-PT, fig.cap="Return year preterminal exploitation rate at age (medians and 95% prediction interval)."}
CM_ER(report, brood = FALSE, type = "PT", year1, r = d$r_matt)
```

```{r BYER-PT, fig.cap="Brood year preterminal exploitation rate at age (medians and 95% prediction interval)."}
CM_ER(report, brood = TRUE, type = "PT", year1, r = d$r_matt)
```

### Return

```{r matt-RY, fig.cap="Posterior estimates of maturity by age and return year (medians and 95% prediction interval). Dashed, horizontal lines represent prior means."}
CM_maturity(report, d, year1, r = d$r_matt, brood = FALSE, rs_names = rs_names)
```

```{r matt-BY, fig.cap="Posterior estimates of maturity by age and brood year (medians and 95% prediction interval). Dashed, horizontal lines represent prior means."}
CM_maturity(report, d, year1, r = d$r_matt, brood = TRUE, rs_names = rs_names)
```

```{r recr, fig.cap="Recruitment at age (medians and 95% prediction interval)."}
CM_recr(report, year1, ci = TRUE)
```

### Terminal fishery

```{r vulT, fig.cap="Posterior estimates of terminal vulnerability at age (medians and 95% prediction interval)."}
CM_vul(report, type = "vulT")
```

```{r FT, fig.cap="Terminal instantaneous fishing mortality (medians and 95% prediction interval)."}
CM_F(report, PT = FALSE, year1)
```

```{r CYER-T, fig.cap="Return year terminal exploitation rate at age (medians and 95% prediction interval)."}
CM_ER(report, brood = FALSE, type = "T", year1, r = d$r_matt)
```

```{r BYER-T, fig.cap="Brood year terminal exploitation rate at age (medians and 95% prediction interval)."}
CM_ER(report, brood = TRUE, type = "T", year1, r = d$r_matt)
```

### Escapement

```{r esc, fig.cap="Escapement at age (medians and 95% prediction interval)."}
CM_esc(report, year1)
```

## About

This report was generated on: `r Sys.time()`<br />
salmonMSE version `r packageVersion("salmonMSE")`<br />
`r R.version.string`

---
title: "Comparison of salmonMSE projections"
date: "`r Sys.Date()`"
output:
  html_document:
    highlight: rstudio
    df_print: paged
---

<style type="text/css">
h1 { /* Header 1 */
  font-size: 24px;
}
</style>

```{r setup, include = FALSE, echo = FALSE}
library(dplyr)
knitr::opts_chunk$set(
  collapse = TRUE, echo = FALSE, message = FALSE,
  fig.width = 6, fig.height = 4.5, out.width = "650px", comment = "#>"
)
```

# Time series {.tabset}

<!-- ADD RMD BY STOCK -->

## About

This report was generated on: `r Sys.time()`<br />
salmonMSE version `r packageVersion("salmonMSE")`<br />
`r R.version.string`

---
title: "salmonMSE conditioning model"
date: "`r Sys.Date()`"
output:
  html_document:
    highlight: rstudio
    df_print: paged
---

<style type="text/css">
h1 { /* Header 1 */
  font-size: 24px;
}
</style>

```{r setup, include = FALSE, echo = FALSE}
library(ggplot2)
library(salmonMSE)
knitr::opts_chunk$set(
  collapse = TRUE, echo = FALSE, message = FALSE,
  fig.width = 6, fig.height = 4.5, out.width = "650px", comment = "#>"
)
theme_set(theme_bw())
```

# NAME {.tabset}

## Summary

```{r}
data.frame(
  Parameter = c("Historical years", "Age classes", "MCMC iterations"),
  Value = c(d$Ldyr, d$Nages, length(report))
)
```

```{r}
df_pars <- rstan::summary(stanfit)$summary %>%
  round(3) %>%
  as.data.frame()
df_pars[, c("mean", "sd", "2.5%", "50%", "97.5%", "n_eff", "Rhat")]
```

### Legend

```{r par-key, results="asis"}
knitr::kable(make_CM_table(fit), row.names = FALSE)
```

## MCMC

```{r trace-cr, fig.cap="Wormplots of productivity parameters, scaling parameters and the log-posterior from MCMC iterations after thinning and warmup."}
pars_core <- c("cr", "log_so", "moadd", "log_FbasePT", "log_FbaseT", "lp")
CM_trace(stanfit, pars_core)
```

```{r trace-cov, fig.cap="Wormplots of coefficients for mortality covariates from MCMC iterations after thinning and warmup."}
pars_cov <- c("^b")
CM_trace(stanfit, pars_cov)
```

```{r trace-vul, fig.cap="Wormplots of fishery vulnerability at age parameters from MCMC iterations after thinning and warmup."}
pars_vul <- c("^logit_vul")
CM_trace(stanfit, pars_cov)
```

```{r trace-var, fig.cap="Wormplots of variance parameters from MCMC iterations after thinning and warmup."}
pars_var <- c("lnE_sd", "wt_sd", "wto_sd", "fanomalyPT_sd", "fanomalyT_sd", "sd_matt")
CM_trace(stanfit, pars_var)
```

```{r pairs-cr, fig.cap="Marginal posterior distributions and correlations of productivity parameters, scaling parameters and the log-posterior (from MCMC iterations after thinning and warmup)."}
CM_pairs(stanfit, pars_core)
```

```{r pairs-cov, fig.cap="Marginal posterior distributions and correlations of coefficients for mortality covariates."}
CM_pairs(stanfit, pars_cov)
```

```{r pairs-vul, fig.cap="Marginal posterior distributions and correlations for fishery vulnerability at age parameters."}
CM_pairs(stanfit, pars_vul)
```

```{r pairs-var, fig.cap="Marginal posterior distributions and correlations of variance parameters."}
CM_pairs(stanfit, pars_var)
```

```{r wt, fig.cap="Posterior estimates of egg mortality deviates from the stock-recruit relationship (medians and 95% prediction interval)."}
CM_wt(stanfit, year1)
```

```{r wto, fig.cap="Posterior estimates of age 1 mortality deviates (medians and 95% prediction interval)."}
CM_wto(stanfit, year1)
```

## Data and fits {.tabset}

### Hatchery and escapement

```{r hatch-rel, fig.cap="Total hatchery releases."}
CM_data(d$hatchrelease, c(year, max(year) + 1), ylab = "Hatchery release", xlab = "Release Year")
```

```{r fit-esc, fig.cap="Observed total escapement (return to river, points with dotted lines) and posterior predicted values (medians and 95% prediction interval)."}
CM_fit_esc(report, d, year)
```

```{r fit-pHOS, fig.cap="Observed pHOS (points with dotted lines) and posterior predicted values (medians and 95% prediction interval)."}
CM_fit_pHOS(report, d, year)
```

```{r cov1, fig.cap="Linear covariates to natural mortality in age 1."}
CM_covariate(d[["covariate1"]], cov1_names, year1, ylab = "M covariates (age 1)")
```

```{r cov, fig.cap="Linear covariates to natural mortality in ages 2+."}
CM_covariate(d[["covariate"]], cov_names, year1, ylab = "M covariates (age 2+)")
```

### CWT

```{r cwt-rel, fig.cap="Total CWT releases."}
CM_CWTrel(d$cwtrelease, year1, rs_names)
```

```{r fit-cwt-catpt, fig.cap="Observed coded wire tag preterminal catch at age (points with dotted lines) and posterior predicted values (medians and 95% prediction interval)."}
CM_fit_CWTcatch(report, d, PT = TRUE, year1, rs_names)
```

```{r fit-cwt-catt, fig.cap="Observed coded wire tag terminal catch at age (points with dotted lines) and posterior predicted values (medians and 95% prediction interval)."}
CM_fit_CWTcatch(report, d, PT = FALSE, year1, rs_names)
```

```{r fit-cwt-esc, fig.cap="Observed coded wire tag escapement at age (points with dotted lines) and posterior predicted values (medians and 95% prediction interval)."}
CM_fit_CWTesc(report, d, year1, rs_names)
```

```{r mat-cwt-rs, fig.cap="Brood year maturity by release strategy."}
if (d$n_r > 1) {
  CM_maturity(report, d, year1, rs_names = rs_names, annual = TRUE)
}
```

## State variables {.tabset}

### Spawners/Broodtake

```{r Spawners, fig.cap="Number of spawners by origin (medians and 95% prediction interval)."}
CM_ts_origin(report, year1, var = "Spawners", xlab = "Return Year")
```

```{r pHOS, fig.cap="Proportion hatchery origin spawners (medians and 95% prediction interval)."}
.CM_ts(report, year1, var = "pHOScensus", ci = TRUE, ylab = "pHOScensus", xlab = "Return Year")
```

```{r pNOB, fig.cap="Proportion natural origin broodtake, assuming non-selective broodtake (medians and 95% prediction interval)."}
.CM_ts(report, year1, var = "pNOB", ci = TRUE, ylab = "pNOB", xlab = "Brood Year")
```

```{r PNI, fig.cap="Proportionate natural influence."}
report <- lapply(report, function(i) {
  i$PNI <- i$pNOB/(i$pNOB + i$pHOSeff)
  return(i)
})
.CM_ts(report, year1, var = "PNI", ci = TRUE, ylab = "PNI", xlab = "Brood Year")
```

### Egg production

```{r SRR, fig.cap="Estimated stock-recruit relationship (medians and 95% prediction interval). Numbers correspond to years of egg production (predicts juvenile production for the following calendar year). Points show annual smolt production (medians)."}
CM_SRR(report, year1)
```

```{r egg, fig.cap="Egg production (medians and 95% prediction interval)."}
.CM_ts(report, year1, var = "egg", ci = TRUE, ylab = "Egg production", xlab = "Return Year")
```

```{r smolt, fig.cap="Natural smolt production (outmigrating juveniles, medians and 95% prediction interval)."}
report <- lapply(report, function(i) {
  i$Smolt <- i$N[, 1, 1]
  return(i)
})
.CM_ts(report, year1 - 1, var = "Smolt", ci = TRUE, ylab = "Smolt production", xlab = "Brood Year")
```

```{r Megg, fig.cap="Estimated egg-smolt instantaneous mortality rate (medians and 95% prediction interval)."}
CM_Megg(report, year1, ci = TRUE, surv = FALSE)
```

```{r Meggsurv, fig.cap="Estimated egg-smolt survival of natural origin fish (medians and 95% prediction interval)."}
CM_Megg(report, year1, ci = TRUE, surv = TRUE) +
  coord_cartesian(ylim = c(0, 1))
```

```{r prod, fig.cap="Realized productivity (medians and 95% prediction interval) from annual maturity and juvenile natural mortality values and constant density-dependent egg-juvenile survival. Values are less than 1 when there is insufficient spawners because juvenile natural mortality is high and/or maturity is late."}
CM_prod(report, d, year1)
```

```{r Srep, fig.cap="Realized spawners at replacement (medians and 95% prediction interval) from annual maturity and natural mortality values and constant density-dependent egg-juvenile survival. Values are less than zero when realized productivity is less than 1."}
CM_Srep(report, d, year1)
```

```{r Erep, fig.cap="Realized egg production at replacement (medians and 95% prediction interval) from annual maturity and natural mortality values and constant density-dependent egg-juvenile survival. Values are less than zero when realized productivity is less than 1."}
CM_Srep(report, d, year1, type = "egg")
```

### Juvenile abundance

```{r Njuv, fig.cap="Juvenile abundance at age (medians and 95% prediction interval). Natural origin age 1 fish corresponds to smolt production (outmigrating juveniles)."}
CM_Njuv(report, year1, ci = FALSE)
```

```{r M, fig.cap="Juvenile instantaneous natural mortality at age (medians and 95% prediction interval)."}
CM_M(report, year1)
```

```{r Msurv, fig.cap="Juvenile natural survival at age (medians and 95% prediction interval)."}
CM_surv(report, year1) +
  coord_cartesian(ylim = c(0, 1))
```

```{r M2, fig.cap="Survival to age 2. All fish experience age 1 mortality (first year of marine life stage) and hatchery origin fish may have lower survival if release mortality is specified."}
CM_surv2(report, year1)
```

```{r Mcov1, fig.cap="Contribution to natural mortality in age 1 from covariates."}
CM_covariate(d[["covariate1"]], cov1_names, year1, b = rstan::extract(stanfit, "b1")[["b1"]], ylab = "Natural mortality (age 1)")
```

```{r Mcov, fig.cap="Contribution to natural mortality in age 1 from covariates."}
CM_covariate(d[["covariate"]], cov_names, year1, b = rstan::extract(stanfit, "b")[["b"]], ylab = "Natural mortality (age 2+)")
```

### Preterminal fishery

```{r vulPT, fig.cap="Posterior estimates of preterminal vulnerability at age (medians and 95% prediction interval)."}
FPT <- sapply(report, getElement, "FPT")
if (sum(FPT)) CM_vul(report, type = "vulPT")
```

```{r FPT, fig.cap="Preterminal instantaneous fishing mortality (medians and 95% prediction interval)."}
if (sum(FPT)) CM_F(report, PT = TRUE, year1)
```

```{r CYER-PT, fig.cap="Return year preterminal exploitation rate at age (medians and 95% prediction interval)."}
if (sum(FPT)) CM_ER(report, brood = FALSE, type = "PT", year1, r = d$r_matt)
```

```{r BYER-PT, fig.cap="Brood year preterminal exploitation rate at age (medians and 95% prediction interval)."}
if (sum(FPT)) CM_ER(report, brood = TRUE, type = "PT", year1, r = d$r_matt)
```

### Return

```{r matt-RY, fig.cap="Posterior estimates of maturity by age and return year (medians and 95% prediction interval). Dashed, horizontal lines represent prior means."}
CM_maturity(report, d, year1, r = d$r_matt, brood = FALSE, rs_names = rs_names)
```

```{r matt-BY, fig.cap="Posterior estimates of maturity by age and brood year (medians and 95% prediction interval). Dashed, horizontal lines represent prior means."}
CM_maturity(report, d, year1, r = d$r_matt, brood = TRUE, rs_names = rs_names)
```

```{r recr, fig.cap="Recruitment at age (mature abundance, medians and 95% prediction interval)."}
CM_recr(report, year1, ci = TRUE)
```

### Terminal fishery

```{r vulT, fig.cap="Posterior estimates of terminal vulnerability at age (medians and 95% prediction interval)."}
FT <- sapply(report, getElement, "FT")
if (sum(FT)) CM_vul(report, type = "vulT")
```

```{r FT, fig.cap="Terminal instantaneous fishing mortality (medians and 95% prediction interval)."}
if (sum(FT)) CM_F(report, PT = FALSE, year1)
```

```{r CYER-T, fig.cap="Return year terminal exploitation rate at age (medians and 95% prediction interval)."}
if (sum(FT)) CM_ER(report, brood = FALSE, type = "T", year1, r = d$r_matt)
```

```{r BYER-T, fig.cap="Brood year terminal exploitation rate at age (medians and 95% prediction interval)."}
if (sum(FT)) CM_ER(report, brood = TRUE, type = "T", year1, r = d$r_matt)
```

### Escapement

```{r esc, fig.cap="Escapement at age (medians and 95% prediction interval)."}
CM_esc(report, year1)
```

## About

This report was generated on: `r Sys.time()`<br />
salmonMSE version `r packageVersion("salmonMSE")`<br />
`r R.version.string`

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CM_int.R
\name{CM_int}
\alias{CM_int}
\title{Estimation function for conditioning model}
\usage{
CM_int(p, d)
}
\arguments{
\item{p}{List of parameter variables. See \code{\link[=fit_CM]{fit_CM()}}.}

\item{d}{List of data variables. See \code{\link[=fit_CM]{fit_CM()}}.}
}
\value{
Numeric, objective function value (log-posterior)
}
\description{
Population dynamics model of an age structured salmon population. Used with RTMB to estimate historical reconstruction from data.
}
\author{
Q. Huynh with Stan code provided by J. Korman and C. Walters
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pm.R
\name{PNI50}
\alias{PNI50}
\alias{PNI80}
\alias{WILD50}
\alias{SMSY85}
\alias{Sgen100}
\title{Example performance metrics}
\usage{
PNI50(SMSE, Ref = 0.5, Yrs = NULL)

PNI80(SMSE, Ref = 0.8, Yrs = NULL)

WILD50(SMSE, Ref = 0.5, Yrs = NULL)

SMSY85(SMSE, Ref = 0.85, Yrs = NULL)

Sgen100(SMSE, Ref = 1, Yrs = NULL)
}
\arguments{
\item{SMSE}{SMSE object returned by \code{\link[=salmonMSE]{salmonMSE()}}}

\item{Ref}{Threshold for the performance metric, used to calculate the probability that the metric exceeds this value}

\item{Yrs}{Numeric vector of length 2 to indicate the year range over which to summarize performance.
If NULL, the performance is summarized over all projection years.}
}
\value{
A vector of probabilities corresponding to population
}
\description{
Functions that evaluate return probabilities of outcomes from the simulations.
}
\details{
\itemize{
\item \code{PNI50} calculates the probability that PNI exceeds 0.50 (threshold for an integrated-transition population, Withler et al. 2018)
\item \code{PNI80} calculates the probability that PNI exceeds 0.80 (threshold for an integrated-wild population, Withler et al. 2018)
\item \code{WILD50} calculates the probability that at least 50 percent of natural spawners are wild
\item \code{SMSY85} calculates the probability that NOS/SMSY exceeds 0.85
\item \code{Sgen100} calculates the probability that NOS/Sgen exceeds 1
}
}
\references{
Withler et al. 2018. Genetically Based Targets for Enhanced Contributions to Canadian Pacific Chinook Salmon Populations.
DFO Can. Sci. Advis. Sec. Res. Doc. 2018/019. xii + 88 p.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AHA_int.R, R/MMSE2SMSE.R
\name{calc_pwild}
\alias{calc_pwild}
\alias{calc_pwild_age}
\title{Proportion wild spawners}
\usage{
calc_pwild(pHOS_cur, pHOS_prev, gamma)

calc_pwild_age(NOS_a, HOS_a, fec, gamma)
}
\arguments{
\item{pHOS_cur}{Numeric, proportion of hatchery spawners in current generation}

\item{pHOS_prev}{Numeric, proportion of hatchery spawners in previous generation}

\item{gamma}{Numeric, reduced reproductive success of hatchery spawners}

\item{NOS_a}{Array \verb{[nsim, maxage, years]} for natural spawners}

\item{HOS_a}{Array \verb{[nsim, maxage, years]} for hatchery spawners}

\item{fec}{Array \verb{[nsim, maxage, years]} for age class fecundity}
}
\value{
\code{calc_pwild_age()} a matrix of pWILD by simulation and year. \code{calc_pwild()} returns a numeric
}
\description{
Calculate the proportion of wild spawners from a time series of spawners
\itemize{
\item \code{calc_pwild()} is the simple calculation based on the proportion of hatchery spawners
\item \code{calc_pwild_age()} performs the calculation weighted by age class fecundity
}
}
\examples{

calc_pwild(0.9, 0.4, 0.8)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hatchery.R
\name{calc_smolt}
\alias{calc_smolt}
\title{Smolt production}
\usage{
calc_smolt(
  N1,
  N2 = N1,
  kappa,
  capacity,
  Smax,
  phi = 1,
  tau = 1,
  fitness_loss = 1,
  SRrel = c("BH", "Ricker"),
  per_recruit = FALSE
)
}
\arguments{
\item{N1}{Egg production for the density-independent component of the stock-recruit relationship. Can be the number of spawners if \code{phi = 1}
and \code{Smax} is in units of spawners.}

\item{N2}{Egg production for the density-dependent component of the stock-recruit relationship (only used if \code{per_recruit = FALSE})}

\item{kappa}{Base productivity parameter}

\item{capacity}{Base capacity parameter if \code{SRrel = "BH"}}

\item{Smax}{Base Smax parameter if \code{SRrel = "Ricker"}}

\item{phi}{Unfished egg per smolt (\code{1/phi} corresponds to the one-to-one adult/spawner replacement line)}

\item{tau}{Unfished spawner per smolt}

\item{fitness_loss}{Survival term to reduce smolt production due to fitness, between 0-1}

\item{SRrel}{Character for the stock-recruit function}

\item{per_recruit}{Logical, whether N1 is a per recruit quantity (TRUE) or in absolute numbers (FALSE)}
}
\value{
Numeric
}
\description{
Calculate smolt production from base stock-recruit parameters and fitness loss
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rel.R
\name{SAR_fitness}
\alias{SAR_fitness}
\alias{makeRel_SAR}
\title{Update natural mortality of juveniles}
\usage{
SAR_fitness(
  x = -1,
  y = 1,
  envir = c("natural", "hatchery"),
  rel_loss = 1,
  s = 1,
  nyears,
  Mbase
)

makeRel_SAR(
  p_smolt = 1,
  s = 1,
  envir = c("natural", "hatchery"),
  rel_loss,
  nyears,
  Mbase
)
}
\arguments{
\item{x}{Integer, simulation number from openMSE}

\item{y}{Integer, simulation year (including historical years)}

\item{envir}{Character, whether to obtain the fitness value for the natural or hatchery environment.}

\item{rel_loss}{Numeric, the loss exponent for the juveniles}

\item{s}{Integer, the salmonMSE population index. Used to search for the fitness value}

\item{nyears}{Integer, the number of historical years in the operating model}

\item{Mbase}{Array \verb{[nsim, n_age, proyears]} the base natural mortality value in the openMSE operating model.}

\item{p_smolt}{Integer, the population index for the juvenile population in the openMSE model}
}
\value{
\itemize{
\item \code{smolt_func()} returns a numeric for the ratio of the realized smolt production vs. the hypothetical value if there were no
hatchery, en route mortality, or habitat improvement
\item \code{makeRel_smolt} returns a list that is passed to openMSE as a inter-population relationship
}
}
\description{
Internal function that updates juvenile natural mortality in the marine environement due
to fitness
\itemize{
\item \code{SAR_fitness()} calculates the new natural mortality value
\item \code{makeRel_SAR} generates a list for openMSE to use in the simulations
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CM_int.R, R/CMfun.R
\name{get_report}
\alias{get_report}
\alias{get_CMdata}
\alias{CMfigures}
\alias{CM_trace}
\alias{CM_pairs}
\alias{CM_fit_esc}
\alias{CM_fit_pHOS}
\alias{CM_fit_CWTesc}
\alias{CM_fit_CWTcatch}
\alias{CM_maturity}
\alias{CM_vul}
\alias{CM_SRR}
\alias{CM_prod}
\alias{CM_Srep}
\alias{CM_M}
\alias{CM_Megg}
\alias{CM_Njuv}
\alias{CM_recr}
\alias{CM_esc}
\alias{CM_F}
\alias{CM_surv}
\alias{CM_wt}
\alias{CM_surv2}
\alias{CM_wto}
\alias{CM_ER}
\alias{CM_CWT_ER}
\alias{CM_covariate}
\title{Figures for conditioning model results}
\usage{
get_report(stanfit, sims, inc_warmup = FALSE)

get_CMdata(fit)

CM_trace(stanfit, vars, inc_warmup = FALSE)

CM_pairs(stanfit, vars = c("log_so", "log_cr"), inc_warmup = FALSE)

CM_fit_esc(report, d, year)

CM_fit_pHOS(report, d, year)

CM_fit_CWTesc(report, d, year1 = 1, rs_names)

CM_fit_CWTcatch(report, d, PT = TRUE, year1 = 1, rs_names)

CM_maturity(
  report,
  d,
  year1 = 1,
  r = 1,
  brood = TRUE,
  annual = FALSE,
  rs_names
)

CM_vul(report, type = c("vulPT", "vulT"))

CM_SRR(report, year1 = 1)

CM_prod(report, d, year1 = 1)

CM_Srep(report, d, year1 = 1, type = c("spawner", "egg"))

CM_M(report, year1 = 1, ci = TRUE)

CM_Megg(report, year1 = 1, ci = TRUE, surv = FALSE)

CM_Njuv(report, year1 = 1, ci = TRUE)

CM_recr(report, year1 = 1, ci = TRUE)

CM_esc(report, year1 = 1, ci = TRUE)

CM_F(report, PT = TRUE, year1 = 1, ci = TRUE)

CM_surv(report, year1 = 1, ci = TRUE)

CM_wt(stanfit, year1 = 1, ci = TRUE)

CM_surv2(report, year1 = 1, ci = TRUE, ylab = "Survival to age 2")

CM_wto(stanfit, year1 = 1, ci = TRUE)

CM_ER(
  report,
  brood = TRUE,
  type = c("PT", "T", "all"),
  year1 = 1,
  ci = TRUE,
  at_age = TRUE,
  r = 1
)

CM_CWT_ER(
  report,
  brood = TRUE,
  type = c("PT", "T", "all"),
  year1 = 1,
  ci = TRUE,
  rs_names
)

CM_covariate(x, names, year1 = 1, b, ylab = "Covariate")
}
\arguments{
\item{stanfit}{Output from \code{\link[=sample_CM]{sample_CM()}}}

\item{sims}{Optional integer vector for subset of MCMC iterations}

\item{inc_warmup}{Logical, whether to include warmup MCMC samples}

\item{fit}{Output from \verb{[fit_CM()]}}

\item{vars}{Character vector for variable names (see \code{names(stanfit@sim$samples[[1]])}). Regex and partial matching supported because it is passed to the \code{pattern} argument of \code{\link[=grepl]{grepl()}}}

\item{report}{List, output of state variables from individual MCMC samples, obtained with \code{get_report()}}

\item{d}{List of data variables, obtained with \code{get_CMdata()}}

\item{year}{Vector of years}

\item{year1}{Numeric, first year of model}

\item{rs_names}{Character vector of hatchery release strategies}

\item{PT}{Logical, whether to plot preterminal catch, otherwise (plot terminal catch)}

\item{r}{Integer, the release strategy for the figure (only if \code{annual = FALSE})}

\item{brood}{Logical, whether to show results by brood year or return year (FALSE)}

\item{annual}{Logical, whether to show panel figure by individual year (TRUE) or a single time series figure}

\item{type}{Character, indicates type of variable to plot}

\item{ci}{Logical whether to show posterior intervals in addition to posterior median}

\item{surv}{Logical, whether to plot survival (values between 0 - 1) or instantaneous mortality rates}

\item{ylab}{Character y axis label}

\item{at_age}{Logical, whether to make figure by individual age}

\item{x}{Matrix of covariates by year x covariate}

\item{names}{Character of covariate names}

\item{b}{Matrix of fixed effect coefficients by simulation x covariate. If missing only the covariates (\code{x}) are plotted,
otherwise, the dot product \code{sum(x * b)} is calculated by individual simulation and quantiles are plotted}
}
\value{
\itemize{
\item \code{get_report()} returns the list of state variables by individual MCMC samples
}

\itemize{
\item \code{get_CMdata()} returns the list of data variables used in the conditioning model
}

\itemize{
\item \code{CM_trace()} returns a ggplot showing the MCMC trace plot (aka wormplot)
}

\itemize{
\item \code{CM_pairs()} returns output from \code{\link[graphics:pairs]{graphics::pairs()}}, a matrix of scatterplots of MCMC posterior samples
}

\itemize{
\item \code{CM_fit_esc()} returns base graphics with fit to total escapement time series
}

\itemize{
\item \code{CM_fit_pHOS()} returns base graphics with fit to pHOS (census) observations
}

\itemize{
\item \code{CM_fit_CWTesc()} returns ggplot of fit to CWT escapement at age
}

\itemize{
\item \code{CM_fit_CWTcatch()} returns ggplot of fit to CWT catch at age
}

\itemize{
\item \code{CM_maturity()} returns ggplot of estimated maturity at age
}

\itemize{
\item \code{CM_vul()} returns ggplot of estimated fishery vulnerability at age
}

\itemize{
\item \code{CM_SRR()} returns ggplot of estimated stock-recruit relationship (density-dependent juvenile production from egg production) with average
relationship and realized annual values. Years correspond to return years of egg production (predicts juvenile production of the following calendar year).
}

\itemize{
\item \code{CM_prod()} returns ggplot of realized productivity in the absence of fishery harvest, annual values are based on natural mortality and maturity at age
}

\itemize{
\item \code{CM_Srep()} returns ggplot of realized spawner or egg production at replacement, annual values are based on natural mortality and maturity at age
}

\itemize{
\item \code{CM_M()} returns ggplot of estimated natural mortality time series by age (marine stage)
}

\itemize{
\item \code{CM_Megg()} returns ggplot of egg-juvenile mortality time series
}

\itemize{
\item \code{CM_Njuv()} returns ggplot of juvenile abundance
}

\itemize{
\item \code{CM_recr()} returns ggplot of recruitment (mature return)
}

\itemize{
\item \code{CM_esc()} returns ggplot of escapement (after terminal harvest)
}

\itemize{
\item \code{CM_F()} returns ggplot of instantaneous fishing mortality
}

\itemize{
\item \code{CM_surv()} returns ggplot of natural survival (converting from instantaneous units of natural mortality)
}

\itemize{
\item \code{CM_wt()} returns ggplot of annual deviations in egg-juvenile mortality from the Ricker function
}

\itemize{
\item \code{CM_surv2()} returns ggplot of annual survival to age 2, which includes age-1 mortality (marine life stage) for both natural and hatchery origin fish.
Hatchery fish experience additional mortality specified by release mortality.
}

\itemize{
\item \code{CM_wt()} returns ggplot of annual deviations in age 1 natural mortality (first year in marine life stage, deviations from time series average)
}

\itemize{
\item \code{CM_ER()} returns ggplot of exploitation rate either by individual age or aggregate values using adult equivalents
}

\itemize{
\item \code{CM_CWT_ER()} returns ggplot of CWT exploitation rate (by release strategy)
}

\itemize{
\item \code{CM_covariate()} returns ggplot of mortality covariates
}
}
\description{
Functions used by the markdown report to generate summary figures from the age-structured conditoning model
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/salmonMSE.R
\name{salmonMSE}
\alias{salmonMSE}
\title{Run salmonMSE}
\usage{
salmonMSE(SOM, Hist = FALSE, silent = FALSE, trace = FALSE, convert = TRUE)
}
\arguments{
\item{SOM}{An object of class \linkS4class{SOM}}

\item{Hist}{Logical, whether to stop the function stop after historical simulations?}

\item{silent}{Logical, whether to report progress in console}

\item{trace}{Logical, whether to report additional messages from openMSE}

\item{convert}{Logical, whether to convert the output into a salmon MSE (SHist or SMSE, depending on \code{Hist}) object}
}
\value{
If \code{Hist = TRUE}: if \code{convert = TRUE}, a \linkS4class{SHist} object or if \code{convert = FALSE}, a multiHist object (list).

If \code{Hist = FALSE}: if \code{convert = TRUE}, a \linkS4class{SMSE} object or if \code{convert = FALSE}, a \link[MSEtool:MMSE-class]{MSEtool::MMSE} object.
}
\description{
\code{salmonMSE()} runs a salmon management strategy evaluation through the following steps:
\itemize{
\item Converts a salmon operating model (\linkS4class{SOM}) to a multi-stock operating model (\link[MSEtool:MOM-class]{MSEtool::MOM}) via \code{\link[=SOM2MOM]{SOM2MOM()}}
\item Creates a harvest management procedure specifying the harvest control rule
\item Generates the historical reconstruction of the state variables
\item Runs projection (if \code{Hist = FALSE})
\item Converts the openMSE output, along with additional state variables recorded in \link{salmonMSE_env}, into a salmon MSE object (SMSE) via \code{\link[=MMSE2SMSE]{MMSE2SMSE()}}
}
}
\examples{
\dontrun{
SMSE <- salmonMSE(simple_SOM)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Harvest_MMP.R
\name{Harvest_MMP}
\alias{Harvest_MMP}
\title{Harvest component of operating model}
\usage{
Harvest_MMP(
  x = 1,
  DataList,
  reps = 1,
  u_terminal,
  u_preterminal,
  K_PT,
  K_T,
  type_PT = "u",
  type_T = "u",
  MSF_PT = FALSE,
  MSF_T = FALSE,
  m,
  release_mort,
  p_terminal = c(2, 5),
  p_preterminal = c(1, 4),
  pkey = data.frame(p = 1:6, s = 1),
  p_natural = 1:3,
  p_hatchery = 4:6,
  ...
)
}
\arguments{
\item{x}{Simulation number}

\item{DataList}{A nested list of \link[MSEtool:Data-class]{MSEtool::Data} objects by stock then fleet, generated by \code{\link[MSEtool:multiMSE]{MSEtool::multiMSE()}}}

\item{reps}{The number of stochastic replicates to be returned by the function}

\item{u_terminal}{Numeric vector by population (s). Harvest rate of retained catch in the terminal fishery}

\item{u_preterminal}{Single numeric. Harvest rate of retained catch in the pre-terminal fishery}

\item{K_PT}{Single numeric. Total retained catch in the pre-terminal fishery}

\item{K_T}{Numeric vector by population (s). Total retained catch in the terminal fishery}

\item{type_PT}{Single character, containing either "catch" or "u". Indicates whether to manage by harvest rate or total catch numbers for preterminal fisheries}

\item{type_T}{Character vector by population (s), containing either "catch" or "u". indicates whether to manage by harvest rate or total catch numbers for terminal fisheries}

\item{MSF_PT}{Single logical, whether to implement mark-selective fishing for the preterminal fishery}

\item{MSF_T}{Logical vector by population (s), whether to implement mark-selective fishing for the terminal fishery}

\item{m}{Numeric vector by population (s). Mark rate of hatchery origin fish, as a proxy for fishery retention. Only used to calculate the fishing effort.
Retention in the operating model is specified in the \link[MSEtool:MOM-class]{MSEtool::MOM} object}

\item{release_mort}{Matrix \verb{[2, s]}. Release mortality of discarded fish in the pre-terminal (1st row) and terminal (2nd row) fishery. Only used
if \code{MSF_PT = TRUE} or \code{MSF_T[s] = TRUE}. Only used to calculate the fishing effort.
Release mortality in the operating model is specified in the \link[MSEtool:MOM-class]{MSEtool::MOM} object}

\item{p_terminal}{Numeric vector. Population index (p) for the recruitment that experiences the terminal fishing mortality}

\item{p_preterminal}{Numeric vector. Population index (p) for immature fish that experience the pre-terminal fishing mortality}

\item{pkey}{Data frame that assigns openMSE population by life stage and origin (p) to salmonMSE population (s)}

\item{p_natural}{Numeric vector. Population index (p) for natural origin fish}

\item{p_hatchery}{Numeric vector. Population index (p) for hatchery origin fish}

\item{...}{Not used}
}
\value{
A nested list of \link[MSEtool:Rec-class]{MSEtool::Rec} objects, same dimension as \code{DataList}
}
\description{
A function used by openMSE to specify the fishing effort during the projections.
salmonMSE updates the arguments of this function from the salmon operating model.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CMfun.R
\name{report_CM}
\alias{report_CM}
\title{Conditioning model markdown report}
\usage{
report_CM(
  stanfit,
  year,
  cov1_names,
  cov_names,
  rs_names,
  name,
  filename = "CM",
  dir = tempdir(),
  open_file = TRUE,
  render_args = list(),
  ...
)
}
\arguments{
\item{stanfit}{Output from \code{\link[=sample_CM]{sample_CM()}}}

\item{year}{Optional vector of calendar years}

\item{cov1_names}{Optional character vector for names of covariates that predict age-1 natural mortality}

\item{cov_names}{Optional character vector for names of covariates that predict age-2+ natural mortality}

\item{rs_names}{Optional character vector for names of hatchery release strategies}

\item{name}{Optional character string for the model name to include in the report, e.g., model run number}

\item{filename}{Character string for the name of the markdown and HTML files}

\item{dir}{The directory in which the markdown and HTML files will be saved.}

\item{open_file}{Logical, whether the HTML document is opened after it is rendered}

\item{render_args}{List of arguments to pass to \code{\link[rmarkdown:render]{rmarkdown::render()}}}

\item{...}{Additional arguments (not used)}
}
\value{
Returns invisibly the output of \code{\link[rmarkdown:render]{rmarkdown::render()}}, typically the path of the output file
}
\description{
Generate a markdown report to plot time series and MCMC posteriors of estimates from the conditioning model. See \code{\link[=get_report]{get_report()}} for the
various plotting functions used in the report.
}
\details{
Report excludes MCMC values from warmup iterations
}
\seealso{
\code{\link[=fit_CM]{fit_CM()}} \code{\link[=get_report]{get_report()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/define-SOM.R
\docType{class}
\name{SOM-class}
\alias{SOM-class}
\alias{SOM}
\title{Class \code{"SOM"}}
\description{
An object containing all the parameters for a salmon operating model (SOM).
}
\section{Slots}{

\describe{
\item{\code{Name}}{Character. Identifying name}

\item{\code{nsim}}{Integer. Number of simulations}

\item{\code{proyears}}{Integer. The number of projected years}

\item{\code{seed}}{Integer. A random seed to ensure users can reproduce results exactly}

\item{\code{Bio}}{\linkS4class{Bio} object informing biological parameters and natural production. Provide a list of Bio objects for multi-population models.}

\item{\code{Habitat}}{\linkS4class{Habitat} object containing management levers for controlling survival in the freshwater environment. Provide a list of Habitat objects for multi-population models.}

\item{\code{Hatchery}}{\linkS4class{Hatchery} object containing management levers for hatchery production and in-river removals. Provide a list of Hatchery objects for multi-population models.}

\item{\code{Harvest}}{\linkS4class{Harvest} object containing management levers for marine harvest. Provide a list of Harvest objects for multi-population models.}

\item{\code{Historical}}{\linkS4class{Historical} object to inform historical reconstruction and informing starting abundance for the projection. Provide a list of Historical objects for multi-population models.}

\item{\code{stray}}{Matrix \verb{[np, np]} where \code{np = length(Bio)} and row \code{p} indicates the re-assignment of hatchery fish to each population when they mature (at the recruitment life stage). For example,
\code{SOM@stray <- matrix(c(0.75, 0.25, 0.25, 0.75), 2, 2)} indicates that 75 percent of mature fish return to their natal river and 25 percent stray in both populations. By default, an identity matrix is used (no straying).}
}}

\section{Objects from the Class}{
 Objects can be created by calls of the form
\code{new("SOM", Bio, Habitat, Hatchery, Harvest, Historical)}.
}

\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot-statevar.R
\name{compare_statevar_ts}
\alias{compare_statevar_ts}
\alias{compare_statevar_hist}
\title{Compare state variables from simulation runs}
\usage{
compare_statevar_ts(
  SMSE_list,
  var = "PNI",
  s = 1,
  figure = TRUE,
  xlab = "Projection Year",
  quant = FALSE,
  ylab = var,
  ylim,
  agg.fun = sum,
  names,
  col_vec,
  ...
)

compare_statevar_hist(
  SMSE_list,
  var = "PNI",
  s = 1,
  y,
  figure = TRUE,
  xlab = var,
  names,
  col_vec,
  type = c("density", "hist"),
  ...
)
}
\arguments{
\item{SMSE_list}{List of SMSE objects for multiple model runs returned by \code{\link[=salmonMSE]{salmonMSE()}}}

\item{var}{Character. Slot for the state variable in \code{SMSE} object. See \code{slotNames(SMSE)} for options. Additional supported options are:
\code{"ESS"} (egg-smolt survival), \code{"pbrood"} (broodtake to escapement ratio), \code{"pNOSesc"} (NOS/natural escapement), \code{"pHOSesc"} (HOS/hatchery escapement),
\verb{Total Spawners} (NOS + HOS), \code{NOS/SMSY}, \code{S/SMSY}, and \code{NOS/Sgen}.}

\item{s}{Integer. Population index for multi-population model (e.g., \code{s = 1} is the first population in the model)}

\item{figure}{Logical, whether to generate a figure (set to FALSE if only using the function to return the data matrix)}

\item{xlab}{Character. Name of time variable for the figure}

\item{quant}{Logical, whether to plot individual simulations (FALSE) or the median with 95 percent confidence intervals (TRUE)}

\item{ylab}{Character. Name of the state variable for the figure}

\item{ylim}{Vector. Y-axis limits}

\item{agg.fun}{Function. Defines how to aggregate state variables that are reported by age. Typically, \code{sum} is used but \code{max} is also
possible for reporting apical exploitation rates.}

\item{names}{Character vector \code{length(SMSE_list)} to label individual model runs}

\item{col_vec}{Character vector \code{length(SMSE_list)} for custom colour schemes for comparing across model scenarios in figures}

\item{...}{Additional arguments to base plot function}

\item{y}{Integer. Projection year for the state variable to plot the histogram. If missing, the last projection year is used.}

\item{type}{Character, whether to generate a density figure or histogram}
}
\value{
An array invisibly. Also generates base graphics if \code{figure = TRUE}
}
\description{
Compare outputs from multiple simulations to evaluate performance across states of nature and/or management levers (identified by colour):
\itemize{
\item \code{compare_statevar_ts()} produces a time series for all simulations, or with medians and 95th percentile intervals
\item \code{compare_statevar_hist()} produces a histogram or density plot across all simulations for a particular year
}
}
\seealso{
\code{\link[=plot_statevar_ts]{plot_statevar_ts()}} \code{\link[=compare]{compare()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CM.R
\name{CM2SOM}
\alias{CM2SOM}
\title{Convert conditioning model to operating model}
\usage{
CM2SOM(stanfit, sims, nsim = 2, seed = 1, proyears = 40)
}
\arguments{
\item{stanfit}{Output from \code{\link[=sample_CM]{sample_CM()}}}

\item{sims}{Optional, a vector of integers indicating the MCMC iterations to convert to operating model simulations. Otherwise,
use argument \code{nsim} in order to sample a subset of the MCMC.}

\item{nsim}{Integer, total number of simulations in the operating model. Only used if \code{sims} is missing.}

\item{seed}{Integer, seed for sampling the MCMC output. Only used if \code{sims} is missing.}

\item{proyears}{Integer, the number of projection years in the operating model}
}
\value{
\linkS4class{SOM} object.
}
\description{
Creates an operating model from MCMC samples and data inputs of the conditioning model.
Management actions for habitat, hatchery production, and harvest still need to be specified in the operating model.

\strong{Note: the function assumes the maturity values in the last conditioning year for the projection, which are likely not well informed by CWT data. Consider updating the maturity using some historical average (e.g., across most recent completed brood years).}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/define-SOM.R
\docType{class}
\name{Bio-class}
\alias{Bio-class}
\title{Class \code{"Bio"}}
\description{
The component of the operating model that controls biological dynamics, i.e., natural production.
}
\details{
Various parameters can be stochastic (length \code{nsim}) or input as a single numeric
(value identical across all simulations).
}
\section{Slots}{

\describe{
\item{\code{Name}}{Character. Identifying name}

\item{\code{maxage}}{Integer. The maximum age of the population age structure.}

\item{\code{n_g}}{Integer. Number of life history groups within a cohort.
Life history groups (LHGs) are sub-units of a cohort that have different marine survival.
Default is 1.}

\item{\code{p_LHG}}{The proportion of the total egg production assigned to each life history group within a cohort. For example,
if \code{Bio@n_g <- 2}, then \code{Bio@p_LHG <- c(0.9, 0.1)}, then 90 percent of the egg production in the first population is assigned to the first life history group and
ten percent to the second LHG. Default is \code{rep(1/Bio@n_g, Bio@n_g)}}

\item{\code{p_mature}}{Either vector by age (length \code{maxage}) or an array with dimension \verb{[nsim, maxage, proyears]}. The proportion mature by age.}

\item{\code{SRrel}}{Character, stock-recruit relationship for density-dependent smolt production from fry. Either "BH" (Beverton-Holt) or "Ricker".
Not used if habitat component is used. See \code{Habitat} object.}

\item{\code{capacity}}{Vector length \code{nsim}. Only used if \code{SRrel = "BH"}. The asymptote, i.e., maximum juvenile production, of the Beverton-Holt stock-recruit function for density-dependent natural smolt production from egg production.
\strong{Units of smolts.} Not used if habitat component is used.}

\item{\code{kappa}}{Vector length \code{nsim}. The adult productivity ratio for the stock-recruit function. \strong{Units of recruits per spawner.}
Natural per-capita production of recruits as the population approaches zero (density-independent component).
Not used if habitat component is used.}

\item{\code{Smax}}{Vector length \code{nsim}. Only used if \code{SRrel = "Ricker"}. The spawner abundance that maximizes smolt production in the Ricker stock-recruit function. \strong{Units of spawners.}
Not used if habitat component is used.}

\item{\code{phi}}{Optional, vector length \code{nsim}. Egg production per smolt at unfished replacement. \strong{Units of egg per smolt}.
Converts productivity from units of recruits/spawner (\code{kappa}) to smolts/egg (\code{alpha}), where \code{alpha = kappa/phi}. In simple models,
\code{phi} is the product of marine survival, fecundity, and proportion female. If not provided, \code{phi} will be calculated from \code{Mjuv_NOS}, \code{p_mature}, \code{s_enroute}, \code{p_female}, \code{fec}, and \code{p_LHG} corresponding
to the first year and weighted by life history groups. Not used if habitat component is used.}

\item{\code{tau}}{Optional, vector length \code{nsim}. Spawner per smolt at unfished replacement, only used if \code{SRrel = "Ricker"}. \strong{Units of spawner per smolt}.
Used to convert \code{Smax} to \code{Emax}, the corresponding egg production that maximizes smolt production, where \code{Emax = Smax * phi/tau} and the Ricker parameter \code{beta = 1/Emax}. In simple models,
\code{tau} is the product of marine survival and proportion female. If not provided, \code{tau} will be calculated from \code{Mjuv_NOS}, \code{p_mature}, \code{s_enroute}, \code{p_female}, and \code{p_LHG} corresponding
to the first year and weighted by life history groups. Not used if habitat component is used.}

\item{\code{Mjuv_NOS}}{Either vector by age (length \code{maxage-1}) or an array with dimension \verb{[nsim, maxage-1, proyears, n_g]}.
Natural mortality of immature natural origin fish, the value for the first age represents natural mortality from age 1 to 2, second age is mortality from age 2 to 3, and so on.
To replicate the SAR parameter of a stage-specific model, set \code{Mjuv_NOS[a] = -log(SAR)} where \code{a} is the age class prior to maturation (and zero for all other ages).}

\item{\code{fec}}{Vector by age (length \code{maxage}) or an array with dimension \verb{[nsim, maxage, proyears]}. Female fecundity of natural origin spawners.}

\item{\code{p_female}}{Numeric. The proportion of females in the spawning population. Default is 0.5. Can also be a vector \verb{[maxage]} (for situations where older spawners are predominantly female)}

\item{\code{s_enroute}}{Numeric. Survival of escapement to the spawning grounds (for spawning and for broodtake). Default is 1.}
}}

\section{Creating Object}{

Objects can be created by calls of the form \code{new("Bio")}
}

\examples{
showClass("Bio")
}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare.R
\name{compare_spawners}
\alias{compare_spawners}
\alias{compare_fitness}
\alias{compare_escapement}
\title{Compare simulation runs}
\usage{
compare_spawners(SMSE_list, Design, prop = FALSE, FUN = median)

compare_fitness(SMSE_list, Design, FUN = median)

compare_escapement(SMSE_list, Design, FUN = median)
}
\arguments{
\item{SMSE_list}{A list of SMSE objects returned by \code{\link[=salmonMSE]{salmonMSE()}}}

\item{Design}{A data frame with two columns that describes the factorial design of the simulations. Used to label the figure.
Rows correspond to each object in \code{SMSE_list}. There two columns are variables against which to plot the result. See
example in \url{https://docs.salmonmse.com/articles/decision-table.html}.}

\item{prop}{Logical, whether to plot absolute numbers over proportions}

\item{FUN}{Summarizing function across simulations, typically \code{\link[stats:median]{stats::median()}} or \code{\link[base:mean]{base::mean()}}}
}
\value{
A ggplot object
}
\description{
Create figures that compare results across two dimensions
}
\details{
\itemize{
\item \code{\link[=compare_spawners]{compare_spawners()}} generates a time series of the composition of spawners
\item \code{\link[=compare_fitness]{compare_fitness()}} generates a time series of metrics (fitness, PNI, pHOS, and pWILD) related to hatchery production
\item \code{\link[=compare_escapement]{compare_escapement()}} generates a time series of the proportion of spawners and broodtake to escapement
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/define-SOM.R
\docType{class}
\name{Historical-class}
\alias{Historical-class}
\title{Class \code{"Historical"}}
\description{
Optional component of the operating model that specifies the historical dynamics. The default option starts the projection with 1000
natural-origin juveniles and 1000 hatchery-origin juveniles (if there is hatchery production) in the oldest age class, representing
single-brood year returns since there is only one age class in the population. Specify the abundance in all age class to simulate multiple
brood-year returns.
}
\section{Slots}{

\describe{
\item{\code{Name}}{Character. Identifying name}

\item{\code{InitNjuv_NOS}}{Array by \verb{[nsim, maxage, n_g]}. The abundance of immature natural origin fish at the beginning of the projection.
Default assumes 1000 in the oldest age class, which creates a population with single brood year returns.}

\item{\code{InitNjuv_HOS}}{Array by \verb{[nsim, maxage, n_r]}. The abundance of immature hatchery origin fish at the beginning of the projection.
Default assumes 1000 in the oldest age class, which creates a population with single brood year returns.}
}}

\section{Creating Object}{

Objects can be created by calls of the form \code{new("Historical")}
}

\examples{
showClass("Historical")
}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/salmonMSE_env.R
\docType{data}
\name{salmonMSE_env}
\alias{salmonMSE_env}
\title{Environment to store salmon specific variables, e.g, fitness, during the simulation}
\format{
An object of class \code{environment} of length 5.
}
\usage{
salmonMSE_env
}
\description{
Environment to store salmon specific variables, e.g, fitness, during the simulation
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SRR.R
\name{calc_SRRpars}
\alias{calc_SRRpars}
\title{Convert density-dependent survival parameters}
\usage{
calc_SRRpars(p, capacity, f = 1, p_female = 1, type = c("BH", "Ricker", "HS"))
}
\arguments{
\item{p}{Numeric, the productivity parameter that sets the maximum survival as the initial abundance approaches zero}

\item{capacity}{Numeric, the capacity parameter that set the maximum survivors}

\item{f}{Fecundity, the spawning output per mature female}

\item{p_female}{The proportion of females per spawner}

\item{type}{Character, the functional form of the stock-recruit relationship}
}
\value{
Numeric vector length 2 for alpha and beta value, respectively
}
\description{
Converts from capacity/productivity parameters to alpha/beta stock-recruit parameters where productivity is in terms
of smolts per spawner and alpha is terms of smolts per egg.
}
\details{
\deqn{\alpha = \dfrac{P}{f \times p_{female}}}

For the Beverton-Holt stock recruit relationship:
\deqn{\beta = \dfrac{\alpha}{C}}

For the Ricker stock recruit relationship:
\deqn{\beta = \dfrac{\alpha}{Ce}}, \eqn{e} is Euler's number.
}
\examples{

N1 <- 100
N2 <- 200
p <- 10
capacity <- 250

calc_SRRpars(p, capacity, type = "BH")
}
\seealso{
\code{\link[=calc_SRR]{calc_SRR()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot-LHG.R
\name{plot_LHG}
\alias{plot_LHG}
\alias{plot_RS}
\title{Plot life history groups and release strategies}
\usage{
plot_LHG(
  SMSE,
  var = "NOS",
  type = c("prop", "abs"),
  s = 1,
  FUN = median,
  figure = TRUE,
  xlab = "Projection Year",
  ylab,
  name,
  ylim
)

plot_RS(
  SMSE,
  var = "HOS",
  type = c("prop", "abs"),
  s = 1,
  FUN = median,
  figure = TRUE,
  xlab = "Projection Year",
  ylab,
  name,
  ylim
)
}
\arguments{
\item{SMSE}{Class \linkS4class{SMSE} object returned by \code{\link[=salmonMSE]{salmonMSE()}}}

\item{var}{Character. Slot for the state variables in \code{SMSE@Misc$LHG[[1]]} or \code{SMSE@Misc$RS[[1]]}.}

\item{type}{Character to indicate whether to plot proportion or absolute numbers}

\item{s}{Integer. Population index for multi-population model (e.g., \code{s = 1} is the first population in the model)}

\item{FUN}{Summarizing function across simulations, typically \code{\link[=median]{median()}} or \code{\link[=mean]{mean()}}}

\item{figure}{Logical, whether to generate a figure (set to FALSE if only using the function to return the data matrix)}

\item{xlab}{Character. Name of time variable for the figure}

\item{ylab}{Character. Name of the state variable for the figure}

\item{name}{Character. Vector of names for the life history groups or release strategies}

\item{ylim}{Vector length 2, y-axis limits}
}
\value{
Base graphics figure, barplot of distribution or total numbers by LHG or RS. Returns invisibly the matrix of plotted values
}
\description{
Plot the annual proportions of life history groups (natural origin fish) or release strategies (hatchery origin) at various life stages
}
\seealso{
\code{\link[=plot_statevar_ts]{plot_statevar_ts()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rel.R
\name{smolt_func}
\alias{smolt_func}
\alias{makeRel_smolt}
\title{Predict smolt production}
\usage{
smolt_func(
  Nage_NOS,
  Nage_HOS,
  Nage_stray,
  x = -1,
  y,
  output = c("natural", "hatchery"),
  s_enroute,
  p_female,
  fec,
  SRRpars,
  hatchery_args,
  fitness_args,
  habitat_args,
  stray_args,
  s,
  g,
  prop_LHG,
  r
)

makeRel_smolt(
  p_smolt = 1,
  s = 1,
  p_natural,
  p_hatchery = NULL,
  p_stray = NULL,
  maxage,
  output = c("natural", "hatchery"),
  s_enroute,
  p_female,
  fec,
  SRRpars,
  hatchery_args,
  fitness_args,
  habitat_args,
  stray_args,
  g,
  prop_LHG,
  r
)
}
\arguments{
\item{Nage_NOS}{Matrix \verb{[n_age, n_g]} of natural escapement from openMSE. \code{n_g} is the number of life history groups (sub groups within cohorts)
that contribute to spawning.}

\item{Nage_HOS}{Matrix \verb{[n_age, n_r]} of hatchery escapement from openMSE. \code{n_r} is the number of release strategies (sub groups within cohorts)
that contribute to spawning.}

\item{Nage_stray}{Matrix \verb{[nage, n_stray]} of hatchery escapement from donor populations that will stray to the recipient population. \code{stray_args$stray_matrix}
determines the proportions of the donor escapement that will move to the recipient population.}

\item{x}{Integer, simulation number from openMSE}

\item{y}{Integer, simulation year (including historical years)}

\item{output}{Character, whether to predict the natural origin or hatchery origin smolt production}

\item{s_enroute}{Numeric, en route survival of the escapement to the spawning grounds}

\item{p_female}{Vector \verb{[maxage]}, proportion female for calculating the egg production.}

\item{fec}{Array \verb{[nsim, maxage, nyears+proyears]}. The fecundity at age schedule of spawners}

\item{SRRpars}{Data frame containing stock recruit parameters for natural smolt production from egg production.
Column names include: SRrel, kappa, capacity, Smax, phi}

\item{hatchery_args}{Named list containing various arguments controlling broodtake and hatchery production. See details below.}

\item{fitness_args}{Named list containing various arguments controlling population fitness from hatchery production.
Names include: fitness_type, omega2, theta, fitness_variance, heritability, zbar_start, fitness_floor, rel_loss}

\item{habitat_args}{Named list with arguments sontrolling freshwater life stage survival.
Names include: use_habitat (logical), Habitat (\linkS4class{Habitat} object)}

\item{stray_args}{Named list with arguments controlling strays in and out of the local population
Names include:}

\item{s}{Integer, salmonMSE population index. Used to report variables to \link{salmonMSE_env}.}

\item{g}{Integer for the life history group of natural origin fish to pass the parameter back to openMSE (if \code{output = "natural"})}

\item{prop_LHG}{Numeric vector, proportion of the egg production assign to life history groups (sums up to one) (only used if \code{output = "natural"})}

\item{r}{Integer for the release strategy of hatchery origin fish to pass the parameter back to openMSE (if \code{output = "hatchery"})}

\item{p_smolt}{Integer, the population index for the smolt production in the openMSE model, corresponding to \code{output}}

\item{p_natural}{Integer vector, the population index for the natural origin escapement in the openMSE model. Can be more than one
if spawning is from multiple life history groups}

\item{p_hatchery}{Integer vector, the population index for the hatchery origin escapement in the openMSE model. Can be more than one
if there are multiple release strategies. Set to \code{NULL} for no hatchery production}

\item{p_stray}{Integer vector, population index for the hatchery strays in multi-system models}
}
\value{
\itemize{
\item \code{smolt_func()} returns a numeric for the ratio of the realized smolt production vs. the hypothetical value if there were no
hatchery, en route mortality, or habitat improvement
\item \code{makeRel_smolt()} returns a list that is passed to openMSE as a inter-population relationship
}
}
\description{
Internal function that predicts the natural origin or hatchery origin smolt production from the
escapement, and saves state variables to \link{salmonMSE_env}.
\itemize{
\item \code{smolt_func()} is the population dynamics function
\item \code{makeRel_smolt()} generates a list for openMSE to use in the simulations
}
}
\section{hatchery_args}{

Hatchery control parameters are included in a named list with the following arguments:
\itemize{
\item \code{egg_target} Numeric, target egg production for hatchery. Set to zero for no hatchery production.
\item \code{ptarget_NOB} Numeric, the target proportion of the natural origin broodtake relative to the overall broodtake
\item \code{pmax_NOB} Numeric, the maximum proportion of the natural origin escapement to be used as broodtake
\item \code{fec_brood} Array \verb{[nsim, maxage, nyears+proyears]}. the fecundity at age schedule of broodtake to calculate the total hatchery egg production
\item \code{s_yearling} Numeric, the survival of eggs to the smolt life stage (for yearling release)
\item \code{s_subyearling} Numeric. the survival of eggs to subyearling life stage (for subyearling release)
\item \code{p_yearling} Numeric, the proportion of annual releases at the yearling life stage (vs. subyearling)
\item \code{phatchery} Numeric, the proportion of the hatchery origin escapement that return to the hatchery,
for example, by removal from spawning grounds or swim-in facilities. These fish are available for broodtake.
\item \code{premove_HOS} Numeric or function, the proportion of the hatchery origin fish to be removed from the spawning
grounds (in order to ensure a high proportion of NOS or for an in-river fishery). These fish are not available for broodtake.
For example, a value less than one can represent imperfect implementation of weir removal.
\item \code{premove_NOS} Numeric or function, the proportion of the natural origin fish to be removed from the spawning
grounds, for example, an in-river fishery.
\item \code{s_prespawn} Numeric, the survival of broodtake prior to egg production. \code{1 - s_prespawn} is the proportion of fish
not used for hatchery purposes, e.g., mortality or other resesarch purposes. Used to back-calculate the broodtake.
\item \code{gamma} Numeric, the relative reproductive success of hatchery origin spawners (relative to natural origin spawners)
\item \code{m} Numeric, mark rate for selective broodtake
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/define-SOM.R
\docType{class}
\name{Habitat-class}
\alias{Habitat-class}
\title{Class \code{"Habitat"}}
\description{
The component of the operating model that controls survival in the freshwater environment. Includes changes in survival from either
environmental/climate effects or habitat mitigation.
}
\section{Slots}{

\describe{
\item{\code{Name}}{Character. Identifying name}

\item{\code{use_habitat}}{Logical. If \code{TRUE}, utilize stage-specific density-dependent functions from egg production from incubation mortality,
egg-to-fry production, and fry-to-smolt production with annual deviations. Otherwise, the density-dependence is modeled for egg-to-smolt survival.
See \code{Bio} object.}

\item{\code{prespawn_rel}}{Character, density-dependent function for pre-spawn mortality, e.g., for spawners to reach spawning sites. Choices are "BH" (Beverton-Holt) or "HS" (hockey stick).
Default is "BH".}

\item{\code{prespawn_prod}}{Numeric, productivity for pre-spawn mortality. Default is 1.
Default if \code{Inf}.}

\item{\code{prespawn_capacity}}{Numeric, capacity for pre-spawn mortality. Default is \code{Inf}, i.e., density-independence.
Default is \code{Inf}.}

\item{\code{egg_rel}}{Character, density-dependent function for egg production from total spawning output. Choices are "BH" (Beverton-Holt) or "HS" (hockey stick).
Default is "BH".}

\item{\code{egg_prod}}{Numeric, productivity for egg production from total spawning output (incubation). Default is 1.
Default if \code{Inf}.}

\item{\code{egg_capacity}}{Numeric, capacity for egg production from total spawning output (incubation). Default is \code{Inf}, i.e., density-independence.
Default is \code{Inf}.}

\item{\code{fry_rel}}{Character, density-dependent function for egg-to-fry production. Choices are "BH" (Beverton-Holt) or "HS" (hockey stick).
Default is "BH".}

\item{\code{fry_prod}}{Numeric between 0-1, productivity for egg production from total spawning output, i.e., maximum survival as egg production approaches zero.
Default is 0.4.}

\item{\code{fry_capacity}}{Numeric, capacity for fry production from egg production. Default is \code{Inf}, i.e., for density-independence.
Default is \code{Inf}.}

\item{\code{fry_sdev}}{Matrix \verb{[nsim, proyears]}, deviations from the density-dependent egg-fry survival. Can be utilized to incorporate
time-varying environmental, climate, or habitat mitigation effects. Default is \code{matrix(1, nsim, proyears)}.}

\item{\code{smolt_rel}}{Character, density-dependent function for fry-to-smolt production. Choices are "BH" (Beverton-Holt) or "HS" (hockey stick).
Default is "BH".}

\item{\code{smolt_prod}}{Numeric between 0-1, productivity for smolt production from fry, i.e., maximum survival as fry production approaches zero.
Default is 1.}

\item{\code{smolt_capacity}}{Numeric, capacity for smolt production from fry production. Set to \code{Inf} for density-independence.
Default is \code{Inf}.}

\item{\code{smolt_sdev}}{Matrix \verb{[nsim, proyears]}, deviations from the density-dependent fry-smolt survival. Can be utilized to incorporate
time-varying environmental, climate, or habitat mitigation effects. Default is \code{matrix(1, nsim, proyears)}.}
}}

\section{Creating Object}{

Objects can be created by calls of the form \code{new("Habitat")}
}

\examples{
showClass("Habitat")
}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot-statevar.R
\name{plot_statevar_ts}
\alias{plot_statevar_ts}
\alias{plot_statevar_hist}
\alias{plot_spawners}
\alias{plot_fitness}
\alias{plot_escapement}
\alias{plot_fishery}
\alias{plot_Kobe}
\title{Plot core output from salmonMSE}
\usage{
plot_statevar_ts(
  SMSE,
  var = "PNI",
  s = 1,
  figure = TRUE,
  xlab = "Projection Year",
  quant = FALSE,
  ylab = var,
  ylim,
  agg.fun = sum,
  ...
)

plot_statevar_hist(SMSE, var = "PNI", s = 1, y, figure = TRUE, xlab = var, ...)

plot_spawners(SMSE, s = 1, prop = TRUE, FUN = median, figure = TRUE, ylim)

plot_fitness(SMSE, s = 1, FUN = median, figure = TRUE, ylim)

plot_escapement(SMSE, s = 1, FUN = median, figure = TRUE, ylim)

plot_fishery(
  SMSE,
  s = 1,
  type = c("catch", "exploit", "harvest"),
  FUN = median,
  figure = TRUE,
  ylim,
  ylab,
  ...
)

plot_Kobe(
  SMSE,
  s = 1,
  FUN = median,
  figure = TRUE,
  xlim,
  ylim,
  xlab = expression(NOS/S[MSY]),
  ylab = expression(U/U[MSY]),
  type = c("T", "PT")
)
}
\arguments{
\item{SMSE}{Class \linkS4class{SMSE} object returned by \code{\link[=salmonMSE]{salmonMSE()}}}

\item{var}{Character. Slot for the state variable in \code{SMSE} object. See \code{slotNames(SMSE)} for options. Additional supported options are:
\code{"ESS"} (egg-smolt survival), \code{"pbrood"} (broodtake to escapement ratio), \code{"pNOSesc"} (NOS/natural escapement), \code{"pHOSesc"} (HOS/hatchery escapement),
\verb{Total Spawners} (NOS + HOS), \code{NOS/SMSY}, \code{S/SMSY}, and \code{NOS/Sgen}.}

\item{s}{Integer. Population index for multi-population model (e.g., \code{s = 1} is the first population in the model)}

\item{figure}{Logical, whether to generate a figure (set to FALSE if only using the function to return the data matrix)}

\item{xlab}{Character. Name of time variable for the figure}

\item{quant}{Logical, whether to plot individual simulations (FALSE) or the median with 95 percent confidence intervals (TRUE)}

\item{ylab}{Character. Name of the state variable for the figure}

\item{ylim}{Vector. Y-axis limits}

\item{agg.fun}{Function. Defines how to aggregate state variables that are reported by age. Typically, \code{sum} is used but \code{max} is also
possible for reporting apical exploitation rates.}

\item{...}{Additional arguments to base plot function}

\item{y}{Integer. Projection year for the state variable to plot the histogram. If missing, the last projection year is used.}

\item{prop}{Logical, whether to plot proportions or absolute numbers}

\item{FUN}{Summarizing function across simulations, typically \code{\link[=median]{median()}} or \code{\link[=mean]{mean()}}}

\item{type}{For \code{plot_Kobe}, the fishery state variable to plot. Whether to plot the exploitation rate for the terminal (T) or pre-terminal fishery (PT).}

\item{xlim}{Vector. X-axis limits}
}
\value{
Functions return the matrix of plotted values invisibly. Figure plotted from base graphics
}
\description{
Various functions that plot the state variables from salmonMSE projections:
\itemize{
\item \code{plot_statevar_ts()} produces a time series for all simulations, or with medians and 95th percentile intervals
\item \code{plot_statevar_hist()} produces a histogram across all simulations for a particular year
\item \code{plot_spawners()} produces a summary barplot of spawners, including NOS, HOS, and wild spawners
\item \code{plot_escapement()} produces a summary figure of the proportion of spawners and broodtake to escapement
\item \code{plot_fitness()} produces a summary figure of metrics (fitness, PNI, pHOS, and pWILD) related to hatchery production
\item \code{plot_fishery()} produces a summary figure of metrics related to the fishery, e.g., median catch, exploitation rate or harvest rate
}
}
\seealso{
\code{\link[=plot_decision_table]{plot_decision_table()}} \code{\link[=plot_LHG]{plot_LHG()}} \code{\link[=compare_statevar_ts]{compare_statevar_ts()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_obj.R
\docType{data}
\name{simple_SOM}
\alias{simple_SOM}
\alias{multi_SOM}
\title{Example objects from salmonMSE}
\format{
An object of class \linkS4class{SOM}.

An object of class \code{SOM} of length 1.
}
\source{
See salmonMSE \href{https://docs.salmonmse.com/articles/example.html}{website}
}
\usage{
simple_SOM

multi_SOM
}
\description{
S4 objects containing operating model inputs to run the projection.
}
\examples{
data(simple_SOM)

data(multi_SOM)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AHA.R
\name{AHA}
\alias{AHA}
\title{All-H Analyzer}
\usage{
AHA(SOM, ngen = 100, silent = FALSE)
}
\arguments{
\item{SOM}{An object of class \linkS4class{SOM}}

\item{ngen}{Integer, the number of generations for which to run the simulation}

\item{silent}{Logical, indicates whether to silence messages to the R console}
}
\value{
A named list containing vectors of state variables (by simulation, population, and generation). See \linkS4class{SMSE} object description.
}
\description{
Wrapper function for an implementation of All-H Analyzer (\href{https://www.streamnet.org/home/data-maps/hatchery-reform/hsrg-tools/}{AHA}) in R. Can be used to compare outputs between AHA
and salmonMSE.
}
\references{
Hatchery Scientific Review Group. 2020. All-H Analyzer Tool Guide and Documentation. May 2020.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare.R
\name{compare}
\alias{compare}
\title{Compare scenarios in markdown}
\usage{
compare(
  SMSE_list,
  names,
  col_vec,
  filename = "SMSEcompare",
  dir = tempdir(),
  open_file = TRUE,
  render_args = list(),
  ...
)
}
\arguments{
\item{SMSE_list}{List of \linkS4class{SMSE} objects}

\item{names}{Character vector \code{length(SMSE_list)} to label individual model runs}

\item{col_vec}{Character vector \code{length(SMSE_list)} for custom colour schemes for comparing across model scenarios in figures}

\item{filename}{Character string for the name of the markdown and HTML files.}

\item{dir}{The directory in which the markdown and HTML files will be saved.}

\item{open_file}{Logical, whether the HTML document is opened after it is rendered.}

\item{render_args}{List of arguments to pass to \code{\link[rmarkdown:render]{rmarkdown::render()}}.}

\item{...}{Additional arguments (not used)}
}
\value{
Returns invisibly the output of \code{\link[rmarkdown:render]{rmarkdown::render()}}, typically the path of the output file
}
\description{
Generate a markdown report for multiple model runs to compare scenarios. 3-5 is likely the ideal number of scenarios for comparison.
}
\seealso{
\code{\link[=report]{report()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SOM2MOM_int.R
\name{calc_phi}
\alias{calc_phi}
\title{Calculate equilibrium quantities with life history groups}
\usage{
calc_phi(
  Mjuv,
  p_mature,
  p_female,
  fec,
  s_enroute = 1,
  n_g = 1,
  p_LHG,
  output = c("egg", "spawner")
)
}
\arguments{
\item{Mjuv}{Matrix \verb{[maxage, n_g]}, but can be a vector if \code{n_g = 1}. Juvenile natural mortality}

\item{p_mature}{Matrix \verb{[maxage, n_g]}, but can be a vector if \code{n_g = 1}. Maturity at age}

\item{p_female}{Numeric. Proportion female}

\item{fec}{Matrix \verb{[maxage, n_g]}, but can be a vector if \code{n_g = 1}. Fecundity at age. Only used if \code{output = "egg"}}

\item{s_enroute}{Numeric, en-route survival of escapement to spawning grounds}

\item{n_g}{Integer. Number of life history groups}

\item{p_LHG}{Vector length \code{n_g} of proportion of life history groups per recruit. Default is \code{rep(1/n_g, n_g)}}

\item{output}{Character to indicate the output units, e.g., "egg" returns eggs per smolt, and "spawner" returns spawners per smolt}
}
\value{
Numeric, units depend on \code{"output"} argument
}
\description{
Calculate eggs/smolt or spawners/smolt based on life history parameters (survival, maturity, fecundity)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SOM2MOM.R
\name{check_SOM}
\alias{check_SOM}
\title{Check inputs to SOM object}
\usage{
check_SOM(SOM, silent = FALSE)
}
\arguments{
\item{SOM}{\linkS4class{SOM} object}

\item{silent}{Logical, whether to report progress in console}
}
\value{
Updated \linkS4class{SOM} object with full dimensions in various slots
}
\description{
Ensures that the slots in the \linkS4class{SOM} object have the correct dimensions. Function will
update some slots to their full dimensions.
}
\examples{
SOM_checked <- check_SOM(simple_SOM, silent = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/define-SOM.R
\docType{class}
\name{SMSE-class}
\alias{SMSE-class}
\title{Class \code{"SMSE"}}
\description{
Stores the outputs from the simulation of salmon operating models.
}
\details{
In generation \eqn{t}, proportionate natural influence (PNI) is defined as:

\deqn{\textrm{PNI}_t = \dfrac{p^\textrm{NOB}_t}{p^\textrm{NOB}_t + p^\textrm{HOSeff}_t}}

with \eqn{p^\textrm{HOSeff} = \textrm{HOSeff}/(\textrm{NOS} + \textrm{HOSeff})}.

The proportion of wild salmon is defined as:

\deqn{p^{\textrm{WILD}}_t = q^\textrm{HOScen}_t
\dfrac{(q^\textrm{HOScen}_{t-1})^2}
{(q^\textrm{HOScen}_{t-1})^2 + 2\gamma \times p^\textrm{HOScen}_{t-1} q^\textrm{HOScen}_{t-1} +
\gamma^2 (p^\textrm{HOScen}_{t-1})^2}}

where \eqn{q = 1-p} and \eqn{p^\textrm{HOScen} = \textrm{HOS}/(\textrm{NOS} + \textrm{HOS})}.
}
\section{Slots}{

\describe{
\item{\code{Name}}{Character. Identifying name}

\item{\code{proyears}}{Integer. The number of projected years}

\item{\code{nsim}}{Integer. The number of simulations}

\item{\code{nstocks}}{Integer. The number of stocks}

\item{\code{Snames}}{Character. Stock names}

\item{\code{Egg_NOS}}{Array \verb{[nsim, nstocks, proyears]}. Spawning output, i.e., egg production, of natural origin spawners.}

\item{\code{Egg_HOS}}{Array \verb{[nsim, nstocks, proyears]}. Spawning output of hatchery origin spawners.}

\item{\code{Fry_NOS}}{Array \verb{[nsim, nstocks, proyears]}. Fry that are offspring of natural origin spawners.}

\item{\code{Fry_HOS}}{Array \verb{[nsim, nstocks, proyears]}. Fry that are offspring of hatchery origin spawners.}

\item{\code{Smolt_NOS}}{Array \verb{[nsim, nstocks, proyears]}. Smolts that are offspring of natural origin spawners.}

\item{\code{Smolt_HOS}}{Array \verb{[nsim, nstocks, proyears]}. Smolts that are offspring of hatchery origin spawners.}

\item{\code{Smolt_Rel}}{Array \verb{[nsim, nstocks, proyears]}. Smolts that are offspring of broodtake, i.e., hatchery releases.}

\item{\code{Njuv_NOS}}{Array \verb{[nsim, nstocks, nage, proyears]}. Abundance of juvenile natural origin fish at the beginning of the year.}

\item{\code{Njuv_HOS}}{Array \verb{[nsim, nstocks, nage, proyears]}. Abundance of juvenile hatchery origin fish at the beginning of the year.}

\item{\code{Return_NOS}}{Array \verb{[nsim, nstocks, nage, proyears]}. Mature fish that will be natural origin spawners.}

\item{\code{Return_HOS}}{Array \verb{[nsim, nstocks, nage, proyears]}. Mature fish that will be hatchery origin spawners.}

\item{\code{Escapement_NOS}}{Array \verb{[nsim, nstocks, nage, proyears]}. The escapement of mature fish that will be natural origin spawners.}

\item{\code{Escapement_HOS}}{Array \verb{[nsim, nstocks, nage, proyears]}. The escapement of mature fish that will be hatchery origin spawners.}

\item{\code{NOB}}{Array \verb{[nsim, nstocks, proyears]}. Natural origin broodtake.}

\item{\code{HOB}}{Array \verb{[nsim, nstocks, proyears]}. Hatchery origin broodtake (local + strays).}

\item{\code{HOB_stray}}{Array \verb{[nsim, nstocks, proyears]}. Hatchery origin broodtake (strays only).}

\item{\code{HOB_import}}{Array \verb{[nsim, nstocks, proyears]}. Imported hatchery origin broodtake used for hatchery production.}

\item{\code{NOS}}{Array \verb{[nsim, nstocks, nage, proyears]}. Natural origin spawners.}

\item{\code{HOS}}{Array \verb{[nsim, nstocks, nage, proyears]}. Hatchery origin spawners (local + strays).}

\item{\code{HOS_stray}}{Array \verb{[nsim, nstocks, nage, proyears]}. Hatchery origin spawners (strays only).}

\item{\code{HOS_effective}}{Array \verb{[nsim, nstocks, nage, proyears]}. Hatchery origin spawners (local + strays) discounted by \code{gamma}.}

\item{\code{KPT_NOS}}{Array \verb{[nsim, nstocks, proyears]}. Pre-terminal fishery kept catch of natural origin spawners.}

\item{\code{KT_NOS}}{Array \verb{[nsim, nstocks, proyears]}. Terminal fishery kept catch of natural origin spawners.}

\item{\code{KPT_HOS}}{Array \verb{[nsim, nstocks, proyears]}. Pre-terminal fishery kept catch of hatchery origin spawners.}

\item{\code{KT_HOS}}{Array \verb{[nsim, nstocks, proyears]}. Terminal fishery kept catch of hatchery origin spawners.}

\item{\code{DPT_NOS}}{Array \verb{[nsim, nstocks, proyears]}. Pre-terminal fishery released catch (live and dead) of natural origin spawners.}

\item{\code{DT_NOS}}{Array \verb{[nsim, nstocks, proyears]}. Terminal fishery released catch (live and dead) of natural origin spawners.}

\item{\code{DPT_HOS}}{Array \verb{[nsim, nstocks, proyears]}. Pre-terminal fishery released catch (live and dead) of hatchery origin spawners.}

\item{\code{DT_HOS}}{Array \verb{[nsim, nstocks, proyears]}. Terminal fishery released catch (live and dead) hatchery origin spawners.}

\item{\code{UPT_NOS}}{Array \verb{[nsim, nstocks, nage, proyears]}. Pre-terminal fishery harvest rate (from kept catch) of natural origin spawners.}

\item{\code{UT_NOS}}{Array \verb{[nsim, nstocks, nage, proyears]}. Terminal fishery harvest rate of natural origin spawners.}

\item{\code{UPT_HOS}}{Array \verb{[nsim, nstocks, nage, proyears]}. Pre-terminal fishery harvest rate of hatchery origin spawners.}

\item{\code{UT_HOS}}{Array \verb{[nsim, nstocks, nage, proyears]}. Terminal fishery harvest rate of hatchery origin spawners.}

\item{\code{ExPT_NOS}}{Array \verb{[nsim, nstocks, nage, proyears]}. Pre-terminal fishery exploitation rate (from kept catch and dead releases) of natural origin spawners.}

\item{\code{ExT_NOS}}{Array \verb{[nsim, nstocks, nage, proyears]}. Terminal fishery exploitation rate of natural origin spawners.}

\item{\code{ExPT_HOS}}{Array \verb{[nsim, nstocks, nage, proyears]}. Pre-terminal fishery exploitation rate of hatchery origin spawners.}

\item{\code{ExT_HOS}}{Array \verb{[nsim, nstocks, nage, proyears]}. Terminal fishery exploitation rate of hatchery origin spawners.}

\item{\code{fitness}}{Array \verb{[nsim, nstocks, 2, proyears]}. Fitness of the population in the natural (1) and hatchery (2) environments.}

\item{\code{pNOB}}{Array \verb{[nsim, nstocks, proyears]}. Proportion of natural fish in the brood.}

\item{\code{pHOS_census}}{Array \verb{[nsim, nstocks, proyears]}. Proportion of spawners of hatchery origin, weighted by age class fecundity.}

\item{\code{pHOS_effective}}{Array \verb{[nsim, nstocks, proyears]}. Proportion of spawners of hatchery origin, discounted by \code{gamma}, weighted by age class fecundity.}

\item{\code{PNI}}{Array \verb{[nsim, nstocks, proyears]}. Proportionate natural influence, index of gene flow from hatchery to the natural environment.}

\item{\code{p_wild}}{Array \verb{[nsim, nstocks, proyears]}. Proportion of wild spawners, natural spawners whose parents were also produced in the natural environment assuming
non-assortative mating, defined under Canada's Wild Salmon Policy.}

\item{\code{Mjuv_loss}}{Array \verb{[nsim, nstocks, nage, proyears]}. Realized juvenile natural mortality, which may differ from inputs due to fitness loss.}

\item{\code{Misc}}{List. Miscellaneous output:
\itemize{
\item \code{Ref} for reference points
\item \code{SHist} for the \linkS4class{SHist} object (primarily for developer use)
\item \code{SOM} for the \linkS4class{SOM} object (updated by \code{check_SOM()}).
\item \code{LHG} list \code{nstocks} long containing state variables by natural-origin life history group
\item \code{RS} list \code{nstocks} long containing state variables by hatchery-origin release strategy
}}
}}

\section{Creating Object}{

Objects can be created by calls of the form \code{new("SMSE")}
}

\examples{
showClass("SMSE")
}
\references{
Withler et al. 2018. Genetically Based Targets for Enhanced Contributions to Canadian Pacific Chinook Salmon Populations.
DFO Can. Sci. Advis. Sec. Res. Doc. 2018/019. xii + 88 p.
}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CM.R
\name{fit_CM}
\alias{fit_CM}
\alias{sample_CM}
\title{Fit conditioning model to historical data}
\usage{
fit_CM(
  data,
  start = list(),
  map = list(),
  lower = list(),
  upper = list(),
  do_fit = TRUE,
  silent = TRUE,
  control = list(eval.max = 1e+05, iter.max = 1e+05),
  ...
)

sample_CM(fit, ...)
}
\arguments{
\item{data}{A list containing data inputs. See details.}

\item{start}{An optional list containing parameter starting values. See details.}

\item{map}{An optional list that describes how parameters are fixed in the model. See \code{\link[TMB:MakeADFun]{TMB::MakeADFun()}}.}

\item{lower}{Named list containing lower bounds for parameters. See details.}

\item{upper}{Named list containing upper bounds for parameters. See details.}

\item{do_fit}{Logical, whether to do the fit and estimate the Hessian.}

\item{silent}{Logical, whether to silence output from RTMB to the console.}

\item{control}{List, \code{control} argument to pass to \code{\link[stats:nlminb]{stats::nlminb()}}.}

\item{...}{For \code{fit_CM}, arguments to \code{\link[RTMB:TMB-interface]{RTMB::MakeADFun()}}. For \code{sample_CM}, arguments to \code{rstan::sampling()}}

\item{fit}{List of output from \code{fit_CM()}}
}
\value{
\itemize{
\item \code{fit_CM()} returns a named list containing the RTMB model (\code{obj}), nlminb output (\code{opt}), standard errors (\code{SD}),
and parameter bounds (\code{lower} and \code{upper})
\item \code{sample_CM()} returns a \code{stanfit} object containing the MCMC chains
}
}
\description{
{
Bayesian stock reconstruction model of natural and hatchery origin fish population.
Maturity and age-1 natural mortality are estimated from coded wire tag catch and escapement at age.
A separate series of observed escapement, and hatchery releases reconstructs the population of interest,
informed by natural mortality and maturity from CWT (\href{https://publications.gc.ca/site/eng/9.940685/publication.html}{Korman and Walters 2024}).
The model estimates time-varying maturity rate as well as time-varying ocean survival as a linear model of covariates (separate covariates
for age 1 vs. ages 2+).
The model can include either a preterminal juvenile fishery, terminal return fishery, or both (see Data and start sections of the documentation).

\code{fit_CM()} generates the RTMB model from data which can then be passed to \code{sample_CM()} to run the MCMC in Stan. Generate a markdown report with \code{\link[=report_CM]{report_CM()}}.

More information is available on the \href{https://docs.salmonmse.com/articles/conditioning.html}{salmonMSE} website
}
}
\section{Data}{

Data should passed through a named list with the following entries.
\itemize{
\item \code{Nages} Integer, number of age classes in the model
\item \code{Ldyr} Integer, number of years in the model
\item \code{lht} Integer, life history type. Should be 1 for now
\item \code{n_r} Integer, number of release strategies for CWT, subset of a hatchery-origin brood year that differ in maturity rate. Default is 1.
\item \code{cwtrelease} Matrix \verb{[Ldyr, n_r]}, coded wire tag (CWT) releases by \strong{release year and release strategy}
\item \code{cwtesc} Array \verb{[Ldyr, Nages, n_r]}. CWT escapement \strong{by release year, age, and release strategy} (assumes broodyear is release year minus 1). Poisson likelihood.
\item \code{cwtcatPT} Array \verb{[Ldyr, Nages, n_r]}. CWT preterminal catch (juvenile fish), \strong{by release year, age, and release strategy}. Poisson likelihood. Set all values to zero to turn off
parameters related to the preterminal fishery.
\item \code{cwtcatT} Array \verb{[Ldyr, Nages, n_r]}. CWT terminal catch (returning, mature fish), \strong{by release year, age, and release strategy}. Poisson likelihood. Set all values to zero to turn off
parameters related to the terminal fishery.
\item \code{bvulPT} Vector length \code{Nages}. Prior mean for the vulnerability at age to the preterminal fishery.
\item \code{bvulT} Vector length \code{Nages}. Prior mean for the vulnerability at age to the terminal fishery.
\item \code{RelRegFPT} Vector \code{Ldyr}. Trend in relative regional preterminal fishing mortality. Fishing mortality is estimated by estimating a scaling
coefficient and annual deviations from this vector. Default is \code{rep(1, d$Ldyr)} (no prior trend) if \code{cwtcatPT} is provided, otherwise zero.
\item \code{RelRegFT} Vector \code{Ldyr}. Trend in relative regional terminal fishing mortality.
Default is \code{rep(1, d$Ldyr)} (no prior trend) if \code{cwtcatT} is provided, otherwise zero.
\item \code{bmatt} Vector length \code{Nages}. Proportion maturity at age, base values for calculating the unfished replacement line. Also the prior means if year-specific
maturity rates are estimated.
\item \code{mobase}. Vector length \code{Nages}. Natural mortality at age, base values for calculating the unfished replacement line and the
the equilibrium spawners at age.
\item \code{covariate1} \emph{Optional}. Matrix \verb{Ldyr, ncov1} of linear covariates that predict natural mortality for age 1.
\item \code{covariate} \emph{Optional}. Matrix \verb{Ldyr, ncov} of linear covariates that predict natural mortality for ages 2+.
\item \code{hatchsurv} Numeric, survival of hatchery releases into the smolt life stage. Density-independent. Default is 1. If less than 1, then hatchery origin fish have
lower survival to age 2 (after first year of marine life stage) compared to natural origin fish.
\item \code{gamma} \emph{Optional}. Numeric, the relative spawning success of hatchery origin spawners. Default is 1.
\item \code{ssum} Numeric, proportion of spawners that is female. Can also be a vector \code{Nages}
\item \code{fec} Vector length \code{Nages}. Fecundity, egg production at age
\item \code{r_matt} Integer, the release strategy for which to use maturity parameter for the natural system. Default is 1.
\item \code{obsescape} Vector length \code{Ldyr} by \strong{return year}, total observed escapement from fisheries, i.e., return to river (all ages and both hatchery/natural fish). Lognormal likelhood.
\item \code{propwildspawn} Vector length \code{Ldyr} by \strong{return year}, proportion of the escapement that spawn (accounts for en-route mortality and broodtake)
\item \code{hatchrelease} Vector length \code{Ldyr+1} by \strong{release year}, number of hatchery releases for the population. Default is zero.
\item \code{obs_pHOS} \emph{Optional}. Vector length \code{Ldyr} by \strong{return year}, observations of proportion of hatchery origin spawners (census) (between 0-1) \emph{by brood year}. Fitted to model with logistic-normal likelihood.
\item \code{pHOS_sd} Numeric, logistic-normal standard deviation of pHOS observations. Default is 1.
\item \code{pHOS_init} Numeric, initial pHOS for equilibrium abundance in the first year of the model. Default is 0.
\item \code{s_enroute} Numeric, survival of escapement to spawning grounds. Default is 1.
\item \code{so_mu} Numeric, the prior mean for spawners at unfished replacement in logspace. Default is \code{log(3 * max(data$obsescape))}.
\item \code{so_sd} Numeric, the prior standard deviation for spawners at unfished replacement in logspace. Default is 0.5.
\item \code{finitPT} Numeric, initial preterminal fishing mortality for calculating the equilibrium juvenile proportions at age in the first year of the model. Default is 0. Set to \code{"estimate"} to allow the model to estimate
the equilibrium condition.
\item \code{finitT} Numeric, initial terminal fishing mortality for calculating the equilibrium juvenile proportions at age in the first year of the model. Default is 0. Set to \code{"estimate"} to allow the model to estimate
the equilibrium condition.
\item \code{spawn_init} Numeric, initial spawners to calculate equilibrium abundance in the first year of the model. Default is \code{obsescape[1]}.
\item \code{cwtExp} Numeric, the CWT expansion factor, typically the reciprocal of the catch sampling rate (higher factors for lower sampling rate).
The model scales down the CWT predictions to match the observations. In other words,
the model assumes that the CWT catch and escapement are not expanded. For example, \code{cwtExp = 10} divides the CWT predictions by 10 for the likelihood. Default is 1.
The Poisson distribution is used for the likelihood of the CWT observations, and the expansion parameter can be used to downweight the CWT likelihood relative to the escapement time series.
However it requires adjustments of the CWT catches prior to fitting to ensure the proper population scale.
If the expanded catch is 100, then the input CWT catch should be 10 and 50 with \code{cwtExp} of 10 and 2, respectively, to maintain the same population scale.
The Poisson variance scales with the mean and is higher with \code{cwtExp = 2}.
\item \code{fitness} Logical, whether to calculate fitness effects on survival. Default is \code{FALSE}.
\item \code{theta} Vector length 2, the optimum phenotype value for the natural and hatchery environments. Default is 100 and 80, respectively. See
\href{https://docs.salmonmse.com/articles/equations.html#fitness-effects-on-survival}{online article} for more information.
\item \code{rel_loss} Vector length 3, the loss in fitness apportioned between the egg, fry (both prior to density-dependence), and smolt (after density-dependence) life stage. The three values should sum to 1.
\item \code{zbar_start} Vector length 2, the mean phenotype of the spawners and broodtake in the natural and hatchery environment, respectively, at the start of the model. Default values of 100 and 100, implying maximum fitness at
for the natural environment at the start of the model.
\item \code{fitness_variance} Numeric. The variance (omega-squared) of the fitness function. Assumed identical between the natural and hatchery environments. Default is 100.
\item \code{phenotype_variance} Numeric. The variance (sigma-squared) of the phenotypic trait (with optimum theta). Assumed identical between the natural and hatchery environments. Default is 10.
\item \code{heritability} Numeric. The heritability (h-squared) of the phenotypic trait. Between 0-1. Default is 0.5.
\item \code{fitness_floor} Numeric. The minimum fitness value in the natural and hatchery environments. Default is 0.5.
}
}

\section{start}{

Starting values for parameters can be provided through a named list:
\itemize{
\item \code{log_cr} Numeric, log of the compensation ratio (productivity). Default is 3.
\item \code{log_so} Numeric, unfished spawners in logspace. Default is \code{log(3 * max(data$obsescape))}.
\item \code{moadd} Numeric, additive term to base natural mortality rate for age 1 juveniles. Default is zero.
\item \code{wt} Vector \code{Ldyr}. Annual deviates in natural mortality during the freshwater life stage (affects egg to smolt survival).
Estimated with normal prior with mean zero and standard deviation \code{p$wt_sd}. Default is zero.
\item \code{wto} Vector \code{Ldyr}. Annual deviates in natural mortality for age 1 juveniles (marine life stage).
Estimated with normal prior with mean zero and standard deviation \code{p$wto_sd}. Default is zero.
\item \code{log_FbasePT} Numeric, scaling coefficient to estimate preterminal fishing mortality from \code{data$RelRegFPT}. Default is \code{log(0.1)}.
\item \code{log_FbaseT} Numeric, scaling coefficient to estimate preterminal fishing mortality from \code{data$RelRegFT}. Default is \code{log(0.1)}.
\item \code{log_fanomalyPT} Vector \code{Ldyr}. Annual lognormal deviates from \code{exp(log_FbasePT) * data$RelRegFPT} to estimate preterminal fishing mortality.
Estimated with normal prior with mean zero and standard deviation \code{p$fanomaly_sd}. Default is zero.
\item \code{log_fanomalyT} Vector \code{Ldyr}. Annual lognormal deviates from \code{exp(log_FbaseT) * data$RelRegFT} to estimate terminal fishing mortality.
Estimated with normal prior with mean zero and standard deviation \code{p$fanomalyPT_sd}. Default is zero.
\item \code{lnE_sd} Numeric, lognormal standard deviation of the observed escapement. Estimated with hierarchical \code{gamma(2, 5)} prior. Default is 0.1.
\item \code{wt_sd} Numeric, lognormal standard deviation of the egg to smolt (freshwater) natural mortality deviates. Estimated with hierarchical \code{gamma(2, 5)} prior. Default is 1.
\item \code{wto_sd} Numeric, lognormal standard deviation of the age 1 (marine) natural mortality deviates. Estimated with hierarchical \code{gamma(2, 5)} prior. Default is 1.
\item \code{fanomalyPT_sd} Numeric, lognormal standard deviation of \code{fanomalyPT}. Estimated with hierarchical \code{gamma(2, 5)} prior. Default is 1.
\item \code{fanomalyT_sd} Numeric, lognormal standard deviation of \code{fanomalyT}. Estimated with hierarchical \code{gamma(2, 5)} prior. Default is 1.
\item \code{logit_vulPT} Vector \code{Nages-2} of preterminal vulnerability at age in logit space. Fixed to zero and one at age 1 and the maximum age, respectively.
Default is \code{qlogis(data$bvul_PT[-c(1, data$Nages)])}.
\item \code{logit_vulT} Vector \code{Nages-2} of terminal vulnerability at age in logit space. Fixed to zero and one at age 1 and the maximum age, respectively.
Default is \code{qlogis(data$bvul_T[-c(1, data$Nages)])}.
\item \code{logit_matt} Matrix \verb{Ldyr, Nages-2} maturity by year and age in logit space. Maturity is fixed to zero and one at age 1 and the maximum age, respectively.
Default is \code{matrix(qlogis(data$bmatt[-c(1, data$Nages)]), data$Ldyr, data$Nages-2, byrow = TRUE)}.
\item \code{sd_matt} Vector \code{Nages-2}. Logit standard deviation of maturity (\code{logit_matt}) by age class. Default is 0.5.
\item \code{b1} Vector \code{ncov1} of coefficients for linear covariates that predict natural mortality for age 1. Default is zero.
\item \code{b} Vector \code{ncov} of coefficients for linear covariates that predict natural mortality for ages 2+. Default is zero.
}
}

\section{Bounds}{


By default, the standard deviation parameters and parameters in normal space (e.g., \code{FbasePT}, \code{Fbase_T}) have a lower bound of zero.
\code{moadd} has a lower bound of zero by default, but it is feasible that this parameter can be negative as well.
Deviation parameters centred around zero are bounded between -3 to 3.
The \code{log_cr} parameter has a lower bound of zero.

All other parameters are unbounded.
}

\section{Covariates on natural mortality}{


Natural mortality is modeled as the sum of a base value \eqn{M^\textrm{base}}, additional scaling factor for age 1 \eqn{M^\textrm{add}},
a linear system of covariates \eqn{X} and coefficients \eqn{b}:

\deqn{
M_{y,a} =
\begin{cases}
M^\textrm{base}_a + M^\textrm{add} + \sum_j b^1_j X^1_{y,j} & \quad a = 1\\
M^\textrm{base}_a + \sum_j b_j X_{y,j} & \quad a = 2, \ldots, A
\end{cases}
}
}

\references{
Korman, J. and Walters, C. 2024. A life cycle model for Chinook salmon population dynamics. Canadian Contractor Report of Hydrography and Ocean
Sciences 62: vi + 60 p.
}
\seealso{
\code{\link[=report_CM]{report_CM()}}

\code{\link[=CM2SOM]{CM2SOM()}}
}
\author{
Q. Huynh from Stan code provided by J. Korman and C. Walters
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\name{glossary}
\alias{glossary}
\title{salmonMSE glossary}
\description{
Glossary of terms and parameters used in salmonMSE
}
\examples{
data(glossary)
glossary[1:2, ]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/define-SOM.R
\docType{class}
\name{SHist-class}
\alias{SHist-class}
\title{Class \code{"SHist"}}
\description{
Stores the outputs from the historical reconstruction of salmon operating models. Primarily used for development and debugging.
}
\section{Slots}{

\describe{
\item{\code{Name}}{Character. Identifying name}

\item{\code{nyears}}{Integer, number of historica years}

\item{\code{nsim}}{Integer. The number of simulations}

\item{\code{nstocks}}{Integer. The number of stocks}

\item{\code{Snames}}{Character. Stock names}

\item{\code{Egg_NOS}}{Array \verb{[nsim, nstocks, nyears]}. Spawning output, i.e., egg production, of natural origin spawners.}

\item{\code{Egg_HOS}}{Array \verb{[nsim, nstocks, nyears]}. Spawning output of hatchery origin spawners.}

\item{\code{Smolt}}{Array \verb{[nsim, nstocks, nyears]}. Natural smolt production (sum of offspring of natural and hatchery spawners).}

\item{\code{Smolt_Rel}}{Array \verb{[nsim, nstocks, proyears]}. Smolts that are offspring of broodtake, i.e., hatchery releases.}

\item{\code{Njuv_NOS}}{Array \verb{[nsim, nstocks, nage, nyears]}. Abundance of juvenile natural origin fish at the beginning of the year.}

\item{\code{Njuv_HOS}}{Array \verb{[nsim, nstocks, nage, nyears]}. Abundance of juvenile hatchery origin fish at the beginning of the year.}

\item{\code{Return_NOS}}{Array \verb{[nsim, nstocks, nage, nyears]}. Mature fish that will be natural origin spawners.}

\item{\code{Return_HOS}}{Array \verb{[nsim, nstocks, nage, nyears]}. Mature fish that will be hatchery origin spawners.}

\item{\code{Escapement_NOS}}{Array \verb{[nsim, nstocks, nage, nyears]}. The escapement of mature fish that will be natural origin spawners.}

\item{\code{Escapement_HOS}}{Array \verb{[nsim, nstocks, nage, nyears]}. The escapement of mature fish that will be hatchery origin spawners.}

\item{\code{NOS}}{Array \verb{[nsim, nstocks, proyears]}. Natural origin spawners.}

\item{\code{HOS}}{Array \verb{[nsim, nstocks, proyears]}. Hatchery origin spawners.}

\item{\code{HOS_effective}}{Array \verb{[nsim, nstocks, proyears]}. Hatchery origin spawners discounted by \code{gamma}.}

\item{\code{KPT_NOS}}{Array \verb{[nsim, nstocks, proyears]}. Pre-terminal fishery kept catch of natural origin spawners.}

\item{\code{KT_NOS}}{Array \verb{[nsim, nstocks, proyears]}. Terminal fishery kept catch of natural origin spawners.}

\item{\code{KPT_HOS}}{Array \verb{[nsim, nstocks, proyears]}. Pre-terminal fishery kept catch of hatchery origin spawners.}

\item{\code{KT_HOS}}{Array \verb{[nsim, nstocks, proyears]}. Terminal fishery kept catch of hatchery origin spawners.}

\item{\code{DPT_NOS}}{Array \verb{[nsim, nstocks, proyears]}. Pre-terminal fishery released catch (live and dead) of natural origin spawners.}

\item{\code{DT_NOS}}{Array \verb{[nsim, nstocks, proyears]}. Terminal fishery released catch (live and dead) of natural origin spawners.}

\item{\code{DPT_HOS}}{Array \verb{[nsim, nstocks, proyears]}. Pre-terminal fishery released catch (live and dead) of hatchery origin spawners.}

\item{\code{DT_HOS}}{Array \verb{[nsim, nstocks, proyears]}. Terminal fishery released catch (live and dead) hatchery origin spawners.}

\item{\code{UPT_NOS}}{Array \verb{[nsim, nstocks, proyears]}. Pre-terminal fishery harvest rate (from kept catch) of natural origin spawners.}

\item{\code{UT_NOS}}{Array \verb{[nsim, nstocks, proyears]}. Terminal fishery harvest rate of natural origin spawners.}

\item{\code{UPT_HOS}}{Array \verb{[nsim, nstocks, proyears]}. Pre-terminal fishery harvest rate of hatchery origin spawners.}

\item{\code{UT_HOS}}{Array \verb{[nsim, nstocks, proyears]}. Terminal fishery harvest rate of hatchery origin spawners.}

\item{\code{ExPT_NOS}}{Array \verb{[nsim, nstocks, proyears]}. Pre-terminal fishery exploitation rate (from kept catch and dead releases) of natural origin spawners.}

\item{\code{ExT_NOS}}{Array \verb{[nsim, nstocks, proyears]}. Terminal fishery exploitation rate of natural origin spawners.}

\item{\code{ExPT_HOS}}{Array \verb{[nsim, nstocks, proyears]}. Pre-terminal fishery exploitation rate of hatchery origin spawners.}

\item{\code{ExT_HOS}}{Array \verb{[nsim, nstocks, proyears]}. Terminal fishery exploitation rate of hatchery origin spawners.}

\item{\code{Misc}}{List. Miscellaneous output}
}}

\examples{
showClass("SHist")
}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Harvest_MMP.R, R/MMSE2SMSE.R, R/SOM2MOM.R,
%   R/SOM2MOM_int.R, R/multiHist2SHist.R
\name{make_Harvest_MMP}
\alias{make_Harvest_MMP}
\alias{MMSE2SMSE}
\alias{salmonMSE-int}
\alias{SOM2MOM}
\alias{make_Stock}
\alias{make_Fleet}
\alias{multiHist2SHist}
\title{Internal salmonMSE functions for converting operating model inputs and outputs}
\usage{
make_Harvest_MMP(SOM, check = TRUE)

MMSE2SMSE(MMSE, SOM, Harvest_MMP, N, stateN, Ford, H, stateH)

SOM2MOM(SOM, check = TRUE)

make_Stock(
  SOM,
  s = 1,
  g = 1,
  r = 1,
  NOS = TRUE,
  stage = c("immature", "return", "escapement")
)

make_Fleet(SOM, s, NOS = TRUE, stage = c("immature", "return", "escapement"))

multiHist2SHist(multiHist, SOM, check = TRUE)
}
\arguments{
\item{SOM}{An object of class \linkS4class{SOM}}

\item{check}{Logical, whether to check the \code{SOM} object using \code{\link[=check_SOM]{check_SOM()}}}

\item{MMSE}{Object of class \link[MSEtool:MMSE-class]{MSEtool::MMSE} returned from MSEtool}

\item{Harvest_MMP}{Optional harvest function created by \code{\link[=make_Harvest_MMP]{make_Harvest_MMP()}}}

\item{N}{Data frame of natural origin abundance at age saved in the \link{salmonMSE_env} environment during the simulation}

\item{stateN}{Data frame of natural origin state variables saved in the \link{salmonMSE_env} environment during the simulation}

\item{Ford}{Data frame of phenotypic trait values saved in the \link{salmonMSE_env} environment during the simulation}

\item{H}{Data frame of hatchery origin abundance at age saved in the \link{salmonMSE_env} environment during the simulation}

\item{stateH}{Data frame of hatchery origin state variables saved in the \link{salmonMSE_env} environment during the simulation}

\item{s}{Integer, the population integer for which to create the Stock or Fleet object}

\item{g}{Integer, the life history group for which to create the Stock object. Not relevant if \code{NOS = FALSE}}

\item{r}{Integer, the hatchery release group for which to create the Stock object. Not relevant if \code{NOS = TRUE}}

\item{NOS}{Logical, whether the Stock or Fleet object corresponds to natural origin or hatchery origin fish}

\item{stage}{Character indicating the corresponding salmon life stage of the Stock or Fleet object}

\item{multiHist}{Class multiHist object returned from MSEtool}
}
\value{
\code{make_Harvest_MMP}: Function of class "MMP" by updating the formal arguments for \code{\link[=Harvest_MMP]{Harvest_MMP()}}

\code{MMSE2SMSE}: \linkS4class{SMSE} object

\code{SOM2MOM}: \link[MSEtool:MOM-class]{MSEtool::MOM} object

\code{make_Stock}: List containing a \link[MSEtool:Stock-class]{MSEtool::Stock} object and accompanying custom parameters list

\code{make_Stock}: List containing a \link[MSEtool:Fleet-class]{MSEtool::Fleet} object and accompanying custom parameters list

\code{multiHist2SHist}: \linkS4class{SHist} object
}
\description{
\itemize{
\item \code{\link[=SOM2MOM]{SOM2MOM()}} converts a salmon operating model (\linkS4class{SOM}) to a multi-stock operating model (\link[MSEtool:MOM-class]{MSEtool::MOM})
\item \code{\link[=make_Stock]{make_Stock()}} creates the \link[MSEtool:Stock-class]{MSEtool::Stock} object (openMSE) corresponding to salmon life stage
\item \code{\link[=make_Fleet]{make_Fleet()}} creates the \link[MSEtool:Fleet-class]{MSEtool::Fleet} object (openMSE) corresponding to the fishery that interacts with the various salmon life stages
\item \code{\link[=multiHist2SHist]{multiHist2SHist()}} converts the openMSE historical reconstruction into a salmon Hist object (\linkS4class{SHist})
\item \code{\link[=MMSE2SMSE]{MMSE2SMSE()}} converts the openMSE projection output, along with additional state variables recorded in \link{salmonMSE_env}, into a salmon MSE object (\linkS4class{SMSE})
\item \code{\link[=make_Harvest_MMP]{make_Harvest_MMP()}} creates a multi-stock management procedure for the harvest component of the operating model by
specifying exploitation rates through updating the formal arguments for \code{\link[=Harvest_MMP]{Harvest_MMP()}}
}

\code{\link[=salmonMSE]{salmonMSE()}} is the wrapper function that coordinates the simulation and the output.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/define-SOM.R
\docType{class}
\name{Hatchery-class}
\alias{Hatchery-class}
\title{Class \code{"Hatchery"}}
\description{
The component of the operating model that controls the hatchery management and in-river removals.
}
\details{
Various parameters can be stochastic (length \code{nsim}) or input as a single numeric
(value identical across all simulations).

A description of the fitness parameters and hatchery dynamics is available in the
\href{https://docs.salmonmse.com/articles/equations.html#hatchery-production}{online documentation}.
}
\section{Slots}{

\describe{
\item{\code{Name}}{Character. Identifying name}

\item{\code{n_r}}{Integer. Number of release strategies, sub-groups of fish with different survival schedules. Default is one.}

\item{\code{n_yearling}}{Vector length \code{n_r}. The target number of yearlings released by release strategy. No hatchery is modeled if \code{sum(n_yearling, n_subyearling) = 0}.
are zero. Default is zero. Yearlings are differentiated from sub-yearlings only in the hatchery survival parameter, e.g., lower egg-yearling survival than for egg-subyearling, so more brood is needed for yearling releases.}

\item{\code{n_subyearling}}{Vector length \code{n_r}. The target number of subyearlings released. No hatchery is modeled if \code{sum(n_yearling, n_subyearling) = 0} are zero.
Default is zero. Yearlings are differentiated from sub-yearlings only in the hatchery survival parameter, e.g., lower egg-yearling survival than for egg-subyearling, so more brood is needed for yearling releases.}

\item{\code{yearling_DD}}{Logical, whether freshwater yearling survival after release is density-dependent, in competition with natural-origin juveniles. Default is FALSE. Likely relevant only for stream types where there is substantial residence time before outmigration.}

\item{\code{subyearling_DD}}{Logical, whether freshwater subyearling survival after release is density-dependent, in competition with natural-origin juveniles. Default is FALSE. Likely relevant only for stream types where there is substantial residence time before outmigration.}

\item{\code{s_prespawn}}{Numeric. The survival of broodtake prior to egg production. \code{1 - s_prespawn} is the proportion of fish not used for hatchery purposes, e.g., mortality or other resesarch purposes.
Used to back-calculate the broodtake from \code{n_yearling} and \code{n_subyearling}. Default is 1.}

\item{\code{s_egg_smolt}}{Numeric. The survival of eggs to the smolt life stage (for yearling release).
Used to back-calculate the broodtake from \code{n_yearling} and \code{n_subyearling}. Default is 1.}

\item{\code{s_egg_subyearling}}{Numeric. The survival of eggs to subyearling life stage (for subyearling release).
Used to back-calculate the broodtake from \code{n_yearling} and \code{n_subyearling}. Default is 1.}

\item{\code{Mjuv_HOS}}{Either vector by age (length \code{maxage-1}) or an array with dimension \verb{[nsim, maxage-1, proyears, n_r]}.
Natural mortality of immature hatchery origin fish.
To replicate the SAR parameter of a stage-specific model, set \code{Mjuv_HOS[a] = -log(SAR)} for the age class prior to maturation (and zero for all other ages).}

\item{\code{p_mature_HOS}}{Vector by age (length \code{maxage}) or an array with dimension \verb{[nsim, maxage, proyears, n_r]} for the maturity of hatchery spawners.
Default is set equal to \code{Bio@p_mature} for all release strategies.}

\item{\code{stray_external}}{Matrix by age (length \code{maxage}) and release strategy \code{n_r} that denotes the annual number of hatchery origin strays from other populations/systems
not included in the operating model. Default is zero. External strays are added at the escapement life stage, assumed unmarked. For multi-population models with straying within the system, see also \code{SOM@stray} matrix.}

\item{\code{gamma}}{Numeric. The relative reproductive success of hatchery origin spawners (relative to natural origin spawners). Default is 1.}

\item{\code{m}}{Numeric. The mark rate of hatchery origin fish, which affects selective broodtake and fishery retention if mark-selective fishing is utilized.
Set m = 1 for AHA compatibility with \code{ptarget_NOB}. Default is zero.}

\item{\code{f_brood}}{Function that calculates the natural origin brood and hatchery origin brood from the escapement (after en-route mortality). Function should be of the form
\code{function(NO, HO, stray, m) {return(list(NOB, HOB_marked, HOB_unmarked, HOB_stray))}}. Allows for bespoke rules for broodtake.}

\item{\code{brood_import}}{\strong{Not used if \code{f_brood} is provided} Matrix by age (length \code{maxage}) for the number of annual imported hatchery origin broodstock. Egg production is weighted by \code{fec_brood}.
To meet hatchery production target releases, imported brood and local marked brood are used indiscriminately. Default imported brood is zero.}

\item{\code{pmax_esc}}{\strong{Not used if \code{f_brood} is provided} Numeric. The maximum proportion of total escapement (after en route mortality) that could be used as broodtake. Set to 1 for AHA compatibility. Default is 0.75.}

\item{\code{pmax_NOB}}{\strong{Not used if \code{f_brood} is provided} Numeric. The maximum proportion of the natural origin escapement (after en route mortality and \code{pmax_esc}) to be used as broodtake. If broodstock
is limited by \code{pmax_esc < 1}, then this parameter should be 1. Default is 1.}

\item{\code{ptarget_NOB}}{\strong{Not used if \code{f_brood} is provided} Numeric. The target proportion of the natural origin broodtake relative to the overall broodtake, assuming the mark rate is 1 and natural origin fish
can be identified in the hatchery. The realized proportion may be lower if there are insufficient natural origin escapement. If the mark rate < 1, then
this target proportion identifies the proportion of unmarked fished in the broodtake. If mark rate = 0, then pNOB is equal to the proportion in the escapement. Default is 0.9.}

\item{\code{phatchery}}{\strong{Not used if \code{f_brood} is provided} Numeric. Optional parameter (default is \code{NA}). If set to a numeric between 0-1, this value is the proportion of the hatchery origin escapement that return to the hatchery, for example, by removal from spawning grounds
or swim-in facilities. These fish are available for broodtake. None of these fish will spawn in the natural environment.
With the default option, \code{NA} allows all hatchery origin escapement to be available for brood (the remainder go to the spawning grounds).}

\item{\code{premove_HOS}}{Numeric or function. The target proportion of the hatchery origin fish to be removed from the spawning grounds (in order to
ensure a high proportion of NOS). The proportion of hatchery fish removed is discounted by the mark rate, i.e., \code{p = premove_HOS * m}.
The removed hatchery-origin fish do not spawn and are not available for broodtake. A value less than one can represent
imperfect implementation of weir removal. Default is zero. This slot can take a function that returns the proportion (p) based on hatchery-origin and natural-origin abundance (after brood removal), thus
allowing for bespoke rules for in-river harvest. The function should be of the form: \code{function(NO, HO, m) {return(p)}}.}

\item{\code{premove_NOS}}{Numeric or function. The target proportion of the natural origin fish to be removed from the spawning grounds, for example, through an in-river fishery.
The proportion of natural fish removed is adjusted by the mark rate, i.e., \code{p = premove_NOS * (1-m)}.
Default is zero. This slot can take a function that returns the proportion (p) based on hatchery-origin and natural-origin abundance (after brood removal), thus
allowing for bespoke rules for in-river harvest. The function should be of the form: \code{function(NO, HO, m) {return(p)}}.}

\item{\code{fec_brood}}{Vector of length \code{maxage} or an array with dimension \verb{[nsim, maxage, proyears]}. The fecundity schedule of broodtake to calculate the total egg production for the hatchery. If missing, uses \code{Bio@fec}.}

\item{\code{fitness_type}}{Character vector length 2. The fitness function to apply in the natural and hatchery environment, respectively. For each, either "Ford" or "none".}

\item{\code{theta}}{Vector length 2. The optimum phenotype value for the natural and hatchery environments.}

\item{\code{rel_loss}}{Vector length 3. The loss in fitness apportioned among the egg, fry, and smolt life stages which reduces survival. Theoretically, the three values should sum to 1. Alternatively,
set to zero to set fitness loss to zero for that specific life stage (survival is one).}

\item{\code{zbar_start}}{Vector length 2. The mean phenotype value in the natural and hatchery populations at the start of the projection. Alternatively,
an array by dimension \verb{[nsim, maxage, 2]}, where the age slot corresponds to cohort.}

\item{\code{fitness_variance}}{Numeric. The variance (omega-squared) of the fitness function. Assumed identical between the natural and hatchery environments. Default is 100.}

\item{\code{phenotype_variance}}{Numeric. The variance (sigma-squared) of the phenotypic trait (theta). Assumed identical between the natural and hatchery environments. Default is 10.}

\item{\code{heritability}}{Numeric or vector length \verb{[nsim]}. The heritability (h-squared) of the phenotypic trait. Between 0-1. Default is 0.5}

\item{\code{fitness_floor}}{Numeric. The minimum fitness value in the natural and hatchery environments, i.e., fitness cannot drop below this threshold. Default is 0.5.}
}}

\section{Creating Object}{

Objects can be created by calls of the form \code{new("Hatchery")}
}

\examples{
showClass("Hatchery")
}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ref.R
\name{calc_ref}
\alias{calc_ref}
\alias{calc_MSY}
\alias{calc_Sgen}
\title{Reference points}
\usage{
calc_ref(SOM, rel_F, check = TRUE, maximize = c("MSY", "MER"))

calc_MSY(
  Mjuv,
  fec,
  p_female,
  rel_F,
  vulPT,
  vulT,
  p_mature,
  s_enroute,
  n_g = 1,
  p_LHG = 1,
  SRRpars,
  maximize = c("MSY", "MER"),
  F_search = c(1e-08, 5)
)

calc_Sgen(
  Mjuv,
  fec,
  p_female,
  rel_F,
  vulPT,
  vulT,
  p_mature,
  s_enroute,
  n_g = 1,
  p_LHG = 1,
  SRRpars,
  SMSY,
  F_search = c(1e-08, 100),
  nyears
)
}
\arguments{
\item{SOM}{An object of class \linkS4class{SOM}}

\item{rel_F}{Numeric length 2, indicates the relative effort in the preterminal and terminal fisheries, with a maximum value of 1.
The default is \code{c(0, 1)} which indicates a yield calculation with only the terminal fishery.}

\item{check}{Logical, whether to check the SOM object using \code{\link[=check_SOM]{check_SOM()}}}

\item{maximize}{Character, whether the MSY calculation is the optimum that maximizes catch (\code{"MSY"}) or excess recruitment (\code{"MER"}). The two
methods should be equivalent when \code{rel_F = c(0, 1)}.}

\item{Mjuv}{Numeric \code{maxage} for juvenile natural mortality. Can be a matrix \verb{[maxage, n_g]}.}

\item{fec}{Numeric \code{maxage} for fecundity. Can be a matrix \verb{[maxage, n_g]}.}

\item{p_female}{Numeric for proportion female spawners}

\item{vulPT}{Numeric \code{maxage} for preterminal vulnerability at age}

\item{vulT}{Numeric \code{maxage} for terminal vulnerability at age}

\item{p_mature}{Numeric \code{maxage} for maturity proportions at age. Can be a matrix \verb{[maxage, n_g]}.}

\item{s_enroute}{Numeric for en-route survival of escapement to spawning grounds}

\item{n_g}{Integer, number of life history groups within a cohort}

\item{p_LHG}{Numeric \code{n_g} for proportion of the total egg production assigned to each life history group within a cohort}

\item{SRRpars}{Data frame, one row, that contains the stock recruit parameters that predicts density-dependent survival at the egg-smolt life stage}

\item{F_search}{Numeric, length 2 for the range of F values to search for the instantaneous fishing mortality that produces MSY}

\item{SMSY}{Numeric, spawning abundance at MSY}

\item{nyears}{Integer, number of years to project the population with no fishing to reach \code{SMSY}. Default
is the minimum age of maturity.}
}
\value{
\itemize{
\item \code{calc_MSY} returns a vector of various state variables (catch, exploitation rate, egg production, spawners) at MSY
\item \code{calc_Sgen} returns a numeric
\item \code{calc_ref} returns a list by stock, each containing a matrix of MSY state variables and Sgen by simulation
}
}
\description{
Calculate MSY and Sgen reference points for the operating model. Uses the biological parameters (maturity, natural mortality)
in the last year of the projection.
\itemize{
\item \code{calc_MSY()} calculates the MSY reference points from a set of biological and fishery parameters
\item \code{calc_Sgen()} calculates the Sgen, the spawner abundance that would reach the spawner abundance at MSY after one generation without fishing
\item \code{calc_ref()} is a wrapper function that calculates MSY and Sgen for an operating model
}
}
\examples{
ref_simple <- calc_ref(simple_SOM)
ref_multi <- calc_ref(multi_SOM)

}
\seealso{
\code{\link[=calc_Smsy_Ricker]{calc_Smsy_Ricker()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ref.R
\name{calc_Smsy_Ricker}
\alias{calc_Smsy_Ricker}
\alias{calc_Umsy_Ricker}
\alias{calc_Sgen_Ricker}
\title{Ricker reference points}
\usage{
calc_Smsy_Ricker(loga, b)

calc_Umsy_Ricker(loga)

calc_Sgen_Ricker(loga, b)
}
\arguments{
\item{loga}{Numeric, alpha parameter (returns per spawner) in the Ricker function: \eqn{R=S\exp(\log(a)-bS)}
where \code{S} is the number of spawners and \code{R} is the return}

\item{b}{Numeric, beta parameter}
}
\value{
All three functions return a numeric
}
\description{
Compute reference points (Umsy, Smsy, and Sgen) from Ricker stock-recruit function
based on Scheuerell (2016).
}
\examples{
a <- 3
Smax <- 100
b <- 1/Smax

calc_Smsy_Ricker(log(a), b)
calc_Umsy_Ricker(log(a))
calc_Sgen_Ricker(log(a), b)
}
\references{
Scheuerell, M.D. 2016. An explicit solution for
calculating optimum spawning stock size from Ricker’s stock recruitment model. PeerJ 4:e1623. \doi{10.7717/peerj.1623}
}
\seealso{
\code{\link[=calc_ref]{calc_ref()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SRR.R
\name{calc_SRR}
\alias{calc_SRR}
\title{Calculate abundance from density-dependent mortality}
\usage{
calc_SRR(N1, N2 = N1, p, capacity, type = c("BH", "Ricker", "HS"))
}
\arguments{
\item{N1}{Numeric, the initial abundance that scales the density-independent survival term}

\item{N2}{Numeric, the initial abundance that scales the density-dependent survival term}

\item{p}{Numeric, the productivity parameter that sets the maximum survival as the initial abundance approaches zero}

\item{capacity}{Numeric, the capacity parameter that set the maximum survivors}

\item{type}{Character, the functional form of the stock-recruit relationship}
}
\value{
Numeric, the abundance of survivors
}
\description{
Calculates the abundance of survivors after applying either a Beverton-Holt or Ricker stock-recruit relationship.
}
\details{
The Beverton-Holt stock recruit relationship is of the following form:
\deqn{\textrm{Smolt} = \dfrac{\alpha N_1}{1 + \beta N_2}}
where \eqn{\alpha = P}, \eqn{\beta = P/C}.

The Ricker stock recruit relationship is of the following form:
\deqn{\textrm{Smolt} = \alpha N_1 \exp(-\beta N_2)}
where \eqn{\alpha = P}, \eqn{\beta = P/(Ce)}, \eqn{e} is Euler's number.

Productivity \eqn{P} is in terms of abundance per unit of \eqn{N_1} and \eqn{N_2}.

The hockey stick is of the following form:

\deqn{
\textrm{Smolt} =
\begin{cases}
p N_1 &, N_1 \le \frac{N_1}{N_2} \times C\\
\frac{N_1}{N_2} \times C &, \textrm{otherwise}
\end{cases}
}
}
\examples{
N1 <- 100
N2 <- 200
p <- 10
capacity <- 250

calc_SRR(N1, N2, p, capacity, type = "BH")

# Validation
SRRpars <- calc_SRRpars(p, capacity, type = "BH")
a <- SRRpars[1]
b <- SRRpars[2]
a * N1/(1 + b * N2)
}
\seealso{
\code{\link[=calc_SRRpars]{calc_SRRpars()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Harvest_MMP.R
\name{get_F}
\alias{get_F}
\title{Calculate F from harvest rate}
\usage{
get_F(
  u = 0,
  K = 0,
  type = c("u", "catch"),
  M,
  N = 1,
  vul = 1,
  ret = 1,
  release_mort = 0,
  Fmax = 20
)
}
\arguments{
\item{u}{Harvest rate, between 0-1}

\item{K}{Catch, between 0-Inf}

\item{type}{Character, either \code{"catch"}, or \code{"u"}, whether to solve for catch or harvest rate, respectively}

\item{M}{Instantaneous natural mortality rate}

\item{N}{Abundance}

\item{vul}{Vulnerability}

\item{ret}{Retention rate}

\item{release_mort}{Release mortality as a proportion, between 0-1. Only relevant if \code{ret < 1}.}

\item{Fmax}{Maximum allowable value of F}
}
\value{
Numeric for the apical F
}
\description{
Solves for apical instantaneous fishing mortality rate (F), proportional to fishing effort, from harvest rate (total retained catch over total abundance).
The apical F can be greater than the realized F, if retention < 1.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot-statevar.R
\name{plot_decision_table}
\alias{plot_decision_table}
\alias{plot_decision_table2}
\title{Decision table of performance metrics}
\usage{
plot_decision_table(
  x,
  y,
  z,
  title,
  xlab,
  ylab,
  scenario,
  ncol = NULL,
  dir = "v"
)

plot_decision_table2(
  x,
  y,
  z,
  title,
  xlab,
  ylab,
  zlab,
  scenario,
  ncol = NULL,
  dir = "v",
  bin = c(0, 0.05, 0.25, 0.5, 0.75, 0.95),
  bin_labels = c("0-0.04", "0.05-0.24", "0.25-0.49", "0.5-0.74", "0.75-0.94", "0.95-1"),
  bin_col = c("purple4", "deeppink", "pink", "white", "green", "green4"),
  cell_border = FALSE,
  add_values = FALSE
)
}
\arguments{
\item{x}{Atomic, vector of values for the x axis (same length as z). Will be converted to factors}

\item{y}{Atomic, vector of values for the y axis (same length as z). Will be converted to factors}

\item{z}{Numeric, vector of values for the performance metric}

\item{title}{Character, optional title of figure}

\item{xlab}{Character, optional x-axis label}

\item{ylab}{Character, optional y-axis label}

\item{scenario}{Atomic, vector of faceting variables (same length as z) used to generate a grid of decision tables}

\item{ncol}{Integer, number of columns for decision table grid, only used if \code{scenario} is provided}

\item{dir}{Character, either "h" or "v" to describe how the grid of tables should be organized (horizontally or vertically)}

\item{zlab}{Character, optional color legend}

\item{bin}{Numeric vector of bins to sort values of \code{z}}

\item{bin_labels}{Character vector for bin names for the figure}

\item{bin_col}{Character vector of colors for the bins in the figure}

\item{cell_border}{Logical, whether to add borders for each cell in the figure}

\item{add_values}{Logical, whether to add the values of \code{z} in the figure}
}
\value{
ggplot object
}
\description{
Generates a coloured table of a performance metric across two axes, which may be a population dynamics variable
(e.g., productivity) or a management action (e.g., hatchery production levels or harvest strategy).
See example at \url{https://docs.salmonmse.com/articles/decision-table.html}. More examples below.
\itemize{
\item \code{plot_decision_table()} is a simple figure where colour range is intended to continuously transition from pink to white to green corresponding to values of 0, 0.5, and 1, respectively.
\item \code{plot_decision_table2()} is converts performance metrics values into bins and provides more user control in the colour scheme
}
}
\examples{
# Simple decision table
results <- data.frame(
  PNI = c(0.7, 0.23, 0.05, 0.9, 0.85, 0.74, 0.95, 0.92, 0.9),
  pNOB = rep(c(0.5, 0.75, 1), each = 3),
  ER = rep(c(0.2, 0.3, 0.4), 3),
  scenario = "High productivity"
)
plot_decision_table(
  x = results$ER,
  y = results$pNOB,
  z = results$PNI,
  title = "PNI",
  xlab = "Exploitation rate",
  ylab = "pNOB target"
)

# Multiple decision tables organized by scenario
# Continuing from above
results_low <- results
results_low$scenario <- "Low productivity"
results_low$PNI <- 0.5 * results$PNI

results_all <- rbind(results, results_low)
plot_decision_table(
  x = results_all$ER,
  y = results_all$pNOB,
  z = results_all$PNI,
  title = "PNI",
  xlab = "Exploitation rate",
  ylab = "pNOB target",
  scenario = results_all$scenario
)
# Example of binned decision table

df <- expand.grid(
  SAR = seq(0.005, 0.03, 0.005),
  ER = seq(0, 0.5, 0.1)
)
df$value <- ifelse(5 * df$SAR + 0.2 > df$ER, 0.75, 0.05)
df$value <- ifelse(df$SAR < 0.02 & df$ER > 0.4, 0.04, df$value)

plot_decision_table2(
  x = df$SAR,
  y = df$ER,
  z = df$value,
  xlab = "Marine survival",
  ylab = "Exploitation rate",
  zlab = "Probability\nof objective"
)
}
\seealso{
\code{\link[=plot_statevar_ts]{plot_statevar_ts()}} \code{\link[=plot_tradeoff]{plot_tradeoff()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/define-SOM.R
\name{report}
\alias{report}
\alias{report,SMSE-method}
\title{Generate markdown reports}
\usage{
\S4method{report}{SMSE}(
  object,
  name = object@Name,
  filename = "SMSE",
  dir = tempdir(),
  open_file = TRUE,
  render_args = list(),
  ...
)
}
\arguments{
\item{object}{\linkS4class{SMSE} object}

\item{name}{Character string for the model name to include in the report, e.g., model run number.}

\item{filename}{Character string for the name of the markdown and HTML files.}

\item{dir}{The directory in which the markdown and HTML files will be saved.}

\item{open_file}{Logical, whether the HTML document is opened after it is rendered.}

\item{render_args}{List of arguments to pass to \code{\link[rmarkdown:render]{rmarkdown::render()}}.}

\item{...}{Additional arguments (not used)}
}
\value{
Returns invisibly the output of \code{\link[rmarkdown:render]{rmarkdown::render()}}, typically the path of the output file
}
\description{
Generate a markdown report for outcomes from a single operating model projection
}
\seealso{
\code{\link[=compare]{compare()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/define-SOM.R
\docType{class}
\name{Harvest-class}
\alias{Harvest-class}
\title{Class \code{"Harvest"}}
\description{
The component of the operating model that controls marine harvest.
}
\section{Slots}{

\describe{
\item{\code{Name}}{Character. Identifying name}

\item{\code{type_PT}}{Character. Whether to manage preterminal fishery catch from exploitation rate ("u") or catch target ("catch"). Default is "u",}

\item{\code{type_T}}{Character. Whether to manage terminal fishery catch from exploitation rate ("u") or catch target ("catch"). Default is "u",}

\item{\code{u_preterminal}}{Numeric. If \code{type_PT = "u"}, the exploitation rate of the immature stock in the pre-terminal fishery. This will be converted
to an instantaneous fishing mortality rate, i.e., \code{F_preterminal = -log(1 - u_preterminal)}.}

\item{\code{u_terminal}}{Numeric. If \code{type_T = "u"}, The exploitation rate of the return in the terminal fishery. This will be converted
to an instantaneous fishing mortality rate, i.e., \code{F_terminal = -log(1 - u_terminal)}.}

\item{\code{K_PT}}{Numeric. If \code{type_PT = "catch"}, the catch target of the immature stock in the pre-terminal fishery.}

\item{\code{K_T}}{Numeric. If \code{type_T = "catch"}, the catch target of the return in the terminal fishery.}

\item{\code{MSF_PT}}{Logical. Whether to implement mark-selective fishing in the preterminal fishery, with no retention on unmarked fish.}

\item{\code{MSF_T}}{Logical. Whether to implement mark-selective fishing in the terminal fishery, with no retention on unmarked fish.}

\item{\code{release_mort}}{Vector length 2. The proportion of released fish that die after release, in the pre-terminal and terminal fishery.
Implemented to model mark-selective fishing. Not used if either \code{MSF_PT} or \code{MSF_T} is \code{ FALSE}.}

\item{\code{vulPT}}{Vector length \code{maxage} or matrix \verb{[nsim, maxage]}. Vulnerability schedule (between 0-1) in the preterminal fishery. Values indicate
the proportion of fishing mortality experienced by each age class, where \code{F_preterminal = -log(1 - u_preterminal)}.}

\item{\code{vulT}}{Vector length \code{maxage} or matrix \verb{[nsim, maxage]}. Vulnerability schedule (between 0-1) in the terminal fishery. Values indicate
the proportion of fishing mortality experienced by each age class, where \code{F_terminal = -log(1 - u_terminal)}.}
}}

\section{Creating Object}{

Objects can be created by calls of the form \code{new("Harvest")}
}

\examples{
showClass("Harvest")
}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot-statevar.R
\name{plot_tradeoff}
\alias{plot_tradeoff}
\title{Tradeoff figure}
\usage{
plot_tradeoff(
  pm1,
  pm2,
  x1,
  x2,
  xlab,
  ylab,
  x1lab,
  x2lab,
  scenario,
  scenario_rows,
  scenario_cols,
  ncol = NULL,
  dir = "v",
  add_letters = FALSE
)
}
\arguments{
\item{pm1}{Numeric or matrix. A vector of values for the first performance metric on the x-axis. Alternatively, provide a three column matrix corresponding
to the lower bound, central tendency, and upper bound.}

\item{pm2}{Numeric or matrix. A vector of values for the second performance metric on the y-axis (same length as pm1). Alternatively, provide a three column matrix corresponding
to the lower bound, central tendency, and upper bound.}

\item{x1}{Atomic, vector of values for the first grouping variable. Various levels are represented by colours. Same length as pm1. If numeric, you may want to convert values to factors.}

\item{x2}{Atomic, vector of values for the second grouping variable. Various levels are represented by shapes. Same length as pm1. If numeric, will be converted to factors.}

\item{xlab}{Character, optional x-axis label}

\item{ylab}{Character, optional y-axis label}

\item{x1lab}{Character, optional label for the first grouping variable}

\item{x2lab}{Character, optional label for the second grouping variable}

\item{scenario}{Atomic, vector of scenario names (same length as \code{pm1}, \code{pm2}) used to identify and create separate panels}

\item{scenario_rows}{Atomic, vector of scenario variables (same length as \code{pm1}, \code{pm2}) used to identify the rows in a grid of tradeoff figures. Use with \code{scenario_cols}}

\item{scenario_cols}{Atomic, vector of scenario variables (same length as \code{pm1}, \code{pm2}) used to identify the columns in a grid of tradeoff figures. Use with \code{scenario_cols}}

\item{ncol}{Integer, number of columns in the grid, only used if \code{scenario} is provided}

\item{dir}{Character, either "h" or "v" to describe how the grid of tables should be organized (horizontally or vertically) , only used if \code{scenario} is provided}

\item{add_letters}{Logical, whether to identify separate panels by letters (only used if scenarios are identified). Helpful for publication-level figures}
}
\value{
ggplot object
}
\description{
Generates a tradeoff figure, a comparison between two performance metrics, across two variables which may represent
a population dynamics variable (e.g., productivity) or a management action (e.g., hatchery production levels or harvest strategy).
See example at \url{https://docs.salmonmse.com/articles/decision-table.html}
}
\examples{

# Single tradeoff panel
results <- data.frame(
  PNI = c(0.7, 0.23, 0.05, 0.9, 0.85, 0.74, 0.95, 0.92, 0.9),
  Catch = c(10, 14, 12, 8, 7, 7, 8.2, 7.1, 6.9),
  pNOB = rep(c(0.5, 0.75, 1), each = 3),
  ER = rep(c(0.2, 0.3, 0.4), 3),
  scenario = "High productivity"
)

plot_tradeoff(
  pm1 = results$PNI,
  pm2 = results$Catch,
  x1 = results$ER,
  x2 = results$pNOB,
  xlab = "PNI",
  ylab = "Catch",
  x1lab = "Exploitation\nrate",
  x2lab = "pNOB\ntarget"
)

# Multiple panels, continuing from above code
results_low <- results
results_low$scenario <- "Low productivity"
results_low$PNI <- 0.5 * results$PNI
results_low$Catch <- c(0.9, 0.8, 0.7) * results$Catch

results_all <- rbind(results, results_low)
g <- plot_tradeoff(
  pm1 = results_all$PNI,
  pm2 = results_all$Catch,
  x1 = results_all$ER,
  x2 = results_all$pNOB,
  xlab = "PNI",
  ylab = "Catch",
  x1lab = "Exploitation\nrate",
  x2lab = "pNOB\ntarget",
  scenario = results_all$scenario
)

# Change legend with ggplot2
library(ggplot2)
g + scale_shape_manual(values = c(1, 4, 16))


}
\seealso{
\code{\link[=plot_statevar_ts]{plot_statevar_ts()}} \code{\link[=plot_decision_table]{plot_decision_table()}}
}
